@@ -2580,8 +2580,8 @@ Windows line endings first:
 
     >>> import tempfile, os
     >>> fn = tempfile.mktemp()
-    >>> with open(fn, 'w') as fobj:
-    ...     fobj.write('Test:\r\n\r\n  >>> x = 1 + 1\r\n\r\nDone.\r\n')
+    >>> with open(fn, 'wb') as f:
+    ...     f.write('Test:\r\n\r\n  >>> x = 1 + 1\r\n\r\nDone.\r\n')
     >>> doctest.testfile(fn, False)
     TestResults(failed=0, attempted=1)
     >>> os.remove(fn)
@@ -2589,8 +2589,8 @@ Windows line endings first:
 And now *nix line endings:
 
     >>> fn = tempfile.mktemp()
-    >>> with open(fn, 'w') as fobj:
-    ...     fobj.write('Test:\n\n  >>> x = 1 + 1\n\nDone.\n')
+    >>> with open(fn, 'wb') as f:
+    ...     f.write('Test:\n\n  >>> x = 1 + 1\n\nDone.\n')
     >>> doctest.testfile(fn, False)
     TestResults(failed=0, attempted=1)
     >>> os.remove(fn)
