@@ -9,6 +9,8 @@ import warnings
 import sys
 import signal
 import subprocess
+import sysconfig
+import textwrap
 import time
 try:
     import resource
@@ -571,6 +573,12 @@ class URandomTests (unittest.TestCase):
         data2 = self.get_urandom_subprocess(16)
         self.assertNotEqual(data1, data2)
 
+
+HAVE_GETENTROPY = (sysconfig.get_config_var('HAVE_GETENTROPY') == 1)
+
+@unittest.skipIf(HAVE_GETENTROPY,
+                 "getentropy() does not use a file descriptor")
+class URandomFDTests(unittest.TestCase):
     @unittest.skipUnless(resource, "test requires the resource module")
     def test_urandom_failure(self):
         # Check urandom() failing when it is not able to open /dev/random.
@@ -886,6 +894,7 @@ def test_main():
         MakedirTests,
         DevNullTests,
         URandomTests,
+        URandomFDTests,
         ExecvpeTests,
         Win32ErrorTests,
         TestInvalidFD,
