@@ -803,7 +803,7 @@ class ThreadingExceptionTests(BaseTestCase):
                 running = True
                 while running:
                     time.sleep(0.01)
-                1/0
+                1.0/0.0
             t = threading.Thread(target=run)
             t.start()
             while not running:
@@ -830,7 +830,7 @@ class ThreadingExceptionTests(BaseTestCase):
                 running = True
                 while running:
                     time.sleep(0.01)
-                1/0
+                1.0/0.0
             t = threading.Thread(target=run)
             t.start()
             while not running:
@@ -858,7 +858,7 @@ class ThreadingExceptionTests(BaseTestCase):
                 running = True
                 while running:
                     time.sleep(0.01)
-                1/0
+                1.0/0.0
             sys.stderr = None
             t = threading.Thread(target=run)
             t.start()
