@@ -28,7 +28,7 @@ class PointerType(_CDataMeta):
         # XXX check if typedict['_type_'] is any sane
         # XXX remember about paramfunc
         obj = type.__new__(self, name, cls, typedict)
-        for k, v in d.items():
+        for k, v in d.iteritems():
             setattr(obj, k, v)
         if '_type_' in typedict:
             self.set_type(obj, typedict['_type_'])
@@ -67,22 +67,15 @@ class PointerType(_CDataMeta):
                 self._buffer = ffiarray(1, autofree=True)
             if value is not None:
                 self.contents = value
-        def _init_no_arg_(self):
-            self._buffer = ffiarray(1, autofree=True)
         self._ffiarray = ffiarray
         self.__init__ = __init__
-        self._init_no_arg_ = _init_no_arg_
         self._type_ = TP
-
-    def _build_ffiargtype(self):
-        return _ffi.types.Pointer(self._type_.get_ffi_argtype())
-
-    def _deref_ffiargtype(self):
-        return self._type_.get_ffi_argtype()
+        self._ffiargtype = _ffi.types.Pointer(TP.get_ffi_argtype())
 
     from_address = cdata_from_address
 
-class _Pointer(_CData, metaclass=PointerType):
+class _Pointer(_CData):
+    __metaclass__ = PointerType
 
     def getcontents(self):
         addr = self._buffer[0]
@@ -121,11 +114,9 @@ class _Pointer(_CData, metaclass=PointerType):
         cobj = self._type_.from_param(value)
         if ensure_objects(cobj) is not None:
             store_reference(self, index, cobj._objects)
-        address = self._buffer[0]
-        address += index * sizeof(self._type_)
-        cobj._copy_to(address)
+        self._subarray(index)[0] = cobj._get_buffer_value()
 
-    def __bool__(self):
+    def __nonzero__(self):
         return self._buffer[0] != 0
 
     contents = property(getcontents, setcontents)
@@ -139,21 +130,23 @@ def _cast_addr(obj, _, tp):
     if not (isinstance(tp, _CDataMeta) and tp._is_pointer_like()):
         raise TypeError("cast() argument 2 must be a pointer type, not %s"
                         % (tp,))
-    result = tp._newowninstance_()
-    if isinstance(obj, int):
+    if isinstance(obj, (int, long)):
+        result = tp()
         result._buffer[0] = obj
         return result
     elif obj is None:
+        result = tp()
         return result
     elif isinstance(obj, Array):
-        result._buffer[0] = obj._buffer
-    elif isinstance(obj, bytes):
-        result._buffer[0] = memoryview(obj)._pypy_raw_address()
-        return result
+        ptr = tp.__new__(tp)
+        ptr._buffer = tp._ffiarray(1, autofree=True)
+        ptr._buffer[0] = obj._buffer
+        result = ptr
     elif not (isinstance(obj, _CData) and type(obj)._is_pointer_like()):
         raise TypeError("cast() argument 1 must be a pointer, not %s"
                         % (type(obj),))
     else:
+        result = tp()
         result._buffer[0] = obj._buffer[0]
 
     # The casted objects '_objects' member:
