@@ -3,15 +3,11 @@
 import sys
 if sys.platform == 'win32':
     #This module does not exist in windows
-    raise ModuleNotFoundError('No module named _curses', name='_curses')
-import locale
+    raise ImportError('No module named _curses')
 from functools import wraps
 
 from _curses_cffi import ffi, lib
 
-version = b"2.2"
-__version__ = b"2.2"
-
 def _copy_to_globals(name):
     globals()[name] = getattr(lib, name)
 
@@ -58,12 +54,16 @@ def _setup():
             if key_n == b"UNKNOWN KEY":
                 continue
             if not isinstance(key_n, str):   # python 3
-                key_n = key_n.decode('utf-8')
+                key_n = key_n.decode()
             key_n = key_n.replace('(', '').replace(')', '')
             globals()[key_n] = key
 
 _setup()
 
+# Do we want this?
+# version = "2.2"
+# __version__ = "2.2"
+
 
 # ____________________________________________________________
 
@@ -84,9 +84,7 @@ def _ensure_initialised():
 
 
 def _ensure_initialised_color():
-    if not _initialised:
-        raise error("must call initscr() first")
-    if not _initialised_color:
+    if not _initialised and _initialised_color:
         raise error("must call start_color() first")
 
 
@@ -167,49 +165,15 @@ def _mk_w_return_val(method_name):
 def _chtype(ch):
     return int(ffi.cast("chtype", ch))
 
-def _bytestype(text):
-    if isinstance(text, bytes):
+def _texttype(text):
+    if isinstance(text, str):
         return text
-    elif isinstance(text, str):
-        return text.encode()
+    elif isinstance(text, unicode):
+        return str(text)   # default encoding
     else:
-        raise TypeError("bytes or str expected, got a '%s' object"
+        raise TypeError("str or unicode expected, got a '%s' object"
                         % (type(text).__name__,))
 
-def _convert_to_chtype(win, obj):
-    if isinstance(obj, bytes) and len(obj) == 1:
-        value = ord(obj)
-    elif isinstance(obj, str):
-        if len(obj) != 1:
-            raise TypeError("expect bytes or str of length 1 or int, "
-                            "got a str of length %d", len(obj))
-        value = ord(obj)
-        if (128 < value):
-            if win:
-                encoding = win.encoding
-            else:
-                encoding = screen_encoding
-            b = obj.encode(encoding)
-            if len(bytes) == 1:
-                value = ord(b)
-            else:
-                OverflowError("byte doesn't fit in chtype")
-    elif isinstance(obj, int):
-        value = obj
-    else:
-        raise TypeError('expect bytes or str of length 1, or int, got %s' % type(obj))
-    return value
-
-def _convert_to_string(win, obj):
-    if isinstance(obj, str):
-        value = obj.encode(win.encoding)
-    elif isinstance(obj, bytes):
-        value = obj
-    else:
-        raise TypeError('expect bytes or str, got %s' % type(obj))
-    if b'\0' in value:
-        raise ValueError('embedded null character') 
-    return value
 
 def _extract_yx(args):
     if len(args) >= 2:
@@ -253,17 +217,8 @@ class error(Exception):
 
 
 class Window(object):
-    def __init__(self, window, encoding=None):
-        if encoding is None:
-            # CPython has a win32 branch here, but _curses is not supported
-            # on win32
-            codeset = locale.nl_langinfo(locale.CODESET)
-            if codeset:
-                encoding = codeset
-            else:
-                encoding = 'utf-8'
+    def __init__(self, window):
         self._win = window
-        self._encoding = encoding
 
     def __del__(self):
         if self._win != lib.stdscr:
@@ -332,7 +287,7 @@ class Window(object):
 
     @_argspec(1, 1, 2)
     def addstr(self, y, x, text, attr=None):
-        text = _convert_to_string(self, text)
+        text = _texttype(text)
         if attr is not None:
             attr_old = lib.getattrs(self._win)
             lib.wattrset(self._win, attr)
@@ -346,7 +301,7 @@ class Window(object):
 
     @_argspec(2, 1, 2)
     def addnstr(self, y, x, text, n, attr=None):
-        text = _convert_to_string(self, text)
+        text = _texttype(text)
         if attr is not None:
             attr_old = lib.getattrs(self._win)
             lib.wattrset(self._win, attr)
@@ -379,15 +334,7 @@ class Window(object):
                     _chtype(tl), _chtype(tr), _chtype(bl), _chtype(br))
         return None
 
-    def box(self, *args):
-        if len(args) == 0:
-            vertint = 0
-            horint = 0
-        elif len(args) == 2:
-            vertint = _convert_to_chtype(self, args[0])
-            horint = _convert_to_chtype(self, args[1])
-        else:
-            raise TypeError('verch,horch required')
+    def box(self, vertint=0, horint=0):
         lib.box(self._win, vertint, horint)
         return None
 
@@ -457,20 +404,6 @@ class Window(object):
             raise error("getch requires 0 or 2 arguments")
         return val
 
-    def get_wch(self, *args):
-        wch = ffi.new("wint_t[1]")
-        if len(args) == 0:
-            val = lib.wget_wch(self._win, wch)
-        elif len(args) == 2:
-            val = lib.mvwget_wch(self._win, *args, wch)
-        else:
-            raise error("get_wch requires 0 or 2 arguments")
-        _check_ERR(val, "get_wch")
-        if val == lib.KEY_CODE_YES:
-            return wch[0]
-        else:
-            return chr(wch[0])
-
     def getkey(self, *args):
         if len(args) == 0:
             val = lib.wgetch(self._win)
@@ -488,16 +421,11 @@ class Window(object):
             val = lib.keyname(val)
             if val == ffi.NULL:
                 return ""
-            key_n = ffi.string(val)
-            if not isinstance(key_n, str):
-                key_n = key_n.decode('utf-8')
-            return key_n
+            return ffi.string(val)
 
     @_argspec(0, 1, 2)
     def getstr(self, y, x, n=1023):
         n = min(n, 1023)
-        if n < 0:
-            raise ValueError("'n' must be nonnegative")
         buf = ffi.new("char[1024]")  # /* This should be big enough.. I hope */
 
         if y is None:
@@ -540,8 +468,6 @@ class Window(object):
     @_argspec(0, 1, 2)
     def instr(self, y, x, n=1023):
         n = min(n, 1023)
-        if n < 0:
-            raise ValueError("'n' must be nonnegative")
         buf = ffi.new("char[1024]")  # /* This should be big enough.. I hope */
         if y is None:
             code = lib.winnstr(self._win, buf, n)
@@ -554,7 +480,7 @@ class Window(object):
 
     @_argspec(1, 1, 2)
     def insstr(self, y, x, text, attr=None):
-        text = _convert_to_string(self, text)
+        text = _texttype(text)
         if attr is not None:
             attr_old = lib.getattrs(self._win)
             lib.wattrset(self._win, attr)
@@ -568,7 +494,7 @@ class Window(object):
 
     @_argspec(2, 1, 2)
     def insnstr(self, y, x, text, n, attr=None):
-        text = _convert_to_string(self, text)
+        text = _texttype(text)
         if attr is not None:
             attr_old = lib.getattrs(self._win)
             lib.wattrset(self._win, attr)
@@ -628,9 +554,6 @@ class Window(object):
     def putwin(self, filep):
         # filestar = ffi.new("FILE *", filep)
         return _check_ERR(lib.putwin(self._win, filep), "putwin")
-        # XXX CPython 3.5 says: We have to simulate this by writing to
-        # a temporary FILE*, then reading back, then writing to the
-        # argument stream.
 
     def redrawln(self, beg, num):
         return _check_ERR(lib.wredrawln(self._win, beg, num), "redrawln")
@@ -662,7 +585,7 @@ class Window(object):
             win = lib.subpad(self._win, nlines, ncols, begin_y, begin_x)
         else:
             win = lib.subwin(self._win, nlines, ncols, begin_y, begin_x)
-        return Window(_check_NULL(win), self.encoding)
+        return Window(_check_NULL(win))
 
     def scroll(self, nlines=None):
         if nlines is None:
@@ -686,23 +609,6 @@ class Window(object):
             _check_ERR(lib.wmove(self._win, y, x), "wmove")
         return _check_ERR(lib.wvline(self._win, ch | attr, n), "vline")
 
-    @property
-    def encoding(self):
-        return self._encoding
-
-    @encoding.setter
-    def encoding(self, val):
-        if not val:
-            raise TypeError('encoding may not be deleted')
-        if not isinstance(val, str):
-            raise TypeError('setting encoding to a non-string')
-        encoding = val.encode('ascii')
-        self._encoding = val 
-
-    @encoding.deleter
-    def encoding(self):
-        raise TypeError('encoding may not be deleted')
-        
 
 beep = _mk_no_return("beep")
 def_prog_mode = _mk_no_return("def_prog_mode")
@@ -798,7 +704,6 @@ if lib._m_NCURSES_MOUSE_VERSION:
 
 
 def getwin(filep):
-    # XXX CPython 3.5: there's logic to use a temp file instead
     return Window(_check_NULL(lib.getwin(filep)))
 
 
@@ -890,9 +795,7 @@ def initscr():
     globals()["LINES"] = lib.LINES
     globals()["COLS"] = lib.COLS
 
-    window = Window(win)
-    globals()['screen_encoding'] = window.encoding
-    return window
+    return Window(win)
 
 
 def setupterm(term=None, fd=-1):
@@ -905,8 +808,6 @@ def setupterm(term=None, fd=-1):
 
     if term is None:
         term = ffi.NULL
-    elif isinstance(term, str):
-        term = term.encode()
     err = ffi.new("int *")
     if lib.setupterm(term, fd, err) == lib.ERR:
         err = err[0]
@@ -999,7 +900,7 @@ def pair_number(pairvalue):
 
 
 def putp(text):
-    text = _bytestype(text)
+    text = _texttype(text)
     return _check_ERR(lib.putp(text), "putp")
 
 
@@ -1012,29 +913,101 @@ def qiflush(flag=True):
     return None
 
 
-# Internal helper used for updating curses.LINES, curses.COLS, _curses.LINES
-# and _curses.COLS
-def update_lines_cols():
-    globals()["LINES"] = lib.LINES
-    globals()["COLS"] = lib.COLS
-    try:
-        m = sys.modules["curses"]
-        m.LINES = lib.LINES
-        m.COLS = lib.COLS
-    except (KeyError, AttributeError):
-        pass
+# XXX: Do something about the following?
+# /* Internal helper used for updating curses.LINES, curses.COLS, _curses.LINES
+#  * and _curses.COLS */
+# #if defined(HAVE_CURSES_RESIZETERM) || defined(HAVE_CURSES_RESIZE_TERM)
+# static int
+# update_lines_cols(void)
+# {
+#     PyObject *o;
+#     PyObject *m = PyImport_ImportModuleNoBlock("curses");
 
+#     if (!m)
+#         return 0;
 
-def resizeterm(lines, columns):
-    _ensure_initialised()
-    _check_ERR(lib.resizeterm(lines, columns), "resizeterm")
-    update_lines_cols()
+#     o = PyInt_FromLong(LINES);
+#     if (!o) {
+#         Py_DECREF(m);
+#         return 0;
+#     }
+#     if (PyObject_SetAttrString(m, "LINES", o)) {
+#         Py_DECREF(m);
+#         Py_DECREF(o);
+#         return 0;
+#     }
+#     if (PyDict_SetItemString(ModDict, "LINES", o)) {
+#         Py_DECREF(m);
+#         Py_DECREF(o);
+#         return 0;
+#     }
+#     Py_DECREF(o);
+#     o = PyInt_FromLong(COLS);
+#     if (!o) {
+#         Py_DECREF(m);
+#         return 0;
+#     }
+#     if (PyObject_SetAttrString(m, "COLS", o)) {
+#         Py_DECREF(m);
+#         Py_DECREF(o);
+#         return 0;
+#     }
+#     if (PyDict_SetItemString(ModDict, "COLS", o)) {
+#         Py_DECREF(m);
+#         Py_DECREF(o);
+#         return 0;
+#     }
+#     Py_DECREF(o);
+#     Py_DECREF(m);
+#     return 1;
+# }
+# #endif
 
+# #ifdef HAVE_CURSES_RESIZETERM
+# static PyObject *
+# PyCurses_ResizeTerm(PyObject *self, PyObject *args)
+# {
+#     int lines;
+#     int columns;
+#     PyObject *result;
 
-def resize_term(lines, columns):
-    _ensure_initialised()
-    _check_ERR(lib.resize_term(lines, columns), "resize_term")
-    update_lines_cols()
+#     PyCursesInitialised;
+
+#     if (!PyArg_ParseTuple(args,"ii:resizeterm", &lines, &columns))
+#         return NULL;
+
+#     result = PyCursesCheckERR(resizeterm(lines, columns), "resizeterm");
+#     if (!result)
+#         return NULL;
+#     if (!update_lines_cols())
+#         return NULL;
+#     return result;
+# }
+
+# #endif
+
+# #ifdef HAVE_CURSES_RESIZE_TERM
+# static PyObject *
+# PyCurses_Resize_Term(PyObject *self, PyObject *args)
+# {
+#     int lines;
+#     int columns;
+
+#     PyObject *result;
+
+#     PyCursesInitialised;
+
+#     if (!PyArg_ParseTuple(args,"ii:resize_term", &lines, &columns))
+#         return NULL;
+
+#     result = PyCursesCheckERR(resize_term(lines, columns), "resize_term");
+#     if (!result)
+#         return NULL;
+#     if (!update_lines_cols())
+#         return NULL;
+#     return result;
+# }
+# #endif /* HAVE_CURSES_RESIZE_TERM */
 
 
 def setsyx(y, x):
@@ -1053,17 +1026,23 @@ def start_color():
 
 def tigetflag(capname):
     _ensure_initialised_setupterm()
-    return lib.tigetflag(capname.encode())
+    if isinstance(capname, unicode):
+        capname = capname.encode('ascii')
+    return lib.tigetflag(capname)
 
 
 def tigetnum(capname):
     _ensure_initialised_setupterm()
-    return lib.tigetnum(capname.encode())
+    if isinstance(capname, unicode):
+        capname = capname.encode('ascii')
+    return lib.tigetnum(capname)
 
 
 def tigetstr(capname):
     _ensure_initialised_setupterm()
-    val = lib.tigetstr(capname.encode())
+    if isinstance(capname, unicode):
+        capname = capname.encode('ascii')
+    val = lib.tigetstr(capname)
     if int(ffi.cast("intptr_t", val)) in (0, -1):
         return None
     return ffi.string(val)
@@ -1071,13 +1050,6 @@ def tigetstr(capname):
 
 def tparm(fmt, i1=0, i2=0, i3=0, i4=0, i5=0, i6=0, i7=0, i8=0, i9=0):
     args = [ffi.cast("int", i) for i in (i1, i2, i3, i4, i5, i6, i7, i8, i9)]
-    # fmt is expected to be a byte string; CPython 3.x complains
-    # "TypeError: 'str' does not support the buffer interface", but we
-    # can do better.
-    if isinstance(fmt, str):
-        # error message modeled on "TypeError: must be str, not bytes"
-        # that you get if you call curses.tigetstr(b'...') on CPython 3.x
-        raise TypeError('must be bytes, not str')
     result = lib.tparm(fmt, *args)
     if result == ffi.NULL:
         raise error("tparm() returned NULL")
@@ -1099,17 +1071,6 @@ def ungetch(ch):
     return _check_ERR(lib.ungetch(_chtype(ch)), "ungetch")
 
 
-def unget_wch(ch):
-    _ensure_initialised()
-    if isinstance(ch, str):
-        if len(ch) != 1:
-            raise TypeError("expect bytes or str of length1, or int, "
-                            "got a str of length %d" % len(ch))
-    elif isinstance(ch, int): 
-        ch = chr(ch)
-    return _check_ERR(lib.unget_wch(ch), "unget_wch")
-
-
 def use_env(flag):
     lib.use_env(flag)
     return None
