@@ -8,7 +8,7 @@ def parse_doc(s):
     startrev = None
     branches = set()
     def parseline(line):
-        _, value = line.split('#')[0].split(':', 1)
+        _, value = line.split(':', 1)
         return value.strip()
     #
     for line in s.splitlines():
@@ -85,12 +85,10 @@ def test_whatsnew():
     #whatsnew_list = doc.listdir('whatsnew-*.rst')
     #whatsnew_list.sort()
     #last_whatsnew = whatsnew_list[-1].read()
-    last_whatsnew = doc.join('whatsnew-pypy3-head.rst').read()
+    last_whatsnew = doc.join('whatsnew-head.rst').read()
     startrev, documented = parse_doc(last_whatsnew)
     merged, branch = get_merged_branches(ROOT, startrev, '')
     merged.discard('default')
-    merged.discard('py3.7')
-    merged.discard('py3.6')
     merged.discard('')
     not_documented = merged.difference(documented)
     not_merged = documented.difference(merged)
@@ -101,10 +99,8 @@ def test_whatsnew():
     print '\n'.join(not_merged)
     print
     assert not not_documented
-    if branch == 'py3.7':
+    if branch == 'default':
         assert not not_merged
-    else:
-        assert branch in documented, 'Please document this branch before merging: %s' % branch
 
 def test_startrev_on_default():
     doc = ROOT.join('pypy', 'doc')
@@ -114,4 +110,4 @@ def test_startrev_on_default():
         "hg log -r %s --template '{branch}'" % startrev)
     if errcode != 0:
         py.test.skip('no Mercurial repo')
-    assert wc_branch in ('default', "'default'") # sometimes the ' leaks (windows)
+    assert wc_branch == 'default'
