@@ -56,7 +56,6 @@ class ASTNodeVisitor(ASDLVisitor):
 
     def visitSum(self, sum, base):
         if is_simple_sum(sum):
-            assert not sum.attributes
             self.emit("class %s(AST):" % (base,))
             self.emit("@staticmethod", 1)
             self.emit("def from_object(space, w_node):", 1)
@@ -86,7 +85,7 @@ class ASTNodeVisitor(ASDLVisitor):
             self.emit("class %s(AST):" % (base,))
             if sum.attributes:
                 self.emit("")
-                args = ", ".join(attr.name for attr in sum.attributes)
+                args = ", ".join(attr.name.value for attr in sum.attributes)
                 self.emit("def __init__(self, %s):" % (args,), 1)
                 for attr in sum.attributes:
                     self.visit(attr)
@@ -102,8 +101,8 @@ class ASTNodeVisitor(ASDLVisitor):
                           % (typ.name,), 3)
             self.emit("raise oefmt(space.w_TypeError,", 2)
             self.emit("        \"Expected %s node, got %%T\", w_node)" % (base,), 2)
-            self.emit("State.ast_type(%r, 'AST', None, %s)" %
-                      (base, [attr.name for attr in sum.attributes]))
+            self.emit("State.ast_type('%r', 'AST', None, %s)" %
+                      (base, [repr(attr.name) for attr in sum.attributes]))
             self.emit("")
             for cons in sum.types:
                 self.visit(cons, base, sum.attributes)
@@ -112,77 +111,47 @@ class ASTNodeVisitor(ASDLVisitor):
     def visitProduct(self, product, name):
         self.emit("class %s(AST):" % (name,))
         self.emit("")
-        self.make_constructor(product.fields + product.attributes, product)
+        self.make_constructor(product.fields, product)
         self.emit("")
         self.make_mutate_over(product, name)
         self.emit("def walkabout(self, visitor):", 1)
         self.emit("visitor.visit_%s(self)" % (name,), 2)
         self.emit("")
-        self.make_converters(product.fields + product.attributes, name)
-        if product.attributes:
-            attr_names = ', %s' % ([a.name for a in product.attributes],)
-        else:
-            attr_names = ''
-        self.emit("State.ast_type(%r, 'AST', %s%s)" %
-                  (name, [f.name for f in product.fields], attr_names))
+        self.make_converters(product.fields, name)
+        self.emit("State.ast_type('%r', 'AST', %s)" %
+                  (name, [repr(f.name) for f in product.fields]))
         self.emit("")
 
     def get_value_converter(self, field, value):
-        if field.type in self.data.simple_types:
+        if field.type.value in self.data.simple_types:
             return "%s_to_class[%s - 1]().to_object(space)" % (field.type, value)
-        elif field.type in ("object", "singleton", "constant",
-                            "bytes", "string"):
+        elif field.type.value in ("object", "string"):
             return value
-        elif field.type == "bool":
-            return "space.newbool(%s)" % (value,)
-        elif field.type == "int":
-            return "space.newint(%s)" % (value,)
-        elif field.type == "identifier":
-            if field.opt:
-                return "space.newtext_or_none(%s)" % (value,)
-            return "space.newtext(%s)" % (value,)
+        elif field.type.value in ("identifier", "int", "bool"):
+            return "space.wrap(%s)" % (value,)
         else:
             wrapper = "%s.to_object(space)" % (value,)
-            allow_none = field.opt
-            # Some sequences allow None values:
-            # - arguments.kw_defaults (for mandatory kw-only arguments)
-            # - Dict.keys (for **nested_dict elements)
-            if field.name in ('kw_defaults', 'keys'):
-                allow_none = True
-            if allow_none:
+            if field.opt:
                 wrapper += " if %s is not None else space.w_None" % (value,)
             return wrapper
-
+        
     def get_value_extractor(self, field, value):
-        if field.type in self.data.simple_types:
+        if field.type.value in self.data.simple_types:
             return "%s.from_object(space, %s)" % (field.type, value)
-        elif field.type in ("object", "singleton", "constant"):
+        elif field.type.value in ("object",):
             return value
-        elif field.type in ("string", "bytes"):
-            return "check_string(space, %s, %d)" % (value, field.opt)
-        elif field.type in ("identifier",):
+        elif field.type.value in ("string",):
+            return "check_string(space, %s)" % (value,)
+        elif field.type.value in ("identifier",):
             if field.opt:
-                return "space.text_or_none_w(%s)" % (value,)
-            return "space.text_w(%s)" % (value,)
-        elif field.type in ("int",):
-            return "obj_to_int(space, %s, %s)" % (value, field.opt)
-        elif field.type in ("bool",):
+                return "space.str_or_None_w(%s)" % (value,)
+            return "space.realstr_w(%s)" % (value,)
+        elif field.type.value in ("int",):
+            return "space.int_w(%s)" % (value,)
+        elif field.type.value in ("bool",):
             return "space.bool_w(%s)" % (value,)
         else:
-            extractor = "%s.from_object(space, %s)" % (field.type, value)
-            if field.opt:
-                if field.type == 'expr':
-                    # the expr.from_object() method should accept w_None and
-                    # return None; nothing more to do here
-                    pass
-                elif field.type == 'arg':
-                    # the method arg.from_object() doesn't accept w_None
-                    extractor += (
-                        ' if not space.is_w(%s, space.w_None) else None'
-                        % (value,))
-                else:
-                    raise NotImplementedError(field.type)
-            return extractor
+            return "%s.from_object(space, %s)" % (field.type, value)
 
     def get_field_converter(self, field):
         if field.seq:
@@ -210,11 +179,6 @@ class ASTNodeVisitor(ASDLVisitor):
         else:
             value = self.get_value_extractor(field, "w_%s" % (field.name,))
             lines = ["_%s = %s" % (field.name, value)]
-            if not field.opt and field.type not in ("int",):
-                lines.append("if _%s is None:" % (field.name,))
-                lines.append("    raise_required_value(space, w_node, '%s')"
-                             % (field.name,))
-
         return lines
 
     def make_converters(self, fields, name, extras=None):
@@ -225,7 +189,7 @@ class ASTNodeVisitor(ASDLVisitor):
             wrapping_code = self.get_field_converter(field)
             for line in wrapping_code:
                 self.emit(line, 2)
-            self.emit("space.setattr(w_node, space.newtext(%r), w_%s)" % (
+            self.emit("space.setattr(w_node, space.wrap(%r), w_%s)" % (
                     str(field.name), field.name), 2)
         self.emit("return w_node", 2)
         self.emit("")
@@ -252,31 +216,24 @@ class ASTNodeVisitor(ASDLVisitor):
             if extras:
                 base_args = ", ".join(str(field.name) for field in extras)
                 self.emit("%s.__init__(self, %s)" % (base, base_args), 2)
-
+    
     def make_mutate_over(self, cons, name):
         self.emit("def mutate_over(self, visitor):", 1)
         for field in cons.fields:
-            if (field.type not in asdl.builtin_types and
-                field.type not in self.data.simple_types):
+            if (field.type.value not in asdl.builtin_types and
+                field.type.value not in self.data.simple_types):
                 if field.opt or field.seq:
                     level = 3
                     self.emit("if self.%s:" % (field.name,), 2)
                 else:
                     level = 2
                 if field.seq:
-                    sub = field.name
-                    self.emit("for i in range(len(self.{})):".format(sub),
-                        level)
-                    self.emit("if self.{}[i] is not None:".format(sub),
-                        level + 1)
-                    self.emit(
-                        "self.{0}[i] = self.{0}[i].mutate_over(visitor)".format(sub),
-                        level + 2)
+                    sub = (field.name,)
+                    self.emit("visitor._mutate_sequence(self.%s)" % sub, level)
                 else:
-                    sub = field.name
-                    self.emit(
-                        "self.{0} = self.{0}.mutate_over(visitor)".format(sub),
-                        level)
+                    sub = (field.name, field.name)
+                    self.emit("self.%s = self.%s.mutate_over(visitor)" % sub,
+                              level)
         self.emit("return visitor.visit_%s(self)" % (name,), 2)
         self.emit("")
 
@@ -290,8 +247,8 @@ class ASTNodeVisitor(ASDLVisitor):
         self.emit("")
         self.make_mutate_over(cons, cons.name)
         self.make_converters(cons.fields, cons.name, extra_attributes)
-        self.emit("State.ast_type(%r, '%s', %s)" %
-                  (cons.name, base, [f.name for f in cons.fields]))
+        self.emit("State.ast_type('%r', '%s', %s)" % 
+                  (cons.name, base, [repr(f.name) for f in cons.fields]))
         self.emit("")
 
     def visitField(self, field):
@@ -307,18 +264,15 @@ class ASTVisitorVisitor(ASDLVisitor):
         self.emit("def visit_sequence(self, seq):", 1)
         self.emit("if seq is not None:", 2)
         self.emit("for node in seq:", 3)
-        self.emit("if node is not None:", 4)
-        self.emit("node.walkabout(self)", 5)
-        self.emit("")
-        self.emit("def visit_kwonlydefaults(self, seq):", 1)
-        self.emit("if seq is not None:", 2)
-        self.emit("for node in seq:", 3)
-        self.emit("if node:", 4)
-        self.emit("node.walkabout(self)", 5)
+        self.emit("node.walkabout(self)", 4)
         self.emit("")
         self.emit("def default_visitor(self, node):", 1)
         self.emit("raise NodeVisitorNotImplemented", 2)
         self.emit("")
+        self.emit("def _mutate_sequence(self, seq):", 1)
+        self.emit("for i in range(len(seq)):", 2)
+        self.emit("seq[i] = seq[i].mutate_over(self)", 3)
+        self.emit("")
         super(ASTVisitorVisitor, self).visitModule(mod)
         self.emit("")
 
@@ -341,9 +295,6 @@ class GenericASTVisitorVisitor(ASDLVisitor):
     def visitModule(self, mod):
         self.emit("class GenericASTVisitor(ASTVisitor):")
         self.emit("")
-        self.emit("def visited(self, node):", 1)
-        self.emit("pass  # base implementation", 2)
-        self.emit("")
         super(GenericASTVisitorVisitor, self).visitModule(mod)
         self.emit("")
 
@@ -360,7 +311,6 @@ class GenericASTVisitorVisitor(ASDLVisitor):
 
     def make_visitor(self, name, fields):
         self.emit("def visit_%s(self, node):" % (name,), 1)
-        self.emit("self.visited(node)", 2)
         have_body = False
         for field in fields:
             if self.visitField(field):
@@ -370,8 +320,8 @@ class GenericASTVisitorVisitor(ASDLVisitor):
         self.emit("")
 
     def visitField(self, field):
-        if (field.type not in asdl.builtin_types and
-            field.type not in self.data.simple_types):
+        if (field.type.value not in asdl.builtin_types and 
+            field.type.value not in self.data.simple_types):
             level = 2
             template = "node.%s.walkabout(self)"
             if field.seq:
@@ -412,7 +362,7 @@ class ASDLData(object):
             if isinstance(tp.value, asdl.Sum):
                 sum = tp.value
                 if is_simple_sum(sum):
-                    simple_types.add(tp.name)
+                    simple_types.add(tp.name.value)
                 else:
                     attrs = [field for field in sum.attributes]
                     for cons in sum.types:
@@ -420,7 +370,7 @@ class ASDLData(object):
                         cons_attributes[cons] = attrs
             else:
                 prod = tp.value
-                prod_simple.add(tp.name)
+                prod_simple.add(tp.name.value)
                 add_masks(prod.fields, prod)
         prod_simple.update(simple_types)
         self.cons_attributes = cons_attributes
@@ -434,35 +384,23 @@ class ASDLData(object):
 HEAD = r"""# Generated by tools/asdl_py.py
 from rpython.tool.pairtype import extendabletype
 from rpython.tool.sourcetools import func_with_new_name
-from rpython.rlib.objectmodel import specialize
 
 from pypy.interpreter import typedef
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import interp2app
 
-@specialize.arg(0)
-def build(cls, *args):
-    ''' works like calling cls, but replace the four positions that come at the
-    end of the arguments with the positions in the concrete syntax tree node
-    passed as the last argument. '''
-    newargs = args[:-1] + (args[-1].get_lineno(), args[-1].get_column(),
-        args[-1].get_end_lineno(), args[-1].get_end_column())
-    return cls(*newargs)
+
+def raise_attriberr(space, w_obj, name):
+    raise oefmt(space.w_AttributeError,
+                "'%T' object has no attribute '%s'", w_obj, name)
 
 
-def raise_required_value(space, w_obj, name):
-    raise oefmt(space.w_ValueError,
-                "field %s is required for %T", name, w_obj)
-
-def check_string(space, w_obj, allow_none=False):
-    if allow_none and space.is_w(w_obj, space.w_None):
-        return w_obj
-
-    if not (space.isinstance_w(w_obj, space.w_bytes) or
+def check_string(space, w_obj):
+    if not (space.isinstance_w(w_obj, space.w_str) or
             space.isinstance_w(w_obj, space.w_unicode)):
-        raise oefmt(space.w_TypeError,
-                    "AST string must be of type str or unicode")
+        raise OperationError(space.w_TypeError, space.wrap(
+                'AST string must be of type str or unicode'))
     return w_obj
 
 def get_field(space, w_node, name, optional):
@@ -474,18 +412,9 @@ def get_field(space, w_node, name, optional):
         w_obj = space.w_None
     return w_obj
 
-def obj_to_int(space, w_value, optional):
-    if optional and space.is_w(w_value, space.w_None):
-        return 0
-    if not space.isinstance_w(w_value, space.w_long):
-        raise oefmt(space.w_ValueError,
-                    "invalid integer value: %R", w_value)
-    return space.int_w(w_value)
-
 
 class AST(object):
     __metaclass__ = extendabletype
-    _attrs_ = ['lineno', 'col_offset', 'end_lineno', 'end_col_offset']
 
     def walkabout(self, visitor):
         raise AssertionError("walkabout() implementation not provided")
@@ -493,15 +422,6 @@ class AST(object):
     def mutate_over(self, visitor):
         raise AssertionError("mutate_over() implementation not provided")
 
-    def copy_location(self, node_start, node_end=None):
-        if node_end is None:
-            node_end = node_start
-        self.lineno = node_start.get_lineno()
-        self.col_offset = node_start.get_column()
-        self.end_lineno = node_end.get_end_lineno()
-        self.end_col_offset = node_end.get_end_column()
-        return self
-
 
 class NodeVisitorNotImplemented(Exception):
     pass
@@ -511,10 +431,10 @@ class _FieldsWrapper(W_Root):
     "Hack around the fact we can't store tuples on a TypeDef."
 
     def __init__(self, fields):
-        assert fields == []
+        self.fields = fields
 
-    def spacebind(self, space):
-        return space.newtuple([])
+    def __spacebind__(self, space):
+        return space.newtuple([space.wrap(field) for field in self.fields])
 
 
 class W_AST(W_Root):
@@ -530,14 +450,14 @@ class W_AST(W_Root):
         if w_dict is None:
             w_dict = space.newdict()
         w_type = space.type(self)
-        w_fields = space.getattr(w_type, space.newtext("_fields"))
+        w_fields = space.getattr(w_type, space.wrap("_fields"))
         for w_name in space.fixedview(w_fields):
             try:
                 space.setitem(w_dict, w_name,
                           space.getattr(self, w_name))
             except OperationError:
                 pass
-        w_attrs = space.findattr(w_type, space.newtext("_attributes"))
+        w_attrs = space.findattr(w_type, space.wrap("_attributes"))
         if w_attrs:
             for w_name in space.fixedview(w_attrs):
                 try:
@@ -556,35 +476,28 @@ class W_AST(W_Root):
 
 def W_AST_new(space, w_type, __args__):
     node = space.allocate_instance(W_AST, w_type)
-    return node
+    return space.wrap(node)
 
 def W_AST_init(space, w_self, __args__):
     args_w, kwargs_w = __args__.unpack()
     fields_w = space.fixedview(space.getattr(space.type(w_self),
-                               space.newtext("_fields")))
+                               space.wrap("_fields")))
     num_fields = len(fields_w) if fields_w else 0
-    if args_w and len(args_w) > num_fields:
-        suffix = 's' if num_fields == 1 else ''
-        raise oefmt(space.w_TypeError,
-            "%T constructor takes at most %d positional argument%s", w_self,
-            num_fields, suffix)
-    if args_w:
-        for i in range(min(len(fields_w), len(args_w))):
-            w_field = fields_w[i]
-            w_arg = args_w[i]
-            space.setattr(w_self, w_field, w_arg)
-    # XXX bit wrong complexity but should be fine
-    for field, w_value in kwargs_w.iteritems():
-        found = len(args_w)
-        for i, w_field in enumerate(fields_w):
-            if space.text_w(w_field) == field:
-                found = i
-                break
-        if found < len(args_w):
+    if args_w and len(args_w) != num_fields:
+        if num_fields == 0:
             raise oefmt(space.w_TypeError,
-                "%T got multiple values for argument '%8'",
-                w_self, field)
-        space.setattr(w_self, space.newtext(field), w_value)
+                "%T constructor takes 0 positional arguments", w_self)
+        elif num_fields == 1:
+            raise oefmt(space.w_TypeError,
+                "%T constructor takes either 0 or %d positional argument", w_self, num_fields)
+        else:
+            raise oefmt(space.w_TypeError,
+                "%T constructor takes either 0 or %d positional arguments", w_self, num_fields)
+    if args_w:
+        for i, w_field in enumerate(fields_w):
+            space.setattr(w_self, w_field, args_w[i])
+    for field, w_value in kwargs_w.iteritems():
+        space.setattr(w_self, space.wrap(field), w_value)
 
 
 W_AST.typedef = typedef.TypeDef("_ast.AST",
@@ -609,20 +522,20 @@ class State:
         self.w_AST = space.gettypeobject(W_AST.typedef)
         for (name, base, fields, attributes) in self.AST_TYPES:
             self.make_new_type(space, name, base, fields, attributes)
-
+        
     def make_new_type(self, space, name, base, fields, attributes):
         w_base = getattr(self, 'w_%s' % base)
         w_dict = space.newdict()
-        space.setitem_str(w_dict, '__module__', space.newtext('_ast'))
+        space.setitem_str(w_dict, '__module__', space.wrap('_ast'))
         if fields is not None:
             space.setitem_str(w_dict, "_fields",
-                              space.newtuple([space.newtext(f) for f in fields]))
+                              space.newtuple([space.wrap(f) for f in fields]))
         if attributes is not None:
             space.setitem_str(w_dict, "_attributes",
-                              space.newtuple([space.newtext(a) for a in attributes]))
+                              space.newtuple([space.wrap(a) for a in attributes]))
         w_type = space.call_function(
-            space.w_type,
-            space.newtext(name), space.newtuple([w_base]), w_dict)
+            space.w_type, 
+            space.wrap(name), space.newtuple([w_base]), w_dict)
         setattr(self, 'w_%s' % name, w_type)
 
 def get(space):
