@@ -35,18 +35,12 @@ class Function(W_Root):
                           'w_func_globals?',
                           'closure?[*]',
                           'defs_w?[*]',
-                          'name?',
-                          'qualname?',
-                          'w_kw_defs?']
+                          'name?']
 
-    w_kw_defs = None
-
-    def __init__(self, space, code, w_globals=None, defs_w=[], kw_defs_w=None,
-                 closure=None, w_ann=None, forcename=None, qualname=None):
+    def __init__(self, space, code, w_globals=None, defs_w=[], closure=None,
+                 forcename=None):
         self.space = space
         self.name = forcename or code.co_name
-        self.qualname = qualname or self.name
-        assert isinstance(self.qualname, str)
         self.w_doc = None   # lazily read from code.getdocstring()
         self.code = code       # Code instance
         self.w_func_globals = w_globals  # the globals dictionary
@@ -54,10 +48,6 @@ class Function(W_Root):
         self.defs_w = defs_w
         self.w_func_dict = None # filled out below if needed
         self.w_module = None
-        self.w_ann = w_ann
-        #
-        if kw_defs_w is not None:
-            self.init_kwdefaults_dict(kw_defs_w)
 
     def __repr__(self):
         # return "function %s.%s" % (self.space, self.name)
@@ -119,7 +109,7 @@ class Function(W_Root):
                 for i in funccallunrolling:
                     if i < nargs:
                         new_frame.locals_cells_stack_w[i] = args_w[i]
-                return new_frame.run(self.name, self.qualname)
+                return new_frame.run()
         elif nargs >= 1 and fast_natural_arity == Code.PASSTHROUGHARGS1:
             assert isinstance(code, gateway.BuiltinCodePassThroughArguments1)
             return code.funcrun_obj(self, args_w[0],
@@ -127,8 +117,7 @@ class Function(W_Root):
                                               list(args_w[1:])))
         return self.call_args(Arguments(self.space, list(args_w)))
 
-    def funccall_valuestack(self, nargs, frame, methodcall=False): # speed hack
-        # methodcall is only for better error messages
+    def funccall_valuestack(self, nargs, frame): # speed hack
         from pypy.interpreter import gateway
         from pypy.interpreter.pycode import PyCode
 
@@ -172,10 +161,10 @@ class Function(W_Root):
         elif fast_natural_arity == Code.PASSTHROUGHARGS1 and nargs >= 1:
             assert isinstance(code, gateway.BuiltinCodePassThroughArguments1)
             w_obj = frame.peekvalue(nargs-1)
-            args = frame.make_arguments(nargs-1, w_function=self)
+            args = frame.make_arguments(nargs-1)
             return code.funcrun_obj(self, w_obj, args)
 
-        args = frame.make_arguments(nargs, methodcall=methodcall, w_function=self)
+        args = frame.make_arguments(nargs)
         return self.call_args(args)
 
     @jit.unroll_safe
@@ -187,7 +176,7 @@ class Function(W_Root):
             w_arg = frame.peekvalue(nargs-1-i)
             new_frame.locals_cells_stack_w[i] = w_arg
 
-        return new_frame.run(self.name, self.qualname)
+        return new_frame.run()
 
     @jit.unroll_safe
     def _flat_pycall_defaults(self, code, nargs, frame, defs_to_load):
@@ -204,7 +193,7 @@ class Function(W_Root):
         for j in xrange(start, ndefs):
             new_frame.locals_cells_stack_w[i] = self.defs_w[j]
             i += 1
-        return new_frame.run(self.name, self.qualname)
+        return new_frame.run()
 
     def getdict(self, space):
         if self.w_func_dict is None:
@@ -213,17 +202,18 @@ class Function(W_Root):
 
     def setdict(self, space, w_dict):
         if not space.isinstance_w(w_dict, space.w_dict):
-            raise oefmt(space.w_TypeError,
-                        "setting function's dictionary to a non-dict")
+            raise OperationError(space.w_TypeError,
+                space.wrap("setting function's dictionary to a non-dict")
+            )
         self.w_func_dict = w_dict
 
     def descr_function__new__(space, w_subtype, w_code, w_globals,
                               w_name=None, w_argdefs=None, w_closure=None):
         code = space.interp_w(Code, w_code)
         if not space.isinstance_w(w_globals, space.w_dict):
-            raise oefmt(space.w_TypeError, "expected dict")
+            raise OperationError(space.w_TypeError, space.wrap("expected dict"))
         if not space.is_none(w_name):
-            name = space.text_w(w_name)
+            name = space.str_w(w_name)
         else:
             name = None
         if not space.is_none(w_argdefs):
@@ -237,36 +227,36 @@ class Function(W_Root):
         if space.is_none(w_closure) and nfreevars == 0:
             closure = None
         elif not space.is_w(space.type(w_closure), space.w_tuple):
-            raise oefmt(space.w_TypeError, "invalid closure")
+            raise OperationError(space.w_TypeError, space.wrap("invalid closure"))
         else:
             from pypy.interpreter.nestedscope import Cell
             closure_w = space.unpackiterable(w_closure)
             n = len(closure_w)
             if nfreevars == 0:
-                raise oefmt(space.w_ValueError, "no closure needed")
+                raise OperationError(space.w_ValueError, space.wrap("no closure needed"))
             elif nfreevars != n:
-                raise oefmt(space.w_ValueError, "closure is wrong size")
+                raise OperationError(space.w_ValueError, space.wrap("closure is wrong size"))
             closure = [space.interp_w(Cell, w_cell) for w_cell in closure_w]
         func = space.allocate_instance(Function, w_subtype)
-        Function.__init__(func, space, code, w_globals, defs_w, None, closure,
-                          None,name)
-        return func
+        Function.__init__(func, space, code, w_globals, defs_w, closure, name)
+        return space.wrap(func)
 
     def descr_function_call(self, __args__):
         return self.call_args(__args__)
 
     def descr_function_repr(self):
-        return self.getrepr(self.space, 'function %s' % self.qualname)
+        return self.getrepr(self.space, 'function %s' % (self.name,))
 
+    # delicate
+    _all = {'': None}
 
     def _cleanup_(self):
-        # delicate
         from pypy.interpreter.gateway import BuiltinCode
         if isinstance(self.code, BuiltinCode):
             # we have been seen by other means so rtyping should not choke
             # on us
             identifier = self.code.identifier
-            previous = self.space._builtin_functions_by_identifier.get(identifier, self)
+            previous = Function._all.get(identifier, self)
             assert previous is self, (
                 "duplicate function ids with identifier=%r: %r and %r" % (
                 identifier, previous, self))
@@ -274,14 +264,13 @@ class Function(W_Root):
         return False
 
     def add_to_table(self):
-        self.space._builtin_functions_by_identifier[self.code.identifier] = self
+        Function._all[self.code.identifier] = self
 
-    def find(space, identifier):
-        return space._builtin_functions_by_identifier[identifier]
+    def find(identifier):
+        return Function._all[identifier]
     find = staticmethod(find)
 
     def descr_function__reduce__(self, space):
-        XXX   # This is not used any more
         from pypy.interpreter.gateway import BuiltinCode
         from pypy.interpreter.mixedmodule import MixedModule
         w_mod = space.getbuiltinmodule('_pickle_support')
@@ -290,13 +279,14 @@ class Function(W_Root):
         if isinstance(code, BuiltinCode):
             new_inst = mod.get('builtin_function')
             return space.newtuple([new_inst,
-                                   space.newtuple([space.newtext(code.identifier)])])
+                                   space.newtuple([space.wrap(code.identifier)])])
 
         new_inst = mod.get('func_new')
+        w = space.wrap
         if self.closure is None:
             w_closure = space.w_None
         else:
-            w_closure = space.newtuple([cell for cell in self.closure])
+            w_closure = space.newtuple([w(cell) for cell in self.closure])
         if self.w_doc is None:
             w_doc = space.w_None
         else:
@@ -313,10 +303,9 @@ class Function(W_Root):
         nt = space.newtuple
         tup_base = []
         tup_state = [
-            space.newtext(self.name),
-            space.newtext(self.qualname),
+            w(self.name),
             w_doc,
-            self.code,
+            w(self.code),
             w_func_globals,
             w_closure,
             nt(self.defs_w),
@@ -326,19 +315,17 @@ class Function(W_Root):
         return nt([new_inst, nt(tup_base), nt(tup_state)])
 
     def descr_function__setstate__(self, space, w_args):
-        XXX   # This is not used any more
         args_w = space.unpackiterable(w_args)
         try:
-            (w_name, w_qualname, w_doc, w_code, w_func_globals, w_closure,
-             w_defs, w_func_dict, w_module) = args_w
+            (w_name, w_doc, w_code, w_func_globals, w_closure, w_defs,
+             w_func_dict, w_module) = args_w
         except ValueError:
             # wrong args
-            raise oefmt(space.w_ValueError,
-                        "Wrong arguments to function.__setstate__")
+            raise OperationError(space.w_ValueError,
+                         space.wrap("Wrong arguments to function.__setstate__"))
 
         self.space = space
-        self.name = space.text_w(w_name)
-        self.qualname = space.utf8_w(w_qualname)
+        self.name = space.str_w(w_name)
         self.code = space.interp_w(Code, w_code)
         if not space.is_w(w_closure, space.w_None):
             from pypy.interpreter.nestedscope import Cell
@@ -358,11 +345,6 @@ class Function(W_Root):
         self.defs_w = space.fixedview(w_defs)
         self.w_module = w_module
 
-    def _check_code_mutable(self, attr):
-        if not self.can_change_code:
-            raise oefmt(self.space.w_TypeError,
-                        "Cannot change %s attribute of builtin functions", attr)
-
     def fget_func_defaults(self, space):
         values_w = self.defs_w
         # the `None in values_w` check here is to ensure that interp-level
@@ -372,118 +354,60 @@ class Function(W_Root):
             return space.w_None
         return space.newtuple(values_w)
 
-    def fget_defaults_count(self, space):
-        # PyPy extension.  This is mostly to make inspect.py happy about
-        # methods like dict.pop(), which take a default but is None.
-        # Previously, inspect.py would build a signature saying that all
-        # arguments are required, and then unittest.mock would complain
-        # if we call it with less arguments.  Now inspect.py looks up
-        # '__defaults_count__' too.
-        return space.newint(len(self.defs_w))
-
     def fset_func_defaults(self, space, w_defaults):
-        self._check_code_mutable("__defaults__")
         if space.is_w(w_defaults, space.w_None):
             self.defs_w = []
             return
         if not space.isinstance_w(w_defaults, space.w_tuple):
-            raise oefmt(space.w_TypeError,
-                        "__defaults__ must be set to a tuple object or None")
+            raise OperationError(space.w_TypeError, space.wrap("func_defaults must be set to a tuple object or None"))
         self.defs_w = space.fixedview(w_defaults)
 
     def fdel_func_defaults(self, space):
-        self._check_code_mutable("__defaults__")
         self.defs_w = []
 
-    def fget_func_kwdefaults(self, space):
-        if self.w_kw_defs is None:
-            return space.w_None
-        return self.w_kw_defs
-
-    def fset_func_kwdefaults(self, space, w_new):
-        if space.is_w(w_new, space.w_None):
-            self.w_kw_defs = None
-        else:
-            if not space.isinstance_w(w_new, space.w_dict):
-                raise oefmt(space.w_TypeError, "__kwdefaults__ must be a dict")
-            self.w_kw_defs = w_new
-
-    def fdel_func_kwdefaults(self, space):
-        self.w_kw_defs = None
-
-    def init_kwdefaults_dict(self, kw_defs_w):
-        # use the moduledict logic to get normally-constant entries
-        space = self.space
-        w_dict = space.newdict(module=True)
-        for w_name, w_value in kw_defs_w:
-            space.setitem(w_dict, w_name, w_value)
-        self.w_kw_defs = w_dict
-
     def fget_func_doc(self, space):
         if self.w_doc is None:
             self.w_doc = self.code.getdocstring(space)
         return self.w_doc
 
     def fset_func_doc(self, space, w_doc):
-        self._check_code_mutable("__doc__")
         self.w_doc = w_doc
 
-    def fdel_func_doc(self, space):
-        self._check_code_mutable("__doc__")
-        self.w_doc = space.w_None
-
     def fget_func_name(self, space):
-        return space.newtext(self.name)
+        return space.wrap(self.name)
 
     def fset_func_name(self, space, w_name):
-        self._check_code_mutable("__name__")
-        if space.isinstance_w(w_name, space.w_text):
-            self.name = space.text_w(w_name)
+        if space.isinstance_w(w_name, space.w_str):
+            self.name = space.str_w(w_name)
         else:
-            raise oefmt(space.w_TypeError,
-                        "__name__ must be set to a string object")
+            raise OperationError(space.w_TypeError,
+                space.wrap("__name__ must be set to a string object"))
 
-    def fget_func_qualname(self, space):
-        return space.newtext(self.qualname)
-
-    def set_qualname(self, qualname):
-        self.qualname = qualname
-
-    def fset_func_qualname(self, space, w_name):
-        self._check_code_mutable("__qualname__")
-        try:
-            qualname = space.realutf8_w(w_name)
-        except OperationError as e:
-            if e.match(space, space.w_TypeError):
-                raise oefmt(space.w_TypeError,
-                            "__qualname__ must be set to a string object")
-            raise
-        self.set_qualname(qualname)
+    def fdel_func_doc(self, space):
+        self.w_doc = space.w_None
 
     def fget___module__(self, space):
         if self.w_module is None:
             if self.w_func_globals is not None and not space.is_w(self.w_func_globals, space.w_None):
-                self.w_module = space.call_method(self.w_func_globals, "get", space.newtext("__name__"))
+                self.w_module = space.call_method(self.w_func_globals, "get", space.wrap("__name__"))
             else:
                 self.w_module = space.w_None
         return self.w_module
 
     def fset___module__(self, space, w_module):
-        self._check_code_mutable("__module__")
         self.w_module = w_module
 
     def fdel___module__(self, space):
-        self._check_code_mutable("__module__")
         self.w_module = space.w_None
 
     def fget_func_code(self, space):
-        return self.getcode()
+        return space.wrap(self.code)
 
     def fset_func_code(self, space, w_code):
         from pypy.interpreter.pycode import PyCode
         if not self.can_change_code:
-            raise oefmt(space.w_TypeError,
-                        "Cannot change code attribute of builtin functions")
+            raise OperationError(space.w_TypeError,
+                    space.wrap("Cannot change code attribute of builtin functions"))
         code = space.interp_w(Code, w_code)
         closure_len = 0
         if self.closure:
@@ -497,85 +421,126 @@ class Function(W_Root):
 
     def fget_func_closure(self, space):
         if self.closure is not None:
-            w_res = space.newtuple([cell for cell in self.closure])
+            w_res = space.newtuple([space.wrap(i) for i in self.closure])
         else:
             w_res = space.w_None
         return w_res
 
-    def fget_func_annotations(self, space):
-        if self.w_ann is None:
-            self.w_ann = space.newdict()
-        return self.w_ann
-
-    def fset_func_annotations(self, space, w_new):
-        self._check_code_mutable("__annotations__")
-        if space.is_w(w_new, space.w_None):
-            w_new = None
-        elif not space.isinstance_w(w_new, space.w_dict):
-            raise oefmt(space.w_TypeError, "__annotations__ must be a dict")
-        self.w_ann = w_new
-
-    def fdel_func_annotations(self, space):
-        self._check_code_mutable("__annotations__")
-        self.w_ann = None
-
 
 def descr_function_get(space, w_function, w_obj, w_cls=None):
     """functionobject.__get__(obj[, type]) -> method"""
     # this is not defined as a method on Function because it's generally
     # useful logic: w_function can be any callable.  It is used by Method too.
-    if w_obj is None or space.is_w(w_obj, space.w_None):
-        return w_function
+    asking_for_bound = (space.is_none(w_cls) or
+                        not space.is_w(w_obj, space.w_None) or
+                        space.is_w(w_cls, space.type(space.w_None)))
+    if asking_for_bound:
+        return space.wrap(Method(space, w_function, w_obj, w_cls))
     else:
-        return Method(space, w_function, w_obj)
+        return space.wrap(Method(space, w_function, None, w_cls))
 
 
-class _Method(W_Root):
-    """A method is a function bound to a specific instance."""
-    _immutable_fields_ = ['w_function', 'w_instance']
+class Method(W_Root):
+    """A method is a function bound to a specific instance or class."""
+    _immutable_fields_ = ['w_function', 'w_instance', 'w_class']
 
-    def __init__(self, space, w_function, w_instance):
+    def __init__(self, space, w_function, w_instance, w_class):
         self.space = space
-        assert w_instance is not None   # unbound methods only exist in Python 2
         self.w_function = w_function
-        self.w_instance = w_instance
+        self.w_instance = w_instance   # or None
+        if w_class is None:
+            w_class = space.w_None
+        self.w_class = w_class         # possibly space.w_None
+
+    def descr_method__new__(space, w_subtype, w_function, w_instance,
+                            w_class=None):
+        if space.is_w(w_instance, space.w_None):
+            w_instance = None
+        if w_instance is None and space.is_none(w_class):
+            raise OperationError(space.w_TypeError,
+                                 space.wrap("unbound methods must have class"))
+        method = space.allocate_instance(Method, w_subtype)
+        Method.__init__(method, space, w_function, w_instance, w_class)
+        return space.wrap(method)
 
     def __repr__(self):
-        return u"bound method %s" % (self.w_function.getname(self.space),)
+        if self.w_instance:
+            pre = "bound"
+        else:
+            pre = "unbound"
+        return "%s method %s" % (pre, self.w_function.getname(self.space))
 
     def call_args(self, args):
         space = self.space
-        return space.call_obj_args(self.w_function, self.w_instance, args)
+        if self.w_instance is not None:
+            # bound method
+            return space.call_obj_args(self.w_function, self.w_instance, args)
+
+        # unbound method
+        w_firstarg = args.firstarg()
+        if w_firstarg is not None and (
+                space.abstract_isinstance_w(w_firstarg, self.w_class)):
+            pass  # ok
+        else:
+            clsdescr = self.w_class.getname(space)
+            if clsdescr and clsdescr != '?':
+                clsdescr += " instance"
+            else:
+                clsdescr = "instance"
+            if w_firstarg is None:
+                instdescr = "nothing"
+            else:
+                instname = space.abstract_getclass(w_firstarg).getname(space)
+                if instname and instname != '?':
+                    instdescr = instname + " instance"
+                else:
+                    instdescr = "instance"
+            raise oefmt(space.w_TypeError,
+                        "unbound method %N() must be called with %s as first "
+                        "argument (got %s instead)", self, clsdescr, instdescr)
+        return space.call_args(self.w_function, args)
 
     def descr_method_get(self, w_obj, w_cls=None):
-        return self     # already bound
+        space = self.space
+        if self.w_instance is not None:
+            return space.wrap(self)    # already bound
+        else:
+            # only allow binding to a more specific class than before
+            if (w_cls is not None and
+                not space.is_w(w_cls, space.w_None) and
+                not space.abstract_issubclass_w(w_cls, self.w_class,
+                                                allow_override=True)):
+                return space.wrap(self)    # subclass test failed
+            else:
+                return descr_function_get(space, self.w_function, w_obj, w_cls)
 
     def descr_method_call(self, __args__):
         return self.call_args(__args__)
 
     def descr_method_repr(self):
         space = self.space
-        w_name = space.findattr(self.w_function, space.newtext('__qualname__'))
-        if w_name is None:
-            name = self.w_function.getname(self.space)
+        name = self.w_function.getname(self.space)
+        # XXX do we handle all cases sanely here?
+        if space.is_w(self.w_class, space.w_None):
+            w_class = space.type(self.w_instance)
         else:
-            try:
-                name = space.utf8_w(w_name)
-            except OperationError as e:
-                if not e.match(space, space.w_TypeError):
-                    raise
-                name = '?'
-        objrepr = space.utf8_w(space.repr(self.w_instance))
-        s = b'<bound method %s of %s>' % (name, objrepr)
-        return space.newtext(s)
+            w_class = self.w_class
+        typename = w_class.getname(self.space)
+        if self.w_instance is None:
+            s = "<unbound method %s.%s>" % (typename, name)
+            return space.wrap(s)
+        else:
+            objrepr = space.str_w(space.repr(self.w_instance))
+            s = '<bound method %s.%s of %s>' % (typename, name, objrepr)
+            return space.wrap(s)
 
     def descr_method_getattribute(self, w_attr):
         space = self.space
-        if space.text_w(w_attr) != '__doc__':
+        if space.str_w(w_attr) != '__doc__':
             try:
                 return space.call_method(space.w_object, '__getattribute__',
-                                         self, w_attr)
-            except OperationError as e:
+                                         space.wrap(self), w_attr)
+            except OperationError, e:
                 if not e.match(space, space.w_AttributeError):
                     raise
         # fall-back to the attribute of the underlying 'im_func'
@@ -583,16 +548,43 @@ class _Method(W_Root):
 
     def descr_method_eq(self, w_other):
         space = self.space
-        if not isinstance(w_other, _Method):
+        if not isinstance(w_other, Method):
             return space.w_NotImplemented
-        if not space.is_w(self.w_instance, w_other.w_instance):
-            return space.w_False
-        return space.is_(self.w_function, w_other.w_function)
+        if self.w_instance is None:
+            if w_other.w_instance is not None:
+                return space.w_False
+        else:
+            if w_other.w_instance is None:
+                return space.w_False
+            if not space.eq_w(self.w_instance, w_other.w_instance):
+                return space.w_False
+        return space.eq(self.w_function, w_other.w_function)
+
+    def is_w(self, space, other):
+        if not isinstance(other, Method):
+            return False
+        return (self.w_instance is other.w_instance and
+                self.w_function is other.w_function and
+                self.w_class is other.w_class)
+
+    def immutable_unique_id(self, space):
+        from pypy.objspace.std.util import IDTAG_METHOD as tag
+        from pypy.objspace.std.util import IDTAG_SHIFT
+        if self.w_instance is not None:
+            id = space.bigint_w(space.id(self.w_instance))
+            id = id.lshift(LONG_BIT)
+        else:
+            id = rbigint.fromint(0)
+        id = id.or_(space.bigint_w(space.id(self.w_function)))
+        id = id.lshift(LONG_BIT).or_(space.bigint_w(space.id(self.w_class)))
+        id = id.lshift(IDTAG_SHIFT).int_or_(tag)
+        return space.newlong_from_rbigint(id)
 
     def descr_method_hash(self):
         space = self.space
         w_result = space.hash(self.w_function)
-        w_result = space.xor(w_result, space.id(self.w_instance))
+        if self.w_instance is not None:
+            w_result = space.xor(w_result, space.hash(self.w_instance))
         return w_result
 
     def descr_method__reduce__(self, space):
@@ -600,47 +592,29 @@ class _Method(W_Root):
         from pypy.interpreter.gateway import BuiltinCode
         w_mod    = space.getbuiltinmodule('_pickle_support')
         mod      = space.interp_w(MixedModule, w_mod)
-        w_instance = self.w_instance
+        new_inst = mod.get('method_new')
+        w_instance = self.w_instance or space.w_None
         w_function = self.w_function
         if (isinstance(w_function, Function) and
                 isinstance(w_function.code, BuiltinCode)):
             new_inst = mod.get('builtin_method_new')
-            tup = [w_instance, space.newtext(w_function.name)]
+            if space.is_w(w_instance, space.w_None):
+                tup = [self.w_class, space.wrap(w_function.name)]
+            else:
+                tup = [w_instance, space.wrap(w_function.name)]
+        elif space.is_w(self.w_class, space.w_None):
+            tup = [self.w_function, w_instance]
         else:
-            w_builtins = space.getbuiltinmodule('builtins')
-            new_inst = space.getattr(w_builtins, space.newtext('getattr'))
-            tup = [w_instance, space.newtext(w_function.getname(space))]
+            tup = [self.w_function, w_instance, self.w_class]
         return space.newtuple([new_inst, space.newtuple(tup)])
 
-class Method(_Method):
-    def descr_method__new__(space, w_subtype, w_function, w_instance):
-        if space.is_w(w_instance, space.w_None):
-            w_instance = None
-        if w_instance is None:
-            raise oefmt(space.w_TypeError, "self must not be None")
-        method = space.allocate_instance(Method, w_subtype)
-        _Method.__init__(method, space, w_function, w_instance)
-        return method
 
 class StaticMethod(W_Root):
     """The staticmethod objects."""
-    _immutable_fields_ = ['w_function?']
+    _immutable_fields_ = ['w_function']
 
     def __init__(self, w_function):
         self.w_function = w_function
-        self.w_dict = None
-
-    def getdict(self, space):
-        if self.w_dict is None:
-            self.w_dict = space.newdict(instance=True)
-        return self.w_dict
-
-    def setdict(self, space, w_dict):
-        if not space.isinstance_w(w_dict, space.w_dict):
-            raise oefmt(space.w_TypeError,
-                        "__dict__ must be set to a dictionary, not a %T",
-                        w_dict)
-        self.w_dict = w_dict
 
     def descr_staticmethod_get(self, w_obj, w_cls=None):
         """staticmethod(x).__get__(obj[, type]) -> x"""
@@ -648,52 +622,27 @@ class StaticMethod(W_Root):
 
     def descr_staticmethod__new__(space, w_subtype, w_function):
         instance = space.allocate_instance(StaticMethod, w_subtype)
-        instance.__init__(space.w_None)
-        return instance
-
-    def descr_init(self, space, w_function):
-        self.w_function = w_function
-
-    def descr_isabstract(self, space):
-        return space.newbool(space.isabstractmethod_w(self.w_function))
+        instance.__init__(w_function)
+        return space.wrap(instance)
 
 
 class ClassMethod(W_Root):
     """The classmethod objects."""
-    _immutable_fields_ = ['w_function?']
+    _immutable_fields_ = ['w_function']
 
     def __init__(self, w_function):
         self.w_function = w_function
-        self.w_dict = None
-
-    def getdict(self, space):
-        if self.w_dict is None:
-            self.w_dict = space.newdict(instance=True)
-        return self.w_dict
-
-    def setdict(self, space, w_dict):
-        if not space.isinstance_w(w_dict, space.w_dict):
-            raise oefmt(space.w_TypeError,
-                        "__dict__ must be set to a dictionary, not a %T",
-                        w_dict)
-        self.w_dict = w_dict
 
     def descr_classmethod_get(self, space, w_obj, w_klass=None):
         if space.is_none(w_klass):
             w_klass = space.type(w_obj)
-        return Method(space, self.w_function, w_klass)
+        return space.wrap(Method(space, self.w_function, w_klass,
+                                 space.type(w_klass)))
 
     def descr_classmethod__new__(space, w_subtype, w_function):
         instance = space.allocate_instance(ClassMethod, w_subtype)
-        instance.__init__(space.w_None)
-        return instance
-
-    def descr_init(self, space, w_function):
-        self.w_function = w_function
-
-    def descr_isabstract(self, space):
-        return space.newbool(space.isabstractmethod_w(self.w_function))
-
+        instance.__init__(w_function)
+        return space.wrap(instance)
 
 class FunctionWithFixedCode(Function):
     can_change_code = False
@@ -704,26 +653,21 @@ class BuiltinFunction(Function):
     def __init__(self, func):
         assert isinstance(func, Function)
         Function.__init__(self, func.space, func.code, func.w_func_globals,
-                          func.defs_w, None, func.closure,
-                          None, func.name)
+                          func.defs_w, func.closure, func.name)
         self.w_doc = func.w_doc
         self.w_func_dict = func.w_func_dict
         self.w_module = func.w_module
-        self.w_kw_defs = func.w_kw_defs
 
     def descr_builtinfunction__new__(space, w_subtype):
-        raise oefmt(space.w_TypeError,
-                    "cannot create 'builtin_function' instances")
+        raise OperationError(space.w_TypeError,
+                     space.wrap("cannot create 'builtin_function' instances"))
 
     def descr_function_repr(self):
-        return self.space.newtext('<built-in function %s>' % (self.name,))
-
-    def descr__reduce__(self, space):
-        return space.newtext(self.qualname)
+        return self.space.wrap('<built-in function %s>' % (self.name,))
 
 def is_builtin_code(w_func):
     from pypy.interpreter.gateway import BuiltinCode
-    if isinstance(w_func, _Method):
+    if isinstance(w_func, Method):
         w_func = w_func.w_function
     if isinstance(w_func, Function):
         code = w_func.getcode()
