@@ -21,17 +21,13 @@ from pypy.interpreter.argument import Arguments
 from pypy.interpreter.signature import Signature
 from pypy.interpreter.baseobjspace import (W_Root, ObjSpace, SpaceCache,
     DescrMismatch)
-from pypy.interpreter.error import OperationError, oefmt
+from pypy.interpreter.error import OperationError
 from pypy.interpreter.function import ClassMethod, FunctionWithFixedCode
-from rpython.rlib.objectmodel import we_are_translated, not_rpython
+from rpython.rlib import rstackovf
+from rpython.rlib.objectmodel import we_are_translated
 from rpython.rlib.rarithmetic import r_longlong, r_int, r_ulonglong, r_uint
 from rpython.tool.sourcetools import func_with_new_name, compile2
 
-from rpython.rlib.signature import signature, finishsigs
-from rpython.rlib import types as sigtypes
-
-NO_DEFAULT = object()
-
 
 # internal non-translatable parts:
 class SignatureBuilder(object):
@@ -48,29 +44,12 @@ class SignatureBuilder(object):
         self.argnames = argnames
         self.varargname = varargname
         self.kwargname = kwargname
-        self.posonlyargcount = 0
-        self.kwonlystartindex = -1
 
     def append(self, argname):
         self.argnames.append(argname)
 
-    def marker_posonly(self):
-        assert self.posonlyargcount == 0
-        assert self.kwonlystartindex == -1
-        self.posonlyargcount = len(self.argnames)
-
-    def marker_kwonly(self):
-        assert self.kwonlystartindex == -1
-        self.kwonlystartindex = len(self.argnames)
-
     def signature(self):
-        if self.kwonlystartindex == -1:
-            kwonlyargcount = 0
-        else:
-            kwonlyargcount = len(self.argnames) - self.kwonlystartindex
-        return Signature(self.argnames,
-                         self.varargname, self.kwargname,
-                         kwonlyargcount, self.posonlyargcount)
+        return Signature(self.argnames, self.varargname, self.kwargname)
 
 #________________________________________________________________
 
@@ -83,8 +62,8 @@ class Unwrapper(object):
     def _freeze_(self):
         return True
 
-    @not_rpython
     def unwrap(self, space, w_value):
+        """NOT_RPYTHON"""
         raise NotImplementedError
 
 
@@ -144,9 +123,9 @@ class UnwrapSpec_Check(UnwrapSpecRecipe):
     # checks for checking interp2app func argument names wrt unwrap_spec
     # and synthetizing an app-level signature
 
-    def __init__(self, func, argnames):
-        self.func = func
-        self.orig_arg = iter(argnames).next
+    def __init__(self, original_sig):
+        self.func = original_sig.func
+        self.orig_arg = iter(original_sig.argnames).next
 
     def visit_self(self, cls, app_sig):
         self.visit__W_Root(cls, app_sig)
@@ -164,31 +143,10 @@ class UnwrapSpec_Check(UnwrapSpecRecipe):
     def visit_bufferstr(self, el, app_sig):
         self.checked_space_method(el, app_sig)
 
-    def visit_text_or_none(self, el, app_sig):
+    def visit_str_or_None(self, el, app_sig):
         self.checked_space_method(el, app_sig)
 
-    def visit_bytes(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
-    def visit_bytes0(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
-    def visit_text(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
-    def visit_text0(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
-    def visit_unicode(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
-    def visit_utf8(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
-    def visit_fsencode(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
-    def visit_fsencode_or_none(self, el, app_sig):
+    def visit_str0(self, el, app_sig):
         self.checked_space_method(el, app_sig)
 
     def visit_nonnegint(self, el, app_sig):
@@ -209,9 +167,6 @@ class UnwrapSpec_Check(UnwrapSpecRecipe):
     def visit_c_ushort(self, el, app_sig):
         self.checked_space_method(el, app_sig)
 
-    def visit_c_uid_t(self, el, app_sig):
-        self.checked_space_method(el, app_sig)
-
     def visit_truncatedint_w(self, el, app_sig):
         self.checked_space_method(el, app_sig)
 
@@ -261,36 +216,15 @@ class UnwrapSpec_Check(UnwrapSpecRecipe):
         name = int_unwrapping_space_method(typ)
         self.checked_space_method(name, app_sig)
 
-    def visit_posonly(self, _, app_sig):
-        argname = self.orig_arg()
-        assert argname == '__posonly__'
-        app_sig.marker_posonly()
-
-    def visit_kwonly(self, _, app_sig):
-        argname = self.orig_arg()
-        assert argname == '__kwonly__'
-        app_sig.marker_kwonly()
-
 
 class UnwrapSpec_EmitRun(UnwrapSpecEmit):
 
     # collect code to emit for interp2app builtin frames based on unwrap_spec
 
-    def __init__(self, app_sig):
-        self.app_sig = app_sig
+    def __init__(self):
         UnwrapSpecEmit.__init__(self)
         self.run_args = []
 
-    def scope_starargs(self):
-        app_sig = self.app_sig
-        pos = app_sig.num_argnames() + app_sig.num_kwonlyargnames()
-        return "scope_w[%d]" % (pos, )
-
-    def scope_kwargs(self):
-        app_sig = self.app_sig
-        pos = app_sig.num_argnames() + app_sig.num_kwonlyargnames() + app_sig.has_vararg()
-        return "scope_w[%d]" % (pos, )
-
     def scopenext(self):
         return "scope_w[%d]" % self.succ()
 
@@ -315,13 +249,13 @@ class UnwrapSpec_EmitRun(UnwrapSpecEmit):
     def visit__Arguments(self, el):
         self.miniglobals['Arguments'] = Arguments
         self.run_args.append("Arguments.frompacked(space, %s, %s)"
-                             % (self.scope_starargs(), self.scope_kwargs()))
+                             % (self.scopenext(), self.scopenext()))
 
     def visit_args_w(self, el):
-        self.run_args.append("space.fixedview(%s)" % self.scope_starargs())
+        self.run_args.append("space.fixedview(%s)" % self.scopenext())
 
     def visit_w_args(self, el):
-        self.run_args.append(self.scope_starargs())
+        self.run_args.append(self.scopenext())
 
     def visit__object(self, typ):
         name = int_unwrapping_space_method(typ)
@@ -333,34 +267,13 @@ class UnwrapSpec_EmitRun(UnwrapSpecEmit):
                              % (self.scopenext(), ))
 
     def visit_bufferstr(self, typ):
-        self.run_args.append("space.charbuf_w(%s)" % (self.scopenext(),))
+        self.run_args.append("space.bufferstr_w(%s)" % (self.scopenext(),))
 
-    def visit_text_or_none(self, typ):
-        self.run_args.append("space.text_or_none_w(%s)" % (self.scopenext(),))
+    def visit_str_or_None(self, typ):
+        self.run_args.append("space.str_or_None_w(%s)" % (self.scopenext(),))
 
-    def visit_bytes(self, typ):
-        self.run_args.append("space.bytes_w(%s)" % (self.scopenext(),))
-
-    def visit_bytes0(self, typ):
-        self.run_args.append("space.bytes0_w(%s)" % (self.scopenext(),))
-
-    def visit_text(self, typ):
-        self.run_args.append("space.text_w(%s)" % (self.scopenext(),))
-
-    def visit_text0(self, typ):
-        self.run_args.append("space.text0_w(%s)" % (self.scopenext(),))
-
-    def visit_unicode(self, typ):
-        self.run_args.append("space.realunicode_w(%s)" % (self.scopenext(),))
-
-    def visit_utf8(self, typ):
-        self.run_args.append("space.utf8_w(%s)" % (self.scopenext(),))
-
-    def visit_fsencode(self, typ):
-        self.run_args.append("space.fsencode_w(%s)" % (self.scopenext(),))
-
-    def visit_fsencode_or_none(self, typ):
-        self.run_args.append("space.fsencode_or_none_w(%s)" % (self.scopenext(),))
+    def visit_str0(self, typ):
+        self.run_args.append("space.str0_w(%s)" % (self.scopenext(),))
 
     def visit_nonnegint(self, typ):
         self.run_args.append("space.gateway_nonnegint_w(%s)" % (
@@ -381,18 +294,9 @@ class UnwrapSpec_EmitRun(UnwrapSpecEmit):
     def visit_c_ushort(self, typ):
         self.run_args.append("space.c_ushort_w(%s)" % (self.scopenext(),))
 
-    def visit_c_uid_t(self, typ):
-        self.run_args.append("space.c_uid_t_w(%s)" % (self.scopenext(),))
-
     def visit_truncatedint_w(self, typ):
         self.run_args.append("space.truncatedint_w(%s)" % (self.scopenext(),))
 
-    def visit_posonly(self, typ):
-        self.run_args.append("None")
-
-    def visit_kwonly(self, typ):
-        self.run_args.append("None")
-
     def _make_unwrap_activation_class(self, unwrap_spec, cache={}):
         try:
             key = tuple(unwrap_spec)
@@ -427,8 +331,8 @@ class UnwrapSpec_EmitRun(UnwrapSpecEmit):
             cache[key] = activation_cls, self.run_args
             return activation_cls
 
-    def make_activation(unwrap_spec, func, app_sig):
-        emit = UnwrapSpec_EmitRun(app_sig)
+    def make_activation(unwrap_spec, func):
+        emit = UnwrapSpec_EmitRun()
         emit.apply_over(unwrap_spec)
         activation_uw_cls = emit._make_unwrap_activation_class(unwrap_spec)
         return activation_uw_cls(func)
@@ -438,8 +342,8 @@ class UnwrapSpec_EmitRun(UnwrapSpecEmit):
 class BuiltinActivation(object):
     _immutable_ = True
 
-    @not_rpython
     def __init__(self, behavior):
+        """NOT_RPYTHON"""
         self.behavior = behavior
 
     def _run(self, space, scope_w):
@@ -510,34 +414,13 @@ class UnwrapSpec_FastFunc_Unwrap(UnwrapSpecEmit):
                            % (self.nextarg()), )
 
     def visit_bufferstr(self, typ):
-        self.unwrap.append("space.charbuf_w(%s)" % (self.nextarg(),))
+        self.unwrap.append("space.bufferstr_w(%s)" % (self.nextarg(),))
 
-    def visit_text_or_none(self, typ):
-        self.unwrap.append("space.text_or_none_w(%s)" % (self.nextarg(),))
+    def visit_str_or_None(self, typ):
+        self.unwrap.append("space.str_or_None_w(%s)" % (self.nextarg(),))
 
-    def visit_bytes(self, typ):
-        self.unwrap.append("space.bytes_w(%s)" % (self.nextarg(),))
-
-    def visit_bytes0(self, typ):
-        self.unwrap.append("space.bytes0_w(%s)" % (self.nextarg(),))
-
-    def visit_text(self, typ):
-        self.unwrap.append("space.text_w(%s)" % (self.nextarg(),))
-
-    def visit_unicode(self, typ):
-        self.unwrap.append("space.realunicode_w(%s)" % (self.nextarg(),))
-
-    def visit_text0(self, typ):
-        self.unwrap.append("space.text0_w(%s)" % (self.nextarg(),))
-
-    def visit_utf8(self, typ):
-        self.unwrap.append("space.utf8_w(%s)" % (self.nextarg(),))
-
-    def visit_fsencode(self, typ):
-        self.unwrap.append("space.fsencode_w(%s)" % (self.nextarg(),))
-
-    def visit_fsencode_or_none(self, typ):
-        self.unwrap.append("space.fsencode_or_none_w(%s)" % (self.nextarg(),))
+    def visit_str0(self, typ):
+        self.unwrap.append("space.str0_w(%s)" % (self.nextarg(),))
 
     def visit_nonnegint(self, typ):
         self.unwrap.append("space.gateway_nonnegint_w(%s)" % (self.nextarg(),))
@@ -557,19 +440,9 @@ class UnwrapSpec_FastFunc_Unwrap(UnwrapSpecEmit):
     def visit_c_ushort(self, typ):
         self.unwrap.append("space.c_ushort_w(%s)" % (self.nextarg(),))
 
-    def visit_c_uid_t(self, typ):
-        self.unwrap.append("space.c_uid_t_w(%s)" % (self.nextarg(),))
-
     def visit_truncatedint_w(self, typ):
         self.unwrap.append("space.truncatedint_w(%s)" % (self.nextarg(),))
 
-    def visit_posonly(self, typ):
-        raise FastFuncNotSupported
-
-    def visit_kwonly(self, typ):
-        raise FastFuncNotSupported
-
-    @staticmethod
     def make_fastfunc(unwrap_spec, func):
         unwrap_info = UnwrapSpec_FastFunc_Unwrap()
         unwrap_info.apply_over(unwrap_spec)
@@ -600,25 +473,15 @@ class UnwrapSpec_FastFunc_Unwrap(UnwrapSpecEmit):
             exec compile2(source) in unwrap_info.miniglobals, d
             fastfunc = d['fastfunc_%s_%d' % (func.__name__.replace('-', '_'), narg)]
         return narg, fastfunc
+    make_fastfunc = staticmethod(make_fastfunc)
 
 
 def int_unwrapping_space_method(typ):
     assert typ in (int, str, float, unicode, r_longlong, r_uint, r_ulonglong, bool)
     if typ is r_int is r_longlong:
         return 'gateway_r_longlong_w'
-    elif typ is str:
-        return 'utf8_w'
-    elif typ is unicode:
-        return 'realunicode_w'
-    elif typ is bool:
-        # For argument clinic's "bool" specifier: accept any object, and
-        # convert it to a boolean value.  If you don't want this
-        # behavior, you need to say "int" in the unwrap_spec().  Please
-        # use only to emulate "bool" in argument clinic or the 'p'
-        # letter in PyArg_ParseTuple().  Accepting *anything* when a
-        # boolean flag is expected feels like it comes straight from
-        # JavaScript: it is a sure way to hide bugs imho <arigo>.
-        return 'is_true'
+    elif typ in (str, unicode, bool):
+        return typ.__name__ + '_w'
     else:
         return 'gateway_' + typ.__name__ + '_w'
 
@@ -675,10 +538,6 @@ def build_unwrap_spec(func, argnames, self_type=None):
                 unwrap_spec.append('args_w')
             elif argname.startswith('w_'):
                 unwrap_spec.append(W_Root)
-            elif argname == '__posonly__':
-                unwrap_spec.append('posonly')
-            elif argname == '__kwonly__':
-                unwrap_spec.append('kwonly')
             else:
                 unwrap_spec.append(None)
 
@@ -691,8 +550,6 @@ def build_unwrap_spec(func, argnames, self_type=None):
                              "the name of an argument of the following "
                              "function" % (name,))
 
-    assert str not in unwrap_spec   # use 'text' or 'bytes' instead of str
-
     return unwrap_spec
 
 
@@ -706,18 +563,13 @@ class BuiltinCode(Code):
     # When a BuiltinCode is stored in a Function object,
     # you get the functionality of CPython's built-in function type.
 
-    @not_rpython
     def __init__(self, func, unwrap_spec=None, self_type=None,
                  descrmismatch=None, doc=None):
-        from rpython.rlib import rutf8
-        from rpython.flowspace.bytecode import cpython_code_signature
+        "NOT_RPYTHON"
         # 'implfunc' is the interpreter-level function.
         # Note that this uses a lot of (construction-time) introspection.
         Code.__init__(self, func.__name__)
         self.docstring = doc or func.__doc__
-        if self.docstring:
-            # check that it's utf-8
-            rutf8.check_utf8(self.docstring, False)
 
         self.identifier = "%s-%s-%s" % (func.__module__, func.__name__,
                                         getattr(self_type, '__name__', '*'))
@@ -734,10 +586,8 @@ class BuiltinCode(Code):
         # (function, cls) use function to check/unwrap argument of type cls
 
         # First extract the signature from the (CPython-level) code object
-        sig = cpython_code_signature(func.func_code)
-        argnames = sig.argnames
-        varargname = sig.varargname
-        kwargname = sig.kwargname
+        from pypy.interpreter import pycode
+        argnames, varargname, kwargname = pycode.cpython_code_signature(func.func_code)
         self._argnames = argnames
 
         if unwrap_spec is None:
@@ -757,12 +607,11 @@ class BuiltinCode(Code):
             assert descrmismatch is None, (
                 "descrmismatch without a self-type specified")
 
+        orig_sig = SignatureBuilder(func, argnames, varargname, kwargname)
         app_sig = SignatureBuilder(func)
 
-        UnwrapSpec_Check(func, argnames).apply_over(unwrap_spec, app_sig)
-        self.sig = app_sig.signature()
-        argnames = self.sig.argnames
-        varargname = self.sig.varargname
+        UnwrapSpec_Check(orig_sig).apply_over(unwrap_spec, app_sig)
+        self.sig = argnames, varargname, kwargname = app_sig.signature()
 
         self.minargs = len(argnames)
         if varargname:
@@ -770,7 +619,7 @@ class BuiltinCode(Code):
         else:
             self.maxargs = self.minargs
 
-        self.activation = UnwrapSpec_EmitRun.make_activation(unwrap_spec, func, self.sig)
+        self.activation = UnwrapSpec_EmitRun.make_activation(unwrap_spec, func)
         self._bltin = func
         self._unwrap_spec = unwrap_spec
 
@@ -788,7 +637,6 @@ class BuiltinCode(Code):
                     self.func__args__ = func
                 elif unwrap_spec == [self_type, ObjSpace, Arguments]:
                     self.__class__ = BuiltinCodePassThroughArguments1
-                    self.descr_reqcls = self_type
                     miniglobals = {'func': func, 'self_type': self_type}
                     d = {}
                     source = """if 1:
@@ -808,18 +656,18 @@ class BuiltinCode(Code):
         mod = space.interp_w(MixedModule, w_mod)
         builtin_code = mod.get('builtin_code')
         return space.newtuple([builtin_code,
-                               space.newtuple([space.newtext(self.identifier)])])
+                               space.newtuple([space.wrap(self.identifier)])])
 
-    @staticmethod
-    def find(space, identifier):
+    def find(indentifier):
         from pypy.interpreter.function import Function
-        return Function.find(space, identifier).code
+        return Function._all[indentifier].code
+    find = staticmethod(find)
 
     def signature(self):
         return self.sig
 
     def getdocstring(self, space):
-        return space.newtext_or_none(self.docstring)
+        return space.wrap(self.docstring)
 
     def funcrun(self, func, args):
         return BuiltinCode.funcrun_obj(self, func, None, args)
@@ -827,41 +675,40 @@ class BuiltinCode(Code):
     def funcrun_obj(self, func, w_obj, args):
         space = func.space
         activation = self.activation
-
         scope_w = args.parse_obj(w_obj, func.name, self.sig,
-                                 func.defs_w, func.w_kw_defs, self.minargs)
+                                 func.defs_w, self.minargs)
         try:
             w_result = activation._run(space, scope_w)
         except DescrMismatch:
             if w_obj is not None:
                 args = args.prepend(w_obj)
-            return self._type_unwrap_mismatch(space, args)
-        except Exception as e:
+            return scope_w[0].descr_call_mismatch(space,
+                                                  self.descrmismatch_op,
+                                                  self.descr_reqcls,
+                                                  args)
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
             w_result = space.w_None
         return w_result
 
-    def _type_unwrap_mismatch(self, space, args):
-        w_obj = args.firstarg()
-        if w_obj is None:
-            raise oefmt(space.w_SystemError, "unexpected DescrMismatch error")
-        return w_obj.descr_call_mismatch(space,
-                                         self.descrmismatch_op,
-                                         self.descr_reqcls,
-                                         args)
-
     def handle_exception(self, space, e):
         try:
             if not we_are_translated():
                 raise
             raise e
-        except OperationError:
-            raise
-        except Exception as e:      # general fall-back
-            from pypy.interpreter import error
-            raise error.get_converted_unexpected_exception(space, e)
+        except KeyboardInterrupt:
+            raise OperationError(space.w_KeyboardInterrupt,
+                                 space.w_None)
+        except MemoryError:
+            raise OperationError(space.w_MemoryError, space.w_None)
+        except rstackovf.StackOverflow, e:
+            rstackovf.check_stack_overflow()
+            raise OperationError(space.w_RuntimeError,
+                                space.wrap("maximum recursion depth exceeded"))
+        except RuntimeError:   # not on top of py.py
+            raise OperationError(space.w_RuntimeError, space.w_None)
 
 # (verbose) performance hack below
 
@@ -874,8 +721,11 @@ class BuiltinCodePassThroughArguments0(BuiltinCode):
         try:
             w_result = self.func__args__(space, args)
         except DescrMismatch:
-            return self._type_unwrap_mismatch(space, args)
-        except Exception as e:
+            return args.firstarg().descr_call_mismatch(space,
+                                                  self.descrmismatch_op,
+                                                  self.descr_reqcls,
+                                                  args)
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
@@ -892,8 +742,11 @@ class BuiltinCodePassThroughArguments1(BuiltinCode):
         try:
             w_result = self.func__args__(space, w_obj, args)
         except DescrMismatch:
-            return self._type_unwrap_mismatch(space, args.prepend(w_obj))
-        except Exception as e:
+            return args.firstarg().descr_call_mismatch(space,
+                                                  self.descrmismatch_op,
+                                                  self.descr_reqcls,
+                                                  args.prepend(w_obj))
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
@@ -909,32 +762,29 @@ class BuiltinCode0(BuiltinCode):
         try:
             w_result = self.fastfunc_0(space)
         except DescrMismatch:
-            raise oefmt(space.w_SystemError, "unexpected DescrMismatch error")
-        except Exception as e:
+            raise OperationError(space.w_SystemError,
+                                 space.wrap("unexpected DescrMismatch error"))
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
             w_result = space.w_None
         return w_result
 
-w_root_or_none = sigtypes.instance(W_Root, can_be_None=True)
 
-@finishsigs
 class BuiltinCode1(BuiltinCode):
     _immutable_ = True
     fast_natural_arity = 1
 
-    @signature(sigtypes.self(), sigtypes.any(),
-               w_root_or_none,
-               w_root_or_none,
-               returns=w_root_or_none)
     def fastcall_1(self, space, w_func, w1):
         try:
             w_result = self.fastfunc_1(space, w1)
         except DescrMismatch:
-            return self._type_unwrap_mismatch(space,
+            return w1.descr_call_mismatch(space,
+                                          self.descrmismatch_op,
+                                          self.descr_reqcls,
                                           Arguments(space, [w1]))
-        except Exception as e:
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
@@ -942,23 +792,19 @@ class BuiltinCode1(BuiltinCode):
         return w_result
 
 
-@finishsigs
 class BuiltinCode2(BuiltinCode):
     _immutable_ = True
     fast_natural_arity = 2
 
-    @signature(sigtypes.self(), sigtypes.any(),
-               w_root_or_none,
-               w_root_or_none,
-               w_root_or_none,
-               returns=w_root_or_none)
     def fastcall_2(self, space, w_func, w1, w2):
         try:
             w_result = self.fastfunc_2(space, w1, w2)
         except DescrMismatch:
-            return self._type_unwrap_mismatch(space,
+            return w1.descr_call_mismatch(space,
+                                          self.descrmismatch_op,
+                                          self.descr_reqcls,
                                           Arguments(space, [w1, w2]))
-        except Exception as e:
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
@@ -966,51 +812,40 @@ class BuiltinCode2(BuiltinCode):
         return w_result
 
 
-@finishsigs
 class BuiltinCode3(BuiltinCode):
     _immutable_ = True
     fast_natural_arity = 3
 
-    @signature(sigtypes.self(), sigtypes.any(),
-               w_root_or_none,
-               w_root_or_none,
-               w_root_or_none,
-               w_root_or_none,
-               returns=w_root_or_none)
     def fastcall_3(self, space, func, w1, w2, w3):
         try:
             w_result = self.fastfunc_3(space, w1, w2, w3)
         except DescrMismatch:
-            return self._type_unwrap_mismatch(space,
+            return w1.descr_call_mismatch(space,
+                                          self.descrmismatch_op,
+                                          self.descr_reqcls,
                                           Arguments(space, [w1, w2, w3]))
-        except Exception as e:
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
             w_result = space.w_None
         return w_result
 
-@finishsigs
+
 class BuiltinCode4(BuiltinCode):
     _immutable_ = True
     fast_natural_arity = 4
 
-    @signature(sigtypes.self(), sigtypes.any(),
-               w_root_or_none,
-               w_root_or_none,
-               w_root_or_none,
-               w_root_or_none,
-               w_root_or_none,
-               returns=w_root_or_none)
     def fastcall_4(self, space, func, w1, w2, w3, w4):
-        from rpython.rlib.debug import check_annotation
         try:
             w_result = self.fastfunc_4(space, w1, w2, w3, w4)
         except DescrMismatch:
-            return self._type_unwrap_mismatch(space,
+            return w1.descr_call_mismatch(space,
+                                          self.descrmismatch_op,
+                                          self.descr_reqcls,
                                           Arguments(space,
                                                     [w1, w2, w3, w4]))
-        except Exception as e:
+        except Exception, e:
             self.handle_exception(space, e)
             w_result = None
         if w_result is None:
@@ -1041,9 +876,8 @@ def interpindirect2app(unbound_meth, unwrap_spec=None):
     else:
         assert isinstance(unwrap_spec, dict)
         unwrap_spec = unwrap_spec.copy()
-    unwrap_spec['self'] = 'self'
-    return interp2app(globals()['unwrap_spec'](**unwrap_spec)(f),
-                      self_type=base_cls)
+    unwrap_spec['self'] = base_cls
+    return interp2app(globals()['unwrap_spec'](**unwrap_spec)(f))
 
 class interp2app(W_Root):
     """Build a gateway that calls 'f' at interp-level."""
@@ -1052,13 +886,13 @@ class interp2app(W_Root):
 
     instancecache = {}
 
-    @not_rpython
     def __new__(cls, f, app_name=None, unwrap_spec=None, descrmismatch=None,
-                as_classmethod=False, doc=None, self_type=None):
+                as_classmethod=False, doc=None):
 
+        "NOT_RPYTHON"
         # f must be a function whose name does NOT start with 'app_'
+        self_type = None
         if hasattr(f, 'im_func'):
-            assert self_type in (None, f.im_class)
             self_type = f.im_class
             f = f.im_func
         if not isinstance(f, types.FunctionType):
@@ -1088,80 +922,59 @@ class interp2app(W_Root):
         self.name = app_name
         self.as_classmethod = as_classmethod
 
-        argnames = self._code._argnames
-        defaults = f.func_defaults or ()
-        self._staticdefs = dict(zip(
-            argnames[len(argnames) - len(defaults):], defaults))
-
+        if not f.func_defaults:
+            self._staticdefs = []
+        else:
+            argnames = self._code._argnames
+            defaults = f.func_defaults
+            self._staticdefs = zip(argnames[-len(defaults):], defaults)
         return self
 
-    @not_rpython
     def _getdefaults(self, space):
-        alldefs_w = {}
-        assert len(self._code._argnames) == len(self._code._unwrap_spec)
-        for name, spec in zip(self._code._argnames, self._code._unwrap_spec):
-            if name in ('__posonly__', '__kwonly__'):
-                continue
-
-            defaultval = self._staticdefs.get(name, NO_DEFAULT)
-            w_def = Ellipsis
+        "NOT_RPYTHON"
+        defs_w = []
+        for name, defaultval in self._staticdefs:
             if name.startswith('w_'):
-                assert defaultval in (NO_DEFAULT, None), (
+                assert defaultval is None, (
                     "%s: default value for '%s' can only be None, got %r; "
                     "use unwrap_spec(...=WrappedDefault(default))" % (
                     self._code.identifier, name, defaultval))
-                if defaultval is None:
-                    w_def = None
-
-            if isinstance(spec, tuple) and spec[0] is W_Root:
-                assert False, "use WrappedDefault"
-            elif isinstance(spec, WrappedDefault):
-                assert name.startswith('w_')
-                defaultval = spec.default_value
-                w_def = Ellipsis
-
-            if defaultval is not NO_DEFAULT:
-                if name != '__args__' and name != 'args_w':
-                    if w_def is Ellipsis:
-                        if isinstance(defaultval, str) and (
-                                # XXX hackish
-                                spec == 'bytes' or spec == 'bufferstr'
-                                or isinstance(spec, WrappedDefault)):
-                            w_def = space.newbytes(defaultval)
-                        else:
-                            w_def = space.wrap(defaultval)
-                    if name.startswith('w_'):
-                        name = name[2:]
-                    alldefs_w[name] = w_def
-        #
-        # Here, 'alldefs_w' maps some argnames to their wrapped default
-        # value.  We return two lists:
-        #  - a list of defaults for positional arguments, which covers
-        #    some suffix of the sig.argnames list
-        #  - a list of pairs (w_name, w_def) for kwonly arguments
-        #
-        sig = self._code.sig
-        first_defined = 0
-        allposargnames = sig.argnames[:sig.num_argnames()]
-        n_allposargnames = len(allposargnames)
-        while (first_defined < n_allposargnames and
-               allposargnames[first_defined] not in alldefs_w):
-            first_defined += 1
-        defs_w = [alldefs_w.pop(name) for name in allposargnames[first_defined:]]
-
-        kw_defs_w = None
-        if alldefs_w:
-            kw_defs_w = []
-            for name, w_def in sorted(alldefs_w.items()):
-                assert name in sig.argnames[-sig.kwonlyargcount:]
-                w_name = space.newtext(name)
-                kw_defs_w.append((w_name, w_def))
-
-        return defs_w, kw_defs_w
+                defs_w.append(None)
+            elif name != '__args__' and name != 'args_w':
+                defs_w.append(space.wrap(defaultval))
+        if self._code._unwrap_spec:
+            UNDEFINED = object()
+            alldefs_w = [UNDEFINED] * len(self._code.sig[0])
+            if defs_w:
+                alldefs_w[-len(defs_w):] = defs_w
+            code = self._code
+            assert isinstance(code._unwrap_spec, (list, tuple))
+            assert isinstance(code._argnames, list)
+            assert len(code._unwrap_spec) == len(code._argnames)
+            for i in range(len(code._unwrap_spec)-1, -1, -1):
+                spec = code._unwrap_spec[i]
+                argname = code._argnames[i]
+                if isinstance(spec, tuple) and spec[0] is W_Root:
+                    assert False, "use WrappedDefault"
+                if isinstance(spec, WrappedDefault):
+                    w_default = space.wrap(spec.default_value)
+                    assert isinstance(w_default, W_Root)
+                    assert argname.startswith('w_')
+                    argname = argname[2:]
+                    j = self._code.sig[0].index(argname)
+                    assert alldefs_w[j] in (UNDEFINED, None)
+                    alldefs_w[j] = w_default
+            first_defined = 0
+            while (first_defined < len(alldefs_w) and
+                   alldefs_w[first_defined] is UNDEFINED):
+                first_defined += 1
+            defs_w = alldefs_w[first_defined:]
+            assert UNDEFINED not in defs_w
+        return defs_w
 
     # lazy binding to space
 
-    def spacebind(self, space):
+    def __spacebind__(self, space):
         # we first make a real Function object out of it
         # and the result is a wrapped version of this Function.
         return self.get_function(space)
@@ -1174,25 +987,22 @@ class interp2app(W_Root):
 
 
 class GatewayCache(SpaceCache):
-    @not_rpython
     def build(cache, gateway):
+        "NOT_RPYTHON"
         space = cache.space
-        defs_w, kw_defs_w = gateway._getdefaults(space)
+        defs = gateway._getdefaults(space) # needs to be implemented by subclass
         code = gateway._code
-        fn = FunctionWithFixedCode(space, code, None, defs_w, kw_defs_w,
-                                   forcename=gateway.name)
+        fn = FunctionWithFixedCode(space, code, None, defs, forcename=gateway.name)
         if not space.config.translating:
             fn.add_to_table()
         if gateway.as_classmethod:
-            fn = ClassMethod(fn)
+            fn = ClassMethod(space.wrap(fn))
         #
         from pypy.module.sys.vm import exc_info
         if code._bltin is exc_info:
             assert space._code_of_sys_exc_info is None
             space._code_of_sys_exc_info = code
         #
-        if hasattr(space, '_see_interp2app'):
-            space._see_interp2app(gateway)      # only for fake/objspace.py
         return fn
 
 
@@ -1240,14 +1050,14 @@ class ApplevelClass:
 
     def buildmodule(self, space, name='applevel'):
         from pypy.interpreter.module import Module
-        return Module(space, space.newtext(name), self.getwdict(space))
+        return Module(space, space.wrap(name), self.getwdict(space))
 
     def wget(self, space, name):
         w_globals = self.getwdict(space)
-        return space.getitem(w_globals, space.newtext(name))
+        return space.getitem(w_globals, space.wrap(name))
 
-    @not_rpython
     def interphook(self, name):
+        "NOT_RPYTHON"
         def appcaller(space, *args_w):
             if not isinstance(space, ObjSpace):
                 raise TypeError("first argument must be a space instance.")
@@ -1284,18 +1094,17 @@ class ApplevelCache(SpaceCache):
     """NOT_RPYTHON
     The cache mapping each applevel instance to its lazily built w_dict"""
 
-    @not_rpython
     def build(self, app):
-        "Called indirectly by Applevel.getwdict()."
+        "NOT_RPYTHON.  Called indirectly by Applevel.getwdict()."
         return build_applevel_dict(app, self.space)
 
 
 # __________ pure applevel version __________
 
-@not_rpython
 def build_applevel_dict(self, space):
+    "NOT_RPYTHON"
     w_glob = space.newdict(module=True)
-    space.setitem(w_glob, space.newtext('__name__'), space.newtext(self.modname))
+    space.setitem(w_glob, space.wrap('__name__'), space.wrap(self.modname))
     space.exec_(self.source, w_glob, w_glob,
                 hidden_applevel=self.hidden_applevel,
                 filename=self.filename)
@@ -1304,9 +1113,8 @@ def build_applevel_dict(self, space):
 # ____________________________________________________________
 
 
-@not_rpython
 def appdef(source, applevel=ApplevelClass, filename=None):
-    """ build an app-level helper function, like for example:
+    """ NOT_RPYTHON: build an app-level helper function, like for example:
     myfunc = appdef('''myfunc(x, y):
                            return x+y
                     ''')
@@ -1322,15 +1130,15 @@ def appdef(source, applevel=ApplevelClass, filename=None):
             source = source[source.find('\n') + 1:].lstrip()
         assert source.startswith("def "), "can only transform functions"
         source = source[4:]
-        # The following flags have no effect any more in app-level code
-        # (i.e. they are always on anyway), and have been removed:
-        #    CO_FUTURE_DIVISION
-        #    CO_FUTURE_ABSOLUTE_IMPORT
-        #    CO_FUTURE_PRINT_FUNCTION
-        #    CO_FUTURE_UNICODE_LITERALS
-        # Original code was, for each of these flags:
-        #    if flags & __future__.CO_xxx:
-        #        prefix += "from __future__ import yyy\n"
+        import __future__
+        if flags & __future__.CO_FUTURE_DIVISION:
+            prefix += "from __future__ import division\n"
+        if flags & __future__.CO_FUTURE_ABSOLUTE_IMPORT:
+            prefix += "from __future__ import absolute_import\n"
+        if flags & __future__.CO_FUTURE_PRINT_FUNCTION:
+            prefix += "from __future__ import print_function\n"
+        if flags & __future__.CO_FUTURE_UNICODE_LITERALS:
+            prefix += "from __future__ import unicode_literals\n"
     p = source.find('(')
     assert p >= 0
     funcname = source[:p].strip()
@@ -1352,6 +1160,6 @@ class applevel_temp(ApplevelClass):
 
 
 # app2interp_temp is used for testing mainly
-@not_rpython
 def app2interp_temp(func, applevel_temp=applevel_temp, filename=None):
+    """ NOT_RPYTHON """
     return appdef(func, applevel_temp, filename=filename)
