@@ -13,5 +13,5 @@ class W_CTypeVoid(W_CType):
     def __init__(self, space):
         W_CType.__init__(self, space, -1, "void", len("void"))
 
-    def copy_and_convert_to_object(self, cdata):
+    def copy_and_convert_to_object(self, source):
         return self.space.w_None
