@@ -1,24 +1,14 @@
 from __future__ import with_statement
-import sys
 
-from pypy.interpreter.error import OperationError, oefmt
-from pypy.module._rawffi.interp_rawffi import wrap_dlopenerror
+from pypy.interpreter.error import OperationError
 
 from rpython.rlib import jit
-from rpython.rlib.objectmodel import specialize, we_are_translated
+from rpython.rlib.objectmodel import specialize
 from rpython.rlib.rarithmetic import r_uint, r_ulonglong
 from rpython.rlib.unroll import unrolling_iterable
-from rpython.rlib.rdynload import dlopen, DLOpenError, DLLHANDLE
-from rpython.rlib.nonconst import NonConstant
 from rpython.rtyper.lltypesystem import lltype, llmemory, rffi
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
 
-if sys.platform == 'win32':
-    from rpython.rlib.rdynload import dlopenU
-    WIN32 = True
-else:
-    WIN32 = False
-
 
 # ____________________________________________________________
 
@@ -112,14 +102,6 @@ def write_raw_float_data(target, source, size):
 def write_raw_longdouble_data(target, source):
     rffi.cast(rffi.LONGDOUBLEP, target)[0] = source
 
-@jit.dont_look_inside    # lets get_nonmovingbuffer_ll_final_null be inlined
-def write_string_as_charp(target, string):
-    from pypy.module._cffi_backend.ctypefunc import set_mustfree_flag
-    buf, llobj, buf_flag = rffi.get_nonmovingbuffer_ll_final_null(string)
-    set_mustfree_flag(target, ord(buf_flag))   # 4, 5 or 6
-    rffi.cast(rffi.CCHARPP, target)[0] = buf
-    return llobj
-
 # ____________________________________________________________
 
 sprintf_longdouble = rffi.llexternal(
@@ -146,87 +128,88 @@ def as_long_long(space, w_ob):
     # (possibly) convert and cast a Python object to a long long.
     # This version accepts a Python int too, and does convertions from
     # other types of objects.  It refuses floats.
+    if space.is_w(space.type(w_ob), space.w_int):   # shortcut
+        return space.int_w(w_ob)
     try:
-        return space.int_w(w_ob, allow_conversion=False)
-    except OperationError as e:
-        if not (e.match(space, space.w_OverflowError) or
-                e.match(space, space.w_TypeError)):
+        bigint = space.bigint_w(w_ob, allow_conversion=False)
+    except OperationError, e:
+        if not e.match(space, space.w_TypeError):
             raise
         if _is_a_float(space, w_ob):
             raise
-    bigint = space.bigint_w(w_ob, allow_conversion=True)
+        bigint = space.bigint_w(space.int(w_ob), allow_conversion=False)
     try:
         return bigint.tolonglong()
     except OverflowError:
-        raise OperationError(space.w_OverflowError, space.newtext(ovf_msg))
+        raise OperationError(space.w_OverflowError, space.wrap(ovf_msg))
 
 def as_long(space, w_ob):
     # Same as as_long_long(), but returning an int instead.
+    if space.is_w(space.type(w_ob), space.w_int):   # shortcut
+        return space.int_w(w_ob)
     try:
-        return space.int_w(w_ob, allow_conversion=False)
-    except OperationError as e:
-        if not (e.match(space, space.w_OverflowError) or
-                e.match(space, space.w_TypeError)):
+        bigint = space.bigint_w(w_ob, allow_conversion=False)
+    except OperationError, e:
+        if not e.match(space, space.w_TypeError):
             raise
         if _is_a_float(space, w_ob):
             raise
-    return space.int_w(w_ob, allow_conversion=True)
+        bigint = space.bigint_w(space.int(w_ob), allow_conversion=False)
+    try:
+        return bigint.toint()
+    except OverflowError:
+        raise OperationError(space.w_OverflowError, space.wrap(ovf_msg))
 
 def as_unsigned_long_long(space, w_ob, strict):
     # (possibly) convert and cast a Python object to an unsigned long long.
     # This accepts a Python int too, and does convertions from other types of
     # objects.  If 'strict', complains with OverflowError; if 'not strict',
     # mask the result and round floats.
+    if space.is_w(space.type(w_ob), space.w_int):   # shortcut
+        value = space.int_w(w_ob)
+        if strict and value < 0:
+            raise OperationError(space.w_OverflowError, space.wrap(neg_msg))
+        return r_ulonglong(value)
     try:
-        value = space.int_w(w_ob, allow_conversion=False)
-    except OperationError as e:
-        if not (e.match(space, space.w_OverflowError) or
-                e.match(space, space.w_TypeError)):
+        bigint = space.bigint_w(w_ob, allow_conversion=False)
+    except OperationError, e:
+        if not e.match(space, space.w_TypeError):
             raise
         if strict and _is_a_float(space, w_ob):
             raise
-    else:
-        if strict and value < 0:
-            raise OperationError(space.w_OverflowError, space.newtext(neg_msg))
-        return r_ulonglong(value)
-    # note that if not 'strict', then space.int() will round down floats
-    bigint = space.bigint_w(space.int(w_ob), allow_conversion=False)
+        bigint = space.bigint_w(space.int(w_ob), allow_conversion=False)
     if strict:
         try:
             return bigint.toulonglong()
         except ValueError:
-            raise OperationError(space.w_OverflowError, space.newtext(neg_msg))
+            raise OperationError(space.w_OverflowError, space.wrap(neg_msg))
         except OverflowError:
-            raise OperationError(space.w_OverflowError, space.newtext(ovf_msg))
+            raise OperationError(space.w_OverflowError, space.wrap(ovf_msg))
     else:
         return bigint.ulonglongmask()
 
 def as_unsigned_long(space, w_ob, strict):
     # same as as_unsigned_long_long(), but returning just an Unsigned
+    if space.is_w(space.type(w_ob), space.w_int):   # shortcut
+        value = space.int_w(w_ob)
+        if strict and value < 0:
+            raise OperationError(space.w_OverflowError, space.wrap(neg_msg))
+        return r_uint(value)
     try:
-        value = space.int_w(w_ob, allow_conversion=False)
-    except OperationError as e:
-        if not (e.match(space, space.w_OverflowError) or
-                e.match(space, space.w_TypeError)):
+        bigint = space.bigint_w(w_ob, allow_conversion=False)
+    except OperationError, e:
+        if not e.match(space, space.w_TypeError):
             raise
         if strict and _is_a_float(space, w_ob):
             raise
-    else:
-        if strict and value < 0:
-            raise OperationError(space.w_OverflowError, space.newtext(neg_msg))
-        if not we_are_translated():
-            if isinstance(value, NonConstant):   # hack for test_ztranslation
-                return r_uint(0)
-        return r_uint(value)
-    # note that if not 'strict', then space.int() will round down floats
-    bigint = space.bigint_w(space.int(w_ob), allow_conversion=False)
+        bigint = space.bigint_w(space.int(w_ob), allow_conversion=False)
     if strict:
         try:
             return bigint.touint()
         except ValueError:
-            raise OperationError(space.w_OverflowError, space.newtext(neg_msg))
+            raise OperationError(space.w_OverflowError, space.wrap(neg_msg))
         except OverflowError:
-            raise OperationError(space.w_OverflowError, space.newtext(ovf_msg))
+            raise OperationError(space.w_OverflowError, space.wrap(ovf_msg))
     else:
         return bigint.uintmask()
 
@@ -252,12 +235,9 @@ class _NotStandardObject(Exception):
 
 def _standard_object_as_bool(space, w_ob):
     if space.isinstance_w(w_ob, space.w_int):
-        try:
-            return space.int_w(w_ob) != 0
-        except OperationError as e:
-            if not e.match(space, space.w_OverflowError):
-                raise
-            return space.bigint_w(w_ob).tobool()
+        return space.int_w(w_ob) != 0
+    if space.isinstance_w(w_ob, space.w_long):
+        return space.bigint_w(w_ob).tobool()
     if space.isinstance_w(w_ob, space.w_float):
         return space.float_w(w_ob) != 0.0
     raise _NotStandardObject
@@ -273,11 +253,10 @@ _is_nonnull_longdouble = rffi.llexternal(
     sandboxsafe=True)
 
 # split here for JIT backends that don't support floats/longlongs/etc.
-@jit.dont_look_inside
 def is_nonnull_longdouble(cdata):
     return _is_nonnull_longdouble(read_raw_longdouble_data(cdata))
 def is_nonnull_float(cdata, size):
-    return read_raw_float_data(cdata, size) != 0.0    # note: True if a NaN
+    return read_raw_float_data(cdata, size) != 0.0
 
 def object_as_bool(space, w_ob):
     # convert and cast a Python object to a boolean.  Accept an integer
@@ -306,7 +285,24 @@ def object_as_bool(space, w_ob):
     try:
         return _standard_object_as_bool(space, w_io)
     except _NotStandardObject:
-        raise oefmt(space.w_TypeError, "integer/float expected")
+        raise OperationError(space.w_TypeError,
+                             space.wrap("integer/float expected"))
+
+# ____________________________________________________________
+
+def get_new_array_length(space, w_value):
+    if (space.isinstance_w(w_value, space.w_list) or
+        space.isinstance_w(w_value, space.w_tuple)):
+        return (w_value, space.int_w(space.len(w_value)))
+    elif space.isinstance_w(w_value, space.w_basestring):
+        # from a string, we add the null terminator
+        return (w_value, space.int_w(space.len(w_value)) + 1)
+    else:
+        explicitlength = space.getindex_w(w_value, space.w_OverflowError)
+        if explicitlength < 0:
+            raise OperationError(space.w_ValueError,
+                                 space.wrap("negative array length"))
+        return (space.w_None, explicitlength)
 
 # ____________________________________________________________
 
@@ -404,48 +400,3 @@ def unpack_cfloat_list_from_raw_array(float_list, source):
     ptr = rffi.cast(rffi.FLOATP, source)
     for i in range(len(float_list)):
         float_list[i] = rffi.cast(lltype.Float, ptr[i])
-
-# ____________________________________________________________
-
-def dlopen_w(space, w_filename, flags):
-    from pypy.module._cffi_backend.cdataobj import W_CData
-    from pypy.module._cffi_backend import ctypeptr
-
-    autoclose = True
-    if isinstance(w_filename, W_CData):
-        # 'flags' ignored in this case
-        w_ctype = w_filename.ctype
-        if (not isinstance(w_ctype, ctypeptr.W_CTypePointer) or
-            not w_ctype.is_void_ptr):
-            raise oefmt(space.w_TypeError,
-                    "dlopen() takes a file name or 'void *' handle, not '%s'",
-                    w_ctype.name)
-        handle = w_filename.unsafe_escaping_ptr()
-        if not handle:
-            raise oefmt(space.w_RuntimeError, "cannot call dlopen(NULL)")
-        fname = w_ctype.extra_repr(handle)
-        handle = rffi.cast(DLLHANDLE, handle)
-        autoclose = False
-        #
-    elif WIN32 and space.isinstance_w(w_filename, space.w_unicode):
-        fname = space.text_w(w_filename)
-        utf8_name = space.utf8_w(w_filename)
-        uni_len = space.len_w(w_filename)
-        with rffi.scoped_utf82wcharp(utf8_name, uni_len) as ll_libname:
-            try:
-                handle = dlopenU(ll_libname, flags)
-            except DLOpenError as e:
-                raise wrap_dlopenerror(space, e, fname)
-    else:
-        if space.is_none(w_filename):
-            fname = None
-        else:
-            fname = space.fsencode_w(w_filename)
-        with rffi.scoped_str2charp(fname) as ll_libname:
-            if fname is None:
-                fname = "<None>"
-            try:
-                handle = dlopen(ll_libname, flags)
-            except DLOpenError as e:
-                raise wrap_dlopenerror(space, e, fname)
-    return fname, handle, autoclose
