@@ -32,23 +32,18 @@ class BaseAppTestFFI(object):
         #
         c_file.write(py.code.Source('\n'.join(snippets)))
         eci = ExternalCompilationInfo(include_dirs=[cdir])
-        # Windows note: can't reuse the same file name 'x.dll', because
-        # the previous one is likely still opened
-        return str(platform.compile([c_file], eci, 'x' + cls.__name__,
-                                    standalone=False))
+        return str(platform.compile([c_file], eci, 'x', standalone=False))
 
     def setup_class(cls):
         space = cls.space
         cls.w_iswin32 = space.wrap(sys.platform == 'win32')
-        cls.w_iswin64 = space.wrap(sys.platform == 'win32'
-                                   and sys.maxint == 2**63-1)
         cls.w_libfoo_name = space.wrap(cls.prepare_c_example())
         cls.w_libc_name = space.wrap(get_libc_name())
         libm_name = get_libm_name(sys.platform)
         cls.w_libm_name = space.wrap(libm_name)
         libm = CDLL(libm_name)
         pow = libm.getpointer('pow', [], types.void)
-        pow_addr = rffi.cast(rffi.SIGNED, pow.funcsym)
+        pow_addr = rffi.cast(rffi.LONG, pow.funcsym)
         cls._libm = libm     # otherwise it gets unloaded - argh!
         cls.w_pow_addr = space.wrap(pow_addr)
 
@@ -88,20 +83,20 @@ class AppTestFFI(BaseAppTestFFI):
         pow = libm.getfunc('pow', [types.double, types.double], types.double)
         assert pow(2, 3) == 8
 
-    @py.test.mark.skipif("py.test.config.option.runappdirect")
     def test_getaddr(self):
         from _rawffi.alt import CDLL, types
         libm = CDLL(self.libm_name)
         pow = libm.getfunc('pow', [types.double, types.double], types.double)
         assert pow.getaddr() == self.pow_addr
 
-    @py.test.mark.skipif("py.test.config.option.runappdirect")
     def test_getaddressindll(self):
         import sys
         from _rawffi.alt import CDLL
         libm = CDLL(self.libm_name)
         pow_addr = libm.getaddressindll('pow')
-        fff = sys.maxsize*2-1
+        fff = sys.maxint*2-1
+        if sys.platform == 'win32' or sys.platform == 'darwin':
+            fff = sys.maxint*2+1
         assert pow_addr == self.pow_addr & fff
 
     def test_func_fromaddr(self):
@@ -124,7 +119,7 @@ class AppTestFFI(BaseAppTestFFI):
         libfoo = CDLL(self.libfoo_name)
         sum_xy = libfoo.getfunc('sum_xy', [types.sint, types.sint], types.sint)
         assert sum_xy(30, 12) == 42
-        assert sum_xy(sys.maxsize*2, 0) == -2
+        assert sum_xy(sys.maxint*2, 0) == -2
 
     def test_void_result(self):
         """
@@ -183,7 +178,7 @@ class AppTestFFI(BaseAppTestFFI):
                                         types.void)
         assert get_dummy() == 0
         ptr = get_dummy_ptr()
-        assert type(ptr) is int
+        assert type(ptr) in (int, long)
         ptr2 = MyPointerWrapper(ptr)
         set_val_to_ptr(ptr2, 123)
         assert get_dummy() == 123
@@ -210,10 +205,10 @@ class AppTestFFI(BaseAppTestFFI):
         mystrlen = libfoo.getfunc('mystrlen', [types.char_p], types.slong)
         #
         # first, try automatic conversion from a string
-        assert mystrlen(b'foobar') == 6
+        assert mystrlen('foobar') == 6
         # then, try to pass an explicit pointer
         CharArray = _rawffi.Array('c')
-        mystr = CharArray(7, b'foobar')
+        mystr = CharArray(7, 'foobar')
         assert mystrlen(mystr.buffer) == 6
         mystr.free()
         mystrlen.free_temp_buffers()
@@ -260,9 +255,9 @@ class AppTestFFI(BaseAppTestFFI):
         do_nothing = libfoo.getfunc('do_nothing', [types.char_p], types.char_p)
         CharArray = _rawffi.Array('c')
         #
-        ptr = do_nothing(b'foobar')
+        ptr = do_nothing('foobar')
         array = CharArray.fromaddress(ptr, 7)
-        assert bytes(array) == b'foobar\00'
+        assert list(array) == list('foobar\00')
         do_nothing.free_temp_buffers()
 
     def test_typed_pointer_args(self):
@@ -293,7 +288,7 @@ class AppTestFFI(BaseAppTestFFI):
         from _rawffi.alt import CDLL, types
         libfoo = CDLL(self.libfoo_name)
         is_null_ptr = libfoo.getfunc('is_null_ptr', [types.void_p], types.ulong)
-        assert not is_null_ptr(sys.maxsize+1)
+        assert not is_null_ptr(sys.maxint+1)
 
     def test_unsigned_long_args(self):
         """
@@ -307,10 +302,10 @@ class AppTestFFI(BaseAppTestFFI):
         libfoo = CDLL(self.libfoo_name)
         sum_xy = libfoo.getfunc('sum_xy_ul', [types.ulong, types.ulong],
                                 types.ulong)
-        assert sum_xy(sys.maxsize, 12) == sys.maxsize+12
-        assert sum_xy(sys.maxsize+1, 12) == sys.maxsize+13
+        assert sum_xy(sys.maxint, 12) == sys.maxint+12
+        assert sum_xy(sys.maxint+1, 12) == sys.maxint+13
         #
-        res = sum_xy(sys.maxsize*2+3, 0)
+        res = sum_xy(sys.maxint*2+3, 0)
         assert res == 1
 
     def test_unsigned_short_args(self):
@@ -382,9 +377,7 @@ class AppTestFFI(BaseAppTestFFI):
         libfoo = CDLL(self.libfoo_name)
         my_toupper = libfoo.getfunc('my_toupper', [types.char],
                                     types.char)
-        res = my_toupper(b'c')
-        assert type(res) is bytes
-        assert res == b'C'
+        assert my_toupper('c') == 'C'
 
     def test_unichar_args(self):
         """
@@ -398,8 +391,8 @@ class AppTestFFI(BaseAppTestFFI):
         libfoo = CDLL(self.libfoo_name)
         sum_xy = libfoo.getfunc('sum_xy_wc', [types.unichar, types.unichar],
                                 types.unichar)
-        res = sum_xy(chr(1000), chr(2000))
-        assert type(res) is str
+        res = sum_xy(unichr(1000), unichr(2000))
+        assert type(res) is unicode
         assert ord(res) == 3000
 
     def test_single_float_args(self):
@@ -570,22 +563,21 @@ class AppTestFFI(BaseAppTestFFI):
         from _rawffi.alt import CDLL, types
         libfoo = CDLL(self.libfoo_name)
         raises(AttributeError, "libfoo.getfunc('I_do_not_exist', [], types.void)")
-        if self.iswin32 or self.iswin64:
+        if self.iswin32:
             skip("unix specific")
         libnone = CDLL(None)
         raises(AttributeError, "libnone.getfunc('I_do_not_exist', [], types.void)")
 
     def test_calling_convention1(self):
-        # win64 doesn't have __stdcall
-        if not self.iswin32 or self.iswin64:
-            skip("windows 32-bit specific")
+        if not self.iswin32:
+            skip("windows specific")
         from _rawffi.alt import WinDLL, types
         libm = WinDLL(self.libm_name)
         pow = libm.getfunc('pow', [types.double, types.double], types.double)
         try:
             pow(2, 3)
-        except ValueError as e:
-            assert str(e).startswith('Procedure called with')
+        except ValueError, e:
+            assert e.message.startswith('Procedure called with')
         else:
             assert 0, 'test must assert, wrong calling convention'
 
@@ -598,23 +590,21 @@ class AppTestFFI(BaseAppTestFFI):
         sleep(10)
 
     def test_calling_convention3(self):
-        # win64 doesn't have __stdcall
-        if not self.iswin32 or self.iswin64:
-            skip("windows 32-bit specific")
+        if not self.iswin32:
+            skip("windows specific")
         from _rawffi.alt import CDLL, types
         wrong_kernel = CDLL('Kernel32.dll')
         wrong_sleep = wrong_kernel.getfunc('Sleep', [types.uint], types.void)
         try:
             wrong_sleep(10)
-        except ValueError as e:
-            assert str(e).startswith('Procedure called with')
+        except ValueError, e:
+            assert e.message.startswith('Procedure called with')
         else:
             assert 0, 'test must assert, wrong calling convention'
 
     def test_func_fromaddr2(self):
-        # win64 doesn't have __stdcall
-        if not self.iswin32 or self.iswin64:
-            skip("windows 32-bit specific")
+        if not self.iswin32:
+            skip("windows specific")
         from _rawffi.alt import CDLL, types, FuncPtr
         from _rawffi import FUNCFLAG_STDCALL
         libm = CDLL(self.libm_name)
@@ -623,13 +613,12 @@ class AppTestFFI(BaseAppTestFFI):
                 [types.double, types.double], types.double, FUNCFLAG_STDCALL)
         try:
             wrong_pow(2, 3) == 8
-        except ValueError as e:
-            assert str(e).startswith('Procedure called with')
+        except ValueError, e:
+            assert e.message.startswith('Procedure called with')
         else:
             assert 0, 'test must assert, wrong calling convention'
 
     def test_func_fromaddr3(self):
-        # win64: check FUNCFLAG_STDCALL is ignored on win64, as it should be
         if not self.iswin32:
             skip("windows specific")
         from _rawffi.alt import WinDLL, types, FuncPtr
@@ -654,21 +643,3 @@ class AppTestFFI(BaseAppTestFFI):
         f_name = libfoo.getfunc('AAA_first_ordinal_function', [], types.sint)
         f_ordinal = libfoo.getfunc(1, [], types.sint)
         assert f_name.getaddr() == f_ordinal.getaddr()
-
-    def test_cdll_as_integer(self):
-        import _rawffi
-        from _rawffi.alt import CDLL
-        libfoo = CDLL(self.libfoo_name)
-        A = _rawffi.Array('i')
-        a = A(1, autofree=True)
-        a[0] = libfoo      # should cast libfoo to int/long automatically
-
-    def test_windll_as_integer(self):
-        if not self.iswin32:
-            skip("windows specific")
-        import _rawffi
-        from _rawffi.alt import WinDLL
-        libm = WinDLL(self.libm_name)
-        A = _rawffi.Array('i')
-        a = A(1, autofree=True)
-        a[0] = libm        # should cast libm to int/long automatically
