@@ -4,9 +4,9 @@ to app-level with apropriate interface
 """
 
 from pypy.interpreter.gateway import interp2app, unwrap_spec
-from pypy.interpreter.typedef import TypeDef, GetSetProperty, interp_attrproperty_w
+from pypy.interpreter.typedef import TypeDef, GetSetProperty, interp_attrproperty
 from rpython.rtyper.lltypesystem import lltype, rffi
-from pypy.interpreter.error import OperationError, oefmt
+from pypy.interpreter.error import OperationError
 from pypy.module._rawffi.interp_rawffi import segfault_exception
 from pypy.module._rawffi.interp_rawffi import W_DataShape, W_DataInstance
 from pypy.module._rawffi.interp_rawffi import unwrap_value, wrap_value
@@ -36,29 +36,30 @@ class W_Array(W_DataShape):
     def get_basic_ffi_type(self):
         return self.basicffitype
 
-    @unwrap_spec(length=int, autofree=int)
+    @unwrap_spec(length=int, autofree=bool)
     def descr_call(self, space, length, w_items=None, autofree=False):
-        result = self.allocate(space, length, bool(autofree))
+        result = self.allocate(space, length, autofree)
         if not space.is_none(w_items):
             items_w = space.unpackiterable(w_items)
             iterlength = len(items_w)
             if iterlength > length:
-                raise oefmt(space.w_ValueError,
-                            "too many items for specified array length")
+                raise OperationError(space.w_ValueError,
+                                     space.wrap("too many items for specified"
+                                                " array length"))
             for num in range(iterlength):
                 w_item = items_w[num]
                 unwrap_value(space, write_ptr, result.ll_buffer, num,
                              self.itemcode, w_item)
-        return result
+        return space.wrap(result)
 
     def descr_repr(self, space):
-        return space.newtext("<_rawffi.Array '%s' (%d, %d)>" % (self.itemcode,
-                                                                self.size,
-                                                                self.alignment))
+        return space.wrap("<_rawffi.Array '%s' (%d, %d)>" % (self.itemcode,
+                                                             self.size,
+                                                             self.alignment))
 
     @unwrap_spec(address=r_uint, length=int)
     def fromaddress(self, space, address, length):
-        return W_ArrayInstance(space, self, length, address)
+        return space.wrap(W_ArrayInstance(space, self, length, address))
 
 PRIMITIVE_ARRAY_TYPES = {}
 for _code in TYPEMAP:
@@ -91,13 +92,11 @@ class W_ArrayInstance(W_DataInstance):
         W_DataInstance.__init__(self, space, memsize, address)
         self.length = length
         self.shape = shape
-        self.fmt = shape.itemcode
-        self.itemsize = shape.size
 
     def descr_repr(self, space):
         addr = rffi.cast(lltype.Unsigned, self.ll_buffer)
-        return space.newtext("<_rawffi array %x of length %d>" % (addr,
-                                                                  self.length))
+        return space.wrap("<_rawffi array %x of length %d>" % (addr,
+                                                               self.length))
 
     # This only allows non-negative indexes.  Arrays of shape 'c' also
     # support simple slices.
@@ -107,12 +106,13 @@ class W_ArrayInstance(W_DataInstance):
             raise segfault_exception(space, "setting element of freed array")
         if num >= self.length or num < 0:
             raise OperationError(space.w_IndexError, space.w_None)
-        unwrap_value(space, write_ptr, self.ll_buffer, num, self.fmt, w_value)
+        unwrap_value(space, write_ptr, self.ll_buffer, num,
+                     self.shape.itemcode, w_value)
 
     def descr_setitem(self, space, w_index, w_value):
         try:
             num = space.int_w(w_index)
-        except OperationError as e:
+        except OperationError, e:
             if not e.match(space, space.w_TypeError):
                 raise
             self.setslice(space, w_index, w_value)
@@ -124,12 +124,13 @@ class W_ArrayInstance(W_DataInstance):
             raise segfault_exception(space, "accessing elements of freed array")
         if num >= self.length or num < 0:
             raise OperationError(space.w_IndexError, space.w_None)
-        return wrap_value(space, read_ptr, self.ll_buffer, num, self.fmt)
+        return wrap_value(space, read_ptr, self.ll_buffer, num,
+                          self.shape.itemcode)
 
     def descr_getitem(self, space, w_index):
         try:
             num = space.int_w(w_index)
-        except OperationError as e:
+        except OperationError, e:
             if not e.match(space, space.w_TypeError):
                 raise
             return self.getslice(space, w_index)
@@ -137,24 +138,29 @@ class W_ArrayInstance(W_DataInstance):
             return self.getitem(space, num)
 
     def getlength(self, space):
-        return space.newint(self.length)
+        return space.wrap(self.length)
 
     @unwrap_spec(num=int)
     def descr_itemaddress(self, space, num):
-        ptr = rffi.ptradd(self.ll_buffer, self.itemsize * num)
-        return space.newint(rffi.cast(lltype.Unsigned, ptr))
+        itemsize = self.shape.size
+        ptr = rffi.ptradd(self.ll_buffer, itemsize * num)
+        return space.wrap(rffi.cast(lltype.Unsigned, ptr))
 
     def getrawsize(self):
-        return self.itemsize * self.length
+        itemsize = self.shape.size
+        return itemsize * self.length
 
     def decodeslice(self, space, w_slice):
         if not space.isinstance_w(w_slice, space.w_slice):
-            raise oefmt(space.w_TypeError, "index must be int or slice")
-        if self.fmt != 'c':
-            raise oefmt(space.w_TypeError, "only 'c' arrays support slicing")
-        w_start = space.getattr(w_slice, space.newtext('start'))
-        w_stop = space.getattr(w_slice, space.newtext('stop'))
-        w_step = space.getattr(w_slice, space.newtext('step'))
+            raise OperationError(space.w_TypeError,
+                                 space.wrap('index must be int or slice'))
+        letter = self.shape.itemcode
+        if letter != 'c':
+            raise OperationError(space.w_TypeError,
+                                 space.wrap("only 'c' arrays support slicing"))
+        w_start = space.getattr(w_slice, space.wrap('start'))
+        w_stop = space.getattr(w_slice, space.wrap('stop'))
+        w_step = space.getattr(w_slice, space.wrap('step'))
 
         if space.is_w(w_start, space.w_None):
             start = 0
@@ -167,9 +173,11 @@ class W_ArrayInstance(W_DataInstance):
         if not space.is_w(w_step, space.w_None):
             step = space.int_w(w_step)
             if step != 1:
-                raise oefmt(space.w_ValueError, "no step support")
+                raise OperationError(space.w_ValueError,
+                                     space.wrap("no step support"))
         if not (0 <= start <= stop <= self.length):
-            raise oefmt(space.w_ValueError, "slice out of bounds")
+            raise OperationError(space.w_ValueError,
+                                 space.wrap("slice out of bounds"))
         if not self.ll_buffer:
             raise segfault_exception(space, "accessing a freed array")
         return start, stop
@@ -178,26 +186,26 @@ class W_ArrayInstance(W_DataInstance):
         start, stop = self.decodeslice(space, w_slice)
         ll_buffer = self.ll_buffer
         result = [ll_buffer[i] for i in range(start, stop)]
-        return space.newbytes(''.join(result))
+        return space.wrap(''.join(result))
 
     def setslice(self, space, w_slice, w_value):
         start, stop = self.decodeslice(space, w_slice)
-        value = space.bytes_w(w_value)
+        value = space.str_w(w_value)
         if start + len(value) != stop:
-            raise oefmt(space.w_ValueError, "cannot resize array")
+            raise OperationError(space.w_ValueError,
+                                 space.wrap("cannot resize array"))
         ll_buffer = self.ll_buffer
         for i in range(len(value)):
             ll_buffer[start + i] = value[i]
 
-
 W_ArrayInstance.typedef = TypeDef(
-    'ArrayInstance', None, None, "read-write",
+    'ArrayInstance',
     __repr__    = interp2app(W_ArrayInstance.descr_repr),
     __setitem__ = interp2app(W_ArrayInstance.descr_setitem),
     __getitem__ = interp2app(W_ArrayInstance.descr_getitem),
     __len__     = interp2app(W_ArrayInstance.getlength),
     buffer      = GetSetProperty(W_ArrayInstance.getbuffer),
-    shape       = interp_attrproperty_w('shape', W_ArrayInstance),
+    shape       = interp_attrproperty('shape', W_ArrayInstance),
     free        = interp2app(W_ArrayInstance.free),
     byptr       = interp2app(W_ArrayInstance.byptr),
     itemaddress = interp2app(W_ArrayInstance.descr_itemaddress),
@@ -215,13 +223,13 @@ class W_ArrayInstanceAutoFree(W_ArrayInstance):
             self._free()
 
 W_ArrayInstanceAutoFree.typedef = TypeDef(
-    'ArrayInstanceAutoFree', None, None, "read-write",
+    'ArrayInstanceAutoFree',
     __repr__    = interp2app(W_ArrayInstance.descr_repr),
     __setitem__ = interp2app(W_ArrayInstance.descr_setitem),
     __getitem__ = interp2app(W_ArrayInstance.descr_getitem),
     __len__     = interp2app(W_ArrayInstance.getlength),
     buffer      = GetSetProperty(W_ArrayInstance.getbuffer),
-    shape       = interp_attrproperty_w('shape', W_ArrayInstance),
+    shape       = interp_attrproperty('shape', W_ArrayInstance),
     byptr       = interp2app(W_ArrayInstance.byptr),
     itemaddress = interp2app(W_ArrayInstance.descr_itemaddress),
 )
