@@ -1,29 +1,23 @@
-import sys
+
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from rpython.rtyper.lltypesystem import lltype, rffi
 from pypy.module._rawffi.interp_rawffi import write_ptr
 from pypy.module._rawffi.structure import W_Structure
 from pypy.module._rawffi.interp_rawffi import (W_DataInstance, letter2tp,
-     unwrap_value, unpack_argshapes, got_libffi_error, is_narrow_integer_type,
-     LL_TYPEMAP, NARROW_INTEGER_TYPES)
+     unwrap_value, unpack_argshapes, got_libffi_error)
 from rpython.rlib.clibffi import USERDATA_P, CallbackFuncPtr, FUNCFLAG_CDECL
 from rpython.rlib.clibffi import ffi_type_void, LibFFIError
 from rpython.rlib import rweakref
 from pypy.module._rawffi.tracker import tracker
 from pypy.interpreter.error import OperationError
 from pypy.interpreter import gateway
-from rpython.rlib.unroll import unrolling_iterable
-
-BIGENDIAN = sys.byteorder == 'big'
-
-unroll_narrow_integer_types = unrolling_iterable(NARROW_INTEGER_TYPES)
 
 app = gateway.applevel('''
     def tbprint(tb, err):
         import traceback, sys
         traceback.print_tb(tb)
-        print(err, file=sys.stderr)
+        print >>sys.stderr, err
 ''', filename=__file__)
 
 tbprint = app.interphook("tbprint")
@@ -45,23 +39,14 @@ def callback(ll_args, ll_res, ll_userdata):
                     space, rffi.cast(rffi.SIZE_T, ll_args[i]))
             else:
                 # XXX other types?
-                args_w[i] = space.newint(rffi.cast(lltype.Unsigned, ll_args[i]))
+                args_w[i] = space.wrap(rffi.cast(rffi.ULONG, ll_args[i]))
         w_res = space.call(w_callable, space.newtuple(args_w))
         if callback_ptr.result is not None: # don't return void
-            ptr = ll_res
-            letter = callback_ptr.result
-            if BIGENDIAN:
-                # take care of narrow integers!
-                for int_type in unroll_narrow_integer_types:
-                    if int_type == letter:
-                        T = LL_TYPEMAP[int_type]
-                        n = rffi.sizeof(lltype.Signed) - rffi.sizeof(T)
-                        ptr = rffi.ptradd(ptr, n)
-                        break
-            unwrap_value(space, write_ptr, ptr, 0, letter, w_res)
-    except OperationError as e:
-        tbprint(space, e.get_w_traceback(space),
-                space.newtext(e.errorstr(space)))
+            unwrap_value(space, write_ptr, ll_res, 0,
+                         callback_ptr.result, w_res)
+    except OperationError, e:
+        tbprint(space, space.wrap(e.get_traceback()),
+                space.wrap(e.errorstr(space)))
         # force the result to be zero
         if callback_ptr.result is not None:
             resshape = letter2tp(space, callback_ptr.result)
@@ -79,7 +64,7 @@ class W_CallbackPtr(W_DataInstance):
         self.argtypes = unpack_argshapes(space, w_args)
         ffiargs = [tp.get_basic_ffi_type() for tp in self.argtypes]
         if not space.is_w(w_result, space.w_None):
-            self.result = space.text_w(w_result)
+            self.result = space.str_w(w_result)
             ffiresult = letter2tp(space, self.result).get_basic_ffi_type()
         else:
             self.result = None
@@ -135,7 +120,7 @@ def descr_new_callbackptr(space, w_type, w_callable, w_args, w_result,
     return W_CallbackPtr(space, w_callable, w_args, w_result, flags)
 
 W_CallbackPtr.typedef = TypeDef(
-    'CallbackPtr', None, None, "read",
+    'CallbackPtr',
     __new__ = interp2app(descr_new_callbackptr),
     byptr   = interp2app(W_CallbackPtr.byptr),
     buffer  = GetSetProperty(W_CallbackPtr.getbuffer),
