@@ -1,20 +1,12 @@
-import sys
 from pypy.interpreter.baseobjspace import W_Root
-from pypy.interpreter.buffer import SimpleView
 from pypy.interpreter.error import OperationError, oefmt, wrap_oserror
 from pypy.interpreter.gateway import interp2app, unwrap_spec
-from pypy.interpreter.typedef import interp_attrproperty
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
-from pypy.interpreter.unicodehelper import wcharpsize2utf8
-from pypy.interpreter.unicodehelper import wrap_unicode_out_of_range_error
 
 from rpython.rlib.clibffi import *
-from rpython.rlib.objectmodel import we_are_translated
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.rtyper.tool import rffi_platform
 from rpython.rlib.unroll import unrolling_iterable
-from rpython.rlib import rutf8
-from rpython.rlib.objectmodel import specialize, dict_to_switch
 import rpython.rlib.rposix as rposix
 
 _MS_WINDOWS = os.name == "nt"
@@ -27,8 +19,6 @@ from rpython.rlib.rarithmetic import intmask, r_uint
 from pypy.module._rawffi.buffer import RawFFIBuffer
 from pypy.module._rawffi.tracker import tracker
 
-BIGENDIAN = sys.byteorder == 'big'
-
 TYPEMAP = {
     # XXX A mess with unsigned/signed/normal chars :-/
     'c' : ffi_type_uchar,
@@ -52,18 +42,17 @@ TYPEMAP = {
     'O' : ffi_type_pointer,
     'Z' : ffi_type_pointer,
     '?' : cast_type_to_ffitype(lltype.Bool),
-    'v' : ffi_type_sshort
 }
 TYPEMAP_PTR_LETTERS = "POszZ"
-TYPEMAP_NUMBER_LETTERS = "bBhHiIlLqQ?v"
+TYPEMAP_NUMBER_LETTERS = "bBhHiIlLqQ?"
 TYPEMAP_FLOAT_LETTERS = "fd" # XXX long doubles are not propperly supported in
                              # rpython, so we ignore then here
 
 if _MS_WINDOWS:
     TYPEMAP['X'] = ffi_type_pointer
+    TYPEMAP['v'] = ffi_type_sshort
     TYPEMAP_PTR_LETTERS += 'X'
-
-TYPEMAP_FUNCTION = dict_to_switch(TYPEMAP)
+    TYPEMAP_NUMBER_LETTERS += 'v'
 
 def size_alignment(ffi_type):
     return intmask(ffi_type.c_size), intmask(ffi_type.c_alignment)
@@ -90,14 +79,14 @@ LL_TYPEMAP = {
     'O' : rffi.VOIDP,
     'P' : rffi.VOIDP,
     '?' : lltype.Bool,
-    'v' : rffi.SHORT
 }
 
 if _MS_WINDOWS:
     LL_TYPEMAP['X'] = rffi.CCHARP
+    LL_TYPEMAP['v'] = rffi.SHORT
 
 def letter2tp(space, key):
-    from pypy.module._rawffi.interp_array import PRIMITIVE_ARRAY_TYPES
+    from pypy.module._rawffi.array import PRIMITIVE_ARRAY_TYPES
     try:
         return PRIMITIVE_ARRAY_TYPES[key]
     except KeyError:
@@ -105,8 +94,8 @@ def letter2tp(space, key):
 
 def unpack_simple_shape(space, w_shape):
     # 'w_shape' must be either a letter or a tuple (struct, 1).
-    if space.isinstance_w(w_shape, space.w_text):
-        letter = space.text_w(w_shape)
+    if space.isinstance_w(w_shape, space.w_str):
+        letter = space.str_w(w_shape)
         return letter2tp(space, letter)
     else:
         w_shapetype, w_length = space.fixedview(w_shape, expected_length=2)
@@ -116,8 +105,8 @@ def unpack_simple_shape(space, w_shape):
 def unpack_shape_with_length(space, w_shape):
     # Allow 'w_shape' to be a letter or any (shape, number).
     # The result is always a W_Array.
-    if space.isinstance_w(w_shape, space.w_text):
-        letter = space.text_w(w_shape)
+    if space.isinstance_w(w_shape, space.w_str):
+        letter = space.str_w(w_shape)
         return letter2tp(space, letter)
     else:
         w_shapetype, w_length = space.fixedview(w_shape, expected_length=2)
@@ -128,7 +117,7 @@ def unpack_shape_with_length(space, w_shape):
         try:
             result = shape._array_shapes[length]
         except KeyError:
-            from pypy.module._rawffi.interp_array import W_Array
+            from pypy.module._rawffi.array import W_Array
             if isinstance(shape, W_Array) and length == 1:
                 result = shape
             else:
@@ -148,17 +137,13 @@ def unpack_argshapes(space, w_argtypes):
             for w_arg in space.unpackiterable(w_argtypes)]
 
 def got_libffi_error(space):
-    raise oefmt(space.w_SystemError, "not supported by libffi")
+    raise OperationError(space.w_SystemError,
+                         space.wrap("not supported by libffi"))
 
 def wrap_dlopenerror(space, e, filename):
-    if e.msg:
-        # dlerror can return garbage messages under ll2ctypes (not
-        # we_are_translated()), so repr it to avoid potential problems
-        # converting to unicode later
-        msg = e.msg if we_are_translated() else repr(e.msg)
-    else:
-        msg = 'unspecified error'
-    return oefmt(space.w_OSError, 'Cannot load library %s: %8', filename, msg)
+    return oefmt(space.w_OSError,
+                 "Cannot load library %s: %s",
+                 filename, e.msg if e.msg else "unspecified error")
 
 
 class W_CDLL(W_Root):
@@ -168,22 +153,19 @@ class W_CDLL(W_Root):
         self.w_cache = space.newdict()
         self.space = space
 
-    @unwrap_spec(flags=int, variadic_args=int)
-    def ptr(self, space, w_name, w_argtypes, w_restype, flags=FUNCFLAG_CDECL, variadic_args=0):
+    @unwrap_spec(flags=int)
+    def ptr(self, space, w_name, w_argtypes, w_restype, flags=FUNCFLAG_CDECL):
         """ Get a pointer for function name with provided argtypes
         and restype
         """
         resshape = unpack_resshape(space, w_restype)
-        if resshape is None:
-            w_resshape = space.w_None
-        else:
-            w_resshape = resshape
+        w = space.wrap
         argtypes_w = space.fixedview(w_argtypes)
         w_argtypes = space.newtuple(argtypes_w)
-        w_key = space.newtuple([w_name, w_argtypes, w_resshape])
+        w_key = space.newtuple([w_name, w_argtypes, w(resshape)])
         try:
             return space.getitem(self.w_cache, w_key)
-        except OperationError as e:
+        except OperationError, e:
             if e.match(space, space.w_KeyError):
                 pass
             else:
@@ -197,12 +179,12 @@ class W_CDLL(W_Root):
         else:
             ffi_restype = ffi_type_void
 
-        if space.isinstance_w(w_name, space.w_text):
-            name = space.text_w(w_name)
+        if space.isinstance_w(w_name, space.w_str):
+            name = space.str_w(w_name)
 
             try:
                 ptr = self.cdll.getrawpointer(name, ffi_argtypes, ffi_restype,
-                                              flags, variadic_args)
+                                              flags)
             except KeyError:
                 raise oefmt(space.w_AttributeError,
                             "No symbol %s found in library %s",
@@ -214,7 +196,7 @@ class W_CDLL(W_Root):
             ordinal = space.int_w(w_name)
             try:
                 ptr = self.cdll.getrawpointer_byordinal(ordinal, ffi_argtypes,
-                                                        ffi_restype, flags, variadic_args)
+                                                        ffi_restype, flags)
             except KeyError:
                 raise oefmt(space.w_AttributeError,
                             "No symbol %d found in library %s",
@@ -222,46 +204,37 @@ class W_CDLL(W_Root):
             except LibFFIError:
                 raise got_libffi_error(space)
         else:
-            raise oefmt(space.w_TypeError,
-                        "function name must be string or integer")
+            raise OperationError(space.w_TypeError, space.wrap(
+                "function name must be string or integer"))
 
         w_funcptr = W_FuncPtr(space, ptr, argshapes, resshape)
         space.setitem(self.w_cache, w_key, w_funcptr)
         return w_funcptr
 
-    @unwrap_spec(name='text')
+    @unwrap_spec(name=str)
     def getaddressindll(self, space, name):
         try:
             address_as_uint = rffi.cast(lltype.Unsigned,
                                         self.cdll.getaddressindll(name))
         except KeyError:
             raise oefmt(space.w_ValueError, "Cannot find symbol %s", name)
-        return space.newint(address_as_uint)
+        return space.wrap(address_as_uint)
 
-def open_cdll(space, name):
-    try:
-        return CDLL(name)
-    except DLOpenError as e:
-        raise wrap_dlopenerror(space, e, name or "<None>")
-    except OSError as e:
-        raise wrap_oserror(space, e)
-
-if _MS_WINDOWS:
-    name_spec = 'fsencode'
-else:
-    name_spec = 'fsencode_or_none'
-@unwrap_spec(name=name_spec)
+@unwrap_spec(name='str_or_None')
 def descr_new_cdll(space, w_type, name):
-    cdll = open_cdll(space, name)
-    return W_CDLL(space, name, cdll)
+    try:
+        cdll = CDLL(name)
+    except DLOpenError, e:
+        raise wrap_dlopenerror(space, e, name)
+    except OSError, e:
+        raise wrap_oserror(space, e)
+    return space.wrap(W_CDLL(space, name, cdll))
 
 W_CDLL.typedef = TypeDef(
     'CDLL',
     __new__     = interp2app(descr_new_cdll),
     ptr         = interp2app(W_CDLL.ptr),
     getaddressindll = interp2app(W_CDLL.getaddressindll),
-    name        = interp_attrproperty('name', W_CDLL,
-        wrapfn="newtext_or_none"),
     __doc__     = """ C Dynamically loaded library
 use CDLL(libname) to create a handle to a C library (the argument is processed
 the same way as dlopen processes it). On such a library you can call:
@@ -278,7 +251,6 @@ unroll_letters_for_floats = unrolling_iterable(TYPEMAP_FLOAT_LETTERS)
 
 _ARM = rffi_platform.getdefined('__arm__', '')
 
-@specialize.arg(2)
 def read_ptr(ptr, ofs, TP):
     T = lltype.Ptr(rffi.CArray(TP))
     for c in unroll_letters_for_floats:
@@ -298,8 +270,8 @@ def read_ptr(ptr, ofs, TP):
                 return ptr_val
     else:
         return rffi.cast(T, ptr)[ofs]
+read_ptr._annspecialcase_ = 'specialize:arg(2)'
 
-@specialize.argtype(2)
 def write_ptr(ptr, ofs, value):
     TP = lltype.typeOf(value)
     T = lltype.Ptr(rffi.CArray(TP))
@@ -320,11 +292,12 @@ def write_ptr(ptr, ofs, value):
                 return
     else:
         rffi.cast(T, ptr)[ofs] = value
+write_ptr._annspecialcase_ = 'specialize:argtype(2)'
 
 def segfault_exception(space, reason):
     w_mod = space.getbuiltinmodule("_rawffi")
-    w_exception = space.getattr(w_mod, space.newtext("SegfaultException"))
-    return OperationError(w_exception, space.newtext(reason))
+    w_exception = space.getattr(w_mod, space.wrap("SegfaultException"))
+    return OperationError(w_exception, space.wrap(reason))
 
 class W_DataShape(W_Root):
     _array_shapes = None
@@ -344,13 +317,11 @@ class W_DataShape(W_Root):
 
     @unwrap_spec(n=int)
     def descr_size_alignment(self, space, n=1):
-        return space.newtuple2(space.newint(self.size * n),
-                               space.newint(self.alignment))
+        return space.newtuple([space.wrap(self.size * n),
+                               space.wrap(self.alignment)])
 
 
 class W_DataInstance(W_Root):
-    fmt = 'B'
-    itemsize = 1
     def __init__(self, space, size, address=r_uint(0)):
         if address:
             self.ll_buffer = rffi.cast(rffi.VOIDP, address)
@@ -360,46 +331,50 @@ class W_DataInstance(W_Root):
             if tracker.DO_TRACING:
                 ll_buf = rffi.cast(lltype.Signed, self.ll_buffer)
                 tracker.trace_allocation(ll_buf, self)
-        self._ll_buffer = self.ll_buffer
 
     def getbuffer(self, space):
-        return space.newint(rffi.cast(lltype.Unsigned, self.ll_buffer))
-
-    def buffer_advance(self, n):
-        self.ll_buffer = rffi.ptradd(self.ll_buffer, n)
+        return space.wrap(rffi.cast(lltype.Unsigned, self.ll_buffer))
 
     def byptr(self, space):
-        from pypy.module._rawffi.interp_array import ARRAY_OF_PTRS
+        from pypy.module._rawffi.array import ARRAY_OF_PTRS
         array = ARRAY_OF_PTRS.allocate(space, 1)
-        array.setitem(space, 0, self)
-        return array
+        array.setitem(space, 0, space.wrap(self))
+        return space.wrap(array)
 
     def free(self, space):
-        if not self._ll_buffer:
+        if not self.ll_buffer:
             raise segfault_exception(space, "freeing NULL pointer")
         self._free()
 
     def _free(self):
         if tracker.DO_TRACING:
-            ll_buf = rffi.cast(lltype.Signed, self._ll_buffer)
+            ll_buf = rffi.cast(lltype.Signed, self.ll_buffer)
             tracker.trace_free(ll_buf)
-        lltype.free(self._ll_buffer, flavor='raw')
+        lltype.free(self.ll_buffer, flavor='raw')
         self.ll_buffer = lltype.nullptr(rffi.VOIDP.TO)
-        self._ll_buffer = self.ll_buffer
 
     def buffer_w(self, space, flags):
-        return SimpleView(RawFFIBuffer(self), w_obj=self)
+        return RawFFIBuffer(self)
+
+    def readbuf_w(self, space):
+        return RawFFIBuffer(self)
+
+    def writebuf_w(self, space):
+        return RawFFIBuffer(self)
 
     def getrawsize(self):
         raise NotImplementedError("abstract base class")
 
-@specialize.arg(0)
 def unwrap_truncate_int(TP, space, w_arg):
-    return rffi.cast(TP, space.bigint_w(w_arg).ulonglongmask())
+    if space.isinstance_w(w_arg, space.w_int):
+        return rffi.cast(TP, space.int_w(w_arg))
+    else:
+        return rffi.cast(TP, space.bigint_w(w_arg).ulonglongmask())
+unwrap_truncate_int._annspecialcase_ = 'specialize:arg(0)'
 
 
-@specialize.arg(1)
 def unwrap_value(space, push_func, add_arg, argdesc, letter, w_arg):
+    w = space.wrap
     if letter in TYPEMAP_PTR_LETTERS:
         # check for NULL ptr
         if isinstance(w_arg, W_DataInstance):
@@ -416,23 +391,19 @@ def unwrap_value(space, push_func, add_arg, argdesc, letter, w_arg):
         push_func(add_arg, argdesc, rffi.cast(rffi.LONGDOUBLE,
                                               space.float_w(w_arg)))
     elif letter == "c":
-        if space.isinstance_w(w_arg, space.w_int):
-            val = space.byte_w(w_arg)
-        else:
-            s = space.bytes_w(w_arg)
-            if len(s) != 1:
-                raise oefmt(space.w_TypeError,
-                            "Expected bytes of length one as character")
-            val = s[0]
+        s = space.str_w(w_arg)
+        if len(s) != 1:
+            raise OperationError(space.w_TypeError, w(
+                "Expected string of length one as character"))
+        val = s[0]
         push_func(add_arg, argdesc, val)
     elif letter == 'u':
-        s, lgt = space.utf8_len_w(w_arg)
-        if lgt != 1:
-            raise oefmt(space.w_TypeError,
-                        "Expected unicode string of length one as wide "
-                        "character")
-        val = rutf8.codepoint_at_pos(s, 0)
-        push_func(add_arg, argdesc, rffi.cast(rffi.WCHAR_T, val))
+        s = space.unicode_w(w_arg)
+        if len(s) != 1:
+            raise OperationError(space.w_TypeError, w(
+                "Expected unicode string of length one as wide character"))
+        val = s[0]
+        push_func(add_arg, argdesc, val)
     else:
         for c in unroll_letters_for_numbers:
             if letter == c:
@@ -441,64 +412,48 @@ def unwrap_value(space, push_func, add_arg, argdesc, letter, w_arg):
                 push_func(add_arg, argdesc, val)
                 return
         else:
-            raise oefmt(space.w_TypeError, "cannot directly write value")
+            raise OperationError(space.w_TypeError,
+                                 space.wrap("cannot directly write value"))
+unwrap_value._annspecialcase_ = 'specialize:arg(1)'
 
 ll_typemap_iter = unrolling_iterable(LL_TYPEMAP.items())
 
-@specialize.arg(1)
 def wrap_value(space, func, add_arg, argdesc, letter):
     for c, ll_type in ll_typemap_iter:
         if letter == c:
             if c in TYPEMAP_PTR_LETTERS:
                 res = func(add_arg, argdesc, rffi.VOIDP)
-                return space.newint(rffi.cast(lltype.Unsigned, res))
-            if c in TYPEMAP_NUMBER_LETTERS:
-                return space.newint(func(add_arg, argdesc, ll_type))
-            elif c == 'c':
-                return space.newbytes(func(add_arg, argdesc, ll_type))
-            elif c == 'u':
-                code = ord(func(add_arg, argdesc, ll_type))
-                try:
-                    return space.newutf8(rutf8.unichr_as_utf8(
-                        r_uint(code), allow_surrogates=True), 1)
-                except rutf8.OutOfRange:
-                    raise oefmt(space.w_ValueError,
-                        "unicode character %d out of range", code)
+                return space.wrap(rffi.cast(lltype.Unsigned, res))
             elif c == 'f' or c == 'd' or c == 'g':
-                return space.newfloat(float(func(add_arg, argdesc, ll_type)))
+                return space.wrap(float(func(add_arg, argdesc, ll_type)))
             else:
-                assert 0, "unreachable"
-    raise oefmt(space.w_TypeError, "cannot directly read value")
+                return space.wrap(func(add_arg, argdesc, ll_type))
+    raise OperationError(space.w_TypeError,
+                         space.wrap("cannot directly read value"))
+wrap_value._annspecialcase_ = 'specialize:arg(1)'
 
-NARROW_INTEGER_TYPES = 'cbhiBIH?'
-
-def is_narrow_integer_type(letter):
-    return letter in NARROW_INTEGER_TYPES
 
 class W_FuncPtr(W_Root):
     def __init__(self, space, ptr, argshapes, resshape):
         self.ptr = ptr
         self.argshapes = argshapes
         self.resshape = resshape
-        self.narrow_integer = False
-        if resshape is not None:
-            self.narrow_integer = is_narrow_integer_type(resshape.itemcode.lower())
 
     def getbuffer(self, space):
-        return space.newint(rffi.cast(lltype.Unsigned, self.ptr.funcsym))
+        return space.wrap(rffi.cast(lltype.Unsigned, self.ptr.funcsym))
 
     def byptr(self, space):
-        from pypy.module._rawffi.interp_array import ARRAY_OF_PTRS
+        from pypy.module._rawffi.array import ARRAY_OF_PTRS
         array = ARRAY_OF_PTRS.allocate(space, 1)
         array.setitem(space, 0, self.getbuffer(space))
         if tracker.DO_TRACING:
             # XXX this is needed, because functions tend to live forever
             #     hence our testing is not performing that well
             del tracker.alloced[rffi.cast(lltype.Signed, array.ll_buffer)]
-        return array
+        return space.wrap(array)
 
     def call(self, space, args_w):
-        from pypy.module._rawffi.interp_array import W_ArrayInstance
+        from pypy.module._rawffi.array import W_ArrayInstance
         from pypy.module._rawffi.structure import W_StructureInstance
         from pypy.module._rawffi.structure import W_Structure
         argnum = len(args_w)
@@ -542,16 +497,12 @@ class W_FuncPtr(W_Root):
                 result = self.resshape.allocate(space, 1, autofree=True)
                 # adjust_return_size() was used here on result.ll_buffer
                 self.ptr.call(args_ll, result.ll_buffer)
-                if BIGENDIAN and self.narrow_integer:
-                    # we get a 8 byte value in big endian
-                    n = rffi.sizeof(lltype.Signed) - result.shape.size
-                    result.buffer_advance(n)
-                return result
+                return space.wrap(result)
             else:
                 self.ptr.call(args_ll, lltype.nullptr(rffi.VOIDP.TO))
                 return space.w_None
-        except StackCheckError as e:
-            raise OperationError(space.w_ValueError, space.newtext(e.message))
+        except StackCheckError, e:
+            raise OperationError(space.w_ValueError, space.wrap(e.message))
 
 @unwrap_spec(addr=r_uint, flags=int)
 def descr_new_funcptr(space, w_tp, addr, w_args, w_res, flags=FUNCFLAG_CDECL):
@@ -567,7 +518,7 @@ def descr_new_funcptr(space, w_tp, addr, w_args, w_res, flags=FUNCFLAG_CDECL):
                          flags)
     except LibFFIError:
         raise got_libffi_error(space)
-    return W_FuncPtr(space, ptr, argshapes, resshape)
+    return space.wrap(W_FuncPtr(space, ptr, argshapes, resshape))
 
 W_FuncPtr.typedef = TypeDef(
     'FuncPtr',
@@ -579,13 +530,14 @@ W_FuncPtr.typedef = TypeDef(
 W_FuncPtr.typedef.acceptable_as_base_class = False
 
 def _create_new_accessor(func_name, name):
-    @unwrap_spec(tp_letter='text')
+    @unwrap_spec(tp_letter=str)
     def accessor(space, tp_letter):
         if len(tp_letter) != 1:
-            raise oefmt(space.w_ValueError, "Expecting string of length one")
+            raise OperationError(space.w_ValueError, space.wrap(
+                "Expecting string of length one"))
         tp_letter = tp_letter[0] # fool annotator
         try:
-            return space.newint(intmask(getattr(TYPEMAP_FUNCTION(tp_letter), name)))
+            return space.wrap(intmask(getattr(TYPEMAP[tp_letter], name)))
         except KeyError:
             raise oefmt(space.w_ValueError, "Unknown type specification %s",
                         tp_letter)
@@ -603,82 +555,78 @@ def charp2string(space, address, maxlength=-1):
         s = rffi.charp2str(charp_addr)
     else:
         s = rffi.charp2strn(charp_addr, maxlength)
-    return space.newbytes(s)
+    return space.wrap(s)
 
 @unwrap_spec(address=r_uint, maxlength=int)
 def wcharp2unicode(space, address, maxlength=-1):
     if address == 0:
         return space.w_None
     wcharp_addr = rffi.cast(rffi.CWCHARP, address)
-    try:
-        if maxlength == -1:
-            s, lgt = rffi.wcharp2utf8(wcharp_addr)
-        else:
-            s, lgt = rffi.wcharp2utf8n(wcharp_addr, maxlength)
-    except rutf8.OutOfRange as e:
-        raise wrap_unicode_out_of_range_error(space, e)
-    return space.newutf8(s, lgt)
+    if maxlength == -1:
+        s = rffi.wcharp2unicode(wcharp_addr)
+    else:
+        s = rffi.wcharp2unicoden(wcharp_addr, maxlength)
+    return space.wrap(s)
 
 @unwrap_spec(address=r_uint, maxlength=int)
 def charp2rawstring(space, address, maxlength=-1):
     if maxlength == -1:
         return charp2string(space, address)
     s = rffi.charpsize2str(rffi.cast(rffi.CCHARP, address), maxlength)
-    return space.newbytes(s)
+    return space.wrap(s)
 
 @unwrap_spec(address=r_uint, maxlength=int)
 def wcharp2rawunicode(space, address, maxlength=-1):
     if maxlength == -1:
         return wcharp2unicode(space, address)
-    elif maxlength < 0:
-        maxlength = 0
-    s = wcharpsize2utf8(space, rffi.cast(rffi.CWCHARP, address), maxlength)
-    return space.newutf8(s, maxlength)
+    s = rffi.wcharpsize2unicode(rffi.cast(rffi.CWCHARP, address), maxlength)
+    return space.wrap(s)
 
-@unwrap_spec(address=r_uint, newcontent='bufferstr', offset=int, size=int)
-def rawstring2charp(space, address, newcontent, offset=0, size=-1):
+@unwrap_spec(address=r_uint, newcontent='bufferstr')
+def rawstring2charp(space, address, newcontent):
     from rpython.rtyper.annlowlevel import llstr
     from rpython.rtyper.lltypesystem.rstr import copy_string_to_raw
     array = rffi.cast(rffi.CCHARP, address)
-    if size < 0:
-        size = len(newcontent) - offset
-    copy_string_to_raw(llstr(newcontent), array, offset, size)
+    copy_string_to_raw(llstr(newcontent), array, 0, len(newcontent))
 
 if _MS_WINDOWS:
     @unwrap_spec(code=int)
     def FormatError(space, code):
-        return space.newtext(*rwin32.FormatErrorW(code))
+        return space.wrap(rwin32.FormatError(code))
 
     @unwrap_spec(hresult=int)
     def check_HRESULT(space, hresult):
         if rwin32.FAILED(hresult):
-            raise OperationError(space.w_WindowsError, space.newint(hresult))
-        return space.newint(hresult)
+            raise OperationError(space.w_WindowsError, space.wrap(hresult))
+        return space.wrap(hresult)
 
 def get_libc(space):
     name = get_libc_name()
-    cdll = open_cdll(space, name)
-    return W_CDLL(space, name, cdll)
+    try:
+        cdll = CDLL(name)
+    except OSError, e:
+        raise wrap_oserror(space, e)
+    return space.wrap(W_CDLL(space, name, cdll))
 
 def get_errno(space):
-    return space.newint(rposix.get_saved_alterrno())
+    return space.wrap(rposix.get_saved_alterrno())
 
 def set_errno(space, w_errno):
     rposix.set_saved_alterrno(space.int_w(w_errno))
 
 if sys.platform == 'win32':
     # see also
-    # issue #1944 ctypes-on-windows-getlasterror
+    # https://bitbucket.org/pypy/pypy/issue/1944/ctypes-on-windows-getlasterror
     def get_last_error(space):
-        return space.newint(rwin32.GetLastError_alt_saved())
+        return space.wrap(rwin32.GetLastError_alt_saved())
     @unwrap_spec(error=int)
     def set_last_error(space, error):
         rwin32.SetLastError_alt_saved(error)
 else:
     # always have at least a dummy version of these functions
-    # issue 1242
+    # (https://bugs.pypy.org/issue1242)
     def get_last_error(space):
-        return space.newint(0)
+        return space.wrap(0)
     @unwrap_spec(error=int)
     def set_last_error(space, error):
         pass
