@@ -1,9 +1,6 @@
 class AppTestWeakref(object):
     spaceconfig = dict(usemodules=('_weakref',))
-
-    def setup_class(cls):
-        cls.w_runappdirect = cls.space.wrap(cls.runappdirect)
-
+                    
     def test_simple(self):
         import _weakref, gc
         class A(object):
@@ -22,12 +19,6 @@ class AppTestWeakref(object):
         import _weakref
         raises(TypeError, _weakref.ref)
 
-    def test_no_kwargs(self):
-        import _weakref
-        class C(object):
-            pass
-        raises(TypeError, _weakref.ref, C(), callback=None)
-
     def test_callback(self):
         import _weakref, gc
         class A(object):
@@ -38,13 +29,10 @@ class AppTestWeakref(object):
             a2.ref = ref()
         ref1 = _weakref.ref(a1, callback)
         ref2 = _weakref.ref(a1)
-        assert ref1.__callback__ is callback
-        assert ref2.__callback__ is None
         assert _weakref.getweakrefcount(a1) == 2
         del a1
         gc.collect()
         assert ref1() is None
-        assert ref1.__callback__ is None
         assert a2.ref is None
 
     def test_callback_order(self):
@@ -153,14 +141,6 @@ class AppTestWeakref(object):
         assert not (ref1 == [])
         assert ref1 != []
 
-    def test_ne(self):
-        import _weakref
-        class X(object):
-            pass
-        ref1 = _weakref.ref(X())
-        assert ref1.__eq__(X()) is NotImplemented
-        assert ref1.__ne__(X()) is NotImplemented
-
     def test_getweakrefs(self):
         import _weakref, gc
         class A(object):
@@ -237,9 +217,7 @@ class AppTestWeakref(object):
         assert w_bound()() == 42
         del meth
         gc.collect()
-        # it used to be None on py2, but now there is no longer a newly
-        # created unbound method object
-        assert w_unbound() is A.f
+        assert w_unbound() is None
         assert w_bound() is None
 
     def test_set_weakrefable(self):
@@ -258,16 +236,16 @@ class AppTestWeakref(object):
                 yield i
         g = f(10)
         w = _weakref.ref(g)
-        r = next(w())
+        r = w().next()
         assert r == 0
-        r = next(g)
+        r = g.next()
         assert r == 1
         del g
         gc.collect()
         assert w() is None
         g = f(10)
         w = _weakref.ref(g)
-        assert list(g) == list(range(10))
+        assert list(g) == range(10)
         del g
         gc.collect()
         assert w() is None
@@ -309,9 +287,6 @@ class AppTestWeakref(object):
             assert a1 is None
 
     def test_del_and_callback_and_id(self):
-        if not self.runappdirect:
-            skip("the id() doesn't work correctly in __del__ and "
-                 "callbacks before translation")
         import gc, weakref
         seen_del = []
         class A(object):
@@ -437,7 +412,7 @@ class AppTestProxy(object):
                 return 42
             w = getattr(_weakref, kind)(foobaz)
             s = repr(w)
-            print(s)
+            print s
             if kind == 'ref':
                 assert s.startswith('<weakref at ')
             else:
@@ -453,26 +428,20 @@ class AppTestProxy(object):
             except ReferenceError:
                 pass    # only reachable if kind == 'proxy'
             s = repr(w)
-            print(s)
+            print s
             assert "dead" in s
 
-    def test_bytes(self):
+    def test_unicode(self):
         import _weakref
         class C(object):
-            def __bytes__(self):
-                return b"string"
+            def __str__(self):
+                return "string"
+            def __unicode__(self):
+                return u"unicode"
         instance = C()
-        assert "__bytes__" in dir(_weakref.proxy(instance))
-        assert bytes(_weakref.proxy(instance)) == b"string"
-
-    def test_reversed(self):
-        import _weakref
-        class C(object):
-            def __reversed__(self):
-                return b"string"
-        instance = C()
-        assert "__reversed__" in dir(_weakref.proxy(instance))
-        assert reversed(_weakref.proxy(instance)) == b"string"
+        assert "__unicode__" in dir(_weakref.proxy(instance))
+        assert str(_weakref.proxy(instance)) == "string"
+        assert unicode(_weakref.proxy(instance)) == u"unicode"
 
     def test_eq(self):
         import _weakref
@@ -554,56 +523,3 @@ class AppTestProxy(object):
         p1[42] = p2
         assert a1.setkey == 42
         assert a1.setvalue is p2
-
-    def test_error_message_wrong_self(self):
-        import _weakref
-        unboundmeth = _weakref.ref.__repr__
-        e = raises(TypeError, unboundmeth, 42)
-        assert "weakref" in str(e.value)
-        if hasattr(unboundmeth, 'im_func'):
-            e = raises(TypeError, unboundmeth.im_func, 42)
-            assert "'weakref-or-proxy'" in str(e.value)
-
-    def test_reverse_add(self):
-        import _weakref
-        class A:
-            def __add__(self, other):
-                return 17
-
-            def __radd__(self, other):
-                return 20
-
-            def __iadd__(self, other):
-                return 19
-
-        assert A() + 12 == 17
-        assert 12 + A() == 20
-        a = A()
-        a += -1
-        assert a == 19
-
-        a = A()
-        p = _weakref.proxy(a)
-        assert p + 12 == 17
-        print(12 + p)
-        assert 12 + p == 20
-        p += -1
-        assert p == 19
-
-    def test_gt_lt(self):
-        import _weakref
-        class A:
-            def __gt__(self, other):
-                return True
-
-            def __lt__(self, other):
-                return False
-
-        assert (A() > 12) == True
-        assert (12 > A()) == False
-
-        a = A()
-        p = _weakref.proxy(a)
-        assert (p > 12) == True
-        print(12 > p)
-        assert (12 > p) == False
