@@ -1,93 +1,12 @@
 from pypy.module.micronumpy.test.test_base import BaseNumpyAppTest
-from pypy.module.micronumpy.ufuncs import (find_binop_result_dtype,
-        find_unaryop_result_dtype, W_UfuncGeneric)
+from pypy.module.micronumpy.ufuncs import W_UfuncGeneric, unary_ufunc
 from pypy.module.micronumpy.support import _parse_signature
 from pypy.module.micronumpy.descriptor import get_dtype_cache
 from pypy.module.micronumpy.base import W_NDimArray
 from pypy.module.micronumpy.concrete import VoidBoxStorage
-from pypy.interpreter.gateway import interp2app
-from pypy.conftest import option
 from pypy.interpreter.error import OperationError
 
 
-class TestUfuncCoercion(object):
-    def test_binops(self, space):
-        bool_dtype = get_dtype_cache(space).w_booldtype
-        int8_dtype = get_dtype_cache(space).w_int8dtype
-        int32_dtype = get_dtype_cache(space).w_int32dtype
-        float64_dtype = get_dtype_cache(space).w_float64dtype
-        c64_dtype = get_dtype_cache(space).w_complex64dtype
-        c128_dtype = get_dtype_cache(space).w_complex128dtype
-        cld_dtype = get_dtype_cache(space).w_complexlongdtype
-        fld_dtype = get_dtype_cache(space).w_floatlongdtype
-
-        # Basic pairing
-        assert find_binop_result_dtype(space, bool_dtype, bool_dtype) is bool_dtype
-        assert find_binop_result_dtype(space, bool_dtype, float64_dtype) is float64_dtype
-        assert find_binop_result_dtype(space, float64_dtype, bool_dtype) is float64_dtype
-        assert find_binop_result_dtype(space, int32_dtype, int8_dtype) is int32_dtype
-        assert find_binop_result_dtype(space, int32_dtype, bool_dtype) is int32_dtype
-        assert find_binop_result_dtype(space, c64_dtype, float64_dtype) is c128_dtype
-        assert find_binop_result_dtype(space, c64_dtype, fld_dtype) is cld_dtype
-        assert find_binop_result_dtype(space, c128_dtype, fld_dtype) is cld_dtype
-
-        # With promote bool (happens on div), the result is that the op should
-        # promote bools to int8
-        assert find_binop_result_dtype(space, bool_dtype, bool_dtype, promote_bools=True) is int8_dtype
-        assert find_binop_result_dtype(space, bool_dtype, float64_dtype, promote_bools=True) is float64_dtype
-
-        # Coerce to floats
-        assert find_binop_result_dtype(space, bool_dtype, float64_dtype, promote_to_float=True) is float64_dtype
-
-    def test_unaryops(self, space):
-        bool_dtype = get_dtype_cache(space).w_booldtype
-        int8_dtype = get_dtype_cache(space).w_int8dtype
-        uint8_dtype = get_dtype_cache(space).w_uint8dtype
-        int16_dtype = get_dtype_cache(space).w_int16dtype
-        uint16_dtype = get_dtype_cache(space).w_uint16dtype
-        int32_dtype = get_dtype_cache(space).w_int32dtype
-        uint32_dtype = get_dtype_cache(space).w_uint32dtype
-        long_dtype = get_dtype_cache(space).w_longdtype
-        ulong_dtype = get_dtype_cache(space).w_ulongdtype
-        int64_dtype = get_dtype_cache(space).w_int64dtype
-        uint64_dtype = get_dtype_cache(space).w_uint64dtype
-        float16_dtype = get_dtype_cache(space).w_float16dtype
-        float32_dtype = get_dtype_cache(space).w_float32dtype
-        float64_dtype = get_dtype_cache(space).w_float64dtype
-
-        # Normal rules, everything returns itself
-        assert find_unaryop_result_dtype(space, bool_dtype) is bool_dtype
-        assert find_unaryop_result_dtype(space, int8_dtype) is int8_dtype
-        assert find_unaryop_result_dtype(space, uint8_dtype) is uint8_dtype
-        assert find_unaryop_result_dtype(space, int16_dtype) is int16_dtype
-        assert find_unaryop_result_dtype(space, uint16_dtype) is uint16_dtype
-        assert find_unaryop_result_dtype(space, int32_dtype) is int32_dtype
-        assert find_unaryop_result_dtype(space, uint32_dtype) is uint32_dtype
-        assert find_unaryop_result_dtype(space, long_dtype) is long_dtype
-        assert find_unaryop_result_dtype(space, ulong_dtype) is ulong_dtype
-        assert find_unaryop_result_dtype(space, int64_dtype) is int64_dtype
-        assert find_unaryop_result_dtype(space, uint64_dtype) is uint64_dtype
-        assert find_unaryop_result_dtype(space, float32_dtype) is float32_dtype
-        assert find_unaryop_result_dtype(space, float64_dtype) is float64_dtype
-
-        # Coerce to floats, some of these will eventually be float16, or
-        # whatever our smallest float type is.
-        assert find_unaryop_result_dtype(space, bool_dtype, promote_to_float=True) is float16_dtype
-        assert find_unaryop_result_dtype(space, int8_dtype, promote_to_float=True) is float16_dtype
-        assert find_unaryop_result_dtype(space, uint8_dtype, promote_to_float=True) is float16_dtype
-        assert find_unaryop_result_dtype(space, int16_dtype, promote_to_float=True) is float32_dtype
-        assert find_unaryop_result_dtype(space, uint16_dtype, promote_to_float=True) is float32_dtype
-        assert find_unaryop_result_dtype(space, int32_dtype, promote_to_float=True) is float64_dtype
-        assert find_unaryop_result_dtype(space, uint32_dtype, promote_to_float=True) is float64_dtype
-        assert find_unaryop_result_dtype(space, int64_dtype, promote_to_float=True) is float64_dtype
-        assert find_unaryop_result_dtype(space, uint64_dtype, promote_to_float=True) is float64_dtype
-        assert find_unaryop_result_dtype(space, float32_dtype, promote_to_float=True) is float32_dtype
-        assert find_unaryop_result_dtype(space, float64_dtype, promote_to_float=True) is float64_dtype
-
-        # promote bools, happens with sign ufunc
-        assert find_unaryop_result_dtype(space, bool_dtype, promote_bools=True) is int8_dtype
-
-
 class TestGenericUfuncOperation(object):
     def test_signature_parser(self, space):
         class Ufunc(object):
@@ -96,10 +15,10 @@ class TestGenericUfuncOperation(object):
                 self.nout = nout
                 self.nargs = nin + nout
                 self.core_enabled = True
-                self.core_num_dim_ix = 0 
-                self.core_num_dims = [0] * self.nargs  
+                self.core_num_dim_ix = 0
+                self.core_num_dims = [0] * self.nargs
                 self.core_offsets = [0] * self.nargs
-                self.core_dim_ixs = [] 
+                self.core_dim_ixs = []
 
         u = Ufunc(2, 1)
         _parse_signature(space, u, '(m,n), (n,r)->(m,r)')
@@ -116,8 +35,8 @@ class TestGenericUfuncOperation(object):
         b_dtype = get_dtype_cache(space).w_booldtype
 
         ufunc = W_UfuncGeneric(space, [None, None, None], 'eigenvals', None, 1, 1,
-                     [f32_dtype, c64_dtype, 
-                      f64_dtype, c128_dtype, 
+                     [f32_dtype, c64_dtype,
+                      f64_dtype, c128_dtype,
                       c128_dtype, c128_dtype],
                      '')
         f32_array = W_NDimArray(VoidBoxStorage(0, f32_dtype))
@@ -135,6 +54,22 @@ class TestGenericUfuncOperation(object):
         exc = raises(OperationError, ufunc.type_resolver, space, [f32_array], [None],
                                 'i->i', ufunc.dtypes)
 
+    def test_allowed_types(self, space):
+        dt_bool = get_dtype_cache(space).w_booldtype
+        dt_float16 = get_dtype_cache(space).w_float16dtype
+        dt_int32 = get_dtype_cache(space).w_int32dtype
+        ufunc = unary_ufunc(space, None, 'x', int_only=True)
+        assert ufunc._calc_dtype(space, dt_bool, out=None) == (dt_bool, dt_bool)
+        assert ufunc.dtypes  # XXX: shouldn't contain too much stuff
+
+        ufunc = unary_ufunc(space, None, 'x', promote_to_float=True)
+        assert ufunc._calc_dtype(space, dt_bool, out=None) == (dt_float16, dt_float16)
+        assert ufunc._calc_dtype(space, dt_bool, casting='same_kind') == (dt_float16, dt_float16)
+        raises(OperationError, ufunc._calc_dtype, space, dt_bool, casting='no')
+
+        ufunc = unary_ufunc(space, None, 'x')
+        assert ufunc._calc_dtype(space, dt_int32, out=None) == (dt_int32, dt_int32)
+
 class AppTestUfuncs(BaseNumpyAppTest):
     def test_constants(self):
         import numpy as np
@@ -151,6 +86,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_frompyfunc_innerloop(self):
         from numpy import ufunc, frompyfunc, arange, dtype
+        import sys
         def adder(a, b):
             return a+b
         def sumdiff(a, b):
@@ -167,7 +103,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert 'object' in str(e)
             # Use pypy specific extension for out_dtype
             adder_ufunc0 = frompyfunc(adder, 2, 1, dtypes=['match'])
-            sumdiff = frompyfunc(sumdiff, 2, 2, dtypes=['match'], 
+            sumdiff = frompyfunc(sumdiff, 2, 2, dtypes=['match'],
                                     signature='(i),(i)->(i),(i)')
             adder_ufunc1 = frompyfunc([adder, adder], 2, 1,
                             dtypes=[int, int, int, float, float, float])
@@ -188,12 +124,19 @@ class AppTestUfuncs(BaseNumpyAppTest):
         res = int_func12(a)
         assert len(res) == 2
         assert isinstance(res, tuple)
-        assert (res[0] == a).all()
+        if '__pypy__' in sys.builtin_module_names:
+            assert (res[0] == a).all()
+        else:
+            assert all([r is None for r in res[0]]) # ??? no warning or error, just a fail?
         res = sumdiff(2 * a, a)
         assert (res[0] == 3 * a).all()
         assert (res[1] == a).all()
 
     def test_frompyfunc_outerloop(self):
+        import sys
+        from numpy import frompyfunc, dtype, arange
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
         def int_times2(in_array, out_array):
             assert in_array.dtype == int
             in_flat = in_array.flat
@@ -206,7 +149,6 @@ class AppTestUfuncs(BaseNumpyAppTest):
             out_flat = out_array.flat
             for i in range(in_array.size):
                 out_flat[i] = in_flat[i] * 2
-        from numpy import frompyfunc, dtype, arange
         ufunc = frompyfunc([int_times2, double_times2], 1, 1,
                             signature='()->()',
                             dtypes=[dtype(int), dtype(int),
@@ -221,16 +163,18 @@ class AppTestUfuncs(BaseNumpyAppTest):
         af2 = ufunc(af)
         assert all(af2 == af * 2)
         ac = arange(10, dtype=complex)
-        skip('casting not implemented yet')
-        ac1 = ufunc(ac)
+        raises(TypeError, ufunc, ac)
 
     def test_frompyfunc_2d_sig(self):
+        import sys
+        from numpy import frompyfunc, dtype, arange
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
         def times_2(in_array, out_array):
             assert len(in_array.shape) == 2
             assert in_array.shape == out_array.shape
             out_array[:] = in_array * 2
 
-        from numpy import frompyfunc, dtype, arange
         ufunc = frompyfunc([times_2], 1, 1,
                             signature='(m,n)->(n,m)',
                             dtypes=[dtype(int), dtype(int)],
@@ -246,19 +190,56 @@ class AppTestUfuncs(BaseNumpyAppTest):
                             dtypes=[dtype(int), dtype(int)],
                             stack_inputs=True,
                           )
-        ai = arange(18, dtype=int).reshape(2,3,3)
+        ai = arange(12*3*3, dtype='int32').reshape(12,3,3)
         exc = raises(ValueError, ufunc, ai[:,:,0])
         assert "perand 0 has a mismatch in its core dimension 1" in exc.value.message
         ai3 = ufunc(ai[0,:,:])
         ai2 = ufunc(ai)
         assert (ai2 == ai * 2).all()
+        # view
+        aiV = ai[::-2, :, :]
+        assert aiV.strides == (-72, 12, 4)
+        ai2 = ufunc(aiV)
+        assert (ai2 == aiV * 2).all()
 
+        ai = arange(0).reshape(0, 1, 1)
+        ao = ufunc(ai)
+        assert ao.shape == (0, 1, 1)
+
+    def test_frompyfunc_not_contiguous(self):
+        import sys
+        from numpy import frompyfunc, dtype, arange, dot
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
+        def _dot(in0, in1, out):
+            print in0, '\nin1',in1,'\nin1.shape', in1.shape, 'in1.strides', in1.strides
+            out[...] = dot(in0, in1)
+
+        ufunc_dot = frompyfunc(_dot, 2, 1,
+                            signature='(m,m),(m,n)->(m,n)',
+                            dtypes=[dtype(float), dtype(float), dtype(float)],
+                            stack_inputs=True,
+                          )
+        a1 = arange(4, dtype=float).reshape(2,2)
+        # create a non-c-contiguous argument
+        a2 = arange(2, dtype=float).reshape(2,1)
+        a3 = arange(2, dtype=float).reshape(1,2).T
+        b1 = ufunc_dot(a1, a2, sig='dd->d')
+        b2 = dot(a1, a2)
+        assert (b1==b2).all()
+        print 'xxxxxxxxxxxx'
+        b1 = ufunc_dot(a1, a3, sig='dd->d')
+        b2 = dot(a1, a3)
+        assert (b1==b2).all()
+ 
     def test_frompyfunc_needs_nditer(self):
+        import sys
+        from numpy import frompyfunc, dtype, arange
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
         def summer(in0):
-            print 'in summer, in0=',in0,'in0.shape=',in0.shape
             return in0.sum()
 
-        from numpy import frompyfunc, dtype, arange
         ufunc = frompyfunc([summer], 1, 1,
                             signature='(m,m)->()',
                             dtypes=[dtype(int), dtype(int)],
@@ -269,13 +250,16 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert ao.size == 3
 
     def test_frompyfunc_sig_broadcast(self):
+        import sys
+        from numpy import frompyfunc, dtype, arange
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
         def sum_along_0(in_array, out_array):
             out_array[...] = in_array.sum(axis=0)
 
         def add_two(in0, in1, out):
             out[...] = in0 + in1
 
-        from numpy import frompyfunc, dtype, arange
         ufunc_add = frompyfunc(add_two, 2, 1,
                             signature='(m,n),(m,n)->(m,n)',
                             dtypes=[dtype(int), dtype(int), dtype(int)],
@@ -287,13 +271,19 @@ class AppTestUfuncs(BaseNumpyAppTest):
                             stack_inputs=True,
                           )
         ai = arange(18, dtype=int).reshape(3,2,3)
-        aout = ufunc_add(ai, ai[0,:,:])
-        assert aout.shape == (3, 2, 3)
-        aout = ufunc_sum(ai)
-        assert aout.shape == (3, 3)
-
+        aout1 = ufunc_add(ai, ai[0,:,:])
+        assert aout1.shape == (3, 2, 3)
+        aout2 = ufunc_add(ai, ai[0,:,:])
+        aout3 = ufunc_sum(ai)
+        assert aout3.shape == (3, 3)
+        aout4 = ufunc_add(ai, ai[0,:,:][None, :,:])
+        assert (aout1 == aout4).all()
+        
     def test_frompyfunc_fortran(self):
+        import sys
         import numpy as np
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
         def tofrom_fortran(in0, out0):
             out0[:] = in0.T
 
@@ -313,6 +303,80 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert out0.shape == in0.shape
         assert (out0 == in0 * 2).all()
 
+    def test_frompyfunc_casting(self):
+        import sys
+        import numpy as np
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
+
+        def times2_int(in0, out0):
+            assert in0.dtype == int
+            assert out0.dtype == int
+            # hack to assigning to a 0-dim array
+            out0.real = in0 * 2
+
+        def times2_complex(in0, out0):
+            assert in0.dtype == complex
+            assert out0.dtype == complex
+            out0.real = in0.real * 2
+            out0.imag = in0.imag
+
+        def times2_complex0(in0):
+            assert in0.dtype == complex
+            return in0 * 2
+
+        def times2_int0(in0):
+            assert in0.dtype == int
+            return in0 * 2
+
+        times2stacked = np.frompyfunc([times2_int, times2_complex], 1, 1,
+                            dtypes=[np.dtype(int), np.dtype(int),
+                                np.dtype(complex), np.dtype(complex)],
+                            stack_inputs=True, signature='()->()',
+                          )
+        times2 = np.frompyfunc([times2_int0, times2_complex0], 1, 1,
+                            dtypes=[np.dtype(int), np.dtype(int),
+                                np.dtype(complex), np.dtype(complex)],
+                            stack_inputs=False,
+                          )
+        for d in [np.dtype(float), np.dtype('uint8'), np.dtype('complex64')]:
+            in0 = np.arange(4, dtype=d)
+            out0 = times2stacked(in0)
+            assert out0.shape == in0.shape
+            assert out0.dtype in (int, complex) 
+            assert (out0 == in0 * 2).all()
+
+            out0 = times2(in0)
+            assert out0.shape == in0.shape
+            assert out0.dtype in (int, complex) 
+            assert (out0 == in0 * 2).all()
+
+            in0 = np.arange(4, dtype=int)
+            out0 = times2(in0, sig='D->D')
+            assert out0.dtype == complex
+
+    def test_frompyfunc_scalar(self):
+        import sys
+        import numpy as np
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy only frompyfunc extension')
+
+        def summer(in0):
+            out = np.empty(1, in0.dtype)
+            out[0] = in0.sum()
+            return out
+
+        pysummer = np.frompyfunc([summer, summer], 1, 1,
+                            dtypes=[np.dtype(int), np.dtype(int),
+                                np.dtype(complex), np.dtype(complex)],
+                            stack_inputs=False, signature='(m,m)->()',
+                          )
+        for d in [np.dtype(float), np.dtype('uint8'), np.dtype('complex64')]:
+            in0 = np.arange(4, dtype=d).reshape(1, 2, 2)
+            out0 = pysummer(in0)
+            assert out0 == in0.sum()
+            assert out0.dtype in (int, complex)
+
     def test_ufunc_kwargs(self):
         from numpy import ufunc, frompyfunc, arange, dtype
         def adder(a, b):
@@ -326,7 +390,15 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert all(res == args[0] + args[1])
         raises(TypeError, adder_ufunc, *args, blah=True)
         raises(TypeError, adder_ufunc, *args, extobj=True)
-        raises(RuntimeError, adder_ufunc, *args, sig='(d,d)->(d)', dtype=int)
+        raises(RuntimeError, adder_ufunc, *args, sig='dd->d', dtype=int)
+
+    def test_unary_ufunc_kwargs(self):
+        from numpy import array, sin, float16
+        bool_array = array([True])
+        raises(TypeError, sin, bool_array, casting='no')
+        assert sin(bool_array, casting='same_kind').dtype == float16
+        raises(TypeError, sin, bool_array, out=bool_array, casting='same_kind')
+        assert sin(bool_array).dtype == float16
 
     def test_ufunc_attrs(self):
         from numpy import add, multiply, sin
@@ -397,13 +469,15 @@ class AppTestUfuncs(BaseNumpyAppTest):
         for i in range(3):
             assert min_c_b[i] == min(b[i], c)
 
-    def test_scalar(self):
+    def test_all_available(self):
         # tests that by calling all available ufuncs on scalars, none will
         # raise uncaught interp-level exceptions, (and crash the test)
         # and those that are uncallable can be accounted for.
-        # test on the four base-class dtypes: int, bool, float, complex
+        # test on the base-class dtypes: int, bool, float, complex, object
         # We need this test since they have no common base class.
         import numpy as np
+        not_implemented = set(['ldexp', 'frexp', 'cbrt', 'spacing',
+            'hypot', 'modf', 'remainder', 'nextafter'])
         def find_uncallable_ufuncs(dtype):
             uncallable = set()
             array = np.array(1, dtype)
@@ -412,21 +486,35 @@ class AppTestUfuncs(BaseNumpyAppTest):
                 if isinstance(u, np.ufunc):
                     try:
                         u(* [array] * u.nin)
+                    except AttributeError:
+                        pass
+                    except NotImplementedError:
+                        #print s
+                        uncallable.add(s)
                     except TypeError:
                         assert s not in uncallable
                         uncallable.add(s)
             return uncallable
         assert find_uncallable_ufuncs('int') == set()
         assert find_uncallable_ufuncs('bool') == set(['sign'])
-        assert find_uncallable_ufuncs('float') == set(
+        uncallable = find_uncallable_ufuncs('float')
+        uncallable = uncallable.difference(not_implemented)
+        assert uncallable == set(
                 ['bitwise_and', 'bitwise_not', 'bitwise_or', 'bitwise_xor',
                  'left_shift', 'right_shift', 'invert'])
-        assert find_uncallable_ufuncs('complex') == set(
+        uncallable = find_uncallable_ufuncs('complex')
+        uncallable = uncallable.difference(not_implemented)
+        assert uncallable == set(
                 ['bitwise_and', 'bitwise_not', 'bitwise_or', 'bitwise_xor',
                  'arctan2', 'deg2rad', 'degrees', 'rad2deg', 'radians',
                  'fabs', 'fmod', 'invert', 'mod',
                  'logaddexp', 'logaddexp2', 'left_shift', 'right_shift',
                  'copysign', 'signbit', 'ceil', 'floor', 'trunc'])
+        uncallable = find_uncallable_ufuncs('object')
+        uncallable = uncallable.difference(not_implemented)
+        assert uncallable == set(
+                ['isnan', 'logaddexp2', 'copysign', 'isfinite', 'signbit',
+                 'isinf', 'logaddexp'])
 
     def test_int_only(self):
         from numpy import bitwise_and, array
@@ -452,6 +540,12 @@ class AppTestUfuncs(BaseNumpyAppTest):
         b = negative(a + a)
         assert (b == [[-2, -4], [-6, -8]]).all()
 
+        class Obj(object):
+            def __neg__(self):
+                return 'neg'
+        x = Obj()
+        assert type(negative(x)) is str
+
     def test_abs(self):
         from numpy import array, absolute
 
@@ -468,6 +562,11 @@ class AppTestUfuncs(BaseNumpyAppTest):
         c = add(a, b)
         for i in range(3):
             assert c[i] == a[i] + b[i]
+        class Obj(object):
+            def __add__(self, other):
+                return 'add'
+        x = Obj()
+        assert type(add(x, 0)) is str
 
     def test_divide(self):
         from numpy import array, divide
@@ -550,6 +649,15 @@ class AppTestUfuncs(BaseNumpyAppTest):
         for v in [float('inf'), float('-inf'), float('nan'), float('-nan')]:
             assert math.isnan(fmod(v, 2))
 
+    def test_mod(self):
+        from numpy import mod
+        assert mod(5, 3) == 2
+        assert mod(5, -3) == -1
+        assert mod(-5, 3) == 1
+        assert mod(-5, -3) == -2
+        assert mod(2.5, 1) == 0.5
+        assert mod(-1.5, 2) == 0.5
+
     def test_minimum(self):
         from numpy import array, minimum, nan, isnan
 
@@ -1045,6 +1153,17 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert np.equal.reduce([1, 2], dtype=dtype) == True
             assert np.equal.reduce([1, 2, 0], dtype=dtype) == False
 
+    def test_reduce_axes(self):
+        import numpy as np
+        a = np.arange(24).reshape(2, 3, 4)
+        b = np.add.reduce(a, axis=(0, 1))
+        assert b.shape == (4,)
+        assert (b == [60, 66, 72, 78]).all()
+
+    def test_reduce_fmax(self):
+        import numpy as np
+        assert np.fmax.reduce(np.arange(11).astype('b')) == 10
+
     def test_reduceND(self):
         from numpy import add, arange
         a = arange(12).reshape(3, 4)
@@ -1156,6 +1275,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert (logical_xor([True, False, True, False], [1, 2, 0, 0])
                 == [False, True, True, False]).all()
         assert (logical_not([True, False]) == [False, True]).all()
+        assert logical_and.reduce([1.,1.]) == True
 
     def test_logn(self):
         import math
@@ -1320,7 +1440,6 @@ class AppTestUfuncs(BaseNumpyAppTest):
         # dtype
         a = arange(0, 3, 0.5).reshape(2, 3)
         b = add.accumulate(a, dtype=int, axis=1)
-        print b
         assert (b == [[0, 0, 1], [1, 3, 5]]).all()
         assert b.dtype == int
         assert add.accumulate([True]*200)[-1] == 200
@@ -1328,6 +1447,26 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert subtract.accumulate([True]*200).dtype == dtype('bool')
         assert divide.accumulate([True]*200).dtype == dtype('int8')
 
+    def test_accumulate_shapes(self):
+        import numpy as np
+        a = np.arange(6).reshape(2, 1, 3)
+        assert np.add.accumulate(a).shape == (2, 1, 3)
+        raises(ValueError, "np.add.accumulate(a, out=np.zeros((3, 1, 3)))")
+        raises(ValueError, "np.add.accumulate(a, out=np.zeros((2, 3)))")
+        raises(ValueError, "np.add.accumulate(a, out=np.zeros((2, 3, 1)))")
+        b = np.zeros((2, 1, 3))
+        np.add.accumulate(a, out=b, axis=2)
+        assert b[0, 0, 2] == 3
+
+    def test_accumulate_shapes_2(self):
+        import sys
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('PyPy-specific behavior in np.ufunc.accumulate')
+        import numpy as np
+        a = np.arange(6).reshape(2, 1, 3)
+        raises(ValueError, "np.add.accumulate(a, out=np.zeros((2, 1, 3, 2)))")
+
+
     def test_noncommutative_reduce_accumulate(self):
         import numpy as np
         tosubtract = np.arange(5)
@@ -1355,3 +1494,29 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert np.add(np.float16(0), np.longdouble(0)).dtype == np.longdouble
         assert np.add(np.float16(0), np.complex64(0)).dtype == np.complex64
         assert np.add(np.float16(0), np.complex128(0)).dtype == np.complex128
+        assert np.add(np.zeros(5, dtype=np.int8), 257).dtype == np.int16
+        assert np.subtract(np.zeros(5, dtype=np.int8), 257).dtype == np.int16
+        assert np.divide(np.zeros(5, dtype=np.int8), 257).dtype == np.int16
+
+    def test_add_doc(self):
+        import sys
+        if '__pypy__' not in sys.builtin_module_names:
+            skip('cpython sets docstrings differently')
+        try:
+            from numpy import set_docstring
+        except ImportError:
+            from _numpypy.multiarray import set_docstring
+        import numpy as np
+        assert np.add.__doc__ is None
+        add_doc = np.add.__doc__
+        ufunc_doc = np.ufunc.__doc__
+        try:
+            np.add.__doc__ = 'np.add'
+            assert np.add.__doc__ == 'np.add'
+            # Test for interferences between ufunc objects and their class
+            set_docstring(np.ufunc, 'np.ufunc')
+            assert np.ufunc.__doc__ == 'np.ufunc'
+            assert np.add.__doc__ == 'np.add'
+        finally:
+            set_docstring(np.ufunc, ufunc_doc)
+            np.add.__doc__ = add_doc
