@@ -11,24 +11,37 @@ from rpython.tool.jitlogparser.parser import (SimpleParser, Function,
 from rpython.tool.jitlogparser.storage import LoopStorage
 
 
-def find_id_linenos(code):
+def find_ids_range(code):
     """
     Parse the given function and return a dictionary mapping "ids" to
-    "line number".  Ids are identified by comments with a special syntax::
+    "line ranges".  Ids are identified by comments with a special syntax::
 
         # "myid" corresponds to the whole line
         print 'foo' # ID: myid
     """
     result = {}
-    start_lineno = code.startlineno
+    start_lineno = code.co.co_firstlineno
     for i, line in enumerate(py.code.Source(code.source)):
         m = re.search('# ID: (\w+)', line)
         if m:
             name = m.group(1)
             lineno = start_lineno+i
-            result.setdefault(name, lineno)
+            result[name] = xrange(lineno, lineno+1)
     return result
 
+def find_ids(code):
+    """
+    Parse the given function and return a dictionary mapping "ids" to
+    "opcodes".
+    """
+    ids = {}
+    ranges = find_ids_range(code)
+    for name, linerange in ranges.iteritems():
+        opcodes = [opcode for opcode in code.opcodes
+                   if opcode.lineno in linerange]
+        ids[name] = opcodes
+    return ids
+
 
 class Log(object):
     def __init__(self, rawtraces):
@@ -94,18 +107,13 @@ class TraceWithIds(Function):
     def compute_ids(self, ids):
         #
         # 1. compute the ids of self, i.e. the outer function
-        id2lineno = find_id_linenos(self.code)
-        all_my_tracecodes = self.get_list_of_tracecodes()
-        for id, lineno in id2lineno.iteritems():
-            seen = set()
-            opcodes = []
-            for tracecode in all_my_tracecodes:
-                if tracecode.lineno == lineno:
-                    opcode = tracecode.getopcode()
-                    if opcode not in seen:
-                        seen.add(opcode)
-                        opcodes.append(opcode)
-            if opcodes:
+        id2opcodes = find_ids(self.code)
+        all_my_opcodes = self.get_set_of_opcodes()
+        for id, opcodes in id2opcodes.iteritems():
+            if not opcodes:
+                continue
+            target_opcodes = set(opcodes)
+            if all_my_opcodes.intersection(target_opcodes):
                 ids[id] = opcodes
         #
         # 2. compute the ids of all the inlined functions
@@ -113,11 +121,12 @@ class TraceWithIds(Function):
             if isinstance(chunk, TraceWithIds) and chunk.code:
                 chunk.compute_ids(ids)
 
-    def get_list_of_tracecodes(self):
-        result = []
+    def get_set_of_opcodes(self):
+        result = set()
         for chunk in self.chunks:
             if isinstance(chunk, TraceForOpcode):
-                result.append(chunk)
+                opcode = chunk.getopcode()
+                result.add(opcode)
         return result
 
     def has_id(self, id):
@@ -135,7 +144,7 @@ class TraceWithIds(Function):
         for chunk in self.flatten_chunks():
             opcode = chunk.getopcode()
             if opcode_name is None or \
-                   (opcode and opcode.match_name(opcode_name)):
+                   (opcode and opcode.__class__.__name__ == opcode_name):
                 for op in self._ops_for_chunk(chunk, include_guard_not_invalidated):
                     yield op
             else:
@@ -163,7 +172,7 @@ class TraceWithIds(Function):
         for chunk in self.flatten_chunks():
             opcode = chunk.getopcode()
             if opcode in target_opcodes and (opcode_name is None or
-                                             opcode.match_name(opcode_name)):
+                                             opcode.__class__.__name__ == opcode_name):
                 for op in self._ops_for_chunk(chunk, include_guard_not_invalidated):
                     if op in loop_ops:
                         yield op
@@ -253,7 +262,6 @@ class InvalidMatch(Exception):
 
 
 class OpMatcher(object):
-    DUMMY = 1000
 
     def __init__(self, ops, id=None):
         self.ops = ops
@@ -284,13 +292,6 @@ class OpMatcher(object):
         #
         if line.strip() == 'guard_not_invalidated?':
             return 'guard_not_invalidated', None, [], '...', False
-        if line.strip() == 'dummy_get_utf8?':
-            cls.DUMMY += 1
-            return ('getfield_gc_r',
-                    'p%d' % cls.DUMMY,
-                    ['ConstPtr(ptr%d)' % cls.DUMMY],
-                    '<FieldP pypy.objspace.std.unicodeobject.W_UnicodeObject.inst__utf8 .>',
-                    False)
         # find the resvar, if any
         if ' = ' in line:
             resvar, _, line = line.partition(' = ')
@@ -506,7 +507,7 @@ class OpMatcher(object):
                     #else: optional operation
                     iter_ops.revert_one()
                     continue       # try to match with the next exp_op
-            except InvalidMatch as e:
+            except InvalidMatch, e:
                 e.opindex = iter_ops.index - 1
                 raise
 
@@ -524,7 +525,7 @@ class OpMatcher(object):
         expected_ops = self.parse_ops(expected_src)
         try:
             self.match_loop(expected_ops, ignore_ops)
-        except InvalidMatch as e:
+        except InvalidMatch, e:
             print '@' * 40
             print "Loops don't match"
             print "================="
