@@ -105,8 +105,7 @@ class TestInstance(BaseTestPyPyC):
         # -------------------------------
         entry_bridge, = log.loops_by_filename(self.filepath, is_entry_bridge=True)
         ops = entry_bridge.ops_by_id('mutate', opcode='LOAD_ATTR')
-        assert log.opnames(ops) == ['guard_value',
-                                    'guard_not_invalidated',
+        assert log.opnames(ops) == ['guard_value', 'guard_not_invalidated',
                                     'getfield_gc_i']
         # the STORE_ATTR is folded away
         assert list(entry_bridge.ops_by_id('meth1', opcode='STORE_ATTR')) == []
@@ -154,8 +153,7 @@ class TestInstance(BaseTestPyPyC):
         # -------------------------------
         entry_bridge, = log.loops_by_filename(self.filepath, is_entry_bridge=True)
         ops = entry_bridge.ops_by_id('mutate', opcode='LOAD_ATTR')
-        assert log.opnames(ops) == ['guard_value',
-                                    'guard_not_invalidated',
+        assert log.opnames(ops) == ['guard_value', 'guard_not_invalidated',
                                     'getfield_gc_r', 'guard_nonnull_class',
                                     'getfield_gc_r', 'guard_value', # type check on the attribute
                                     ]
@@ -189,6 +187,39 @@ class TestInstance(BaseTestPyPyC):
 
         """)
 
+    def test_oldstyle_newstyle_mix(self):
+        def main():
+            class A:
+                pass
+
+            class B(object, A):
+                def __init__(self, x):
+                    self.x = x
+
+            i = 0
+            b = B(1)
+            while i < 100:
+                v = b.x # ID: loadattr1
+                v = b.x # ID: loadattr2
+                i += v
+            return i
+
+        log = self.run(main, [], threshold=80)
+        loop, = log.loops_by_filename(self.filepath)
+        assert loop.match_by_id('loadattr1',
+        '''
+        guard_not_invalidated(descr=...)
+        i19 = call_i(ConstClass(ll_call_lookup_function), _, _, _, 0, descr=...)
+        guard_no_exception(descr=...)
+        i22 = int_lt(i19, 0)
+        guard_true(i22, descr=...)
+        i26 = call_i(ConstClass(ll_call_lookup_function), _, _, _, 0, descr=...)
+        guard_no_exception(descr=...)
+        i29 = int_lt(i26, 0)
+        guard_true(i29, descr=...)
+        ''')
+        assert loop.match_by_id('loadattr2', "")   # completely folded away
+
     def test_python_contains(self):
         def main():
             class A(object):
@@ -252,118 +283,3 @@ class TestInstance(BaseTestPyPyC):
             --TICK--
             jump(..., descr=...)
         """)
-
-    def test_super_no_args(self):
-        def main():
-            class A(object):
-                def m(self, x):
-                    return x + 1
-            class B(A):
-                def m(self, x):
-                    return super().m(x)
-            i = 0
-            while i < 300:
-                i = B().m(i)
-            return i
-
-        log = self.run(main, [])
-        loop, = log.loops_by_filename(self.filepath)
-        assert loop.match("""
-            i78 = int_lt(i72, 300)
-            guard_true(i78, descr=...)
-            guard_not_invalidated(descr=...)
-            p1 = force_token()
-            p65 = force_token()
-            p3 = force_token()
-            i81 = int_add(i72, 1)
-
-            # can't use TICK here, because of the extra setfield_gc
-            ticker0 = getfield_raw_i(#, descr=<FieldS pypysig_long_struct.c_value .*>)
-            setfield_gc(p0, p65, descr=<FieldP pypy.interpreter.pyframe.PyFrame.vable_token .>)
-            ticker_cond0 = int_lt(ticker0, 0)
-            guard_false(ticker_cond0, descr=...)
-
-            jump(..., descr=...)
-        """)
-
-    def test_float_instance_field_read(self):
-        def main():
-            class A(object):
-                def __init__(self, x, y):
-                    self.x = float(x)
-                    self.y = float(y)
-
-            l = [A(i, i * 5) for i in range(2000)]
-
-            res = 0
-            for x in l:
-                res += x.x + x.y # ID: get
-            return res
-        log = self.run(main, [])
-        listcomp, loop, = log.loops_by_filename(self.filepath)
-        loop.match_by_id('get', """
-            p67 = getfield_gc_r(p63, descr=...) # map
-            guard_value(p67, ConstPtr(ptr68), descr=...) # promote map
-            guard_not_invalidated(descr=...)
-            p69 = getfield_gc_r(p63, descr=...) # value0
-            i71 = getarrayitem_gc_i(p69, 0, descr=...) # x
-            f71 = convert_longlong_bytes_to_float(i71)
-            i73 = getarrayitem_gc_i(p69, 1, descr=...) # y
-            f73 = convert_longlong_bytes_to_float(i73)
-            f74 = float_add(f71, f73) # add them
-            f75 = float_add(f57, f74)
-            --TICK--
-""")
-
-    def test_float_instance_field_write(self):
-        def main():
-            class A(object):
-                def __init__(self, x):
-                    self.x = float(x)
-
-            l = [A(i) for i in range(2000)]
-
-            for a in l:
-                a.x += 3.4 # ID: set
-        log = self.run(main, [])
-        listcomp, loop, = log.loops_by_filename(self.filepath)
-        loop.match_by_id('set', """
-            p60 = getfield_gc_r(p56, descr=...) # map
-            guard_value(p60, ConstPtr(ptr61), descr=...)
-            guard_not_invalidated(descr=...)
-            p62 = getfield_gc_r(p56, descr=...) # value
-            i64 = getarrayitem_gc_i(p62, 0, descr=...) # x
-            f64 = convert_longlong_bytes_to_float(i64)
-            f66 = float_add(f64, 3.400000) 
-            i66 = convert_float_bytes_to_longlong(f66)
-            i68 = getfield_raw_i(..., descr=...)
-            setarrayitem_gc(p62, 0, i66, descr=...) # store x
-            i71 = int_lt(i68, 0)
-            guard_false(i71, descr=...)
-""")
-
-
-    def test_namedtuple_construction(self):
-        def main():
-            from collections import namedtuple
-            A = namedtuple("A", "x y")
-            res = 0
-            i = 0
-            while i < 2000:
-                res += A(i, 0).x
-                i += 1
-        log = self.run(main, [])
-        loop, = log.loops_by_filename(self.filepath)
-        assert loop.match("""
-            i7 = int_lt(i5, 2000)
-            guard_true(i7, descr=...)
-            guard_not_invalidated(descr=...)
-            p1 = force_token()
-            p2 = force_token()
-            i20 = int_add_ovf(i19, i5)
-            guard_no_overflow(descr=...)
-            i9 = int_add(i5, 1)
-            --TICK--
-            jump(..., descr=...)
-        """)
-
