@@ -96,7 +96,6 @@ class TestMisc(BaseTestPyPyC):
             class A(object):
                 def __init__(self, val):
                     self.val1 = self.val2 = val
-            A("x") # prevent field unboxing
             a = A(1)
             b = A(1)
             sa = 0
@@ -114,7 +113,6 @@ class TestMisc(BaseTestPyPyC):
             i12 = int_is_true(i4)
             guard_true(i12, descr=...)
             guard_not_invalidated(descr=...)
-            guard_nonnull_class(p10, ConstClass(W_IntObject), descr=...)
             i10p = getfield_gc_i(p10, descr=...)
             i10 = int_mul_ovf(2, i10p)
             guard_no_overflow(descr=...)
@@ -129,6 +127,38 @@ class TestMisc(BaseTestPyPyC):
             jump(..., descr=...)
             """)
 
+    def test_xrange_iter(self):
+        def main(n):
+            def g(n):
+                return xrange(n)
+            s = 0
+            for i in xrange(n):  # ID: for
+                tmp = g(n)
+                s += tmp[i]     # ID: getitem
+                a = 0
+            return s
+        #
+        log = self.run(main, [1000])
+        assert log.result == 1000 * 999 / 2
+        loop, = log.loops_by_filename(self.filepath)
+        assert loop.match("""
+        i15 = int_lt(i10, i11)
+        guard_true(i15, descr=...)
+        i17 = int_add(i10, 1)
+        setfield_gc(p9, i17, descr=<.* .*W_XRangeIterator.inst_current .*>)
+        guard_not_invalidated(descr=...)
+        i18 = force_token()
+        i84 = int_sub(i14, 1)
+        i21 = int_lt(i10, 0)
+        guard_false(i21, descr=...)
+        i22 = int_lt(i10, i14)
+        guard_true(i22, descr=...)
+        i23 = int_add_ovf(i6, i10)
+        guard_no_overflow(descr=...)
+        --TICK--
+        jump(..., descr=...)
+        """)
+
     def test_range_iter_simple(self):
         def main(n):
             def g(n):
@@ -143,39 +173,24 @@ class TestMisc(BaseTestPyPyC):
         log = self.run(main, [1000])
         assert log.result == 1000 * 999 / 2
         loop, = log.loops_by_filename(self.filepath)
-        assert loop.match(self.RANGE_ITER_STEP_1)
-
-    RANGE_ITER_STEP_1 = """
+        assert loop.match("""
             guard_not_invalidated?
-            # W_IntRangeStepOneIterator.next()
-            i16 = int_lt(i11, i12)
-            guard_true(i16, descr=...)
+            i16 = int_ge(i11, i12)
+            guard_false(i16, descr=...)
             i20 = int_add(i11, 1)
-            setfield_gc(p4, i20, descr=<.* .*W_IntRangeIterator.inst_current .*>)
+            setfield_gc(p4, i20, descr=<.* .*W_AbstractSeqIterObject.inst_index .*>)
             guard_not_invalidated?
             i21 = force_token()
-            i89 = int_lt(0, i9)
-            guard_true(i89, descr=...)
             i88 = int_sub(i9, 1)
-
-            # Compared with pypy2, we get these two operations extra.
-            # I think the reason is that W_IntRangeStepOneIterator is used
-            # for any 'start' value, which might be negative.
-            i89 = int_lt(i11, 0)
-            guard_false(i89, descr=...)
-
             i25 = int_ge(i11, i9)
             guard_false(i25, descr=...)
             i27 = int_add_ovf(i7, i11)
             guard_no_overflow(descr=...)
             --TICK--
             jump(..., descr=...)
-        """
+        """)
 
     def test_range_iter_normal(self):
-        # Difference: range(n) => range(1, n).
-        # This difference doesn't really change anything in pypy3.5,
-        # but it used to in pypy2.7.
         def main(n):
             def g(n):
                 return range(n)
@@ -189,7 +204,26 @@ class TestMisc(BaseTestPyPyC):
         log = self.run(main, [1000])
         assert log.result == 1000 * 999 / 2
         loop, = log.loops_by_filename(self.filepath)
-        assert loop.match(self.RANGE_ITER_STEP_1)
+        assert loop.match("""
+            guard_not_invalidated?
+            i16 = int_ge(i11, i12)
+            guard_false(i16, descr=...)
+            i17 = int_mul(i11, i14)
+            i18 = int_add(i15, i17)
+            i20 = int_add(i11, 1)
+            setfield_gc(p4, i20, descr=<.* .*W_AbstractSeqIterObject.inst_index .*>)
+            guard_not_invalidated?
+            i21 = force_token()
+            i95 = int_sub(i9, 1)
+            i23 = int_lt(i18, 0)
+            guard_false(i23, descr=...)
+            i25 = int_ge(i18, i9)
+            guard_false(i25, descr=...)
+            i27 = int_add_ovf(i7, i18)
+            guard_no_overflow(descr=...)
+            --TICK--
+            jump(..., descr=...)
+        """)
 
     def test_chain_of_guards(self):
         src = """
@@ -239,7 +273,7 @@ class TestMisc(BaseTestPyPyC):
             i21 = getfield_gc_i(p17, descr=<FieldS .*W_Array.*.inst_len .*>)
             i23 = int_lt(0, i21)
             guard_true(i23, descr=...)
-            i24 = getfield_gc_i(p17, descr=<FieldU .*W_ArrayBase.inst__buffer .*>)
+            i24 = getfield_gc_i(p17, descr=<FieldU .*W_ArrayTypei.inst_buffer .*>)
             i25 = getarrayitem_raw_i(i24, 0, descr=<.*>)
             i27 = int_lt(1, i21)
             guard_false(i27, descr=...)
@@ -251,13 +285,6 @@ class TestMisc(BaseTestPyPyC):
         """)
 
     def test_dont_trace_every_iteration(self):
-        def reference(a, b):
-            i = sa = 0
-            while i < 300:
-                sa += a % b
-                i += 1
-            return sa
-        #
         def main(a, b):
             i = sa = 0
             while i < 300:
@@ -269,12 +296,9 @@ class TestMisc(BaseTestPyPyC):
                 i += 1
             return sa
         #
-        log_ref = self.run(reference, [10, 20])
-        assert log_ref.result == 300 * (10 % 20)
-        #
         log = self.run(main, [10, 20])
         assert log.result == 300 * (10 % 20)
-        assert log.jit_summary.tracing_no == log_ref.jit_summary.tracing_no
+        assert log.jit_summary.tracing_no == 1
         loop, = log.loops_by_filename(self.filepath)
         assert loop.match("""
             i11 = int_lt(i7, 300)
@@ -288,7 +312,7 @@ class TestMisc(BaseTestPyPyC):
         #
         log = self.run(main, [-10, -20])
         assert log.result == 300 * (-10 % -20)
-        assert log.jit_summary.tracing_no == log_ref.jit_summary.tracing_no
+        assert log.jit_summary.tracing_no == 1
 
     def test_overflow_checking(self):
         """
@@ -302,7 +326,7 @@ class TestMisc(BaseTestPyPyC):
                     return -1
                 return a-b
             #
-            total = sys.maxsize - 2147483647
+            total = sys.maxint - 2147483647
             for i in range(100000):
                 total += f(i, 5)
             #
@@ -311,7 +335,6 @@ class TestMisc(BaseTestPyPyC):
         self.run_and_check(main, [])
 
     def test_global(self):
-        # check that the global read is removed even from the entry bridge
         log = self.run("""
         i = 0
         globalinc = 1
@@ -323,10 +346,7 @@ class TestMisc(BaseTestPyPyC):
         """, [1000])
 
         loop, = log.loops_by_id("globalread", is_entry_bridge=True)
-        assert loop.match_by_id("globalread", """
-            # only a dead read
-            dummy_get_utf8?
-        """)
+        assert len(loop.ops_by_id("globalread")) == 0
 
     def test_eval(self):
         def main():
@@ -367,8 +387,7 @@ class TestMisc(BaseTestPyPyC):
     def test_long_comparison(self):
         def main(n):
             while n:
-                x = 12345678901234567890123456
-                x > 1231231231231231231231231  # ID: long_op
+                12345L > 123L  # ID: long_op
                 n -= 1
 
         log = self.run(main, [300])
@@ -391,75 +410,3 @@ class TestMisc(BaseTestPyPyC):
         # the following assertion fails if the loop was cancelled due
         # to "abort: vable escape"
         assert len(loops) == 1
-
-    def test_forward_jump_absolute_doesnt_start_loops(self):
-        def main():
-            def f(x):
-                for i in range(x):
-                    break # jump_absolute, forward
-
-            for i in range(1000000):
-                f(i)
-
-        log = self.run(main, [])
-        loops = log.loops_by_filename(self.filepath)
-        assert len(loops) == 1
-
-    def test_stat_result_virtual(self):
-        def main(n):
-            import os
-            res = 0
-            for i in range(n):
-                res += os.path.islink(__file__) # ID: islink
-            return res
-        log = self.run(main, [3000])
-        loop, = log.loops_by_id("islink")
-        opnames = log.opnames(loop.allops())
-        # one left (used to be 20+)
-        assert opnames.count('new_with_vtable') == 1
-        assert opnames.count('new') == 0
-        assert opnames.count('new_array_clear') == 0
-
-    def test_locals(self):
-        def main(n):
-            res = 0
-            for i in range(n):
-                locals()["abc"] = 1
-                res += locals()["abc"] + locals()["i"]
-            return res
-        log = self.run(main, [3000])
-        loop, = log.loops_by_filename(self.filepath)
-        assert loop.match("""
-            i79 = int_lt(i76, 0)
-            guard_false(i79, descr=...)
-            i80 = int_ge(i76, i33)
-            guard_false(i80, descr=...)
-            i82 = int_add(i76, 1)
-            setfield_gc(p16, i82, descr=...)
-            guard_not_invalidated(descr=...)
-            setarrayitem_gc(p63, 1, i71, descr=...)
-            setarrayitem_gc(p63, 2, i76, descr=...)
-            setarrayitem_gc(p63, 0, i77, descr=...)
-            i87 = int_add_ovf(i71, i82)
-            guard_no_overflow(descr=...)
-
-            --TICK--
-
-            i92 = arraylen_gc(p61, descr=...)
-            i93 = arraylen_gc(p63, descr=...)
-            jump(..., descr=...)
-        """)
-
-    def test_locals_in_inlined_function(self):
-        def main(n):
-            def g():
-                locals()["ABC"] = True
-                return 1
-            res = 0
-            for i in range(n):
-                res += g()
-            return res
-        log = self.run(main, [3000])
-        loop, = log.loops_by_filename(self.filepath)
-        opnames = log.opnames(loop.allops())
-        assert "new" not in opnames
