@@ -1,7 +1,6 @@
-from rpython.rlib.rarithmetic import (r_uint, r_ulonglong, r_longlong,
-                                      maxint, intmask)
-from rpython.rlib import jit, rutf8
+from rpython.rlib import jit
 from rpython.rlib.objectmodel import specialize
+from rpython.rlib.rstring import StringBuilder
 from rpython.rlib.rstruct.error import StructError
 from rpython.rlib.rstruct.formatiterator import FormatIterator
 
@@ -9,15 +8,11 @@ from pypy.interpreter.error import OperationError
 
 
 class PackFormatIterator(FormatIterator):
-    def __init__(self, space, wbuf, args_w):
+    def __init__(self, space, args_w, size):
         self.space = space
         self.args_w = args_w
         self.args_index = 0
-        self.pos = 0
-        self.wbuf = wbuf
-
-    def advance(self, count):
-        self.pos += count
+        self.result = StringBuilder(size)
 
     # This *should* be always unroll safe, the only way to get here is by
     # unroll the interpret function, which means the fmt is const, and thus
@@ -34,10 +29,8 @@ class PackFormatIterator(FormatIterator):
 
     @jit.unroll_safe
     def align(self, mask):
-        pad = (-self.pos) & mask
-        for i in range(self.pos, self.pos+pad):
-            self.wbuf.setitem(i, '\x00')
-        self.advance(pad)
+        pad = (-self.result.getlength()) & mask
+        self.result.append_multiple_char('\x00', pad)
 
     def finished(self):
         if self.args_index != len(self.args_w):
@@ -67,19 +60,28 @@ class PackFormatIterator(FormatIterator):
     def _accept_integral(self, meth):
         space = self.space
         w_obj = self.accept_obj_arg()
-        if space.isinstance_w(w_obj, space.w_int):
+        if (space.isinstance_w(w_obj, space.w_int) or
+            space.isinstance_w(w_obj, space.w_long)):
             w_index = w_obj
         else:
             w_index = None
             if space.lookup(w_obj, '__index__'):
                 try:
                     w_index = space.index(w_obj)
-                except OperationError as e:
+                except OperationError, e:
                     if not e.match(space, space.w_TypeError):
                         raise
                     pass
+            if w_index is None and space.lookup(w_obj, '__int__'):
+                if space.isinstance_w(w_obj, space.w_float):
+                    msg = "integer argument expected, got float"
+                else:
+                    msg = "integer argument expected, got non-integer" \
+                          " (implicit conversion using __int__ is deprecated)"
+                space.warn(space.wrap(msg), space.w_DeprecationWarning)
+                w_index = space.int(w_obj)   # wrapped float -> wrapped int or long
             if w_index is None:
-                raise StructError("required argument is not an integer")
+                raise StructError("cannot convert argument to integer")
         method = getattr(space, meth)
         try:
             return method(w_index)
@@ -94,11 +96,11 @@ class PackFormatIterator(FormatIterator):
 
     def accept_str_arg(self):
         w_obj = self.accept_obj_arg()
-        return self.space.bytes_w(w_obj)
+        return self.space.str_w(w_obj)
 
     def accept_unicode_arg(self):
         w_obj = self.accept_obj_arg()
-        return self.space.utf8_len_w(w_obj)
+        return self.space.unicode_w(w_obj)
 
     def accept_float_arg(self):
         w_obj = self.accept_obj_arg()
@@ -116,7 +118,6 @@ class UnpackFormatIterator(FormatIterator):
         self.buf = buf
         self.length = buf.getlength()
         self.pos = 0
-        self.strides = None
         self.result_w = []     # list of wrapped objects
 
     # See above comment on operate.
@@ -134,61 +135,27 @@ class UnpackFormatIterator(FormatIterator):
         self.pos = (self.pos + mask) & ~mask
 
     def finished(self):
-        value = self.pos
-        if self.strides and self.strides[0] < 0:
-                value = -self.pos
-        if value != self.length:
+        if self.pos != self.length:
             raise StructError("unpack str size too long for format")
 
-    def can_advance(self, count):
-        if self.strides:
-            count = self.strides[0]
-        end = self.pos + count
-        return end <= self.length
-
-    def advance(self, count):
-        if not self.can_advance(count):
-            raise StructError("unpack str size too short for format")
-        self.pos += count
-
     def read(self, count):
-        curpos = self.pos
-        self.advance(count) # raise if we are out of bound
-        return self.buf.getslice(curpos, 1, count)
+        end = self.pos + count
+        if end > self.length:
+            raise StructError("unpack str size too short for format")
+        s = self.buf.getslice(self.pos, end, 1, count)
+        self.pos = end
+        return s
 
     @specialize.argtype(1)
     def appendobj(self, value):
-        if isinstance(value, float):
-            w_value = self.space.newfloat(value)
-        elif isinstance(value, str):
-            w_value = self.space.newbytes(value)
-        elif isinstance(value, unicode):
-            assert not isinstance(value, unicode)
-            w_value = self.space.newutf8(value.decode('utf-8'), len(value))
-        elif isinstance(value, bool):
-            w_value = self.space.newbool(value)
-        else:
-            w_value = self.space.newint(value)
-        self.result_w.append(w_value)
-
-    def append_utf8(self, value):
-        w_ch = self.space.newutf8(rutf8.unichr_as_utf8(r_uint(value)), 1)
-        self.result_w.append(w_ch)
+        self.result_w.append(self.space.wrap(value))
 
     def get_pos(self):
         return self.pos
 
-    def get_buffer_and_pos(self):
-        return self.buf, self.pos
+    def get_buffer_as_string_maybe(self):
+        string, pos = self.buf.as_str_and_offset_maybe()
+        return string, pos+self.pos
 
-    def skip(self, count):
-        # assumption: UnpackFormatIterator only iterates over
-        # flat structures (continous memory) either: forward (index
-        # grows) or reverse
-        if self.strides:
-            assert len(self.strides) == 1
-            count = self.strides[0]
-        end = self.pos + count
-        if end > self.length:
-            raise StructError("unpack str size too short for format")
-        self.pos = end
+    def skip(self, size):
+        self.read(size) # XXX, could avoid taking the slice
