@@ -5,10 +5,12 @@ from rpython.rlib.objectmodel import specialize, newlist_hint
 from rpython.rlib.rarithmetic import ovfcheck
 from rpython.rlib.rstring import (
     find, rfind, count, endswith, replace, rsplit, split, startswith)
+from rpython.rlib.buffer import Buffer
+
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import WrappedDefault, unwrap_spec
-from pypy.interpreter.unicodehelper import str_decode_utf8
-from pypy.objspace.std.sliceobject import W_SliceObject, unwrap_start_stop
+from pypy.objspace.std.sliceobject import (W_SliceObject, unwrap_start_stop,
+    normalize_simple_slice)
 
 
 class StringMethods(object):
@@ -16,7 +18,7 @@ class StringMethods(object):
         assert start >= 0
         assert stop >= 0
         #if start == 0 and stop == len(s) and space.is_w(space.type(orig_obj),
-        #                                                space.w_bytes):
+        #                                                space.w_str):
         #    return orig_obj
         return self._new(s[start:stop])
 
@@ -24,67 +26,31 @@ class StringMethods(object):
         value = self._val(space)
         lenself = len(value)
         start, end = unwrap_start_stop(space, lenself, w_start, w_end)
-        # the None means "no offset"; see bytearrayobject.py
-        return (value, start, end, None)
-
-    @staticmethod
-    def descr_maketrans(space, w_type, w_from, w_to):
-        """B.maketrans(frm, to) -> translation table
-
-        Return a translation table (a bytes object of length 256) suitable
-        for use in the bytes or bytearray translate method where each byte
-        in frm is mapped to the byte at the same position in to.
-        The bytes objects frm and to must be of the same length.
-        """
-        from pypy.objspace.std.bytesobject import makebytesdata_w
-
-        base_table = [chr(i) for i in range(256)]
-        list_from = makebytesdata_w(space, w_from)
-        list_to = makebytesdata_w(space, w_to)
-
-        if len(list_from) != len(list_to):
-            raise oefmt(space.w_ValueError,
-                        "maketrans arguments must have same length")
-
-        for i in range(len(list_from)):
-            pos_from = ord(list_from[i])
-            char_to = list_to[i]
-            base_table[pos_from] = char_to
-
-        return space.newbytes(''.join(base_table))
+        return (value, start, end)
 
     def _multi_chr(self, c):
         return c
 
-    @staticmethod
-    def _single_char(space, w_sub):
-        try:
-            char = space.int_w(w_sub)
-        except OperationError as e:
-            if e.match(space, space.w_OverflowError):
-                char = 256 # arbitrary value which will trigger the ValueError
-                # condition below
-            else:
-                raise
-        if not 0 <= char < 256:
-            raise oefmt(space.w_ValueError,
-                        "byte must be in range(0, 256)")
-        return chr(char)
-
     def descr_len(self, space):
-        return space.newint(self._len())
+        return space.wrap(self._len())
 
-    def descr_iter(self, space):
-        from pypy.objspace.std.iterobject import W_StringIterObject
-        return W_StringIterObject(self, self._iter_getitem_result)
+    #def descr_iter(self, space):
+    #    pass
 
     def descr_contains(self, space, w_sub):
-        value, start, end, _ = self._convert_idx_params(space, None, None)
-        other = self._op_val(space, w_sub, allow_char=True)
+        value = self._val(space)
         if self._use_rstr_ops(space, w_sub):
-            res = value.find(other, start, end)
+            other = self._op_val(space, w_sub)
+            return space.newbool(value.find(other) >= 0)
+
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytesObject):
+            other = self._op_val(space, w_sub)
+            res = find(value, other, 0, len(value))
         else:
-            res = find(value, other, start, end)
+            buffer = _get_buffer(space, w_sub)
+            res = find(value, buffer, 0, len(value))
+
         return space.newbool(res >= 0)
 
     def descr_add(self, space, w_other):
@@ -116,9 +82,6 @@ class StringMethods(object):
 
     descr_rmul = descr_mul
 
-    _KIND1 = "string"
-    _KIND2 = "string"
-
     def descr_getitem(self, space, w_index):
         if isinstance(w_index, W_SliceObject):
             selfvalue = self._val(space)
@@ -133,22 +96,26 @@ class StringMethods(object):
                 ret = _descr_getslice_slowpath(selfvalue, start, step, sl)
                 return self._new_from_list(ret)
 
-        index = space.getindex_w(w_index, space.w_IndexError, self._KIND1)
+        index = space.getindex_w(w_index, space.w_IndexError, "string index")
         return self._getitem_result(space, index)
 
     def _getitem_result(self, space, index):
-        # Returns the result of 'self[index]', where index is an unwrapped int.
-        # Used by descr_getitem() and by descr_iter().
         selfvalue = self._val(space)
         try:
             character = selfvalue[index]
         except IndexError:
-            raise oefmt(space.w_IndexError, self._KIND1 + " index out of range")
-        from pypy.objspace.std.bytesobject import W_BytesObject
-        if isinstance(self, W_BytesObject):
-            return space.newint(ord(character))
+            raise oefmt(space.w_IndexError, "string index out of range")
         return self._new(character)
 
+    def descr_getslice(self, space, w_start, w_stop):
+        selfvalue = self._val(space)
+        start, stop = normalize_simple_slice(space, len(selfvalue), w_start,
+                                             w_stop)
+        if start == stop:
+            return self._empty()
+        else:
+            return self._sliced(space, selfvalue, start, stop, self)
+
     def descr_capitalize(self, space):
         value = self._val(space)
         if len(value) == 0:
@@ -157,7 +124,7 @@ class StringMethods(object):
         builder = self._builder(len(value))
         builder.append(self._upper(value[0]))
         for i in range(1, len(value)):
-            builder.append(self._lower_in_str(value, i))
+            builder.append(self._lower(value[i]))
         return self._new(builder.build())
 
     @unwrap_spec(width=int, w_fillchar=WrappedDefault(' '))
@@ -179,32 +146,43 @@ class StringMethods(object):
         return self._new(centered)
 
     def descr_count(self, space, w_sub, w_start=None, w_end=None):
-        value, start, end, _ = self._convert_idx_params(space, w_start, w_end)
+        value, start, end = self._convert_idx_params(space, w_start, w_end)
 
-        sub = self._op_val(space, w_sub, allow_char=True)
         if self._use_rstr_ops(space, w_sub):
-            return space.newint(value.count(sub, start, end))
+            return space.newint(value.count(self._op_val(space, w_sub), start,
+                                            end))
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytearrayObject):
+            res = count(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = count(value, w_sub._value, start, end)
         else:
-            res = count(value, sub, start, end)
-            assert res >= 0
-        return space.newint(res)
+            buffer = _get_buffer(space, w_sub)
+            res = count(value, buffer, start, end)
+
+        return space.wrap(max(res, 0))
 
     def descr_decode(self, space, w_encoding=None, w_errors=None):
         from pypy.objspace.std.unicodeobject import (
-            get_encoding_and_errors, decode_object)
-        encoding, errors = get_encoding_and_errors(space, w_encoding, w_errors)
-        if encoding is None:
-            encoding = 'utf8'
-        if encoding == 'utf8' or encoding == 'utf-8':
-            # fast path - do not call into app-level codecs.py
-            from pypy.module._codecs.interp_codecs import CodecState
-            state = space.fromcache(CodecState)
-            eh = state.decode_error_handler
-            s = space.charbuf_w(self)
-            ret, lgt, pos = str_decode_utf8(s, errors, True, eh)
-            return space.newtext(ret, lgt)
+            _get_encoding_and_errors, decode_object, unicode_from_string)
+        encoding, errors = _get_encoding_and_errors(space, w_encoding,
+                                                    w_errors)
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        if (encoding is None and errors is None and
+            not isinstance(self, W_BytearrayObject)):
+            return unicode_from_string(space, self)
         return decode_object(space, self, encoding, errors)
 
+    def descr_encode(self, space, w_encoding=None, w_errors=None):
+        from pypy.objspace.std.unicodeobject import (
+            _get_encoding_and_errors, encode_object)
+        encoding, errors = _get_encoding_and_errors(space, w_encoding,
+                                                    w_errors)
+        return encode_object(space, self, encoding, errors)
+
     @unwrap_spec(tabsize=int)
     def descr_expandtabs(self, space, tabsize=8):
         value = self._val(space)
@@ -256,60 +234,82 @@ class StringMethods(object):
         return distance
 
     def descr_find(self, space, w_sub, w_start=None, w_end=None):
-        value, start, end, ofs = self._convert_idx_params(space, w_start, w_end)
+        (value, start, end) = self._convert_idx_params(space, w_start, w_end)
 
-        sub = self._op_val(space, w_sub, allow_char=True)
         if self._use_rstr_ops(space, w_sub):
-            res = value.find(sub, start, end)
+            res = value.find(self._op_val(space, w_sub), start, end)
+            return space.wrap(res)
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytearrayObject):
+            res = find(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = find(value, w_sub._value, start, end)
         else:
-            res = find(value, sub, start, end)
-        if ofs is not None and res >= 0:
-            res -= ofs
-        return space.newint(res)
+            buffer = _get_buffer(space, w_sub)
+            res = find(value, buffer, start, end)
+
+        return space.wrap(res)
 
     def descr_rfind(self, space, w_sub, w_start=None, w_end=None):
-        value, start, end, ofs = self._convert_idx_params(space, w_start, w_end)
+        (value, start, end) = self._convert_idx_params(space, w_start, w_end)
 
-        sub = self._op_val(space, w_sub, allow_char=True)
         if self._use_rstr_ops(space, w_sub):
-            res = value.rfind(sub, start, end)
+            res = value.rfind(self._op_val(space, w_sub), start, end)
+            return space.wrap(res)
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytearrayObject):
+            res = rfind(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = rfind(value, w_sub._value, start, end)
         else:
-            res = rfind(value, sub, start, end)
-        if ofs is not None and res >= 0:
-            res -= ofs
-        return space.newint(res)
+            buffer = _get_buffer(space, w_sub)
+            res = rfind(value, buffer, start, end)
+
+        return space.wrap(res)
 
     def descr_index(self, space, w_sub, w_start=None, w_end=None):
-        value, start, end, ofs = self._convert_idx_params(space, w_start, w_end)
+        (value, start, end) = self._convert_idx_params(space, w_start, w_end)
 
-        sub = self._op_val(space, w_sub, allow_char=True)
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
         if self._use_rstr_ops(space, w_sub):
-            res = value.find(sub, start, end)
+            res = value.find(self._op_val(space, w_sub), start, end)
+        elif isinstance(w_sub, W_BytearrayObject):
+            res = find(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = find(value, w_sub._value, start, end)
         else:
-            res = find(value, sub, start, end)
+            buffer = _get_buffer(space, w_sub)
+            res = find(value, buffer, start, end)
 
         if res < 0:
             raise oefmt(space.w_ValueError,
-                        "substring not found in " + self._KIND2 + ".index")
-        if ofs is not None:
-            res -= ofs
-        return space.newint(res)
+                        "substring not found in string.index")
+        return space.wrap(res)
 
     def descr_rindex(self, space, w_sub, w_start=None, w_end=None):
-        value, start, end, ofs = self._convert_idx_params(space, w_start, w_end)
+        (value, start, end) = self._convert_idx_params(space, w_start, w_end)
 
-        sub = self._op_val(space, w_sub, allow_char=True)
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
         if self._use_rstr_ops(space, w_sub):
-            res = value.rfind(sub, start, end)
+            res = value.rfind(self._op_val(space, w_sub), start, end)
+        elif isinstance(w_sub, W_BytearrayObject):
+            res = rfind(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = rfind(value, w_sub._value, start, end)
         else:
-            res = rfind(value, sub, start, end)
+            buffer = _get_buffer(space, w_sub)
+            res = rfind(value, buffer, start, end)
 
         if res < 0:
             raise oefmt(space.w_ValueError,
-                        "substring not found in " + self._KIND2 + ".rindex")
-        if ofs is not None:
-            res -= ofs
-        return space.newint(res)
+                        "substring not found in string.rindex")
+        return space.wrap(res)
 
     @specialize.arg(2)
     def _is_generic(self, space, func_name):
@@ -424,17 +424,16 @@ class StringMethods(object):
         unwrapped = newlist_hint(size)
         for i in range(size):
             w_s = list_w[i]
-            try:
-                next_string = self._op_val(space, w_s)
-            except OperationError as e:
-                if not e.match(space, space.w_TypeError):
-                    raise
+            check_item = self._join_check_item(space, w_s)
+            if check_item == 1:
                 raise oefmt(space.w_TypeError,
-                            "sequence item %d: expected %s, %T found",
-                            i, self._generic_name(), w_s)
+                            "sequence item %d: expected string, %T found",
+                            i, w_s)
+            elif check_item == 2:
+                return self._join_autoconvert(space, list_w)
             # XXX Maybe the extra copy here is okay? It was basically going to
             #     happen anyway, what with being placed into the builder
-            unwrapped.append(next_string)
+            unwrapped.append(self._op_val(space, w_s))
             prealloc_size += len(unwrapped[i])
 
         sb = self._builder(prealloc_size)
@@ -444,6 +443,9 @@ class StringMethods(object):
             sb.append(unwrapped[i])
         return self._new(sb.build())
 
+    def _join_autoconvert(self, space, list_w):
+        assert False, 'unreachable'
+
     @unwrap_spec(width=int, w_fillchar=WrappedDefault(' '))
     def descr_ljust(self, space, width, w_fillchar):
         value = self._val(space)
@@ -476,14 +478,9 @@ class StringMethods(object):
         value = self._val(space)
         builder = self._builder(len(value))
         for i in range(len(value)):
-            builder.append(self._lower_in_str(value, i))
+            builder.append(self._lower(value[i]))
         return self._new(builder.build())
 
-    def _lower_in_str(self, value, i):
-        # overridden in unicodeobject.py
-        return self._lower(value[i])
-
-    # This is not used for W_UnicodeObject.
     def descr_partition(self, space, w_sub):
         from pypy.objspace.std.bytearrayobject import W_BytearrayObject
         value = self._val(space)
@@ -496,7 +493,7 @@ class StringMethods(object):
 
             pos = value.find(sub)
         else:
-            sub = space.readbuf_w(w_sub)
+            sub = _get_buffer(space, w_sub)
             sublen = sub.getlength()
             if sublen == 0:
                 raise oefmt(space.w_ValueError, "empty separator")
@@ -526,7 +523,7 @@ class StringMethods(object):
 
             pos = value.rfind(sub)
         else:
-            sub = space.readbuf_w(w_sub)
+            sub = _get_buffer(space, w_sub)
             sublen = sub.getlength()
             if sublen == 0:
                 raise oefmt(space.w_ValueError, "empty separator")
@@ -587,7 +584,7 @@ class StringMethods(object):
 
         return self._newlist_unwrapped(space, res)
 
-    @unwrap_spec(keepends=int)
+    @unwrap_spec(keepends=bool)
     def descr_splitlines(self, space, keepends=False):
         value = self._val(space)
         length = len(value)
@@ -606,72 +603,46 @@ class StringMethods(object):
                 eol = pos
             strs.append(value[sol:eol])
         if pos < length:
-            # XXX is this code reachable?
             strs.append(value[pos:length])
         return self._newlist_unwrapped(space, strs)
 
-    def _generic_name(self):
-        return "bytes"
-
-    # This is overridden in unicodeobject, _startswith_tuple is not.
     def descr_startswith(self, space, w_prefix, w_start=None, w_end=None):
-        value, start, end, _ = self._convert_idx_params(space, w_start, w_end)
+        (value, start, end) = self._convert_idx_params(space, w_start, w_end)
         if space.isinstance_w(w_prefix, space.w_tuple):
-            return self._startswith_tuple(space, value, w_prefix, start, end)
-        try:
-            res = self._startswith(space, value, w_prefix, start, end)
-        except OperationError as e:
-            if not e.match(space, space.w_TypeError):
-                raise
-            wanted = self._generic_name()
-            raise oefmt(space.w_TypeError,
-                        "startswith first arg must be %s or a tuple of %s, "
-                        "not %T", wanted, wanted, w_prefix)
-        return space.newbool(res)
+            for w_prefix in space.fixedview(w_prefix):
+                if self._startswith(space, value, w_prefix, start, end):
+                    return space.w_True
+            return space.w_False
+        return space.newbool(self._startswith(space, value, w_prefix, start,
+                                              end))
 
-    def _startswith_tuple(self, space, value, w_prefix, start, end):
-        for w_prefix in space.fixedview(w_prefix):
-            if self._startswith(space, value, w_prefix, start, end):
-                return space.w_True
-        return space.w_False
-
-    # This is overridden in unicodeobject, _startswith_tuple is not.
     def _startswith(self, space, value, w_prefix, start, end):
         prefix = self._op_val(space, w_prefix)
         if start > len(value):
-            return False
+            return self._starts_ends_overflow(prefix)
         return startswith(value, prefix, start, end)
 
-    # This is overridden in unicodeobject, _endswith_tuple is not.
+    def _starts_ends_overflow(self, prefix):
+        return False     # bug-to-bug compat: this is for strings and
+                         # bytearrays, but overridden for unicodes
+
     def descr_endswith(self, space, w_suffix, w_start=None, w_end=None):
-        value, start, end, _ = self._convert_idx_params(space, w_start, w_end)
+        (value, start, end) = self._convert_idx_params(space, w_start, w_end)
         if space.isinstance_w(w_suffix, space.w_tuple):
-            return self._endswith_tuple(space, value, w_suffix, start, end)
-        try:
-            res = self._endswith(space, value, w_suffix, start, end)
-        except OperationError as e:
-            if not e.match(space, space.w_TypeError):
-                raise
-            wanted = self._generic_name()
-            raise oefmt(space.w_TypeError,
-                        "endswith first arg must be %s or a tuple of %s, not "
-                        "%T", wanted, wanted, w_suffix)
-        return space.newbool(res)
+            for w_suffix in space.fixedview(w_suffix):
+                if self._endswith(space, value, w_suffix, start, end):
+                    return space.w_True
+            return space.w_False
+        return space.newbool(self._endswith(space, value, w_suffix, start,
+                                            end))
 
-    def _endswith_tuple(self, space, value, w_suffix, start, end):
-        for w_suffix in space.fixedview(w_suffix):
-            if self._endswith(space, value, w_suffix, start, end):
-                return space.w_True
-        return space.w_False
-
-    # This is overridden in unicodeobject, but _endswith_tuple is not.
     def _endswith(self, space, value, w_prefix, start, end):
         prefix = self._op_val(space, w_prefix)
         if start > len(value):
-            return False
+            return self._starts_ends_overflow(prefix)
         return endswith(value, prefix, start, end)
 
-    def _strip(self, space, w_chars, left, right, name='strip'):
+    def _strip(self, space, w_chars, left, right):
         "internal function called by str_xstrip methods"
         value = self._val(space)
         chars = self._op_val(space, w_chars)
@@ -711,17 +682,17 @@ class StringMethods(object):
     def descr_strip(self, space, w_chars=None):
         if space.is_none(w_chars):
             return self._strip_none(space, left=1, right=1)
-        return self._strip(space, w_chars, left=1, right=1, name='strip')
+        return self._strip(space, w_chars, left=1, right=1)
 
     def descr_lstrip(self, space, w_chars=None):
         if space.is_none(w_chars):
             return self._strip_none(space, left=1, right=0)
-        return self._strip(space, w_chars, left=1, right=0, name='lstrip')
+        return self._strip(space, w_chars, left=1, right=0)
 
     def descr_rstrip(self, space, w_chars=None):
         if space.is_none(w_chars):
             return self._strip_none(space, left=0, right=1)
-        return self._strip(space, w_chars, left=0, right=1, name='rstrip')
+        return self._strip(space, w_chars, left=0, right=1)
 
     def descr_swapcase(self, space):
         selfvalue = self._val(space)
@@ -729,7 +700,7 @@ class StringMethods(object):
         for i in range(len(selfvalue)):
             ch = selfvalue[i]
             if self._isupper(ch):
-                builder.append(self._lower_in_str(selfvalue, i))
+                builder.append(self._lower(ch))
             elif self._islower(ch):
                 builder.append(self._upper(ch))
             else:
@@ -746,20 +717,19 @@ class StringMethods(object):
     def title(self, value):
         builder = self._builder(len(value))
         previous_is_cased = False
-        for i in range(len(value)):
-            ch = value[i]
+        for ch in value:
             if not previous_is_cased:
                 builder.append(self._title(ch))
             else:
-                builder.append(self._lower_in_str(value, i))
+                builder.append(self._lower(ch))
             previous_is_cased = self._iscased(ch)
         return builder.build()
 
     DEFAULT_NOOP_TABLE = ''.join([chr(i) for i in range(256)])
 
     # for bytes and bytearray, overridden by unicode
-    @unwrap_spec(w_delete=WrappedDefault(''))
-    def descr_translate(self, space, w_table, w_delete):
+    @unwrap_spec(w_deletechars=WrappedDefault(''))
+    def descr_translate(self, space, w_table, w_deletechars):
         if space.is_w(w_table, space.w_None):
             table = self.DEFAULT_NOOP_TABLE
         else:
@@ -769,7 +739,7 @@ class StringMethods(object):
                             "translation table must be 256 characters long")
 
         string = self._val(space)
-        deletechars = self._op_val(space, w_delete)
+        deletechars = self._op_val(space, w_deletechars)
         if len(deletechars) == 0:
             buf = self._builder(len(string))
             for char in string:
@@ -816,25 +786,14 @@ class StringMethods(object):
     def descr_getnewargs(self, space):
         return space.newtuple([self._new(self._val(space))])
 
-    def descr_removeprefix(self, space, w_prefix):
-        prefix = self._op_val(space, w_prefix)
-        selfval = self._val(space)
-        if startswith(selfval, prefix):
-            return self._new(selfval[len(prefix):])
-        return self._new(selfval)
-
-    def descr_removesuffix(self, space, w_suffix):
-        suffix = self._op_val(space, w_suffix)
-        selfval = self._val(space)
-        if suffix and endswith(selfval, suffix):
-            end = len(selfval) - len(suffix)
-            assert end >= 0
-            return self._new(selfval[:end])
-        return self._new(selfval)
-
 # ____________________________________________________________
 # helpers for slow paths, moved out because they contain loops
 
 @specialize.argtype(0)
 def _descr_getslice_slowpath(selfvalue, start, step, sl):
     return [selfvalue[start + i*step] for i in range(sl)]
+
+def _get_buffer(space, w_obj):
+    return space.buffer_w(w_obj, space.BUF_SIMPLE)
+
+
