@@ -1,4 +1,3 @@
-# -*- encoding: utf-8 -*-
 """
 The main test for the set implementation is located
 in the stdlibs test/test_set.py which is located in lib-python
@@ -49,9 +48,7 @@ class TestW_SetObject:
 
     def test_space_newset(self):
         s = self.space.newset()
-        assert self.space.text_w(self.space.repr(s)) == 'set()'
-        # check that the second time we don't get 'set(...)'
-        assert self.space.text_w(self.space.repr(s)) == 'set()'
+        assert self.space.str_w(self.space.repr(s)) == 'set([])'
 
     def test_intersection_order(self):
         # theses tests make sure that intersection is done in the correct order
@@ -83,11 +80,10 @@ class TestW_SetObject:
 
     def test_create_set_from_list(self):
         from pypy.interpreter.baseobjspace import W_Root
-        from pypy.objspace.std.setobject import BytesSetStrategy, ObjectSetStrategy, AsciiSetStrategy
+        from pypy.objspace.std.setobject import BytesSetStrategy, ObjectSetStrategy, UnicodeSetStrategy
         from pypy.objspace.std.floatobject import W_FloatObject
 
         w = self.space.wrap
-        wb = self.space.newbytes
         intstr = self.space.fromcache(IntegerSetStrategy)
         tmp_func = intstr.get_storage_from_list
         # test if get_storage_from_list is no longer used
@@ -99,7 +95,7 @@ class TestW_SetObject:
         assert w_set.strategy is intstr
         assert intstr.unerase(w_set.sstorage) == {1:None, 2:None, 3:None}
 
-        w_list = W_ListObject(self.space, [wb("1"), wb("2"), wb("3")])
+        w_list = W_ListObject(self.space, [w("1"), w("2"), w("3")])
         w_set = W_SetObject(self.space)
         _initialize_set(self.space, w_set, w_list)
         assert w_set.strategy is self.space.fromcache(BytesSetStrategy)
@@ -108,7 +104,7 @@ class TestW_SetObject:
         w_list = self.space.iter(W_ListObject(self.space, [w(u"1"), w(u"2"), w(u"3")]))
         w_set = W_SetObject(self.space)
         _initialize_set(self.space, w_set, w_list)
-        assert w_set.strategy is self.space.fromcache(AsciiSetStrategy)
+        assert w_set.strategy is self.space.fromcache(UnicodeSetStrategy)
         assert w_set.strategy.unerase(w_set.sstorage) == {u"1":None, u"2":None, u"3":None}
 
         w_list = W_ListObject(self.space, [w("1"), w(2), w("3")])
@@ -130,26 +126,17 @@ class TestW_SetObject:
 
     def test_listview_bytes_int_on_set(self):
         w = self.space.wrap
-        wb = self.space.newbytes
 
         w_a = W_SetObject(self.space)
-        _initialize_set(self.space, w_a, wb("abcdefg"))
-        assert sorted(self.space.listview_int(w_a)) == [97, 98, 99, 100, 101, 102, 103]
-        assert self.space.listview_bytes(w_a) is None
+        _initialize_set(self.space, w_a, w("abcdefg"))
+        assert sorted(self.space.listview_bytes(w_a)) == list("abcdefg")
+        assert self.space.listview_int(w_a) is None
 
         w_b = W_SetObject(self.space)
         _initialize_set(self.space, w_b, self.space.newlist([w(1),w(2),w(3),w(4),w(5)]))
         assert sorted(self.space.listview_int(w_b)) == [1,2,3,4,5]
         assert self.space.listview_bytes(w_b) is None
 
-    def test_cpyext_add_frozen(self, space):
-        t1 = W_FrozensetObject(space)
-        assert space.len_w(t1) == 0
-        res = t1.cpyext_add_frozen(space.newint(1))
-        assert res
-        assert space.len_w(t1) == 1
-
-
 class AppTestAppSetTest:
 
     def setup_class(self):
@@ -272,7 +259,7 @@ class AppTestAppSetTest:
         raises(KeyError, "b.pop()")
 
         a = set([1,2,3,4,5])
-        for i in range(5):
+        for i in xrange(5):
             a.pop()
         assert a == set()
         raises(KeyError, "a.pop()")
@@ -313,7 +300,7 @@ class AppTestAppSetTest:
         class subset(set):pass
         a = subset()
         b = a | set('abc')
-        assert type(b) is set
+        assert type(b) is subset
 
     def test_init_new_behavior(self):
         s = set.__new__(set, 'abc')
@@ -331,14 +318,14 @@ class AppTestAppSetTest:
         b = subset('abc')
         subset.__new__ = lambda *args: foobar   # not called
         b = b.copy()
-        assert type(b) is set
+        assert type(b) is subset
         assert set(b) == set('abc')
         #
         class frozensubset(frozenset): pass
         b = frozensubset('abc')
         frozensubset.__new__ = lambda *args: foobar   # not called
         b = b.copy()
-        assert type(b) is frozenset
+        assert type(b) is frozensubset
         assert frozenset(b) == frozenset('abc')
 
     def test_union(self):
@@ -350,18 +337,11 @@ class AppTestAppSetTest:
         d = a.union()
         assert d == a
 
-    def test_bytes_items(self):
-        s = set([b'hello'])
-        assert s.pop() == b'hello'
-
-    def test_set_literal(self):
-        """
-        assert {b'a'}.pop() == b'a'
-        """
-
     def test_compare(self):
+        raises(TypeError, cmp, set('abc'), set('abd'))
+        raises(TypeError, cmp, frozenset('abc'), frozenset('abd'))
         assert set('abc') != 'abc'
-        raises(TypeError, "set('abc') < 42")
+        assert not set('abc') < 42
         assert not (set('abc') < set('def'))
         assert not (set('abc') <= frozenset('abd'))
         assert not (set('abc') < frozenset('abd'))
@@ -396,41 +376,10 @@ class AppTestAppSetTest:
         assert set() != set('abc')
         assert set('abc') != set('abd')
 
-    def test_compare_other(self):
-        class TestRichSetCompare:
-            def __gt__(self, some_set):
-                self.gt_called = True
-                return False
-            def __lt__(self, some_set):
-                self.lt_called = True
-                return False
-            def __ge__(self, some_set):
-                self.ge_called = True
-                return False
-            def __le__(self, some_set):
-                self.le_called = True
-                return False
+        class X(set):
+            pass
 
-        # This first tries the builtin rich set comparison, which doesn't know
-        # how to handle the custom object. Upon returning NotImplemented, the
-        # corresponding comparison on the right object is invoked.
-        myset = set(range(3))
-
-        myobj = TestRichSetCompare()
-        myset < myobj
-        assert myobj.gt_called
-
-        myobj = TestRichSetCompare()
-        myset > myobj
-        assert myobj.lt_called
-
-        myobj = TestRichSetCompare()
-        myset <= myobj
-        assert myobj.ge_called
-
-        myobj = TestRichSetCompare()
-        myset >= myobj
-        assert myobj.le_called
+        raises(TypeError, cmp, X(), X())
 
     def test_libpython_equality(self):
         for thetype in [frozenset, set]:
@@ -460,7 +409,7 @@ class AppTestAppSetTest:
         s2 = s1.copy()
         assert s1 is not s2
         assert s1 == s2
-        assert type(s2) is frozenset
+        assert type(s2) is myfrozen
 
     def test_update(self):
         s1 = set('abc')
@@ -484,24 +433,19 @@ class AppTestAppSetTest:
         s1 = set([1, 2.0, "3"])
         s1.update(set(["3", 4, 5.0]))
 
-    def test_update_not_iterable_error(self):
-        with raises(TypeError) as e:
-            set().update(1)
-        assert "'int' object is not iterable" in str(e.value)
-
     def test_recursive_repr(self):
         class A(object):
             def __init__(self, s):
                 self.s = s
             def __repr__(self):
                 return repr(self.s)
-
+        
         s = set([1, 2, 3])
         s.add(A(s))
         therepr = repr(s)
-        assert therepr.startswith("{")
-        assert therepr.endswith("}")
-        inner = set(therepr[1:-1].split(", "))
+        assert therepr.startswith("set([")
+        assert therepr.endswith("])")
+        inner = set(therepr[5:-2].split(", "))
         assert inner == set(["1", "2", "3", "set(...)"])
 
     def test_recursive_repr_frozenset(self):
@@ -512,16 +456,16 @@ class AppTestAppSetTest:
         s = frozenset([1, 2, 3, a])
         a.s = s
         therepr = repr(s)
-        assert therepr.startswith("frozenset({")
-        assert therepr.endswith("})")
+        assert therepr.startswith("frozenset([")
+        assert therepr.endswith("])")
         inner = set(therepr[11:-2].split(", "))
         assert inner == set(["1", "2", "3", "frozenset(...)"])
-
+        
     def test_keyerror_has_key(self):
         s = set()
         try:
             s.remove(1)
-        except KeyError as e:
+        except KeyError, e:
             assert e.args[0] == 1
         else:
             assert 0, "should raise"
@@ -533,7 +477,7 @@ class AppTestAppSetTest:
                 return int(id(self) & 0x7fffffff)
         s = H()
         f = set([s])
-        print(f)
+        print f
         assert s in f
         f.remove(s)
         f.add(s)
@@ -575,7 +519,7 @@ class AppTestAppSetTest:
         key = set([2, 3])
         try:
             s.remove(key)
-        except KeyError as e:
+        except KeyError, e:
             assert e.args[0] is key
 
     def test_contains(self):
@@ -586,39 +530,6 @@ class AppTestAppSetTest:
             assert (c in s) == (c in word)
         raises(TypeError, s.__contains__, [])
 
-        logger = []
-
-        class Foo(object):
-
-            def __init__(self, value, name=None):
-                self.value = value
-                self.name = name or value
-
-            def __repr__(self):
-                return '<Foo %s>' % self.name
-
-            def __eq__(self, other):
-                logger.append((self, other))
-                return self.value == other.value
-
-            def __hash__(self):
-                return 42  # __eq__ will be used given all objects' hashes clash
-
-        foo1, foo2, foo3 = Foo(1), Foo(2), Foo(3)
-        foo42 = Foo(42)
-        foo_set = {foo1, foo2, foo3}
-        del logger[:]
-        foo42 in foo_set
-        logger_copy = set(logger[:])  # prevent re-evaluation during pytest error print
-        assert logger_copy == {(foo3, foo42), (foo2, foo42), (foo1, foo42)}
-
-        del logger[:]
-        foo2_bis = Foo(2, '2 bis')
-        foo2_bis in foo_set
-        logger_copy = set(logger[:])  # prevent re-evaluation during pytest error print
-        assert (foo2, foo2_bis) in logger_copy
-        assert logger_copy.issubset({(foo1, foo2_bis), (foo2, foo2_bis), (foo3, foo2_bis)})
-
     def test_remove(self):
         s = set('abc')
         s.remove('a')
@@ -637,12 +548,12 @@ class AppTestAppSetTest:
         for v1 in ['Q', (1,)]:
             try:
                 s.remove(v1)
-            except KeyError as e:
+            except KeyError, e:
                 v2 = e.args[0]
                 assert v1 == v2
             else:
                 assert False, 'Expected KeyError'
-
+        
     def test_singleton_empty_frozenset(self):
         class Frozenset(frozenset):
             pass
@@ -660,43 +571,10 @@ class AppTestAppSetTest:
             s = subset([2])
             assert s.x == ([2],)
             t = s | base([5])
-            assert type(t) is base, 'base is %s, type(t) is %s' % (base, type(t))
+            # obscure CPython behavior:
+            assert type(t) is subset
             assert not hasattr(t, 'x')
 
-    def test_reverse_ops(self):
-        assert set.__rxor__
-        assert frozenset.__rxor__
-        assert set.__ror__
-        assert frozenset.__ror__
-        assert set.__rand__
-        assert frozenset.__rand__
-        assert set.__rsub__
-        assert frozenset.__rsub__
-
-        # actual behaviour test
-        for base in [set, frozenset]:
-            class S(base):
-                def __xor__(self, other):
-                    if type(other) is not S:
-                        return NotImplemented
-                    return 1
-                __or__ = __and__ = __sub__ = __xor__
-            assert S([1, 2, 3]) ^ S([2, 3, 4]) == 1
-            assert S([1, 2, 3]) ^ {2, 3, 4} == {1, 4}
-            assert {1, 2, 3} ^ S([2, 3, 4]) == {1, 4}
-
-            assert S([1, 2, 3]) & S([2, 3, 4]) == 1
-            assert S([1, 2, 3]) & {2, 3, 4} == {2, 3}
-            assert {1, 2, 3} & S([2, 3, 4]) == {2, 3}
-
-            assert S([1, 2, 3]) | S([2, 3, 4]) == 1
-            assert S([1, 2, 3]) | {2, 3, 4} == {1, 2, 3, 4}
-            assert {1, 2, 3} | S([2, 3, 4]) == {1, 2, 3, 4}
-
-            assert S([1, 2, 3]) - S([2, 3, 4]) == 1
-            assert S([1, 2, 3]) - {2, 3, 4} == {1}
-            assert {1, 2, 3} - S([2, 3, 4]) == {1}
-
     def test_isdisjoint(self):
         assert set([1,2,3]).isdisjoint(set([4,5,6]))
         assert set([1,2,3]).isdisjoint(frozenset([4,5,6]))
@@ -894,7 +772,6 @@ class AppTestAppSetTest:
 
     def test_empty_unhashable(self):
         s = set()
-        raises(TypeError, s.__contains__, [[]])
         raises(TypeError, s.difference, [[]])
         raises(TypeError, s.difference_update, [[]])
         raises(TypeError, s.intersection, [[]])
@@ -1059,7 +936,7 @@ class AppTestAppSetTest:
         it = iter(s)
         s.add(7)
         # 's' is now length 4
-        raises(RuntimeError, it.__next__)
+        raises(RuntimeError, it.next)
 
     def test_iter_set_strategy_only_change_1(self):
         s = set([1, 3, 5])
@@ -1067,44 +944,20 @@ class AppTestAppSetTest:
         class Foo(object):
             def __eq__(self, other):
                 return False
-            def __hash__(self):
-                return 0
         assert Foo() not in s      # this changes the strategy of 'd'
         lst = list(s)  # but iterating still works
         assert sorted(lst) == [1, 3, 5]
 
     def test_iter_set_strategy_only_change_2(self):
-        # on py3k the IntStrategy doesn't work yet. So, we use the
-        # StringSetStrategy for this test
-        s = set(['1', '3', '5'])
+        s = set([1, 3, 5])
         it = iter(s)
-        s.add(42) # change the strategy
-        s.remove('1')
+        s.add('foo')
+        s.remove(1)
         # 's' is still length 3, but its strategy changed.  we are
         # getting a RuntimeError because iterating over the old storage
         # gives us 1, but 1 is not in the set any longer.
         raises(RuntimeError, list, it)
 
-    def test_iter_bytes_strategy(self):
-        l = [b'a', b'b']
-        s = set(l)
-        n = next(iter(s))
-        assert type(n) is bytes
-        assert n in l
-
-    def test_unicodestrategy(self):
-        s = 'àèìòù'
-        myset = set([s])
-        s2 = myset.pop()
-        assert s2 == s
-
-    def test_preserve_identity_of_strings(self):
-        s = 'hello'
-        myset = set([s])
-        s2 = myset.pop()
-        assert s2 == s
-        assert s2 is s
-
     def test_intersect_frozenset_set(self):
         # worked before
         assert type(frozenset([2]) & set([1, 2])) is frozenset
@@ -1149,53 +1002,3 @@ class AppTestAppSetTest:
            raise ValueError
            yield 1
         raises(ValueError, set, f())
-
-    def test_pickle(self):
-        d = {1, 2, 3}
-        it = iter(d)
-        first = next(it)
-        reduced = it.__reduce__()
-        rebuild, args = reduced
-        assert rebuild is iter
-        new = rebuild(*args)
-        items = set(new)
-        assert len(items) == 2
-        items.add(first)
-        assert items == set(d)
-
-    def test_unicode_bug_in_listview_utf8(self):
-        l1 = set(u'\u1234\u2345')
-        assert l1 == set([u'\u1234', '\u2345'])
-
-    def test_frozenset_init_does_nothing(self):
-        f = frozenset([1, 2, 3])
-        f.__init__(4, 5, 6)
-        assert f == frozenset([1, 2, 3])
-
-    def test_error_message_wrong_self(self):
-        e = raises(TypeError, frozenset.copy, 42)
-        assert "frozenset" in str(e.value)
-        if hasattr(frozenset.copy, 'im_func'):
-            e = raises(TypeError, frozenset.copy.im_func, 42)
-            assert "'frozenset' object expected, got 'int' instead" in str(e.value)
-        if hasattr(set.copy, 'im_func'):
-            e = raises(TypeError, set.copy.im_func, 42)
-            assert "'set' object expected, got 'int' instead" in str(e.value)
-
-    def test_cant_mutate_frozenset_via_set(self):
-        x = frozenset()
-        raises(TypeError, set.add, x, 1)
-        raises(TypeError, set.__ior__, x, set([2]))
-
-    def test_class_getitem(self):
-        for cls in set, frozenset:
-            assert set[int, str].__origin__ is set
-            assert set[int, str].__args__ == (int, str)
-
-    def test_frozenset_hash_like_cpython(self):
-        import sys
-        if sys.maxsize != 2**63 - 1:
-            skip("64 bit only")
-        assert hash(frozenset()) == 133146708735736
-        h = hash(frozenset([1, 2, 9]))
-        assert h == (-5390384031640186368)
