@@ -2,10 +2,8 @@ import py
 from rpython.rtyper.lltypesystem import lltype, llmemory, rffi, rstr
 from rpython.rtyper import rclass
 from rpython.jit.backend.test import test_random
-from rpython.jit.backend.test.test_random import getint, getref_base, getref
 from rpython.jit.metainterp.resoperation import ResOperation, rop, optypes
 from rpython.jit.metainterp.history import ConstInt, ConstPtr, getkind
-from rpython.jit.metainterp.support import ptr2int
 from rpython.jit.codewriter import heaptracker
 from rpython.jit.codewriter.effectinfo import EffectInfo
 from rpython.rtyper.annlowlevel import llhelper
@@ -171,7 +169,7 @@ class LLtypeOperationBuilder(test_random.OperationBuilder):
         if length == 0:
             raise test_random.CannotProduceOperation
         v_index = r.choice(self.intvars)
-        if not (0 <= getint(v_index) < length):
+        if not (0 <= v_index.getint() < length):
             v_index = ConstInt(r.random_integer() % length)
         return v_index
 
@@ -219,6 +217,9 @@ class LLtypeOperationBuilder(test_random.OperationBuilder):
 
 # ____________________________________________________________
 
+def ConstAddr(addr, cpu):
+    return ConstInt(heaptracker.adr2int(addr))
+
 class GuardClassOperation(test_random.GuardOperation):
     def gen_guard(self, builder, r):
         ptrvars = [(v, S) for (v, S) in builder.ptrvars
@@ -233,7 +234,7 @@ class GuardClassOperation(test_random.GuardOperation):
             v2, S2 = builder.get_structptr_var(r, must_have_vtable=True)
         vtable = S._hints['vtable']._as_ptr()
         vtable2 = S2._hints['vtable']._as_ptr()
-        c_vtable2 = ConstInt(ptr2int(vtable2))
+        c_vtable2 = ConstAddr(llmemory.cast_ptr_to_adr(vtable2), builder.cpu)
         op = ResOperation(self.opnum, [v, c_vtable2], None)
         return op, (vtable == vtable2)
 
@@ -247,7 +248,8 @@ class GuardNonNullClassOperation(GuardClassOperation):
             builder.loop.operations.append(op)
             v2, S2 = builder.get_structptr_var(r, must_have_vtable=True)
             vtable2 = S2._hints['vtable']._as_ptr()
-            c_vtable2 = ConstInt(ptr2int(vtable2))
+            c_vtable2 = ConstAddr(llmemory.cast_ptr_to_adr(vtable2),
+                                  builder.cpu)
             op = ResOperation(self.opnum, [op, c_vtable2], None)
             return op, False
 
@@ -309,7 +311,7 @@ class GetInteriorFieldOperation(test_random.AbstractOperation):
     def field_descr(self, builder, r):
         v, A = builder.get_structptr_var(r, type=lltype.Array,
                                          array_of_structs=True)
-        array = getref(lltype.Ptr(A), v)
+        array = v.getref(lltype.Ptr(A))
         v_index = builder.get_index(len(array), r)
         choice = []
         for name in A.OF._names:
@@ -342,7 +344,7 @@ class SetFieldOperation(GetFieldOperation):
                 w = ConstInt(r.random_integer())
             else:
                 w = r.choice(builder.intvars)
-            value = getint(w)
+            value = w.getint()
             if rffi.cast(lltype.Signed, rffi.cast(TYPE, value)) == value:
                 break
         builder.do(self.opnum, [v, w], descr)
@@ -355,7 +357,7 @@ class SetInteriorFieldOperation(GetInteriorFieldOperation):
                 w = ConstInt(r.random_integer())
             else:
                 w = r.choice(builder.intvars)
-            value = getint(w)
+            value = w.getint()
             if rffi.cast(lltype.Signed, rffi.cast(TYPE, value)) == value:
                 break
         builder.do(self.opnum, [v, v_index, w], descr)
@@ -387,7 +389,7 @@ class ArrayOperation(test_random.AbstractOperation):
 class GetArrayItemOperation(ArrayOperation):
     def field_descr(self, builder, r):
         v, A = builder.get_arrayptr_var(r)
-        array = getref(lltype.Ptr(A), v)
+        array = v.getref(lltype.Ptr(A))
         v_index = builder.get_index(len(array), r)
         descr = self.array_descr(builder, A)
         return v, A, v_index, descr
@@ -409,7 +411,7 @@ class SetArrayItemOperation(GetArrayItemOperation):
                 w = ConstInt(r.random_integer())
             else:
                 w = r.choice(builder.intvars)
-            value = getint(w)
+            value = w.getint()
             if rffi.cast(lltype.Signed, rffi.cast(A.OF, value)) == value:
                 break
         builder.do(self.opnum, [v, v_index, w], descr)
@@ -453,7 +455,7 @@ class NewSeqOperation(test_random.AbstractOperation):
         v_ptr = builder.do(self.opnum, [v_length])
         getattr(builder, self.builder_cache).append(v_ptr)
         # Initialize the string. Is there a better way to do this?
-        for i in range(getint(v_length)):
+        for i in range(v_length.getint()):
             v_index = ConstInt(i)
             v_char = ConstInt(r.random_integer() % self.max)
             builder.do(self.set_char, [v_ptr, v_index, v_char])
@@ -469,9 +471,9 @@ class AbstractStringOperation(test_random.AbstractOperation):
         current = getattr(builder, self.builder_cache)
         if current and r.random() < .8:
             v_string = r.choice(current)
-            string = getref(self.ptr, v_string)
+            string = v_string.getref(self.ptr)
         else:
-            string = self.alloc(getint(builder.get_index(500, r)))
+            string = self.alloc(builder.get_index(500, r).getint())
             v_string = ConstPtr(lltype.cast_opaque_ptr(llmemory.GCREF, string))
             current.append(v_string)
         for i in range(len(string.chars)):
@@ -482,7 +484,7 @@ class AbstractStringOperation(test_random.AbstractOperation):
 class AbstractGetItemOperation(AbstractStringOperation):
     def produce_into(self, builder, r):
         v_string = self.get_string(builder, r)
-        v_index = builder.get_index(len(getref(self.ptr, v_string).chars), r)
+        v_index = builder.get_index(len(v_string.getref(self.ptr).chars), r)
         builder.do(self.opnum, [v_string, v_index])
 
 class AbstractSetItemOperation(AbstractStringOperation):
@@ -490,7 +492,7 @@ class AbstractSetItemOperation(AbstractStringOperation):
         v_string = self.get_string(builder, r)
         if isinstance(v_string, ConstPtr):
             raise test_random.CannotProduceOperation  # setitem(Const, ...)
-        v_index = builder.get_index(len(getref(self.ptr, v_string).chars), r)
+        v_index = builder.get_index(len(v_string.getref(self.ptr).chars), r)
         v_target = ConstInt(r.random_integer() % self.max)
         builder.do(self.opnum, [v_string, v_index, v_target])
 
@@ -503,15 +505,15 @@ class AbstractCopyContentOperation(AbstractStringOperation):
     def produce_into(self, builder, r):
         v_srcstring = self.get_string(builder, r)
         v_dststring = self.get_string(builder, r)
-        src = getref(self.ptr, v_srcstring)
-        dst = getref(self.ptr, v_dststring)
+        src = v_srcstring.getref(self.ptr)
+        dst = v_dststring.getref(self.ptr)
         if src == dst:                                # because it's not a
             raise test_random.CannotProduceOperation  # memmove(), but memcpy()
         srclen = len(src.chars)
         dstlen = len(dst.chars)
         v_length = builder.get_index(min(srclen, dstlen), r)
-        v_srcstart = builder.get_index(srclen - getint(v_length) + 1, r)
-        v_dststart = builder.get_index(dstlen - getint(v_length) + 1, r)
+        v_srcstart = builder.get_index(srclen - v_length.getint() + 1, r)
+        v_dststart = builder.get_index(dstlen - v_length.getint() + 1, r)
         builder.do(self.opnum, [v_srcstring, v_dststring,
                                 v_srcstart, v_dststart, v_length])
 
@@ -568,7 +570,7 @@ class BaseCallOperation(test_random.AbstractOperation):
             sum = %s
             return %s
         """ % (funcargs, sum, result)).compile()
-        d = {'intmask': intmask}
+        d = {'intmask' : intmask}
         exec code in d
         return subset, d['f']
 
@@ -583,15 +585,15 @@ class BaseCallOperation(test_random.AbstractOperation):
         """ % funcargs).compile()
         vtableptr = v._hints['vtable']._as_ptr()
         d = {
-            'ptr': getref_base(S),
-            'vtable': vtableptr,
-            'LLException': LLException,
+            'ptr': S.getref_base(),
+            'vtable' : vtableptr,
+            'LLException' : LLException,
             }
         exec code in d
         return subset, d['f'], vtableptr
 
     def getresulttype(self):
-        if self.opnum == rop.CALL_I or self.opnum == rop.COND_CALL_VALUE_I:
+        if self.opnum == rop.CALL_I:
             return lltype.Signed
         elif self.opnum == rop.CALL_F:
             return lltype.Float
@@ -613,7 +615,7 @@ class CallOperation(BaseCallOperation):
         RES = self.getresulttype()
         TP = lltype.FuncType([lltype.Signed] * len(subset), RES)
         ptr = llhelper(lltype.Ptr(TP), f)
-        c_addr = ConstInt(ptr2int(ptr))
+        c_addr = ConstAddr(llmemory.cast_ptr_to_adr(ptr), builder.cpu)
         args = [c_addr] + subset
         descr = self.getcalldescr(builder, TP)
         self.put(builder, args, descr)
@@ -630,12 +632,12 @@ class CallOperationException(BaseCallOperation):
         RES = self.getresulttype()
         TP = lltype.FuncType([lltype.Signed] * len(subset), RES)
         ptr = llhelper(lltype.Ptr(TP), f)
-        c_addr = ConstInt(ptr2int(ptr))
+        c_addr = ConstAddr(llmemory.cast_ptr_to_adr(ptr), builder.cpu)
         args = [c_addr] + subset
         descr = self.getcalldescr(builder, TP)
         self.put(builder, args, descr)
         _, vtableptr = builder.get_random_structure_type_and_vtable(r)
-        exc_box = ConstInt(ptr2int(vtableptr))
+        exc_box = ConstAddr(llmemory.cast_ptr_to_adr(vtableptr), builder.cpu)
         op = ResOperation(rop.GUARD_EXCEPTION, [exc_box],
                           descr=builder.getfaildescr())
         op.setfailargs(builder.subset_of_intvars(r))
@@ -652,11 +654,11 @@ class RaisingCallOperation(BaseCallOperation):
         subset, f, exc = self.raising_func_code(builder, r)
         TP = lltype.FuncType([lltype.Signed] * len(subset), lltype.Void)
         ptr = llhelper(lltype.Ptr(TP), f)
-        c_addr = ConstInt(ptr2int(ptr))
+        c_addr = ConstAddr(llmemory.cast_ptr_to_adr(ptr), builder.cpu)
         args = [c_addr] + subset
         descr = self.getcalldescr(builder, TP)
         self.put(builder, args, descr)
-        exc_box = ConstInt(ptr2int(exc))
+        exc_box = ConstAddr(llmemory.cast_ptr_to_adr(exc), builder.cpu)
         op = ResOperation(rop.GUARD_EXCEPTION, [exc_box],
                           descr=builder.getfaildescr())
         op.setfailargs(fail_subset)
@@ -669,13 +671,13 @@ class RaisingCallOperationGuardNoException(BaseCallOperation):
         subset, f, exc = self.raising_func_code(builder, r)
         TP = lltype.FuncType([lltype.Signed] * len(subset), lltype.Void)
         ptr = llhelper(lltype.Ptr(TP), f)
-        c_addr = ConstInt(ptr2int(ptr))
+        c_addr = ConstAddr(llmemory.cast_ptr_to_adr(ptr), builder.cpu)
         args = [c_addr] + subset
         descr = self.getcalldescr(builder, TP)
         self.put(builder, args, descr)
         op = ResOperation(rop.GUARD_NO_EXCEPTION, [],
                           descr=builder.getfaildescr())
-        op._exc_box = ConstInt(ptr2int(exc))
+        op._exc_box = ConstAddr(llmemory.cast_ptr_to_adr(exc), builder.cpu)
         op.setfailargs(builder.subset_of_intvars(r))
         builder.should_fail_by = op
         builder.guard_op = op
@@ -688,7 +690,7 @@ class RaisingCallOperationWrongGuardException(BaseCallOperation):
         subset, f, exc = self.raising_func_code(builder, r)
         TP = lltype.FuncType([lltype.Signed] * len(subset), lltype.Void)
         ptr = llhelper(lltype.Ptr(TP), f)
-        c_addr = ConstInt(ptr2int(ptr))
+        c_addr = ConstAddr(llmemory.cast_ptr_to_adr(ptr), builder.cpu)
         args = [c_addr] + subset
         descr = self.getcalldescr(builder, TP)
         self.put(builder, args, descr)
@@ -696,10 +698,10 @@ class RaisingCallOperationWrongGuardException(BaseCallOperation):
             _, vtableptr = builder.get_random_structure_type_and_vtable(r)
             if vtableptr != exc:
                 break
-        other_box = ConstInt(ptr2int(vtableptr))
+        other_box = ConstAddr(llmemory.cast_ptr_to_adr(vtableptr), builder.cpu)
         op = ResOperation(rop.GUARD_EXCEPTION, [other_box],
                           descr=builder.getfaildescr())
-        op._exc_box = ConstInt(ptr2int(exc))
+        op._exc_box = ConstAddr(llmemory.cast_ptr_to_adr(exc), builder.cpu)
         op.setfailargs(builder.subset_of_intvars(r))
         builder.should_fail_by = op
         builder.guard_op = op
@@ -707,15 +709,9 @@ class RaisingCallOperationWrongGuardException(BaseCallOperation):
 
 # 6. a conditional call (for now always with no exception raised)
 class CondCallOperation(BaseCallOperation):
-
     def produce_into(self, builder, r):
         fail_subset = builder.subset_of_intvars(r)
-        if self.opnum == rop.COND_CALL:
-            RESULT_TYPE = lltype.Void
-            v_cond = builder.get_bool_var(r)
-        else:
-            RESULT_TYPE = lltype.Signed
-            v_cond = r.choice(builder.intvars)
+        v_cond = builder.get_bool_var(r)
         subset = builder.subset_of_intvars(r)[:4]
         for i in range(len(subset)):
             if r.random() < 0.35:
@@ -727,12 +723,10 @@ class CondCallOperation(BaseCallOperation):
                 seen.append(args)
             else:
                 assert seen[0] == args
-            if RESULT_TYPE is lltype.Signed:
-                return len(args) - 42000
         #
-        TP = lltype.FuncType([lltype.Signed] * len(subset), RESULT_TYPE)
+        TP = lltype.FuncType([lltype.Signed] * len(subset), lltype.Void)
         ptr = llhelper(lltype.Ptr(TP), call_me)
-        c_addr = ConstInt(ptr2int(ptr))
+        c_addr = ConstAddr(llmemory.cast_ptr_to_adr(ptr), builder.cpu)
         args = [v_cond, c_addr] + subset
         descr = self.getcalldescr(builder, TP)
         self.put(builder, args, descr)
@@ -774,7 +768,6 @@ for i in range(4):      # make more common
 for i in range(2):
     OPERATIONS.append(GuardClassOperation(rop.GUARD_CLASS))
     OPERATIONS.append(CondCallOperation(rop.COND_CALL))
-    OPERATIONS.append(CondCallOperation(rop.COND_CALL_VALUE_I))
     OPERATIONS.append(RaisingCallOperation(rop.CALL_N))
     OPERATIONS.append(RaisingCallOperationGuardNoException(rop.CALL_N))
     OPERATIONS.append(RaisingCallOperationWrongGuardException(rop.CALL_N))
