@@ -11,9 +11,11 @@ from rpython.jit.metainterp.resoperation import InputArgFloat
 from rpython.jit.metainterp.executor import _execute_arglist, wrap_constant
 from rpython.jit.metainterp.resoperation import opname
 from rpython.jit.codewriter import longlong
-from rpython.rtyper.lltypesystem import lltype, llmemory, rstr
+from rpython.rtyper.lltypesystem import lltype, rstr
 from rpython.rtyper import rclass
 
+class PleaseRewriteMe(Exception):
+    pass
 
 class DummyLoop(object):
     def __init__(self, subops):
@@ -25,44 +27,7 @@ class FakeMetaInterp(object):
     def execute_raised(self, exc, constant=False):
         self._got_exc = exc
 
-
-def getint(v):
-    if isinstance(v, (ConstInt, InputArgInt)):
-        return v.getint()
-    else:
-        return v._example_int
-
-def getfloatstorage(v):
-    if isinstance(v, (ConstFloat, InputArgFloat)):
-        return v.getfloatstorage()
-    else:
-        return v._example_float
-
-def getfloat(v):
-    return longlong.getrealfloat(getfloatstorage(v))
-
-def getref_base(v):
-    if isinstance(v, (ConstPtr, InputArgRef)):
-        return v.getref_base()
-    else:
-        return v._example_ref
-
-def getref(PTR, v):
-    return lltype.cast_opaque_ptr(PTR, getref_base(v))
-
-def constbox(v):
-    if v.type == INT:
-        return ConstInt(getint(v))
-    if v.type == FLOAT:
-        return ConstFloat(getfloatstorage(v))
-    if v.type == REF:
-        return ConstPtr(getref_base(v))
-    assert 0, v.type
-
-
 class OperationBuilder(object):
-    produce_failing_guards = True
-
     def __init__(self, cpu, loop, vars):
         self.cpu = cpu
         if not hasattr(cpu, '_faildescr_keepalive'):
@@ -92,21 +57,11 @@ class OperationBuilder(object):
     def do(self, opnum, argboxes, descr=None):
         self.fakemetainterp._got_exc = None
         op = ResOperation(opnum, argboxes, descr)
-        argboxes = map(constbox, argboxes)
         result = _execute_arglist(self.cpu, self.fakemetainterp,
                                   opnum, argboxes, descr)
         if result is not None:
-            if lltype.typeOf(result) == lltype.Signed:
-                op._example_int = result
-            elif isinstance(result, bool):
-                op._example_int = int(result)
-            elif lltype.typeOf(result) == longlong.FLOATSTORAGE:
-                op._example_float = result
-            elif isinstance(result, float):
-                op._example_float = longlong.getfloatstorage(result)
-            else:
-                assert lltype.typeOf(result) == llmemory.GCREF
-                op._example_ref = result
+            c_result = wrap_constant(result)
+            op.copy_value_from(c_result)
         self.loop.operations.append(op)
         return op
 
@@ -146,7 +101,7 @@ class OperationBuilder(object):
             if v in names:
                 args.append(names[v])
             elif isinstance(v, ConstPtr):
-                assert not getref_base(v) # otherwise should be in the names
+                assert not v.getref_base() # otherwise should be in the names
                 args.append('ConstPtr(lltype.nullptr(llmemory.GCREF.TO))')
             elif isinstance(v, ConstFloat):
                 args.append('ConstFloat(longlong.getfloatstorage(%r))'
@@ -243,10 +198,10 @@ class OperationBuilder(object):
         #
         def writevar(v, nameprefix, init=''):
             if nameprefix == 'const_ptr':
-                if not getref_base(v):
+                if not v.getref_base():
                     return 'lltype.nullptr(llmemory.GCREF.TO)'
-                TYPE = getref_base(v)._obj.ORIGTYPE
-                cont = lltype.cast_opaque_ptr(TYPE, getref_base(v))
+                TYPE = v.getref_base()._obj.ORIGTYPE
+                cont = lltype.cast_opaque_ptr(TYPE, v.getref_base())
                 if TYPE.TO._is_varsize():
                     if isinstance(TYPE.TO, lltype.GcStruct):
                         lgt = len(cont.chars)
@@ -297,9 +252,9 @@ class OperationBuilder(object):
             for i, v in enumerate(self.loop.inputargs):
                 assert not isinstance(v, Const)
                 if v.type == FLOAT:
-                    vals.append("longlong.getfloatstorage(%r)" % getfloat(v))
+                    vals.append("longlong.getfloatstorage(%r)" % v.getfloat())
                 else:
-                    vals.append("%r" % getint(v))
+                    vals.append("%r" % v.getint())
             print >>s, '    loop_args = [%s]' % ", ".join(vals)
         print >>s, '    frame = cpu.execute_token(looptoken, *loop_args)'
         if self.should_fail_by is None:
@@ -309,10 +264,10 @@ class OperationBuilder(object):
         for i, v in enumerate(fail_args):
             if v.type == FLOAT:
                 print >>s, ('    assert longlong.getrealfloat('
-                    'cpu.get_float_value(frame, %d)) == %r' % (i, getfloatstorage(v)))
+                    'cpu.get_float_value(frame, %d)) == %r' % (i, v.getfloatstorage()))
             else:
                 print >>s, ('    assert cpu.get_int_value(frame, %d) == %d'
-                            % (i, getint(v)))
+                            % (i, v.getint()))
         self.names = names
         s.flush()
 
@@ -340,7 +295,7 @@ class AbstractOperation(object):
                 builder.intvars.append(v_result)
                 boolres = self.boolres
                 if boolres == 'sometimes':
-                    boolres = getint(v_result) in [0, 1]
+                    boolres = v_result.getint() in [0, 1]
                 if boolres:
                     builder.boolvars.append(v_result)
             elif v_result.type == FLOAT:
@@ -391,10 +346,10 @@ class BinaryOperation(AbstractOperation):
             v_second = ConstInt((value & self.and_mask) | self.or_mask)
         else:
             v = r.choice(builder.intvars)
-            v_value = getint(v)
+            v_value = v.getint()
             if (v_value & self.and_mask) != v_value:
                 v = builder.do(rop.INT_AND, [v, ConstInt(self.and_mask)])
-            v_value = getint(v)
+            v_value = v.getint()
             if (v_value | self.or_mask) != v_value:
                 v = builder.do(rop.INT_OR, [v, ConstInt(self.or_mask)])
             v_second = v
@@ -440,9 +395,9 @@ class BinaryFloatOperation(AbstractFloatOperation):
             v_second = ConstFloat(r.random_float_storage())
         else:
             v_second = r.choice(builder.floatvars)
-        if abs(getfloat(v_first)) > 1E100 or abs(getfloat(v_second)) > 1E100:
+        if abs(v_first.getfloat()) > 1E100 or abs(v_second.getfloat()) > 1E100:
             raise CannotProduceOperation     # avoid infinities
-        if abs(getfloat(v_second)) < 1E-100:
+        if abs(v_second.getfloat()) < 1E-100:
             raise CannotProduceOperation     # e.g. division by zero error
         self.put(builder, [v_first, v_second])
 
@@ -477,23 +432,16 @@ class CastFloatToIntOperation(AbstractFloatOperation):
         if not builder.floatvars:
             raise CannotProduceOperation
         box = r.choice(builder.floatvars)
-        if not (-sys.maxint-1 <= getfloat(box) <= sys.maxint):
+        if not (-sys.maxint-1 <= box.getfloat() <= sys.maxint):
             raise CannotProduceOperation      # would give an overflow
         self.put(builder, [box])
 
 class GuardOperation(AbstractOperation):
     def gen_guard(self, builder, r):
         v = builder.get_bool_var(r)
-        if not builder.produce_failing_guards:
-            if getint(v):
-                opnum = rop.GUARD_TRUE
-            else:
-                opnum = rop.GUARD_FALSE
-        else:
-            opnum = self.opnum
-        op = ResOperation(opnum, [v])
-        passing = ((opnum == rop.GUARD_TRUE and getint(v)) or
-                   (opnum == rop.GUARD_FALSE and not getint(v)))
+        op = ResOperation(self.opnum, [v])
+        passing = ((self.opnum == rop.GUARD_TRUE and v.getint()) or
+                   (self.opnum == rop.GUARD_FALSE and not v.getint()))
         return op, passing
 
     def produce_into(self, builder, r):
@@ -502,7 +450,6 @@ class GuardOperation(AbstractOperation):
         op.setdescr(builder.getfaildescr())
         op.setfailargs(builder.subset_of_intvars(r))
         if not passing:
-            assert builder.produce_failing_guards
             builder.should_fail_by = op
             builder.guard_op = op
 
@@ -512,27 +459,25 @@ class GuardPtrOperation(GuardOperation):
             raise CannotProduceOperation
         box = r.choice(builder.ptrvars)[0]
         op = ResOperation(self.opnum, [box])
-        passing = ((self.opnum == rop.GUARD_NONNULL and getref_base(box)) or
-                   (self.opnum == rop.GUARD_ISNULL and not getref_base(box)))
+        passing = ((self.opnum == rop.GUARD_NONNULL and box.getref_base()) or
+                   (self.opnum == rop.GUARD_ISNULL and not box.getref_base()))
         return op, passing
 
 class GuardValueOperation(GuardOperation):
     def gen_guard(self, builder, r):
         v = r.choice(builder.intvars)
-        if not builder.produce_failing_guards:
-            other = ConstInt(getint(v))
-        elif r.random() > 0.8:
-            other = ConstInt(getint(r.choice(builder.intvars)))
+        if r.random() > 0.8:
+            other = r.choice(builder.intvars)
         else:
             if r.random() < 0.75:
-                value = getint(v)
+                value = v.getint()
             elif r.random() < 0.5:
-                value = getint(v) ^ 1
+                value = v.getint() ^ 1
             else:
                 value = r.random_integer()
             other = ConstInt(value)
         op = ResOperation(self.opnum, [v, other])
-        return op, (getint(v) == getint(other))
+        return op, (v.getint() == other.getint())
 
 # ____________________________________________________________
 
@@ -544,7 +489,6 @@ for _op in [rop.INT_ADD,
             rop.INT_AND,
             rop.INT_OR,
             rop.INT_XOR,
-            rop.UINT_MUL_HIGH,
             ]:
     OPERATIONS.append(BinaryOperation(_op))
 
@@ -561,8 +505,8 @@ for _op in [rop.INT_LT,
             ]:
     OPERATIONS.append(BinaryOperation(_op, boolres=True))
 
-#OPERATIONS.append(BinaryOperation(rop.INT_FLOORDIV, ~3, 2))
-#OPERATIONS.append(BinaryOperation(rop.INT_MOD, ~3, 2))
+OPERATIONS.append(BinaryOperation(rop.INT_FLOORDIV, ~3, 2))
+OPERATIONS.append(BinaryOperation(rop.INT_MOD, ~3, 2))
 OPERATIONS.append(BinaryOperation(rop.INT_RSHIFT, LONG_BIT-1))
 OPERATIONS.append(BinaryOperation(rop.INT_LSHIFT, LONG_BIT-1))
 OPERATIONS.append(BinaryOperation(rop.UINT_RSHIFT, LONG_BIT-1))
@@ -627,10 +571,9 @@ def Random():
     import random
     seed = pytest.config.option.randomseed
     print
-    print 'Random start seed value is %d.' % (seed,)
+    print 'Random seed value is %d.' % (seed,)
     print
-    r = random.Random()
-    r.seed(seed)
+    r = random.Random(seed)
     def get_random_integer():
         while True:
             result = int(r.expovariate(0.05))
@@ -682,19 +625,17 @@ class RandomLoop(object):
         self.output = output
         if startvars is None:
             startvars = []
-            # vary the number of startvars, between 1 and n_vars
-            n_vars = r.randrange(pytest.config.option.n_vars) + 1
             if cpu.supports_floats:
                 # pick up a single threshold for the whole 'inputargs', so
                 # that some loops have no or mostly no FLOATs while others
                 # have a lot of them
                 k = r.random()
                 # but make sure there is at least one INT
-                at_least_once = r.randrange(n_vars)
+                at_least_once = r.randrange(0, pytest.config.option.n_vars)
             else:
                 k = -1
                 at_least_once = 0
-            for i in range(n_vars):
+            for i in range(pytest.config.option.n_vars):
                 if r.random() < k and i != at_least_once:
                     startvars.append(InputArgFloat(r.random_float_storage()))
                 else:
@@ -734,7 +675,7 @@ class RandomLoop(object):
         assert not hasattr(loop, '_targettoken')
         for i in range(position):
             op = loop.operations[i]
-            if (not rop.has_no_side_effect(op.opnum)
+            if (not op.has_no_side_effect()
                     or op.type not in (INT, FLOAT)):
                 position = i
                 break       # cannot move the LABEL later
@@ -745,8 +686,7 @@ class RandomLoop(object):
                                                       loop._targettoken))
 
     def generate_ops(self, builder, r, loop, startvars, needs_a_label=False):
-        block_length = pytest.config.option.block_length // 2
-        block_length += int(r.expovariate(1.0 / block_length))
+        block_length = pytest.config.option.block_length
         istart = 0
 
         for i in range(block_length):
@@ -788,9 +728,9 @@ class RandomLoop(object):
         self.expected = {}
         for v in endvars:
             if v.type == INT:
-                self.expected[v] = getint(v)
+                self.expected[v] = v.getint()
             elif v.type == FLOAT:
-                self.expected[v] = getfloatstorage(v)
+                self.expected[v] = v.getfloatstorage()
             else:
                 assert 0, v.type
 
@@ -802,7 +742,7 @@ class RandomLoop(object):
             args = []
             for box in self.startvars:
                 if box not in self.loop.inputargs:
-                    box = constbox(box)
+                    box = box.constbox()
                 args.append(box)
             self.cpu.compile_loop(self.loop.inputargs,
                                   [ResOperation(rop.JUMP, args,
@@ -820,7 +760,7 @@ class RandomLoop(object):
 
     def clear_state(self):
         for v, S, fields in self.prebuilt_ptr_consts:
-            container = getref_base(v)._obj.container
+            container = v.getref_base()._obj.container
             for name, value in fields.items():
                 if isinstance(name, str):
                     setattr(container, name, value)
@@ -841,9 +781,9 @@ class RandomLoop(object):
         arguments = []
         for box in self.loop.inputargs:
             if box.type == INT:
-                arguments.append(getint(box))
+                arguments.append(box.getint())
             elif box.type == FLOAT:
-                arguments.append(getfloatstorage(box))
+                arguments.append(box.getfloatstorage())
             else:
                 assert 0, box.type
         deadframe = cpu.execute_token(self.runjitcelltoken(), *arguments)
@@ -855,7 +795,7 @@ class RandomLoop(object):
             if v not in self.expected:
                 assert v.getopnum() == rop.SAME_AS_I   # special case
                 assert isinstance(v.getarg(0), ConstInt)
-                self.expected[v] = getint(v.getarg(0))
+                self.expected[v] = v.getarg(0).getint()
             if v.type == FLOAT:
                 value = cpu.get_float_value(deadframe, i)
             else:
@@ -867,7 +807,7 @@ class RandomLoop(object):
                 )
         exc = cpu.grab_exc_value(deadframe)
         if (self.guard_op is not None and
-            rop.is_guard_exception(self.guard_op.getopnum())):
+            self.guard_op.is_guard_exception()):
             if self.guard_op.getopnum() == rop.GUARD_NO_EXCEPTION:
                 do_assert(exc,
                           "grab_exc_value() should not be %r" % (exc,))
@@ -900,7 +840,7 @@ class RandomLoop(object):
         # generate the branch: a sequence of operations that ends in a FINISH
         subloop = DummyLoop([])
         self.subloops.append(subloop)   # keep around for debugging
-        if rop.is_guard_exception(guard_op.getopnum()):
+        if guard_op.is_guard_exception():
             subloop.operations.append(exc_handling(guard_op))
         bridge_builder = self.builder.fork(self.builder.cpu, subloop,
                                            op.getfailargs()[:])
@@ -936,9 +876,9 @@ class RandomLoop(object):
             args = []
             for x in subset:
                 if x.type == INT:
-                    args.append(InputArgInt(getint(x)))
+                    args.append(InputArgInt(x.getint()))
                 elif x.type == FLOAT:
-                    args.append(InputArgFloat(getfloatstorage(x)))
+                    args.append(InputArgFloat(x.getfloatstorage()))
                 else:
                     assert 0, x.type
             rl = RandomLoop(self.builder.cpu, self.builder.fork,
