@@ -7,29 +7,26 @@ from rpython.jit.backend.x86.regalloc import gpr_reg_mgr_cls, xmm_reg_mgr_cls
 from rpython.jit.backend.x86.profagent import ProfileAgent
 from rpython.jit.backend.llsupport.llmodel import AbstractLLCPU
 from rpython.jit.backend.x86 import regloc
-from rpython.jit.backend.x86.vector_ext import X86VectorExt
-from rpython.jit.backend.x86.arch import WIN64
 
 import sys
 
+from rpython.tool.ansi_print import ansi_log
+log = py.log.Producer('jitbackend')
+py.log.setconsumer('jitbackend', ansi_log)
+
 
 class AbstractX86CPU(AbstractLLCPU):
     debug = True
     supports_floats = True
     supports_singlefloats = True
-    supports_load_effective_address = True
 
     dont_keepalive_stuff = False # for tests
     with_threads = False
     frame_reg = regloc.ebp
 
-    vector_ext = None
-
     # can an ISA instruction handle a factor to the offset?
     load_supported_factors = (1,2,4,8)
 
-    HAS_CODEMAP = True
-
     from rpython.jit.backend.x86.arch import JITFRAME_FIXED_SIZE
     all_reg_indexes = gpr_reg_mgr_cls.all_reg_indexes
     gen_regs = gpr_reg_mgr_cls.all_regs
@@ -46,7 +43,7 @@ class AbstractX86CPU(AbstractLLCPU):
             if config.translation.jit_profiler == "oprofile":
                 from rpython.jit.backend.x86 import oprofile
                 if not oprofile.OPROFILE_AVAILABLE:
-                    raise Exception('oprofile support was explicitly enabled, but oprofile headers seem not to be available')
+                    log.WARNING('oprofile support was explicitly enabled, but oprofile headers seem not to be available')
                 profile_agent = oprofile.OProfileAgent()
             self.with_threads = config.translation.thread
 
@@ -121,10 +118,9 @@ class AbstractX86CPU(AbstractLLCPU):
         looptoken.compiled_loop_token.invalidate_positions = []
 
     def get_all_loop_runs(self):
-        asm = self.assembler
         l = lltype.malloc(LOOP_RUN_CONTAINER,
-                          len(asm.loop_run_counters))
-        for i, ll_s in enumerate(asm.loop_run_counters):
+                          len(self.assembler.loop_run_counters))
+        for i, ll_s in enumerate(self.assembler.loop_run_counters):
             l[i].type = ll_s.type
             l[i].number = ll_s.number
             l[i].counter = ll_s.i
@@ -149,18 +145,16 @@ class CPU386_NO_SSE2(CPU386):
     supports_longlong = False
 
 class CPU_X86_64(AbstractX86CPU):
-    if not WIN64:
-        vector_ext = X86VectorExt()
     backend_name = 'x86_64'
     NUM_REGS = 16
-    if not WIN64:
-        CALLEE_SAVE_REGISTERS = [regloc.ebx, regloc.r12, regloc.r13, regloc.r14,
-                                 regloc.r15]
-    else:
-        CALLEE_SAVE_REGISTERS = [regloc.ebx, regloc.esi, regloc.edi, regloc.r12,
-                                 regloc.r14, regloc.r15]
-        HAS_CODEMAP = False
+    CALLEE_SAVE_REGISTERS = [regloc.ebx, regloc.r12, regloc.r13, regloc.r14, regloc.r15]
 
     IS_64_BIT = True
+    HAS_CODEMAP = True
+
+class CPU_X86_64_SSE4(CPU_X86_64):
+    vector_extension = True
+    vector_register_size = 16
+    vector_horizontal_operations = True
 
 CPU = CPU386
