@@ -64,7 +64,6 @@ class Transformer(object):
         self.cpu = cpu
         self.callcontrol = callcontrol
         self.portal_jd = portal_jd   # non-None only for the portal graph(s)
-        self.graph = None
 
     def transform(self, graph):
         self.graph = graph
@@ -201,12 +200,8 @@ class Transformer(object):
             or v.concretetype != lltype.Bool):
             return False
         for op in block.operations[::-1]:
-            # check if variable is used in block
-            for arg in op.args:
-                if arg == v:
-                    return False
-                if isinstance(arg, ListOfKind) and v in arg.content:
-                    return False
+            if v in op.args:
+                return False   # variable is also used in cur block
             if v is op.result:
                 if op.opname not in ('int_lt', 'int_le', 'int_eq', 'int_ne',
                                      'int_gt', 'int_ge',
@@ -272,9 +267,6 @@ class Transformer(object):
     def rewrite_op_unlikely(self, op):
         return None   # "no real effect"
 
-    def rewrite_op_revdb_do_next_call(self, op):
-        return []    # ignored, only for revdb
-
     def rewrite_op_raw_malloc_usage(self, op):
         if self.cpu.translate_support_code or isinstance(op.args[0], Variable):
             return   # the operation disappears
@@ -287,12 +279,6 @@ class Transformer(object):
     def rewrite_op_jit_record_exact_class(self, op):
         return SpaceOperation("record_exact_class", [op.args[0], op.args[1]], None)
 
-    def rewrite_op_debug_assert_not_none(self, op):
-        if isinstance(op.args[0], Variable):
-            return SpaceOperation('assert_not_none', [op.args[0]], None)
-        else:
-            return []
-
     def rewrite_op_cast_bool_to_int(self, op): pass
     def rewrite_op_cast_bool_to_uint(self, op): pass
     def rewrite_op_cast_char_to_int(self, op): pass
@@ -378,7 +364,7 @@ class Transformer(object):
         return getattr(self, 'handle_%s_indirect_call' % kind)(op)
 
     def rewrite_call(self, op, namebase, initialargs, args=None,
-                     calldescr=None, force_ir=False):
+                     calldescr=None):
         """Turn 'i0 = direct_call(fn, i1, i2, ref1, ref2)'
            into 'i0 = xxx_call_ir_i(fn, descr, [i1,i2], [ref1,ref2])'.
            The name is one of '{residual,direct}_call_{r,ir,irf}_{i,r,f,v}'."""
@@ -388,9 +374,8 @@ class Transformer(object):
         lst_i, lst_r, lst_f = self.make_three_lists(args)
         reskind = getkind(op.result.concretetype)[0]
         if lst_f or reskind == 'f': kinds = 'irf'
-        elif lst_i or force_ir: kinds = 'ir'
+        elif lst_i: kinds = 'ir'
         else: kinds = 'r'
-        if force_ir: assert kinds == 'ir'    # no 'f'
         sublists = []
         if 'i' in kinds: sublists.append(lst_i)
         if 'r' in kinds: sublists.append(lst_r)
@@ -428,8 +413,7 @@ class Transformer(object):
         of 'residual_call_xxx' are the function to call, and its calldescr."""
         calldescr = self.callcontrol.getcalldescr(op, oopspecindex=oopspecindex,
                                                   extraeffect=extraeffect,
-                                                  extradescr=extradescr,
-                                                  calling_graph=self.graph)
+                                                  extradescr=extradescr)
         op1 = self.rewrite_call(op, 'residual_call',
                                 [op.args[0]] + extraargs, calldescr=calldescr)
         if may_call_jitcodes or self.callcontrol.calldescr_canraise(calldescr):
@@ -452,8 +436,6 @@ class Transformer(object):
         # dispatch to various implementations depending on the oopspec_name
         if oopspec_name.startswith('list.') or oopspec_name.startswith('newlist'):
             prepare = self._handle_list_call
-        elif oopspec_name.startswith('int.'):
-            prepare = self._handle_int_special
         elif oopspec_name.startswith('stroruni.'):
             prepare = self._handle_stroruni_call
         elif oopspec_name == 'str.str2unicode':
@@ -468,8 +450,6 @@ class Transformer(object):
             prepare = self._handle_math_sqrt_call
         elif oopspec_name.startswith('rgc.'):
             prepare = self._handle_rgc_call
-        elif oopspec_name.startswith('rvmprof.'):
-            prepare = self._handle_rvmprof_call
         elif oopspec_name.endswith('dict.lookup'):
             # also ordereddict.lookup
             prepare = self._handle_dict_lookup_call
@@ -538,18 +518,28 @@ class Transformer(object):
 
     # XXX some of the following functions should not become residual calls
     # but be really compiled
+    rewrite_op_int_floordiv_ovf_zer   = _do_builtin_call
+    rewrite_op_int_floordiv_ovf       = _do_builtin_call
+    rewrite_op_int_floordiv_zer       = _do_builtin_call
+    rewrite_op_int_mod_ovf_zer        = _do_builtin_call
+    rewrite_op_int_mod_ovf            = _do_builtin_call
+    rewrite_op_int_mod_zer            = _do_builtin_call
+    rewrite_op_int_lshift_ovf         = _do_builtin_call
     rewrite_op_int_abs                = _do_builtin_call
-    rewrite_op_int_floordiv           = _do_builtin_call
-    rewrite_op_int_mod                = _do_builtin_call
     rewrite_op_llong_abs              = _do_builtin_call
     rewrite_op_llong_floordiv         = _do_builtin_call
+    rewrite_op_llong_floordiv_zer     = _do_builtin_call
     rewrite_op_llong_mod              = _do_builtin_call
+    rewrite_op_llong_mod_zer          = _do_builtin_call
     rewrite_op_ullong_floordiv        = _do_builtin_call
+    rewrite_op_ullong_floordiv_zer    = _do_builtin_call
     rewrite_op_ullong_mod             = _do_builtin_call
+    rewrite_op_ullong_mod_zer         = _do_builtin_call
     rewrite_op_gc_identityhash        = _do_builtin_call
     rewrite_op_gc_id                  = _do_builtin_call
     rewrite_op_gc_pin                 = _do_builtin_call
     rewrite_op_gc_unpin               = _do_builtin_call
+    rewrite_op_uint_mod               = _do_builtin_call
     rewrite_op_cast_float_to_uint     = _do_builtin_call
     rewrite_op_cast_uint_to_float     = _do_builtin_call
     rewrite_op_weakref_create         = _do_builtin_call
@@ -604,8 +594,6 @@ class Transformer(object):
         log.WARNING('ignoring hint %r at %r' % (hints, self.graph))
 
     def _rewrite_raw_malloc(self, op, name, args):
-        # NB. the operation 'raw_malloc' is not supported; this is for
-        # the operation 'malloc'/'malloc_varsize' with {flavor: 'gc'}
         d = op.args[1].value.copy()
         d.pop('flavor')
         add_memory_pressure = d.pop('add_memory_pressure', False)
@@ -700,10 +688,6 @@ class Transformer(object):
         ARRAY = op.args[0].concretetype.TO
         if self._array_of_voids(ARRAY):
             return []
-        if isinstance(ARRAY, lltype.FixedSizeArray):
-            raise NotImplementedError(
-                "%r uses %r, which is not supported by the JIT codewriter"
-                % (self.graph, ARRAY))
         if op.args[0] in self.vable_array_vars:     # for virtualizables
             vars = self.vable_array_vars[op.args[0]]
             (v_base, arrayfielddescr, arraydescr) = vars
@@ -734,10 +718,6 @@ class Transformer(object):
         ARRAY = op.args[0].concretetype.TO
         if self._array_of_voids(ARRAY):
             return []
-        if isinstance(ARRAY, lltype.FixedSizeArray):
-            raise NotImplementedError(
-                "%r uses %r, which is not supported by the JIT codewriter"
-                % (self.graph, ARRAY))
         if op.args[0] in self.vable_array_vars:     # for virtualizables
             vars = self.vable_array_vars[op.args[0]]
             (v_base, arrayfielddescr, arraydescr) = vars
@@ -792,7 +772,7 @@ class Transformer(object):
                 return [SpaceOperation('-live-', [], None),
                         SpaceOperation('getfield_vable_%s' % kind,
                                        [v_inst, descr], op.result)]
-        except VirtualizableArrayField as e:
+        except VirtualizableArrayField, e:
             # xxx hack hack hack
             vinfo = e.args[1]
             arrayindex = vinfo.array_field_counter[op.args[1].value]
@@ -802,20 +782,12 @@ class Transformer(object):
                                                 arrayfielddescr,
                                                 arraydescr)
             return []
-        # check for the string or unicode hash field
-        STRUCT = v_inst.concretetype.TO
-        if STRUCT == rstr.STR:
-            assert c_fieldname.value == 'hash'
-            return SpaceOperation('strhash', [v_inst], op.result)
-        elif STRUCT == rstr.UNICODE:
-            assert c_fieldname.value == 'hash'
-            return SpaceOperation('unicodehash', [v_inst], op.result)
         # check for _immutable_fields_ hints
-        immut = STRUCT._immutable_field(c_fieldname.value)
+        immut = v_inst.concretetype.TO._immutable_field(c_fieldname.value)
         need_live = False
         if immut:
             if (self.callcontrol is not None and
-                self.callcontrol.could_be_green_field(STRUCT,
+                self.callcontrol.could_be_green_field(v_inst.concretetype.TO,
                                                       c_fieldname.value)):
                 pure = '_greenfield'
                 need_live = True
@@ -823,9 +795,10 @@ class Transformer(object):
                 pure = '_pure'
         else:
             pure = ''
-        self.check_field_access(STRUCT)
-        argname = getattr(STRUCT, '_gckind', 'gc')
-        descr = self.cpu.fielddescrof(STRUCT, c_fieldname.value)
+        self.check_field_access(v_inst.concretetype.TO)
+        argname = getattr(v_inst.concretetype.TO, '_gckind', 'gc')
+        descr = self.cpu.fielddescrof(v_inst.concretetype.TO,
+                                      c_fieldname.value)
         kind = getkind(RESULT)[0]
         if argname != 'gc':
             assert argname == 'raw'
@@ -842,7 +815,7 @@ class Transformer(object):
         if immut in (IR_QUASIIMMUTABLE, IR_QUASIIMMUTABLE_ARRAY):
             op1.opname += "_pure"
             descr1 = self.cpu.fielddescrof(
-                STRUCT,
+                v_inst.concretetype.TO,
                 quasiimmut.get_mutate_field_name(c_fieldname.value))
             return [SpaceOperation('-live-', [], None),
                    SpaceOperation('record_quasiimmut_field',
@@ -1136,26 +1109,6 @@ class Transformer(object):
                               [op.args[0], op.args[1],
                                op.args[2], op.args[3], c_bytes], op.result)
 
-    def rewrite_op_gc_store_indexed(self, op):
-        T = op.args[2].concretetype
-        kind = getkind(T)[0]
-        assert kind != 'r'
-        descr = self.cpu.arraydescrof(rffi.CArray(T))
-        if (not isinstance(op.args[3], Constant) or
-            not isinstance(op.args[4], Constant)):
-            raise NotImplementedError("gc_store_indexed: 'scale' and 'base_ofs'"
-                                      " should be constants")
-        # According to the comment in resoperation.py, "itemsize is not signed
-        # (always > 0)", so we don't need the "bytes = -bytes" line which is
-        # in rewrite_op_gc_load_indexed
-        bytes = descr.get_item_size_in_bytes()
-        c_bytes = Constant(bytes, lltype.Signed)
-        return SpaceOperation('gc_store_indexed_%s' % kind,
-                              [op.args[0], op.args[1], op.args[2],
-                               op.args[3], op.args[4], c_bytes, descr], None)
-
-
-
     def _rewrite_equality(self, op, opname):
         arg0, arg1 = op.args
         if isinstance(arg0, Constant) and not arg0.value:
@@ -1571,6 +1524,12 @@ class Transformer(object):
                 return self.rewrite_operation(op1)
         ''' % (_old, _new)).compile()
 
+    def rewrite_op_int_neg_ovf(self, op):
+        op1 = SpaceOperation('int_sub_ovf',
+                             [Constant(0, lltype.Signed), op.args[0]],
+                             op.result)
+        return self.rewrite_operation(op1)
+
     def rewrite_op_float_is_true(self, op):
         op1 = SpaceOperation('float_ne',
                              [op.args[0], Constant(0.0, lltype.Float)],
@@ -1609,7 +1568,7 @@ class Transformer(object):
             return []
         return getattr(self, 'handle_jit_marker__%s' % key)(op, jitdriver)
 
-    def _rewrite_op_cond_call(self, op, rewritten_opname):
+    def rewrite_op_jit_conditional_call(self, op):
         have_floats = False
         for arg in op.args:
             if getkind(arg.concretetype) == 'float':
@@ -1618,22 +1577,15 @@ class Transformer(object):
         if len(op.args) > 4 + 2 or have_floats:
             raise Exception("Conditional call does not support floats or more than 4 arguments")
         callop = SpaceOperation('direct_call', op.args[1:], op.result)
-        calldescr = self.callcontrol.getcalldescr(
-                callop,
-                calling_graph=self.graph)
+        calldescr = self.callcontrol.getcalldescr(callop)
         assert not calldescr.get_extra_info().check_forces_virtual_or_virtualizable()
-        op1 = self.rewrite_call(op, rewritten_opname,
+        op1 = self.rewrite_call(op, 'conditional_call',
                                 op.args[:2], args=op.args[2:],
-                                calldescr=calldescr, force_ir=True)
+                                calldescr=calldescr)
         if self.callcontrol.calldescr_canraise(calldescr):
             op1 = [op1, SpaceOperation('-live-', [], None)]
         return op1
 
-    def rewrite_op_jit_conditional_call(self, op):
-        return self._rewrite_op_cond_call(op, 'conditional_call')
-    def rewrite_op_jit_conditional_call_value(self, op):
-        return self._rewrite_op_cond_call(op, 'conditional_call_value')
-
     def handle_jit_marker__jit_merge_point(self, op, jitdriver):
         assert self.portal_jd is not None, (
             "'jit_merge_point' in non-portal graph!")
@@ -1931,8 +1883,7 @@ class Transformer(object):
                              extradescr=None):
         calldescr = self.callcontrol.getcalldescr(op, oopspecindex,
                                                   extraeffect,
-                                                  extradescr=extradescr,
-                                                  calling_graph=self.graph)
+                                                  extradescr=extradescr)
         if extraeffect is not None:
             assert (is_test_calldescr(calldescr)      # for tests
                     or calldescr.get_extra_info().extraeffect == extraeffect)
@@ -1962,8 +1913,7 @@ class Transformer(object):
                             [c_func] + [varoftype(T) for T in argtypes],
                             varoftype(resulttype))
         calldescr = self.callcontrol.getcalldescr(op, oopspecindex,
-                                                  effectinfo,
-                                                  calling_graph=self.graph)
+                                                  effectinfo)
         if isinstance(c_func.value, str):    # in tests only
             func = c_func.value
         else:
@@ -1971,20 +1921,6 @@ class Transformer(object):
                 llmemory.cast_ptr_to_adr(c_func.value))
         self.callcontrol.callinfocollection.add(oopspecindex, calldescr, func)
 
-    def _handle_int_special(self, op, oopspec_name, args):
-        if oopspec_name == 'int.neg_ovf':
-            [v_x] = args
-            op0 = SpaceOperation('int_sub_ovf',
-                                 [Constant(0, lltype.Signed), v_x],
-                                 op.result)
-            return self.rewrite_operation(op0)
-        else:
-            # int.py_div, int.udiv, int.py_mod, int.umod
-            opname = oopspec_name.replace('.', '_')
-            os = getattr(EffectInfo, 'OS_' + opname.upper())
-            return self._handle_oopspec_call(op, args, os,
-                                    EffectInfo.EF_ELIDABLE_CANNOT_RAISE)
-
     def _handle_stroruni_call(self, op, oopspec_name, args):
         SoU = args[0].concretetype     # Ptr(STR) or Ptr(UNICODE)
         can_raise_memoryerror = {
@@ -2135,43 +2071,12 @@ class Transformer(object):
         else:
             raise NotImplementedError(oopspec_name)
 
-    def _handle_rvmprof_call(self, op, oopspec_name, args):
-        if oopspec_name != 'rvmprof.jitted':
-            raise NotImplementedError(oopspec_name)
-        c_entering = Constant(0, lltype.Signed)
-        c_leaving  = Constant(1, lltype.Signed)
-        v_uniqueid = args[0]
-        op1 = SpaceOperation('rvmprof_code', [c_entering, v_uniqueid], None)
-        op2 = SpaceOperation('rvmprof_code', [c_leaving, v_uniqueid], None)
-        #
-        # fish fish inside the oopspec's graph for the ll_func pointer
-        block = op.args[0].value._obj.graph.startblock
-        while True:
-            assert len(block.exits) == 1
-            nextblock = block.exits[0].target
-            if nextblock.operations == ():
-                break
-            block = nextblock
-        last_op = block.operations[-1]
-        assert last_op.opname == 'direct_call'
-        c_ll_func = last_op.args[0]
-        #
-        args = [c_ll_func] + op.args[2:]
-        ops = self.rewrite_op_direct_call(SpaceOperation('direct_call',
-                                                         args, op.result))
-        return [op1] + ops + [op2]
-
     def rewrite_op_ll_read_timestamp(self, op):
         op1 = self.prepare_builtin_call(op, "ll_read_timestamp", [])
         return self.handle_residual_call(op1,
             oopspecindex=EffectInfo.OS_MATH_READ_TIMESTAMP,
             extraeffect=EffectInfo.EF_CANNOT_RAISE)
 
-    def rewrite_op_ll_get_timestamp_unit(self, op):
-        op1 = self.prepare_builtin_call(op, "ll_get_timestamp_unit", [])
-        return self.handle_residual_call(op1,
-            extraeffect=EffectInfo.EF_CANNOT_RAISE)
-
     def rewrite_op_jit_force_quasi_immutable(self, op):
         v_inst, c_fieldname = op.args
         descr1 = self.cpu.fielddescrof(v_inst.concretetype.TO,
