@@ -5,12 +5,11 @@ from rpython.rtyper.lltypesystem import lltype, rstr, llmemory
 from rpython.rlib.rarithmetic import ovfcheck, r_longlong, is_valid_int
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rlib.objectmodel import specialize
-from rpython.rlib.debug import fatalerror
 from rpython.jit.metainterp.history import check_descr
 from rpython.jit.metainterp.history import INT, REF, FLOAT, VOID, AbstractDescr
 from rpython.jit.metainterp.history import ConstInt, ConstFloat, ConstPtr
 from rpython.jit.metainterp import resoperation
-from rpython.jit.metainterp.resoperation import rop, opname
+from rpython.jit.metainterp.resoperation import rop
 from rpython.jit.metainterp.blackhole import BlackholeInterpreter, NULL
 from rpython.jit.codewriter import longlong
 
@@ -52,28 +51,28 @@ def _do_call(cpu, metainterp, argboxes, descr, rettype):
     if rettype == INT:
         try:
             result = cpu.bh_call_i(func, args_i, args_r, args_f, descr)
-        except Exception as e:
+        except Exception, e:
             metainterp.execute_raised(e)
             result = 0
         return result
     if rettype == REF:
         try:
             result = cpu.bh_call_r(func, args_i, args_r, args_f, descr)
-        except Exception as e:
+        except Exception, e:
             metainterp.execute_raised(e)
             result = NULL
         return result
     if rettype == FLOAT:
         try:
             result = cpu.bh_call_f(func, args_i, args_r, args_f, descr)
-        except Exception as e:
+        except Exception, e:
             metainterp.execute_raised(e)
             result = longlong.ZEROF
         return result
     if rettype == VOID:
         try:
             cpu.bh_call_v(func, args_i, args_r, args_f, descr)
-        except Exception as e:
+        except Exception, e:
             metainterp.execute_raised(e)
         return None
     raise AssertionError("bad rettype")
@@ -102,18 +101,6 @@ def do_cond_call(cpu, metainterp, argboxes, descr):
     if condbox.getint():
         do_call_n(cpu, metainterp, argboxes[1:], descr)
 
-def do_cond_call_value_i(cpu, metainterp, argboxes, descr):
-    value = argboxes[0].getint()
-    if value == 0:
-        value = do_call_i(cpu, metainterp, argboxes[1:], descr)
-    return value
-
-def do_cond_call_value_r(cpu, metainterp, argboxes, descr):
-    value = argboxes[0].getref_base()
-    if not value:
-        value = do_call_r(cpu, metainterp, argboxes[1:], descr)
-    return value
-
 def do_getarrayitem_gc_i(cpu, _, arraybox, indexbox, arraydescr):
     array = arraybox.getref_base()
     index = indexbox.getint()
@@ -251,25 +238,6 @@ def do_raw_load(cpu, _, addrbox, offsetbox, arraydescr):
     else:
         return BoxInt(cpu.bh_raw_load_i(addr, offset, arraydescr))
 
-def do_gc_store_indexed(cpu, _, addrbox, indexbox, valuebox, scalebox,
-                        base_ofsbox, bytesbox, arraydescr):
-    addr = addrbox.getref_base()
-    index = indexbox.getint()
-    scale = scalebox.getint()
-    base_ofs = base_ofsbox.getint()
-    bytes = bytesbox.getint()
-    if arraydescr.is_array_of_pointers():
-        raise AssertionError("cannot store GC pointers in gc_store_indexed for now")
-    elif arraydescr.is_array_of_floats():
-        floatval = valuebox.getfloatstorage()
-        cpu.bh_gc_store_indexed_f(addr, index, floatval, scale, base_ofs, bytes,
-                                  arraydescr)
-    else:
-        intval = valuebox.getint()
-        cpu.bh_gc_store_indexed_i(addr, index, intval, scale, base_ofs, bytes,
-                                  arraydescr)
-
-
 def exec_new_with_vtable(cpu, descr):
     return cpu.bh_new_with_vtable(descr)
 
@@ -341,17 +309,12 @@ def do_copyunicodecontent(cpu, _, srcbox, dstbox,
 def do_keepalive(cpu, _, x):
     pass
 
-def do_assert_not_none(cpu, _, box):
-    if not box.getref_base():
-        fatalerror("found during JITting: ll_assert_not_none() failed")
-
 # ____________________________________________________________
 
 
 def _make_execute_list():
     execute_by_num_args = {}
-    for key in opname.values():
-        value = getattr(rop, key)
+    for key, value in rop.__dict__.items():
         if not key.startswith('_'):
             if (rop._FINAL_FIRST <= value <= rop._FINAL_LAST or
                 rop._GUARD_FIRST <= value <= rop._GUARD_LAST):
@@ -402,8 +365,6 @@ def _make_execute_list():
                          rop.CALL_ASSEMBLER_I,
                          rop.CALL_ASSEMBLER_N,
                          rop.INCREMENT_DEBUG_COUNTER,
-                         rop.COND_CALL_VALUE_R,
-                         rop.COND_CALL_VALUE_I,
                          rop.COND_CALL_GC_WB,
                          rop.COND_CALL_GC_WB_ARRAY,
                          rop.ZERO_ARRAY,
@@ -417,31 +378,30 @@ def _make_execute_list():
                          rop.CALL_RELEASE_GIL_F,
                          rop.CALL_RELEASE_GIL_N,
                          rop.QUASIIMMUT_FIELD,
-                         rop.CHECK_MEMORY_ERROR,
+                         rop.CALL_MALLOC_GC,
                          rop.CALL_MALLOC_NURSERY,
                          rop.CALL_MALLOC_NURSERY_VARSIZE,
                          rop.CALL_MALLOC_NURSERY_VARSIZE_FRAME,
                          rop.NURSERY_PTR_INCREMENT,
                          rop.LABEL,
-                         rop.ESCAPE_I,
-                         rop.ESCAPE_N,
-                         rop.ESCAPE_R,
-                         rop.ESCAPE_F,
-                         rop.FORCE_SPILL,
                          rop.SAVE_EXC_CLASS,
                          rop.SAVE_EXCEPTION,
                          rop.RESTORE_EXCEPTION,
-                         rop.VEC_LOAD_I,
-                         rop.VEC_LOAD_F,
+                         rop.VEC_RAW_LOAD_I,
+                         rop.VEC_RAW_LOAD_F,
+                         rop.VEC_RAW_STORE,
+                         rop.VEC_GETARRAYITEM_RAW_I,
+                         rop.VEC_GETARRAYITEM_RAW_F,
+                         rop.VEC_SETARRAYITEM_RAW,
+                         rop.VEC_GETARRAYITEM_GC_I,
+                         rop.VEC_GETARRAYITEM_GC_F,
+                         rop.VEC_SETARRAYITEM_GC,
                          rop.GC_LOAD_I,
                          rop.GC_LOAD_R,
                          rop.GC_LOAD_F,
                          rop.GC_LOAD_INDEXED_R,
-                         rop.VEC_STORE,
                          rop.GC_STORE,
                          rop.GC_STORE_INDEXED,
-                         rop.LOAD_FROM_GC_TABLE,
-                         rop.LOAD_EFFECTIVE_ADDRESS,
                          ):      # list of opcodes never executed by pyjitpl
                 continue
             if rop._VEC_PURE_FIRST <= value <= rop._VEC_PURE_LAST:
