@@ -1,23 +1,14 @@
-import sys
 from rpython.rtyper.extregistry import ExtRegistryEntry
 from rpython.rtyper.lltypesystem import lltype, llmemory, rffi
-from rpython.rtyper.annlowlevel import (
-    cast_gcref_to_instance, cast_instance_to_gcref)
-from rpython.rlib.objectmodel import (
-    we_are_translated, Symbolic, compute_unique_id, specialize, r_dict)
+from rpython.rlib.objectmodel import we_are_translated, Symbolic
+from rpython.rlib.objectmodel import compute_unique_id, specialize
 from rpython.rlib.rarithmetic import r_int64, is_valid_int
-from rpython.rlib.rarithmetic import LONG_BIT, intmask, r_uint
-from rpython.rlib.jit import Counters
 
 from rpython.conftest import option
 
-from rpython.jit.metainterp.resoperation import ResOperation, rop,\
-    AbstractValue, oparity, AbstractResOp, IntOp, RefOp, FloatOp,\
-    opclasses
-from rpython.jit.metainterp.support import ptr2int, int2adr
-from rpython.jit.codewriter import longlong
+from rpython.jit.metainterp.resoperation import ResOperation, rop, AbstractValue
+from rpython.jit.codewriter import heaptracker, longlong
 import weakref
-from rpython.jit.metainterp import jitexc
 
 # ____________________________________________________________
 
@@ -31,15 +22,6 @@ VECTOR = 'V'
 
 FAILARGS_LIMIT = 1000
 
-class SwitchToBlackhole(jitexc.JitException):
-    def __init__(self, reason, raising_exception=False):
-        self.reason = reason
-        self.raising_exception = raising_exception
-        # ^^^ must be set to True if the SwitchToBlackhole is raised at a
-        #     point where the exception on metainterp.last_exc_value
-        #     is supposed to be raised.  The default False means that it
-        #     should just be copied into the blackhole interp, but not raised.
-
 def getkind(TYPE, supports_floats=True,
                   supports_longlong=True,
                   supports_singlefloats=True):
@@ -90,21 +72,70 @@ def repr_rpython(box, typechars):
                         ) #compute_unique_id(box))
 
 
+class XxxAbstractValue(object):
+    __slots__ = ()
+
+    def getint(self):
+        raise NotImplementedError
+
+    def getfloatstorage(self):
+        raise NotImplementedError
+
+    def getfloat(self):
+        return longlong.getrealfloat(self.getfloatstorage())
+
+    def getref_base(self):
+        raise NotImplementedError
+
+    def getref(self, TYPE):
+        raise NotImplementedError
+    getref._annspecialcase_ = 'specialize:arg(1)'
+
+    def constbox(self):
+        raise NotImplementedError
+
+    def getaddr(self):
+        "Only for raw addresses (BoxInt & ConstInt), not for GC addresses"
+        raise NotImplementedError
+
+    def sort_key(self):
+        raise NotImplementedError
+
+    def nonnull(self):
+        raise NotImplementedError
+
+    def repr_rpython(self):
+        return '%s' % self
+
+    def _get_str(self):
+        raise NotImplementedError
+
+    def same_box(self, other):
+        return self is other
+
+    def same_shape(self, other):
+        # only structured containers can compare their shape (vector box)
+        return True
+
+    def getaccum(self):
+        return None
+
 class AbstractDescr(AbstractValue):
-    __slots__ = ('descr_index', 'ei_index')
+    __slots__ = ()
     llopaque = True
-    descr_index = -1
-    ei_index = sys.maxint
 
     def repr_of_descr(self):
         return '%r' % (self,)
 
     def hide(self, cpu):
-        return cast_instance_to_gcref(self)
+        descr_ptr = cpu.ts.cast_instance_to_base_ref(self)
+        return cpu.ts.cast_to_ref(descr_ptr)
 
     @staticmethod
     def show(cpu, descr_gcref):
-        return cast_gcref_to_instance(AbstractDescr, descr_gcref)
+        from rpython.rtyper.annlowlevel import cast_base_ptr_to_instance
+        descr_ptr = cpu.ts.cast_to_baseclass(descr_gcref)
+        return cast_base_ptr_to_instance(AbstractDescr, descr_ptr)
 
     def get_vinfo(self):
         raise NotImplementedError
@@ -173,7 +204,7 @@ class MissingValue(object):
 
 
 class Const(AbstractValue):
-    _attrs_ = ()
+    __slots__ = ()
 
     @staticmethod
     def _new(x):
@@ -182,10 +213,12 @@ class Const(AbstractValue):
         kind = getkind(T)
         if kind == "int":
             if isinstance(T, lltype.Ptr):
-                intval = ptr2int(x)
+                intval = heaptracker.adr2int(llmemory.cast_ptr_to_adr(x))
             else:
                 intval = lltype.cast_primitive(lltype.Signed, x)
             return ConstInt(intval)
+        elif kind == "ref":
+            return cpu.ts.new_ConstRef(x)
         elif kind == "float":
             return ConstFloat(longlong.getfloatstorage(x))
         else:
@@ -229,7 +262,7 @@ class ConstInt(Const):
     getvalue = getint
 
     def getaddr(self):
-        return int2adr(self.value)
+        return heaptracker.int2adr(self.value)
 
     def _get_hash_(self):
         return make_hashable_int(self.value)
@@ -343,14 +376,6 @@ class ConstPtr(Const):
         except lltype.UninitializedMemoryAccess:
             return '<uninitialized string>'
 
-
-class ConstPtrJitCode(ConstPtr):
-    """ a ConstPtr that comes from a constant in the jitcode. has an extra
-    field to cache the encoding in the opencoder. """
-    _attrs_ = ('opencoder_index', )
-    opencoder_index = -1
-
-
 CONST_NULL = ConstPtr(ConstPtr.value)
 
 # ____________________________________________________________
@@ -360,7 +385,7 @@ def make_hashable_int(i):
     from rpython.rtyper.lltypesystem.ll2ctypes import NotCtypesAllocatedStructure
     if not we_are_translated() and isinstance(i, llmemory.AddressAsInt):
         # Warning: such a hash changes at the time of translation
-        adr = int2adr(i)
+        adr = heaptracker.int2adr(i)
         try:
             return llmemory.cast_adr_to_int(adr, "emulated")
         except NotCtypesAllocatedStructure:
@@ -395,20 +420,6 @@ def get_const_ptr_for_unicode(s):
     return result
 _const_ptr_for_unicode = {}
 
-# A dict whose keys are refs (like the .value of ConstPtr).
-# It is an r_dict. Note that NULL is not allowed as a key.
-@specialize.call_location()
-def new_ref_dict():
-    return r_dict(rd_eq, rd_hash, simple_hash_eq=True)
-
-def rd_eq(ref1, ref2):
-    return ref1 == ref2
-
-def rd_hash(ref):
-    assert ref
-    return lltype.identityhash(ref)
-
-
 # ____________________________________________________________
 
 # The JitCellToken class is the root of a tree of traces.  Each branch ends
@@ -421,11 +432,10 @@ class JitCellToken(AbstractDescr):
     was compiled; but the LoopDescr remains alive and points to the
     generated assembler.
     """
-    FORCE_BRIDGE_SEGMENTING = 1 # stored in retraced_count
-
     target_tokens = None
     failed_states = None
     retraced_count = 0
+    terminating = False # see TerminatingLoopToken in compile.py
     invalidated = False
     outermost_jitdriver_sd = None
     # and more data specified by the backend when the loop is compiled
@@ -454,12 +464,6 @@ class JitCellToken(AbstractDescr):
     def dump(self):
         self.compiled_loop_token.cpu.dump_loop_token(self)
 
-    def get_retraced_count(self):
-        return self.retraced_count >> 1
-
-    def set_retraced_count(self, value):
-        self.retraced_count = (value << 1) | (self.retraced_count & 1)
-
 class TargetToken(AbstractDescr):
     _ll_loop_code = 0     # for the backend.  If 0, we know that it is
                           # a LABEL that was not compiled yet.
@@ -634,211 +638,43 @@ def _list_all_operations(result, operations, omit_finish=True):
 # ____________________________________________________________
 
 
-FO_REPLACED_WITH_CONST = r_uint(1)
-FO_POSITION_SHIFT      = 1
-FO_POSITION_MASK       = r_uint(0xFFFFFFFE)
-
-
-class FrontendOp(AbstractResOp):
-    type = 'v'
-    _attrs_ = ('position_and_flags',)
-
-    def __init__(self, pos):
-        # p is the 32-bit position shifted left by one (might be negative,
-        # but casted to the 32-bit UINT type)
-        p = rffi.cast(rffi.UINT, pos << FO_POSITION_SHIFT)
-        self.position_and_flags = r_uint(p)    # zero-extended to a full word
-
-    def get_position(self):
-        # p is the signed 32-bit position, from self.position_and_flags
-        p = rffi.cast(rffi.INT, self.position_and_flags)
-        return intmask(p) >> FO_POSITION_SHIFT
-
-    def set_position(self, new_pos):
-        assert new_pos >= 0
-        self.position_and_flags &= ~FO_POSITION_MASK
-        self.position_and_flags |= r_uint(new_pos << FO_POSITION_SHIFT)
-
-    def is_replaced_with_const(self):
-        return bool(self.position_and_flags & FO_REPLACED_WITH_CONST)
-
-    def set_replaced_with_const(self):
-        self.position_and_flags |= FO_REPLACED_WITH_CONST
-
-    def __repr__(self):
-        return '%s(0x%x)' % (self.__class__.__name__, self.position_and_flags)
-
-class IntFrontendOp(IntOp, FrontendOp):
-    _attrs_ = ('position_and_flags', '_resint')
-
-    def copy_value_from(self, other):
-        self._resint = other.getint()
-
-class FloatFrontendOp(FloatOp, FrontendOp):
-    _attrs_ = ('position_and_flags', '_resfloat')
-
-    def copy_value_from(self, other):
-        self._resfloat = other.getfloatstorage()
-
-class RefFrontendOp(RefOp, FrontendOp):
-    _attrs_ = ('position_and_flags', '_resref', '_heapc_deps')
-    if LONG_BIT == 32:
-        _attrs_ += ('_heapc_flags',)   # on 64 bit, this gets stored into the
-        _heapc_flags = r_uint(0)       # high 32 bits of 'position_and_flags'
-    _heapc_deps = None
-
-    def copy_value_from(self, other):
-        self._resref = other.getref_base()
-
-    if LONG_BIT == 32:
-        def _get_heapc_flags(self):
-            return self._heapc_flags
-        def _set_heapc_flags(self, value):
-            self._heapc_flags = value
-    else:
-        def _get_heapc_flags(self):
-            return self.position_and_flags >> 32
-        def _set_heapc_flags(self, value):
-            self.position_and_flags = (
-                (self.position_and_flags & 0xFFFFFFFF) |
-                (value << 32))
-
-
 class History(object):
-    trace = None
-
     def __init__(self):
-        self.descr_cache = {}
-        self.descrs = {}
-        self.consts = []
-        self._cache = []
-
-    def set_inputargs(self, inpargs, metainterp_sd):
-        from rpython.jit.metainterp.opencoder import Trace
-
-        self.trace = Trace(inpargs, metainterp_sd)
-        self.inputargs = inpargs
-        if self._cache is not None:
-            # hack to record the ops *after* we know our inputargs
-            for (opnum, argboxes, op, descr) in self._cache:
-                pos = self.trace.record_op(opnum, argboxes, descr)
-                op.set_position(pos)
-            self._cache = None
-
-    def length(self):
-        return self.trace._count - len(self.trace.inputargs)
-
-    def trace_tag_overflow(self):
-        return self.trace.tag_overflow
-
-    def trace_tag_overflow_imminent(self):
-        return self.trace.tag_overflow_imminent()
-
-    def get_trace_position(self):
-        return self.trace.cut_point()
-
-    def cut(self, cut_at):
-        self.trace.cut_at(cut_at)
-
-    def any_operation(self):
-        return self.trace._count > self.trace._start
-
-    @specialize.argtype(2)
-    def _record_op(self, opnum, argboxes, descr=None):
-        return self.trace.record_op(opnum, argboxes, descr)
+        self.inputargs = None
+        self.operations = []
 
     @specialize.argtype(3)
     def record(self, opnum, argboxes, value, descr=None):
-        if self.trace is None:
-            pos = 2**14 - 1
-        else:
-            pos = self._record_op(opnum, argboxes, descr)
-        op = self._make_op(pos, value)
-        if self.trace is None:
-            self._cache.append((opnum, argboxes, op, descr))
-        return op
-
-    @specialize.argtype(2)
-    def record0(self, opnum, value, descr=None):
-        if self.trace is None:
-            pos = 2**14 - 1
-        else:
-            pos = self.trace.record_op0(opnum, descr)
-        op = self._make_op(pos, value)
-        if self.trace is None:
-            self._cache.append((opnum, [], op, descr))
-        return op
-
-    @specialize.argtype(3)
-    def record1(self, opnum, argbox1, value, descr=None):
-        if self.trace is None:
-            pos = 2**14 - 1
-        else:
-            pos = self.trace.record_op1(opnum, argbox1, descr)
-        op = self._make_op(pos, value)
-        if self.trace is None:
-            self._cache.append((opnum, [argbox1], op, descr))
-        return op
-
-    @specialize.argtype(4)
-    def record2(self, opnum, argbox1, argbox2, value, descr=None):
-        if self.trace is None:
-            pos = 2**14 - 1
-        else:
-            pos = self.trace.record_op2(opnum, argbox1, argbox2, descr)
-        op = self._make_op(pos, value)
-        if self.trace is None:
-            self._cache.append((opnum, [argbox1, argbox2], op, descr))
-        return op
-
-    @specialize.argtype(5)
-    def record3(self, opnum, argbox1, argbox2, argbox3, value, descr=None):
-        if self.trace is None:
-            pos = 2**14 - 1
-        else:
-            pos = self.trace.record_op3(opnum, argbox1, argbox2, argbox3, descr)
-        op = self._make_op(pos, value)
-        if self.trace is None:
-            self._cache.append((opnum, [argbox1, argbox2, argbox3], op, descr))
-        return op
-
-    @specialize.argtype(2)
-    def _make_op(self, pos, value):
+        op = ResOperation(opnum, argboxes, descr)
         if value is None:
-            op = FrontendOp(pos)
+            assert op.type == 'v'
         elif isinstance(value, bool):
-            op = IntFrontendOp(pos)
+            assert op.type == 'i'
             op.setint(int(value))
         elif lltype.typeOf(value) == lltype.Signed:
-            op = IntFrontendOp(pos)
+            assert op.type == 'i'
             op.setint(value)
         elif lltype.typeOf(value) is longlong.FLOATSTORAGE:
-            op = FloatFrontendOp(pos)
+            assert op.type == 'f'
             op.setfloatstorage(value)
         else:
-            op = RefFrontendOp(pos)
             assert lltype.typeOf(value) == llmemory.GCREF
+            assert op.type == 'r'
             op.setref_base(value)
+        self.operations.append(op)
         return op
 
-    def record_nospec(self, opnum, argboxes, descr=None):
-        tp = opclasses[opnum].type
-        pos = self._record_op(opnum, argboxes, descr)
-        if tp == 'v':
-            return FrontendOp(pos)
-        elif tp == 'i':
-            return IntFrontendOp(pos)
-        elif tp == 'f':
-            return FloatFrontendOp(pos)
-        assert tp == 'r'
-        return RefFrontendOp(pos)
-
     def record_default_val(self, opnum, argboxes, descr=None):
-        assert rop.is_same_as(opnum)
-        op = self.record_nospec(opnum, argboxes, descr)
+        op = ResOperation(opnum, argboxes, descr)
+        assert op.is_same_as()
         op.copy_value_from(argboxes[0])
+        self.operations.append(op)
         return op
 
+    def substitute_operation(self, position, opnum, argboxes, descr=None):
+        resbox = self.operations[position].result
+        op = ResOperation(opnum, argboxes, resbox, descr)
+        self.operations[position] = op
 
 # ____________________________________________________________
 
@@ -884,15 +720,15 @@ class Stats(object):
     compiled_count = 0
     enter_count = 0
     aborted_count = 0
+    operations = None
 
-    def __init__(self, metainterp_sd):
+    def __init__(self):
         self.loops = []
         self.locations = []
         self.aborted_keys = []
         self.invalidated_token_numbers = set()    # <- not RPython
         self.jitcell_token_wrefs = []
         self.jitcell_dicts = []                   # <- not RPython
-        self.metainterp_sd = metainterp_sd
 
     def clear(self):
         del self.loops[:]
@@ -911,7 +747,7 @@ class Stats(object):
         self.jitcell_token_wrefs.append(weakref.ref(token))
 
     def set_history(self, history):
-        self.history = history
+        self.operations = history.operations
 
     def aborted(self):
         self.aborted_count += 1
@@ -948,9 +784,7 @@ class Stats(object):
 
     def check_history(self, expected=None, **check):
         insns = {}
-        t = self.history.trace.get_iter()
-        while not t.done():
-            op = t.next()
+        for op in self.operations:
             opname = op.getopname()
             insns[opname] = insns.get(opname, 0) + 1
         if expected is not None:
@@ -1007,7 +841,7 @@ class Stats(object):
         return insns
 
     def check_simple_loop(self, expected=None, **check):
-        """ Useful in the simplest case when we have only one trace ending with
+        """ Usefull in the simplest case when we have only one trace ending with
         a jump back to itself and possibly a few bridges.
         Only the operations within the loop formed by that single jump will
         be counted.
