@@ -1,13 +1,11 @@
 from rpython.jit.codewriter.effectinfo import EffectInfo
 from rpython.jit.metainterp.history import ConstInt
 from rpython.jit.metainterp.history import CONST_NULL
-from rpython.jit.metainterp.optimize import InvalidLoop
 from rpython.jit.metainterp.optimizeopt import info, optimizer
 from rpython.jit.metainterp.optimizeopt.optimizer import REMOVED
-from rpython.jit.metainterp.optimizeopt.util import (
-    make_dispatcher_method, have_dispatcher_method, get_box_replacement)
+from rpython.jit.metainterp.optimizeopt.util import make_dispatcher_method
+
 from rpython.jit.metainterp.optimizeopt.rawbuffer import InvalidRawOperation
-from .info import getrawptrinfo, getptrinfo
 from rpython.jit.metainterp.resoperation import rop, ResOperation
 
 
@@ -15,7 +13,6 @@ class OptVirtualize(optimizer.Optimization):
     "Virtualize objects until they escape."
 
     _last_guard_not_forced_2 = None
-    _finish_guard_op = None
 
     def make_virtual(self, known_class, source_op, descr):
         opinfo = info.InstancePtrInfo(descr, known_class, is_virtual=True)
@@ -25,13 +22,11 @@ class OptVirtualize(optimizer.Optimization):
         return opinfo
 
     def make_varray(self, arraydescr, size, source_op, clear=False):
-        if not info.reasonable_array_index(size):
-            return False
         if arraydescr.is_array_of_structs():
             assert clear
             opinfo = info.ArrayStructInfo(arraydescr, size, is_virtual=True)
         else:
-            const = self.optimizer.new_const_item(arraydescr)
+            const = self.new_const_item(arraydescr)
             opinfo = info.ArrayPtrInfo(arraydescr, const, size, clear,
                                        is_virtual=True)
         # Replace 'source_op' with a version in which the length is
@@ -40,7 +35,7 @@ class OptVirtualize(optimizer.Optimization):
         newop = self.replace_op_with(source_op, source_op.getopnum(),
                                      args=[ConstInt(size)])
         newop.set_forwarded(opinfo)
-        return True
+        return opinfo
 
     def make_vstruct(self, structdescr, source_op):
         opinfo = info.StructPtrInfo(structdescr, is_virtual=True)
@@ -50,8 +45,7 @@ class OptVirtualize(optimizer.Optimization):
         return opinfo
 
     def make_virtual_raw_memory(self, size, source_op):
-        func = source_op.getarg(0).getint()
-        opinfo = info.RawBufferPtrInfo(self.optimizer.cpu, func, size)
+        opinfo = info.RawBufferPtrInfo(self.optimizer.cpu, size)
         newop = self.replace_op_with(source_op, source_op.getopnum(),
                                      args=[source_op.getarg(0), ConstInt(size)])
         newop.set_forwarded(opinfo)
@@ -67,27 +61,26 @@ class OptVirtualize(optimizer.Optimization):
     def optimize_GUARD_NO_EXCEPTION(self, op):
         if self.last_emitted_operation is REMOVED:
             return
-        return self.emit(op)
+        self.emit_operation(op)
 
     def optimize_GUARD_NOT_FORCED(self, op):
         if self.last_emitted_operation is REMOVED:
             return
-        return self.emit(op)
+        self.emit_operation(op)
 
     def optimize_GUARD_NOT_FORCED_2(self, op):
         self._last_guard_not_forced_2 = op
 
     def optimize_FINISH(self, op):
-        self._finish_guard_op = self._last_guard_not_forced_2
-        return self.emit(op)
-
-    def postprocess_FINISH(self, op):
-        guard_op = self._finish_guard_op
-        if guard_op is not None:
+        if self._last_guard_not_forced_2 is not None:
+            guard_op = self._last_guard_not_forced_2
+            self.emit_operation(op)
             guard_op = self.optimizer.store_final_boxes_in_guard(guard_op, [])
             i = len(self.optimizer._newoperations) - 1
             assert i >= 0
             self.optimizer._newoperations.insert(i, guard_op)
+        else:
+            self.emit_operation(op)
 
     def optimize_CALL_MAY_FORCE_I(self, op):
         effectinfo = op.getdescr().get_extra_info()
@@ -95,7 +88,7 @@ class OptVirtualize(optimizer.Optimization):
         if oopspecindex == EffectInfo.OS_JIT_FORCE_VIRTUAL:
             if self._optimize_JIT_FORCE_VIRTUAL(op):
                 return
-        return self.emit(op)
+        self.emit_operation(op)
     optimize_CALL_MAY_FORCE_R = optimize_CALL_MAY_FORCE_I
     optimize_CALL_MAY_FORCE_F = optimize_CALL_MAY_FORCE_I
     optimize_CALL_MAY_FORCE_N = optimize_CALL_MAY_FORCE_I
@@ -104,10 +97,10 @@ class OptVirtualize(optimizer.Optimization):
         effectinfo = op.getdescr().get_extra_info()
         oopspecindex = effectinfo.oopspecindex
         if oopspecindex == EffectInfo.OS_JIT_FORCE_VIRTUALIZABLE:
-            opinfo = getptrinfo(op.getarg(2))
+            opinfo = self.getptrinfo(op.getarg(2))
             if opinfo and opinfo.is_virtual():
                 return
-        return self.emit(op)
+        self.emit_operation(op)
 
     def optimize_VIRTUAL_REF(self, op):
         # get some constants
@@ -125,9 +118,10 @@ class OptVirtualize(optimizer.Optimization):
         op.set_forwarded(newop)
         newop.set_forwarded(vrefvalue)
         token = ResOperation(rop.FORCE_TOKEN, [])
+        self.emit_operation(token)
         vrefvalue.setfield(descr_virtual_token, newop, token)
-        vrefvalue.setfield(descr_forced, newop, CONST_NULL)
-        return self.emit(token)
+        vrefvalue.setfield(descr_forced, newop,
+                           self.optimizer.cpu.ts.CONST_NULLREF)
 
     def optimize_VIRTUAL_REF_FINISH(self, op):
         # This operation is used in two cases.  In normal cases, it
@@ -164,17 +158,17 @@ class OptVirtualize(optimizer.Optimization):
         # was already forced).
 
     def _optimize_JIT_FORCE_VIRTUAL(self, op):
-        vref = getptrinfo(op.getarg(1))
+        vref = self.getptrinfo(op.getarg(1))
         vrefinfo = self.optimizer.metainterp_sd.virtualref_info
         if vref and vref.is_virtual():
             tokenop = vref.getfield(vrefinfo.descr_virtual_token, None)
             if tokenop is None:
                 return False
-            tokeninfo = getptrinfo(tokenop)
+            tokeninfo = self.getptrinfo(tokenop)
             if (tokeninfo is not None and tokeninfo.is_constant() and
                     not tokeninfo.is_nonnull()):
                 forcedop = vref.getfield(vrefinfo.descr_forced, None)
-                forcedinfo = getptrinfo(forcedop)
+                forcedinfo = self.getptrinfo(forcedop)
                 if forcedinfo is not None and not forcedinfo.is_null():
                     self.make_equal_to(op, forcedop)
                     self.last_emitted_operation = REMOVED
@@ -182,7 +176,7 @@ class OptVirtualize(optimizer.Optimization):
         return False
 
     def optimize_GETFIELD_GC_I(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             fieldop = opinfo.getfield(op.getdescr())
             if fieldop is None:
@@ -190,19 +184,19 @@ class OptVirtualize(optimizer.Optimization):
             self.make_equal_to(op, fieldop)
         else:
             self.make_nonnull(op.getarg(0))
-            return self.emit(op)
+            self.emit_operation(op)
     optimize_GETFIELD_GC_R = optimize_GETFIELD_GC_I
     optimize_GETFIELD_GC_F = optimize_GETFIELD_GC_I
 
     def optimize_SETFIELD_GC(self, op):
         struct = op.getarg(0)
-        opinfo = getptrinfo(struct)
+        opinfo = self.getptrinfo(struct)
         if opinfo is not None and opinfo.is_virtual():
             opinfo.setfield(op.getdescr(), struct,
-                            get_box_replacement(op.getarg(1)))
+                            self.get_box_replacement(op.getarg(1)))
         else:
             self.make_nonnull(struct)
-            return self.emit(op)
+            self.emit_operation(op)
 
     def optimize_NEW_WITH_VTABLE(self, op):
         known_class = ConstInt(op.getdescr().get_vtable())
@@ -213,49 +207,50 @@ class OptVirtualize(optimizer.Optimization):
 
     def optimize_NEW_ARRAY(self, op):
         sizebox = self.get_constant_box(op.getarg(0))
-        if (sizebox is not None and
-            self.make_varray(op.getdescr(), sizebox.getint(), op)):
-            return
-        return self.emit(op)
+        if sizebox is not None:
+            self.make_varray(op.getdescr(), sizebox.getint(), op)
+        else:
+            self.emit_operation(op)
 
     def optimize_NEW_ARRAY_CLEAR(self, op):
         sizebox = self.get_constant_box(op.getarg(0))
-        if (sizebox is not None and
-            self.make_varray(op.getdescr(), sizebox.getint(), op, clear=True)):
-            return
-        return self.emit(op)
+        if sizebox is not None:
+            self.make_varray(op.getdescr(), sizebox.getint(), op, clear=True)
+        else:
+            self.emit_operation(op)        
 
     def optimize_CALL_N(self, op):
         effectinfo = op.getdescr().get_extra_info()
         if effectinfo.oopspecindex == EffectInfo.OS_RAW_MALLOC_VARSIZE_CHAR:
-            return self.do_RAW_MALLOC_VARSIZE_CHAR(op)
+            self.do_RAW_MALLOC_VARSIZE_CHAR(op)
         elif effectinfo.oopspecindex == EffectInfo.OS_RAW_FREE:
-            return self.do_RAW_FREE(op)
+            self.do_RAW_FREE(op)
         elif effectinfo.oopspecindex == EffectInfo.OS_JIT_FORCE_VIRTUALIZABLE:
             # we might end up having CALL here instead of COND_CALL
-            info = getptrinfo(op.getarg(1))
+            info = self.getptrinfo(op.getarg(1))
             if info and info.is_virtual():
                 return
         else:
-            return self.emit(op)
+            self.emit_operation(op)
     optimize_CALL_R = optimize_CALL_N
     optimize_CALL_I = optimize_CALL_N
 
     def do_RAW_MALLOC_VARSIZE_CHAR(self, op):
         sizebox = self.get_constant_box(op.getarg(1))
         if sizebox is None:
-            return self.emit(op)
+            self.emit_operation(op)
+            return
         self.make_virtual_raw_memory(sizebox.getint(), op)
         self.last_emitted_operation = REMOVED
 
     def do_RAW_FREE(self, op):
-        opinfo = getrawptrinfo(op.getarg(1))
+        opinfo = self.getrawptrinfo(op.getarg(1))
         if opinfo and opinfo.is_virtual():
             return
-        return self.emit(op)
+        self.emit_operation(op)
 
     def optimize_INT_ADD(self, op):
-        opinfo = getrawptrinfo(op.getarg(0))
+        opinfo = self.getrawptrinfo(op.getarg(0), create=False)
         offsetbox = self.get_constant_box(op.getarg(1))
         if opinfo and opinfo.is_virtual() and offsetbox is not None:
             offset = offsetbox.getint()
@@ -265,29 +260,29 @@ class OptVirtualize(optimizer.Optimization):
                 isinstance(opinfo, info.RawSlicePtrInfo)):
                 self.make_virtual_raw_slice(offset, opinfo, op)
                 return
-        return self.emit(op)
+        self.emit_operation(op)
 
     def optimize_ARRAYLEN_GC(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             self.make_constant_int(op, opinfo.getlength())
         else:
             self.make_nonnull(op.getarg(0))
-            return self.emit(op)
+            self.emit_operation(op)
 
     def optimize_GETARRAYITEM_GC_I(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             indexbox = self.get_constant_box(op.getarg(1))
             if indexbox is not None:
                 item = opinfo.getitem(op.getdescr(), indexbox.getint())
                 if item is None:   # reading uninitialized array items?
-                    raise InvalidLoop("reading uninitialized virtual "
-                                      "array items")
+                    assert False, "can't read uninitialized items"
+                    itemvalue = value.constvalue     # bah, just return 0
                 self.make_equal_to(op, item)
                 return
         self.make_nonnull(op.getarg(0))
-        return self.emit(op)
+        self.emit_operation(op)
     optimize_GETARRAYITEM_GC_R = optimize_GETARRAYITEM_GC_I
     optimize_GETARRAYITEM_GC_F = optimize_GETARRAYITEM_GC_I
 
@@ -298,16 +293,16 @@ class OptVirtualize(optimizer.Optimization):
     optimize_GETARRAYITEM_GC_PURE_F = optimize_GETARRAYITEM_GC_I
 
     def optimize_SETARRAYITEM_GC(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             indexbox = self.get_constant_box(op.getarg(1))
             if indexbox is not None:
                 opinfo.setitem(op.getdescr(), indexbox.getint(),
-                               get_box_replacement(op.getarg(0)),
-                               get_box_replacement(op.getarg(2)))
+                               self.get_box_replacement(op.getarg(0)),
+                               self.get_box_replacement(op.getarg(2)))
                 return
         self.make_nonnull(op.getarg(0))
-        return self.emit(op)
+        self.emit_operation(op)
 
     def _unpack_arrayitem_raw_op(self, op, indexbox):
         index = indexbox.getint()
@@ -318,7 +313,7 @@ class OptVirtualize(optimizer.Optimization):
         return offset, itemsize, descr
 
     def optimize_GETARRAYITEM_RAW_I(self, op):
-        opinfo = getrawptrinfo(op.getarg(0))
+        opinfo = self.getrawptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             indexbox = self.get_constant_box(op.getarg(1))
             if indexbox is not None:
@@ -332,23 +327,23 @@ class OptVirtualize(optimizer.Optimization):
                     self.make_equal_to(op, itemvalue)
                     return
         self.make_nonnull(op.getarg(0))
-        return self.emit(op)
+        self.emit_operation(op)
     optimize_GETARRAYITEM_RAW_F = optimize_GETARRAYITEM_RAW_I
 
     def optimize_SETARRAYITEM_RAW(self, op):
-        opinfo = getrawptrinfo(op.getarg(0))
+        opinfo = self.getrawptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             indexbox = self.get_constant_box(op.getarg(1))
             if indexbox is not None:
                 offset, itemsize, descr = self._unpack_arrayitem_raw_op(op, indexbox)
-                itemop = get_box_replacement(op.getarg(2))
+                itemop = self.get_box_replacement(op.getarg(2))
                 try:
                     opinfo.setitem_raw(offset, itemsize, descr, itemop)
                     return
                 except InvalidRawOperation:
                     pass
         self.make_nonnull(op.getarg(0))
-        return self.emit(op)
+        self.emit_operation(op)
 
     def _unpack_raw_load_store_op(self, op, offsetbox):
         offset = offsetbox.getint()
@@ -358,7 +353,7 @@ class OptVirtualize(optimizer.Optimization):
         return offset, itemsize, descr
 
     def optimize_RAW_LOAD_I(self, op):
-        opinfo = getrawptrinfo(op.getarg(0))
+        opinfo = self.getrawptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             offsetbox = self.get_constant_box(op.getarg(1))
             if offsetbox is not None:
@@ -370,11 +365,11 @@ class OptVirtualize(optimizer.Optimization):
                 else:
                     self.make_equal_to(op, itemop)
                     return
-        return self.emit(op)
+        self.emit_operation(op)
     optimize_RAW_LOAD_F = optimize_RAW_LOAD_I
 
     def optimize_RAW_STORE(self, op):
-        opinfo = getrawptrinfo(op.getarg(0))
+        opinfo = self.getrawptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             offsetbox = self.get_constant_box(op.getarg(1))
             if offsetbox is not None:
@@ -384,42 +379,40 @@ class OptVirtualize(optimizer.Optimization):
                     return
                 except InvalidRawOperation:
                     pass
-        return self.emit(op)
+        self.emit_operation(op)
 
     def optimize_GETINTERIORFIELD_GC_I(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             indexbox = self.get_constant_box(op.getarg(1))
             if indexbox is not None:
                 descr = op.getdescr()
                 fld = opinfo.getinteriorfield_virtual(indexbox.getint(), descr)
                 if fld is None:
-                    raise InvalidLoop("reading uninitialized virtual interior "
-                                      "array items")
+                    raise Exception("I think this is illegal")
+                    xxx
+                    fieldvalue = self.new_const(descr)
                 self.make_equal_to(op, fld)
                 return
         self.make_nonnull(op.getarg(0))
-        return self.emit(op)
+        self.emit_operation(op)
     optimize_GETINTERIORFIELD_GC_R = optimize_GETINTERIORFIELD_GC_I
     optimize_GETINTERIORFIELD_GC_F = optimize_GETINTERIORFIELD_GC_I
 
     def optimize_SETINTERIORFIELD_GC(self, op):
-        opinfo = getptrinfo(op.getarg(0))
+        opinfo = self.getptrinfo(op.getarg(0))
         if opinfo and opinfo.is_virtual():
             indexbox = self.get_constant_box(op.getarg(1))
             if indexbox is not None:
                 opinfo.setinteriorfield_virtual(indexbox.getint(),
                                                 op.getdescr(),
-                                       get_box_replacement(op.getarg(2)))
+                                       self.get_box_replacement(op.getarg(2)))
                 return
         self.make_nonnull(op.getarg(0))
-        return self.emit(op)
+        self.emit_operation(op)
 
 
 dispatch_opt = make_dispatcher_method(OptVirtualize, 'optimize_',
-                                      default=OptVirtualize.emit)
+        default=OptVirtualize.emit_operation)
 
 OptVirtualize.propagate_forward = dispatch_opt
-dispatch_postprocess = make_dispatcher_method(OptVirtualize, 'postprocess_')
-OptVirtualize.propagate_postprocess = dispatch_postprocess
-OptVirtualize.have_postprocess_op = have_dispatcher_method(OptVirtualize, 'postprocess_')
