@@ -1,10 +1,15 @@
-import py
 from rpython.jit.metainterp.heapcache import HeapCache
 from rpython.jit.metainterp.resoperation import rop, InputArgInt
-from rpython.jit.metainterp.history import ConstInt, ConstPtr, BasicFailDescr
-from rpython.jit.metainterp.history import IntFrontendOp, RefFrontendOp
-from rpython.rtyper.lltypesystem import  llmemory, rffi
+from rpython.jit.metainterp.history import ConstInt, BasicFailDescr
 
+box1 = "box1"
+box2 = "box2"
+box3 = "box3"
+box4 = "box4"
+box5 = "box5"
+lengthbox1 = object()
+lengthbox2 = object()
+lengthbox3 = object()
 descr1 = object()
 descr2 = object()
 descr3 = object()
@@ -24,17 +29,12 @@ class FakeEffectinfo(object):
     EF_RANDOM_EFFECTS                  = 7 #can do whatever
 
     OS_ARRAYCOPY = 0
-    OS_ARRAYMOVE = 9
 
     def __init__(self, extraeffect, oopspecindex, write_descrs_fields, write_descrs_arrays):
         self.extraeffect = extraeffect
         self.oopspecindex = oopspecindex
-        self._write_descrs_fields = write_descrs_fields
-        self._write_descrs_arrays = write_descrs_arrays
-        if len(write_descrs_arrays) == 1:
-            [self.single_write_descr_array] = write_descrs_arrays
-        else:
-            self.single_write_descr_array = None
+        self.write_descrs_fields = write_descrs_fields
+        self.write_descrs_arrays = write_descrs_arrays
 
     def has_random_effects(self):
         return self.extraeffect == self.EF_RANDOM_EFFECTS
@@ -43,14 +43,14 @@ class FakeCallDescr(object):
     def __init__(self, extraeffect, oopspecindex=None, write_descrs_fields=[], write_descrs_arrays=[]):
         self.extraeffect = extraeffect
         self.oopspecindex = oopspecindex
-        self.__write_descrs_fields = write_descrs_fields
-        self.__write_descrs_arrays = write_descrs_arrays
+        self.write_descrs_fields = write_descrs_fields
+        self.write_descrs_arrays = write_descrs_arrays
 
     def get_extra_info(self):
         return FakeEffectinfo(
             self.extraeffect, self.oopspecindex,
-            write_descrs_fields=self.__write_descrs_fields,
-            write_descrs_arrays=self.__write_descrs_arrays,
+            write_descrs_fields=self.write_descrs_fields,
+            write_descrs_arrays=self.write_descrs_arrays,
         )
 
 arraycopydescr1 = FakeCallDescr(FakeEffectinfo.EF_CANNOT_RAISE, FakeEffectinfo.OS_ARRAYCOPY, write_descrs_arrays=[descr1])
@@ -59,83 +59,43 @@ arraycopydescr1 = FakeCallDescr(FakeEffectinfo.EF_CANNOT_RAISE, FakeEffectinfo.O
 class TestHeapCache(object):
     def test_known_class_box(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        assert not h.is_class_known(box1)
-        assert not h.is_class_known(box2)
-        h.class_now_known(box1)
-        assert h.is_class_known(box1)
-        assert not h.is_class_known(box2)
+        assert not h.is_class_known(1)
+        assert not h.is_class_known(2)
+        h.class_now_known(1)
+        assert h.is_class_known(1)
+        assert not h.is_class_known(2)
 
         h.reset()
-        assert not h.is_class_known(box1)
-        assert not h.is_class_known(box2)
+        assert not h.is_class_known(1)
+        assert not h.is_class_known(2)
 
     def test_known_nullity(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        assert not h.is_nullity_known(box1)
-        assert not h.is_nullity_known(box2)
-        h.nullity_now_known(box1)
-        assert h.is_nullity_known(box1)
-        assert not h.is_nullity_known(box2)
+        assert not h.is_nullity_known(1)
+        assert not h.is_nullity_known(2)
+        h.nullity_now_known(1)
+        assert h.is_nullity_known(1)
+        assert not h.is_nullity_known(2)
 
         h.reset()
-        assert not h.is_nullity_known(box1)
-        assert not h.is_nullity_known(box2)
+        assert not h.is_nullity_known(1)
+        assert not h.is_nullity_known(2)
 
-    def test_known_nullity_more_cases(self):
-        h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        h.class_now_known(box1)
-        assert h.is_nullity_known(box1)
-
-        h.new(box2)
-        assert h.is_nullity_known(box2)
-
-        h.reset()
-        assert not h.is_nullity_known(box1)
-        assert not h.is_nullity_known(box2)
 
     def test_nonstandard_virtualizable(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        assert not h.is_known_nonstandard_virtualizable(box1)
-        assert not h.is_known_nonstandard_virtualizable(box2)
-        h.nonstandard_virtualizables_now_known(box1)
-        assert h.is_known_nonstandard_virtualizable(box1)
-        assert not h.is_known_nonstandard_virtualizable(box2)
+        assert not h.is_nonstandard_virtualizable(1)
+        assert not h.is_nonstandard_virtualizable(2)
+        h.nonstandard_virtualizables_now_known(1)
+        assert h.is_nonstandard_virtualizable(1)
+        assert not h.is_nonstandard_virtualizable(2)
 
         h.reset()
-        assert not h.is_known_nonstandard_virtualizable(box1)
-        assert not h.is_known_nonstandard_virtualizable(box2)
-
-    def test_nonstandard_virtualizable_const(self):
-        h = HeapCache()
-        # rare but not impossible situation for some interpreters: we have a
-        # *constant* nonstandard virtualizable
-        c_box = ConstPtr(ConstPtr.value)
-        h.nonstandard_virtualizables_now_known(c_box) # should not crash
-        assert not h.is_known_nonstandard_virtualizable(c_box)
-
-    def test_nonstandard_virtualizable_allocation(self):
-        h = HeapCache()
-        box1 = RefFrontendOp(1)
-        h.new(box1)
-        # we've seen the allocation, so it's not the virtualizable
-        assert h.is_known_nonstandard_virtualizable(box1)
-
-        h.reset()
-        assert not h.is_known_nonstandard_virtualizable(box1)
+        assert not h.is_nonstandard_virtualizable(1)
+        assert not h.is_nonstandard_virtualizable(2)
 
     def test_heapcache_fields(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
         assert h.getfield(box1, descr1) is None
         assert h.getfield(box1, descr2) is None
         h.setfield(box1, box2, descr1)
@@ -159,10 +119,6 @@ class TestHeapCache(object):
 
     def test_heapcache_read_fields_multiple(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.getfield_now_known(box1, descr1, box2)
         h.getfield_now_known(box3, descr1, box4)
         assert h.getfield(box1, descr1) is box2
@@ -178,10 +134,6 @@ class TestHeapCache(object):
 
     def test_heapcache_write_fields_multiple(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.setfield(box1, box2, descr1)
         assert h.getfield(box1, descr1) is box2
         h.setfield(box3, box4, descr1)
@@ -189,10 +141,6 @@ class TestHeapCache(object):
         assert h.getfield(box1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.new(box1)
         h.setfield(box1, box2, descr1)
         assert h.getfield(box1, descr1) is box2
@@ -201,10 +149,6 @@ class TestHeapCache(object):
         assert h.getfield(box1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.new(box1)
         h.new(box3)
         h.setfield(box1, box2, descr1)
@@ -218,10 +162,6 @@ class TestHeapCache(object):
 
     def test_heapcache_arrays(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         assert h.getarrayitem(box1, index1, descr1) is None
         assert h.getarrayitem(box1, index1, descr2) is None
         assert h.getarrayitem(box1, index2, descr1) is None
@@ -264,10 +204,6 @@ class TestHeapCache(object):
 
     def test_heapcache_array_nonconst_index(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index2, box4, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
@@ -278,10 +214,6 @@ class TestHeapCache(object):
 
     def test_heapcache_read_fields_multiple_array(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.getarrayitem_now_known(box1, index1, box2, descr1)
         h.getarrayitem_now_known(box3, index1, box4, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
@@ -297,10 +229,6 @@ class TestHeapCache(object):
 
     def test_heapcache_write_fields_multiple_array(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.setarrayitem(box1, index1, box2, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
         h.setarrayitem(box3, index1, box4, descr1)
@@ -308,10 +236,6 @@ class TestHeapCache(object):
         assert h.getarrayitem(box1, index1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.new(box1)
         h.setarrayitem(box1, index1, box2, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
@@ -320,10 +244,6 @@ class TestHeapCache(object):
         assert h.getarrayitem(box1, index1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.new(box1)
         h.new(box3)
         h.setarrayitem(box1, index1, box2, descr1)
@@ -337,10 +257,6 @@ class TestHeapCache(object):
 
     def test_length_cache(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        lengthbox1 = IntFrontendOp(11)
-        lengthbox2 = IntFrontendOp(12)
         h.new_array(box1, lengthbox1)
         assert h.arraylen(box1) is lengthbox1
 
@@ -351,47 +267,39 @@ class TestHeapCache(object):
 
     def test_invalidate_cache(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box4 = RefFrontendOp(4)
         h.setfield(box1, box2, descr1)
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index2, box4, descr1)
-        h.invalidate_caches(rop.INT_ADD, None)
-        h.invalidate_caches(rop.INT_ADD_OVF, None)
-        h.invalidate_caches(rop.SETFIELD_RAW, None)
-        h.invalidate_caches(rop.SETARRAYITEM_RAW, None)
+        h.invalidate_caches(rop.INT_ADD, None, [])
+        h.invalidate_caches(rop.INT_ADD_OVF, None, [])
+        h.invalidate_caches(rop.SETFIELD_RAW, None, [])
+        h.invalidate_caches(rop.SETARRAYITEM_RAW, None, [])
         assert h.getfield(box1, descr1) is box2
         assert h.getarrayitem(box1, index1, descr1) is box2
         assert h.getarrayitem(box1, index2, descr1) is box4
 
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_ELIDABLE_CANNOT_RAISE), [])
         assert h.getfield(box1, descr1) is box2
         assert h.getarrayitem(box1, index1, descr1) is box2
         assert h.getarrayitem(box1, index2, descr1) is box4
 
-        h.invalidate_caches(rop.GUARD_TRUE, None)
+        h.invalidate_caches(rop.GUARD_TRUE, None, [])
         assert h.getfield(box1, descr1) is box2
         assert h.getarrayitem(box1, index1, descr1) is box2
         assert h.getarrayitem(box1, index2, descr1) is box4
 
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_LOOPINVARIANT_N, FakeCallDescr(FakeEffectinfo.EF_LOOPINVARIANT), [])
 
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
         assert h.getfield(box1, descr1) is None
         assert h.getarrayitem(box1, index1, descr1) is None
         assert h.getarrayitem(box1, index2, descr1) is None
 
-    def test_replace_box_with_box(self):
-        py.test.skip("replacing a box with another box: not supported any more")
+    def test_replace_box(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
         h.setfield(box1, box2, descr1)
         h.setfield(box1, box3, descr2)
         h.setfield(box2, box3, descr3)
@@ -402,22 +310,16 @@ class TestHeapCache(object):
         h.setfield(box4, box3, descr1)
         assert h.getfield(box4, descr1) is box3
 
-    def test_replace_box_with_const(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        c_box3 = ConstPtr(ConstPtr.value)
         h.setfield(box1, box2, descr1)
         h.setfield(box1, box3, descr2)
         h.setfield(box2, box3, descr3)
-        h.replace_box(box3, c_box3)
+        h.replace_box(box3, box4)
         assert h.getfield(box1, descr1) is box2
-        assert c_box3.same_constant(h.getfield(box1, descr2))
-        assert c_box3.same_constant(h.getfield(box2, descr3))
+        assert h.getfield(box1, descr2) is box4
+        assert h.getfield(box2, descr3) is box4
 
     def test_replace_box_twice(self):
-        py.test.skip("replacing a box with another box: not supported any more")
         h = HeapCache()
         h.setfield(box1, box2, descr1)
         h.setfield(box1, box3, descr2)
@@ -441,7 +343,6 @@ class TestHeapCache(object):
         assert h.getfield(box2, descr3) is box5
 
     def test_replace_box_array(self):
-        py.test.skip("replacing a box with another box: not supported any more")
         h = HeapCache()
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index1, box3, descr2)
@@ -461,7 +362,6 @@ class TestHeapCache(object):
         assert h.arraylen(box4) is lengthbox2
 
     def test_replace_box_array_twice(self):
-        py.test.skip("replacing a box with another box: not supported any more")
         h = HeapCache()
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index1, box3, descr2)
@@ -482,45 +382,19 @@ class TestHeapCache(object):
         h.replace_box(lengthbox2, lengthbox3)
         assert h.arraylen(box4) is lengthbox3
 
-    def test_replace_box_with_const_in_array(self):
-        h = HeapCache()
-        box1 = RefFrontendOp(1)
-        lengthbox2 = IntFrontendOp(2)
-        lengthbox2.setint(10)
-        h.arraylen_now_known(box1, lengthbox2)
-        assert h.arraylen(box1) is lengthbox2
-        c10 = ConstInt(10)
-        h.replace_box(lengthbox2, c10)
-        assert c10.same_constant(h.arraylen(box1))
-
-        box2 = IntFrontendOp(2)
-        box2.setint(12)
-        h.setarrayitem(box1, index2, box2, descr1)
-        assert h.getarrayitem(box1, index2, descr1) is box2
-        c12 = ConstInt(12)
-        h.replace_box(box2, c12)
-        assert c12.same_constant(h.getarrayitem(box1, index2, descr1))
-
     def test_ll_arraycopy(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
-        box5 = RefFrontendOp(5)
-        lengthbox1 = IntFrontendOp(11)
-        lengthbox2 = IntFrontendOp(12)
         h.new_array(box1, lengthbox1)
         h.setarrayitem(box1, index1, box2, descr1)
         h.new_array(box2, lengthbox1)
         # Just need the destination box for this call
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box5, box2, index1, index1, index1]
         )
         assert h.getarrayitem(box1, index1, descr1) is box2
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box5, box3, index1, index1, index1]
@@ -529,7 +403,7 @@ class TestHeapCache(object):
 
         h.setarrayitem(box4, index1, box2, descr1)
         assert h.getarrayitem(box4, index1, descr1) is box2
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box3, box5, index1, index1, index2]
@@ -538,14 +412,10 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_differing_descrs(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        lengthbox2 = IntFrontendOp(12)
         h.setarrayitem(box1, index1, box2, descr2)
         assert h.getarrayitem(box1, index1, descr2) is box2
         h.new_array(box2, lengthbox2)
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box3, box2, index1, index1, index2]
@@ -554,12 +424,9 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_differing_descrs_nonconst_index(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
         h.setarrayitem(box1, index1, box2, descr2)
         assert h.getarrayitem(box1, index1, descr2) is box2
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box3, box2, index1, index1, InputArgInt()]
@@ -568,11 +435,8 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_result_propogated(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
         h.setarrayitem(box1, index1, box2, descr1)
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box1, box3, index1, index1, index2]
@@ -581,14 +445,9 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_dest_new(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
-        lengthbox1 = IntFrontendOp(11)
         h.new_array(box1, lengthbox1)
         h.setarrayitem(box3, index1, box4, descr1)
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, index2]
@@ -596,22 +455,16 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_doesnt_escape_arrays(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        lengthbox1 = IntFrontendOp(11)
-        lengthbox2 = IntFrontendOp(12)
-        h.new_array(box1, index2)
-        h.new_array(box2, index2)
-        assert h.is_unescaped(box1)
-        assert h.is_unescaped(box2)
-        h.invalidate_caches_varargs(
+        h.new_array(box1, lengthbox1)
+        h.new_array(box2, lengthbox2)
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, index2]
         )
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, InputArgInt()]
@@ -621,85 +474,71 @@ class TestHeapCache(object):
 
     def test_unescaped(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
         assert not h.is_unescaped(box1)
         h.new(box2)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box2, box1)
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box2, box1])
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
         assert not h.is_unescaped(box2)
 
     def test_unescaped_testing(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
         h.new(box1)
         h.new(box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         # Putting a virtual inside of another virtual doesn't escape it.
-        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
         assert h.is_unescaped(box2)
         # Reading a field from a virtual doesn't escape it.
-        h.invalidate_caches(rop.GETFIELD_GC_I, None, box1)
+        h.invalidate_caches(rop.GETFIELD_GC_I, None, [box1])
         assert h.is_unescaped(box1)
         # Escaping a virtual transitively escapes anything inside of it.
         assert not h.is_unescaped(box3)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box3, box1)
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box3, box1])
         assert not h.is_unescaped(box1)
         assert not h.is_unescaped(box2)
 
     def test_ops_dont_escape(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.INSTANCE_PTR_EQ, None, box1, box2)
+        h.invalidate_caches(rop.INSTANCE_PTR_EQ, None, [box1, box2])
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.INSTANCE_PTR_NE, None, box1, box2)
+        h.invalidate_caches(rop.INSTANCE_PTR_NE, None, [box1, box2])
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
 
     def test_circular_virtuals(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
         h.new(box1)
         h.new(box2)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box2, box1)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box3, box1) # does not crash
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box2, box1])
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box3, box1]) # does not crash
 
     def test_unescaped_array(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        lengthbox1 = IntFrontendOp(11)
-        lengthbox2 = IntFrontendOp(12)
-        h.new_array(box1, index2)
+        h.new_array(box1, lengthbox1)
         assert h.is_unescaped(box1)
-        h.invalidate_caches(rop.SETARRAYITEM_GC, None, box1, index1, box2)
+        h.invalidate_caches(rop.SETARRAYITEM_GC, None, [box1, index1, box2])
         assert h.is_unescaped(box1)
-        h.invalidate_caches(rop.SETARRAYITEM_GC, None, box2, index1, box1)
+        h.invalidate_caches(rop.SETARRAYITEM_GC, None, [box2, index1, box1])
         assert not h.is_unescaped(box1)
 
         h = HeapCache()
-        h.new_array(box1, index2)
+        h.new_array(box1, lengthbox1)
         h.new(box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.SETARRAYITEM_GC, None, box1, lengthbox2, box2)
+        h.invalidate_caches(rop.SETARRAYITEM_GC, None, [box1, lengthbox2, box2])
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches_varargs(
+        h.invalidate_caches(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [box1]
         )
         assert not h.is_unescaped(box1)
@@ -707,12 +546,10 @@ class TestHeapCache(object):
 
     def test_call_doesnt_invalidate_unescaped_boxes(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
         h.new(box1)
         assert h.is_unescaped(box1)
         h.setfield(box1, box2, descr1)
-        h.invalidate_caches_varargs(rop.CALL_N,
+        h.invalidate_caches(rop.CALL_N,
             FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE),
             []
         )
@@ -720,13 +557,10 @@ class TestHeapCache(object):
 
     def test_call_doesnt_invalidate_unescaped_array_boxes(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box3 = RefFrontendOp(3)
-        lengthbox1 = IntFrontendOp(11)
-        h.new_array(box1, index2)
+        h.new_array(box1, lengthbox1)
         assert h.is_unescaped(box1)
         h.setarrayitem(box1, index1, box3, descr1)
-        h.invalidate_caches_varargs(rop.CALL_N,
+        h.invalidate_caches(rop.CALL_N,
             FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE),
             []
         )
@@ -734,18 +568,16 @@ class TestHeapCache(object):
 
     def test_bug_missing_ignored_operations(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
         assert h.getfield(box1, descr1) is box2
-        h.invalidate_caches(rop.STRSETITEM, None)
-        h.invalidate_caches(rop.UNICODESETITEM, None)
-        h.invalidate_caches(rop.SETFIELD_RAW, None)
-        h.invalidate_caches(rop.SETARRAYITEM_RAW, None)
-        h.invalidate_caches(rop.SETINTERIORFIELD_RAW, None)
-        h.invalidate_caches(rop.RAW_STORE, None)
+        h.invalidate_caches(rop.STRSETITEM, None, [])
+        h.invalidate_caches(rop.UNICODESETITEM, None, [])
+        h.invalidate_caches(rop.SETFIELD_RAW, None, [])
+        h.invalidate_caches(rop.SETARRAYITEM_RAW, None, [])
+        h.invalidate_caches(rop.SETINTERIORFIELD_RAW, None, [])
+        h.invalidate_caches(rop.RAW_STORE, None, [])
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is box2
@@ -758,46 +590,40 @@ class TestHeapCache(object):
         # calling some residual code that changes the values on box3: then
         # the content of box2 is still cached at the old value.
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
         assert h.getfield(box1, descr1) is box2
-        h.invalidate_caches_varargs(rop.CALL_MAY_FORCE_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
+        h.invalidate_caches(rop.CALL_MAY_FORCE_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
         assert not h.is_unescaped(box1)
         assert not h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is None
 
     def test_bug_heap_cache_is_cleared_but_not_is_unescaped_2(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
-        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
+        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
         assert h.getfield(box1, descr1) is box2
         descr = BasicFailDescr()
         class XTra:
             oopspecindex = 0
             OS_ARRAYCOPY = 42
-            OS_ARRAYMOVE = 49
             extraeffect = 5
             EF_LOOPINVARIANT = 1
             EF_ELIDABLE_CANNOT_RAISE = 2
             EF_ELIDABLE_OR_MEMORYERROR = 3
             EF_ELIDABLE_CAN_RAISE = 4
         descr.get_extra_info = XTra
-        h.invalidate_caches_varargs(rop.CALL_N, descr, [])
+        h.invalidate_caches(rop.CALL_N, descr, [])
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is box2
 
     def test_is_likely_virtual(self):
         h = HeapCache()
-        box1 = RefFrontendOp(1)
         h.new(box1)
         assert h.is_unescaped(box1)
         assert h.is_likely_virtual(box1)
@@ -807,141 +633,3 @@ class TestHeapCache(object):
         h._escape_box(box1)
         assert not h.is_unescaped(box1)
         assert not h.is_likely_virtual(box1)
-
-    def test_is_likely_virtual_2(self):
-        h = HeapCache()
-        box1 = RefFrontendOp(1)
-        h.new(box1)
-        assert h.is_unescaped(box1)
-        assert h.is_likely_virtual(box1)
-        h.reset_keep_likely_virtuals()
-        assert not h.is_unescaped(box1)
-        assert h.is_likely_virtual(box1)
-        h.reset()     # reset everything
-        assert not h.is_unescaped(box1)
-        assert not h.is_likely_virtual(box1)
-
-    def test_is_likely_virtual_3(self):
-        h = HeapCache()
-        box1 = RefFrontendOp(1)
-        h.new(box1)
-        assert h.is_unescaped(box1)
-        assert h.is_likely_virtual(box1)
-        h.reset_keep_likely_virtuals()
-        assert not h.is_unescaped(box1)
-        assert h.is_likely_virtual(box1)
-        h.class_now_known(box1)     # interaction of the two families of flags
-        assert not h.is_unescaped(box1)
-        assert h.is_likely_virtual(box1)
-
-    def test_is_likely_virtual_array(self):
-        h = HeapCache()
-        box1 = RefFrontendOp(1)
-        h.new_array(box1, index2)
-        assert h.is_likely_virtual(box1)
-        box2 = RefFrontendOp(2)
-        lengthbox = IntFrontendOp(11)
-        # arrays are only virtual if the length is constant
-        h.new_array(box2, lengthbox)
-        assert not h.is_likely_virtual(box2)
-
-    def test_quasiimmut_seen(self):
-        h = HeapCache()
-        box1 = RefFrontendOp(1)
-        box2 = RefFrontendOp(2)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
-        assert not h.is_quasi_immut_known(descr1, box1)
-        assert not h.is_quasi_immut_known(descr1, box2)
-        assert not h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-        h.quasi_immut_now_known(descr1, box1)
-        assert h.is_quasi_immut_known(descr1, box1)
-        assert not h.is_quasi_immut_known(descr1, box2)
-        assert not h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-        h.quasi_immut_now_known(descr1, box2)
-        assert h.is_quasi_immut_known(descr1, box1)
-        assert h.is_quasi_immut_known(descr1, box2)
-        assert not h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-        h.quasi_immut_now_known(descr2, box3)
-        assert h.is_quasi_immut_known(descr1, box1)
-        assert h.is_quasi_immut_known(descr1, box2)
-        assert h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-        h.quasi_immut_now_known(descr2, box4)
-        assert h.is_quasi_immut_known(descr1, box1)
-        assert h.is_quasi_immut_known(descr1, box2)
-        assert h.is_quasi_immut_known(descr2, box3)
-        assert h.is_quasi_immut_known(descr2, box4)
-
-        # invalidate the descr1 cache
-
-        h.setfield(box1, box3, descr1)
-        assert not h.is_quasi_immut_known(descr1, box1)
-        assert not h.is_quasi_immut_known(descr1, box2)
-
-        # a call invalidates everything
-        h.invalidate_caches_varargs(
-            rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE), [])
-        assert not h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-
-
-    def test_quasiimmut_seen_consts(self):
-        h = HeapCache()
-        box1 = ConstPtr(rffi.cast(llmemory.GCREF, 1))
-        box2 = ConstPtr(rffi.cast(llmemory.GCREF, 1))
-        box3 = ConstPtr(rffi.cast(llmemory.GCREF, 1))
-        box4 = ConstPtr(rffi.cast(llmemory.GCREF, 1))
-        assert not h.is_quasi_immut_known(descr1, box1)
-        assert not h.is_quasi_immut_known(descr1, box2)
-        assert not h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-        h.quasi_immut_now_known(descr1, box1)
-        assert h.is_quasi_immut_known(descr1, box1)
-        assert h.is_quasi_immut_known(descr1, box2)
-        assert not h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-        h.quasi_immut_now_known(descr2, box3)
-        assert h.is_quasi_immut_known(descr1, box1)
-        assert h.is_quasi_immut_known(descr1, box2)
-        assert h.is_quasi_immut_known(descr2, box3)
-        assert h.is_quasi_immut_known(descr2, box4)
-
-        # invalidate the descr1 cache
-
-        vbox1 = RefFrontendOp(1)
-        vbox2 = RefFrontendOp(2)
-        h.setfield(vbox1, vbox2, descr1)
-        assert not h.is_quasi_immut_known(descr1, box1)
-        assert not h.is_quasi_immut_known(descr1, box2)
-
-        # a call invalidates everything
-        h.invalidate_caches_varargs(
-            rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE), [])
-        assert not h.is_quasi_immut_known(descr2, box3)
-        assert not h.is_quasi_immut_known(descr2, box4)
-
-    def test_heapcache_on_const(self):
-        h = HeapCache()
-        # two different boxes of the same value
-        box1 = ConstPtr(ConstPtr.value)
-        box2 = ConstPtr(ConstPtr.value)
-        box3 = RefFrontendOp(3)
-        box4 = RefFrontendOp(4)
-
-        # the cache unifies a bit
-        assert h.getfield(box1, descr1) is None
-        assert h.getfield(box2, descr1) is None
-        h.getfield_now_known(box1, descr1, box3)
-        assert h.heap_cache[descr1].last_const_box is box1
-        assert h.heap_cache[descr1]._unique_const_heuristic(box2) is box1
-        assert h.getfield(box1, descr1) is box3
-        assert h.getfield(box2, descr1) is box3
-
-        h.reset()
-        h.setfield(box2, box4, descr1)
-        assert h.getfield(box1, descr1) is box4
-        assert h.getfield(box2, descr1) is box4
