@@ -135,15 +135,6 @@ class ListTests:
         assert res == 7
         self.check_resops(call=0)
 
-    def test_arraymove_simpleoptimize(self):
-        def f():
-            l = [10, 20, 30, 40]
-            l.insert(1, 999)
-            return len(l) + l[1] + l[-1]
-
-        res = self.interp_operations(f, [], listops=True)
-        assert res == 5 + 999 + 40
-
     def test_fold_getitem_1(self):
         jitdriver = JitDriver(greens = ['pc', 'n', 'l'], reds = ['total'])
         def f(n):
@@ -221,10 +212,7 @@ class ListTests:
                 s += lst[0]
                 lst.pop()
                 lst.append(1)
-                lst.insert(0, 5)
-                lst.insert(0, 5)
-                lst.insert(1, 6)
-                s *= lst.pop()
+                s /= lst.pop()
             return s
         res = self.meta_interp(f, [15], listops=True)
         assert res == f(15)
@@ -346,35 +334,6 @@ class ListTests:
                            'guard_true': 2,
                            'jump': 1})
 
-    def test_conditional_call_append(self):
-        jitdriver = JitDriver(greens = [], reds = 'auto')
-
-        def f(n):
-            l = []
-            while n > 0:
-                jitdriver.jit_merge_point()
-                l.append(n)
-                n -= 1
-            return len(l)
-
-        res = self.meta_interp(f, [10])
-        assert res == 10
-        self.check_resops(call=0, cond_call=2)
-
-    def test_conditional_call_pop(self):
-        jitdriver = JitDriver(greens = [], reds = 'auto')
-
-        def f(n):
-            l = range(n)
-            while n > 0:
-                jitdriver.jit_merge_point()
-                l.pop()
-                n -= 1
-            return len(l)
-
-        res = self.meta_interp(f, [10])
-        assert res == 0
-        self.check_resops(call=0, cond_call=2)
 
 class TestLLtype(ListTests, LLJitMixin):
     def test_listops_dont_invalidate_caches(self):
@@ -405,6 +364,36 @@ class TestLLtype(ListTests, LLJitMixin):
         # itself
         self.check_resops(getfield_gc_i=2, getfield_gc_r=5)
 
+    def test_conditional_call_append(self):
+        jitdriver = JitDriver(greens = [], reds = 'auto')
+
+        def f(n):
+            l = []
+            while n > 0:
+                jitdriver.jit_merge_point()
+                l.append(n)
+                n -= 1
+            return len(l)
+
+        res = self.meta_interp(f, [10])
+        assert res == 10
+        self.check_resops(call=0, cond_call=2)
+
+    def test_conditional_call_pop(self):
+        jitdriver = JitDriver(greens = [], reds = 'auto')
+
+        def f(n):
+            l = range(n)
+            while n > 0:
+                jitdriver.jit_merge_point()
+                l.pop()
+                n -= 1
+            return len(l)
+
+        res = self.meta_interp(f, [10])
+        assert res == 0
+        self.check_resops(call=0, cond_call=2)
+
     def test_zero_init_resizable(self):
         def f(n):
             l = [0] * n
