@@ -74,14 +74,9 @@ class TestLLtype(LLJitMixin):
     def test_heap_caching_while_tracing(self):
         class A:
             pass
-
-        @jit.dont_look_inside
-        def get():
-            return A()
-
+        a1 = A()
+        a2 = A()
         def fn(n):
-            a1 = get()
-            a2 = get()
             if n > 0:
                 a = a1
             else:
@@ -96,8 +91,6 @@ class TestLLtype(LLJitMixin):
         self.check_operations_history(getfield_gc_i=0)
 
         def fn(n, ca, cb):
-            a1 = get()
-            a2 = get()
             a1.x = n
             a2.x = n
             a = a1
@@ -112,7 +105,7 @@ class TestLLtype(LLJitMixin):
         self.check_operations_history(getfield_gc_i=1)
         res = self.interp_operations(fn, [-7, 1, 1])
         assert res == -7 * 2
-        self.check_operations_history(getfield_gc_i=0)
+        self.check_operations_history(getfield_gc_i=1)
 
     def test_heap_caching_nonnull(self):
         class A:
@@ -139,18 +132,17 @@ class TestLLtype(LLJitMixin):
     def test_heap_caching_while_tracing_invalidation(self):
         class A:
             pass
+        a1 = A()
+        a2 = A()
         @jit.dont_look_inside
         def f(a):
             a.x = 5
-        @jit.dont_look_inside
-        def get():
-            return A()
         l = [1]
         def fn(n):
             if n > 0:
-                a = get()
+                a = a1
             else:
-                a = get()
+                a = a2
             a.x = n
             x1 = a.x
             f(a)
@@ -164,14 +156,13 @@ class TestLLtype(LLJitMixin):
     def test_heap_caching_dont_store_same(self):
         class A:
             pass
-        @jit.dont_look_inside
-        def get():
-            return A()
+        a1 = A()
+        a2 = A()
         def fn(n):
             if n > 0:
-                a = get()
+                a = a1
             else:
-                a = get()
+                a = a2
             a.x = n
             a.x = n
             return a.x
@@ -218,42 +209,6 @@ class TestLLtype(LLJitMixin):
         assert res == -7 * 2
         self.check_operations_history(getarrayitem_gc_i=1)
 
-    def test_array_caching_float(self):
-        a1 = [0.0, 0.0]
-        a2 = [0.0, 0.0]
-        def fn(n):
-            if n > 0:
-                a = a1
-            else:
-                a = a2
-            a[0] = n + 0.01
-            x1 = a[0]
-            a[n - n] = n + 0.1
-            return a[0] + x1
-        res = self.interp_operations(fn, [7])
-        assert res == 7 + 7 + 0.01 + 0.1
-        self.check_operations_history(getarrayitem_gc_f=1)
-        res = self.interp_operations(fn, [-7])
-        assert res == -7 - 7 + 0.01 + 0.1
-        self.check_operations_history(getarrayitem_gc_f=1)
-
-        def fn(n, ca, cb):
-            a1[0] = n + 0.01
-            a2[0] = n + 0.01
-            a = a1
-            if ca:
-                a = a2
-            b = a1
-            if cb:
-                b = a
-            return a[0] + b[0]
-        res = self.interp_operations(fn, [7, 0, 1])
-        assert res == (7 + 0.01) * 2
-        self.check_operations_history(getarrayitem_gc_f=1)
-        res = self.interp_operations(fn, [-7, 1, 1])
-        assert res == (-7 + 0.01) * 2
-        self.check_operations_history(getarrayitem_gc_f=1)
-
     def test_array_caching_while_tracing_invalidation(self):
         a1 = [0, 0]
         a2 = [0, 0]
@@ -279,16 +234,14 @@ class TestLLtype(LLJitMixin):
 
     def test_array_and_getfield_interaction(self):
         class A: pass
-        @jit.dont_look_inside
-        def get():
-            a = A()
-            a.l = [0, 0]
-            return a
+        a1 = A()
+        a2 = A()
+        a1.l = a2.l = [0, 0]
         def fn(n):
             if n > 0:
-                a = get()
+                a = a1
             else:
-                a = get()
+                a = a2
                 a.l = [0, 0]
             a.x = 0
             a.l[a.x] = n
@@ -305,17 +258,15 @@ class TestLLtype(LLJitMixin):
 
     def test_promote_changes_heap_cache(self):
         class A: pass
-        @jit.dont_look_inside
-        def get():
-            a = A()
-            a.l = [0, 0]
-            a.x = 0
-            return a
+        a1 = A()
+        a2 = A()
+        a1.l = a2.l = [0, 0]
+        a1.x = a2.x = 0
         def fn(n):
             if n > 0:
-                a = get()
+                a = a1
             else:
-                a = get()
+                a = a2
                 a.l = [0, 0]
             jit.promote(a.x)
             a.l[a.x] = n
@@ -332,11 +283,13 @@ class TestLLtype(LLJitMixin):
             getfield_gc_r=1)
 
     def test_promote_changes_array_cache(self):
-        @jit.dont_look_inside
-        def get():
-            return [0, 0]
+        a1 = [0, 0]
+        a2 = [0, 0]
         def fn(n):
-            a = get()
+            if n > 0:
+                a = a1
+            else:
+                a = a2
             a[0] = n
             jit.hint(n, promote=True)
             x1 = a[0]
@@ -352,12 +305,13 @@ class TestLLtype(LLJitMixin):
 
 
     def test_list_caching(self):
-        @jit.dont_look_inside
-        def get():
-            return [0, 0]
+        a1 = [0, 0]
+        a2 = [0, 0]
         def fn(n):
-            a = get()
-            if not n > 0:
+            if n > 0:
+                a = a1
+            else:
+                a = a2
                 if n < -1000:
                     a.append(5)
             a[0] = n
@@ -374,8 +328,6 @@ class TestLLtype(LLJitMixin):
                 getfield_gc_r=1)
 
         def fn(n, ca, cb):
-            a1 = get()
-            a2 = get()
             a1[0] = n
             a2[0] = n
             a = a1
@@ -390,11 +342,11 @@ class TestLLtype(LLJitMixin):
         res = self.interp_operations(fn, [7, 0, 1])
         assert res == 7 * 2
         self.check_operations_history(getarrayitem_gc_i=1,
-                getfield_gc_r=2)
+                getfield_gc_r=3)
         res = self.interp_operations(fn, [-7, 1, 1])
         assert res == -7 * 2
-        self.check_operations_history(getarrayitem_gc_i=0,
-                getfield_gc_r=2)
+        self.check_operations_history(getarrayitem_gc_i=1,
+                getfield_gc_r=3)
 
     def test_list_caching_negative(self):
         def fn(n):
@@ -467,43 +419,6 @@ class TestLLtype(LLJitMixin):
         self.check_history(getarrayitem_gc_i=0, getfield_gc_i=0,
                            getfield_gc_r=0)
 
-
-    def test_nonstandard_virtualizable(self):
-        myjitdriver = jit.JitDriver(greens = [], reds = ['n', 'x', 'i', 'frame'],
-                                    virtualizables = ['frame'])
-
-        class Frame(object):
-            _virtualizable_ = ['s']
-
-            def __init__(self, s):
-                self.s = s
-                self.next = None
-
-        def f(n, a, i):
-            frame = Frame(5)
-            x = 0
-            while n > 0:
-                myjitdriver.can_enter_jit(frame=frame, n=n, x=x, i=i)
-                myjitdriver.jit_merge_point(frame=frame, n=n, x=x, i=i)
-                n -= 1
-                s = frame.s
-                assert s >= 0
-                frame.s += 1
-                # make a new frame
-                f = Frame(7)
-                frame.next = f
-                x += f.s
-                frame.s -= 1
-                frame.next = None
-            return x
-
-        res = self.meta_interp(f, [10, 1, 1], listops=True)
-        assert res == f(10, 1, 1)
-        # we now that f is not the standard virtualizable, since we've seen its
-        # allocation
-        self.check_history(ptr_eq=0)
-
-
     def test_heap_caching_array_pure(self):
         class A(object):
             pass
@@ -529,15 +444,17 @@ class TestLLtype(LLJitMixin):
     def test_heap_caching_and_elidable_function(self):
         class A:
             pass
-        @jit.dont_look_inside
-        def get():
-            return A()
+        class B: pass
+        a1 = A()
+        a1.y = 6
+        a2 = A()
+        a2.y = 13
         @jit.elidable
         def f(b):
             return b + 1
         def fn(n):
             if n > 0:
-                a = get()
+                a = a1
             else:
                 a = A()
             a.x = n
@@ -549,6 +466,7 @@ class TestLLtype(LLJitMixin):
         res = self.interp_operations(fn, [-7])
         assert res == -7 + 7
         self.check_operations_history(getfield_gc_i=0)
+        return
 
     def test_heap_caching_multiple_objects(self):
         class Gbl(object):
@@ -584,57 +502,6 @@ class TestLLtype(LLJitMixin):
         assert res == 4 * 7
         self.check_operations_history(getfield_gc_i=2, getfield_gc_r=2)
 
-    def test_heap_caching_quasi_immutable(self):
-        class A:
-            _immutable_fields_ = ['x?']
-        a1 = A()
-        a1.x = 5
-        a2 = A()
-        a2.x = 7
-
-        @jit.elidable
-        def get(n):
-            if n > 0:
-                return a1
-            return a2
-
-        def g(a):
-            return a.x
-
-        def fn(n):
-            jit.promote(n)
-            a = get(n)
-            return g(a) + a.x
-        res = self.interp_operations(fn, [7])
-        assert res == 10
-        self.check_operations_history(quasiimmut_field=1)
-
-    def test_heap_caching_quasi_immutable_2(self):
-        class A:
-            _immutable_fields_ = ['x?']
-        a1 = A()
-        a1.x = 5
-        a2 = A()
-        a2.x = 7
-
-        @jit.elidable
-        def get(n):
-            if n > 0:
-                return a1
-            return a2
-
-        def g(a):
-            return a.x
-
-        def fn(n):
-            jit.promote(n)
-            return get(n).x + get(n).x
-        res = self.interp_operations(fn, [7])
-        assert res == 10
-        self.check_operations_history(quasiimmut_field=1)
-
-
-
     def test_heap_caching_multiple_tuples(self):
         class Gbl(object):
             pass
@@ -840,68 +707,3 @@ class TestLLtype(LLJitMixin):
         res = self.interp_operations(fn, [0])
         assert res == 0
         self.check_operations_history(setfield_gc=0)
-
-    def test_record_known_class_does_not_invalidate(self):
-        class A:
-            pass
-        class B(A):
-            pass
-        class C(object):
-            _immutable_fields_ = ['x?']
-        c = C()
-        c.x = 5
-        c.b = A()
-        c.b.x = 14
-        def fn(n):
-            if n == 99:
-                c.x = 12
-                c.b = B()
-                c.b.x = 12
-                return 15
-            b = c.b
-            x = b.x
-            jit.record_exact_class(c.b, A)
-            y = b.x
-            return x + y
-        res = self.interp_operations(fn, [1])
-        assert res == 2 * 14
-        self.check_operations_history(getfield_gc_i=1)
-
-    def test_loop_invariant1(self):
-        class A(object):
-            pass
-        a = A()
-        a.current_a = A()
-        a.current_a.x = 1
-        @jit.loop_invariant
-        def f():
-            return a.current_a
-
-        @jit.loop_invariant
-        def f1():
-            return a.current_a
-
-        def g(x):
-            res = 0
-            res += f().x
-            res += f().x
-            res += f().x
-            res += f1().x # not reused!
-            res += f1().x
-            if x > 1000:
-                a.current_a = A()
-                a.current_a.x = 2
-            return res
-        res = self.interp_operations(g, [21])
-        assert res == g(21)
-        self.check_operations_history(call_loopinvariant_r=2)
-
-    def test_heapcache_interiorfields(self):
-        def fn(n):
-            d = {1: n, 2: n}
-            d[4] = n + 1
-            return d[4]
-        res = self.interp_operations(fn, [0])
-        assert res == 1
-        self.check_operations_history(getinteriorfield_gc_i=0)
-
