@@ -1,18 +1,15 @@
 import sys
 import weakref
 
-from rpython.jit.codewriter import support, longlong
-from rpython.jit.metainterp import resoperation, history, jitexc
-from rpython.jit.metainterp.support import ptr2int, int2adr
+from rpython.jit.codewriter import support, heaptracker, longlong
+from rpython.jit.metainterp import resoperation, history
 from rpython.rlib.debug import debug_start, debug_stop, debug_print
 from rpython.rlib.debug import have_debug_prints_for
 from rpython.rlib.jit import PARAMETERS
-from rpython.rlib.rjitlog import rjitlog as jl
 from rpython.rlib.nonconst import NonConstant
 from rpython.rlib.objectmodel import specialize, we_are_translated, r_dict
 from rpython.rlib.rarithmetic import intmask, r_uint
 from rpython.rlib.unroll import unrolling_iterable
-from rpython.rlib import rstack
 from rpython.rtyper.annlowlevel import (hlstr, cast_base_ptr_to_instance,
     cast_object_to_ptr)
 from rpython.rtyper.lltypesystem import lltype, llmemory, rstr, rffi
@@ -48,7 +45,8 @@ def unspecialize_value(value):
         if lltype.typeOf(value).TO._gckind == 'gc':
             return lltype.cast_opaque_ptr(llmemory.GCREF, value)
         else:
-            return ptr2int(value)
+            adr = llmemory.cast_ptr_to_adr(value)
+            return heaptracker.adr2int(adr)
     elif isinstance(value, float):
         return longlong.getfloatstorage(value)
     else:
@@ -62,7 +60,7 @@ def unwrap(TYPE, box):
         if TYPE.TO._gckind == "gc":
             return box.getref(TYPE)
         else:
-            adr = int2adr(box.getint())
+            adr = heaptracker.int2adr(box.getint())
             return llmemory.cast_adr_to_ptr(adr, TYPE)
     if TYPE == lltype.Float:
         return box.getfloat()
@@ -77,11 +75,10 @@ def wrap(cpu, value, in_const_box=False):
             if in_const_box:
                 return history.ConstPtr(value)
             else:
-                res = history.RefFrontendOp(0)
-                res.setref_base(value)
-                return res
+                return resoperation.InputArgRef(value)
         else:
-            value = ptr2int(value)
+            adr = llmemory.cast_ptr_to_adr(value)
+            value = heaptracker.adr2int(adr)
             # fall through to the end of the function
     elif (isinstance(value, float) or
           longlong.is_longlong(lltype.typeOf(value))):
@@ -92,9 +89,7 @@ def wrap(cpu, value, in_const_box=False):
         if in_const_box:
             return history.ConstFloat(value)
         else:
-            res = history.FloatFrontendOp(0)
-            res.setfloatstorage(value)
-            return res
+            return resoperation.InputArgFloat(value)
     elif isinstance(value, str) or isinstance(value, unicode):
         assert len(value) == 1     # must be a character
         value = ord(value)
@@ -105,9 +100,7 @@ def wrap(cpu, value, in_const_box=False):
     if in_const_box:
         return history.ConstInt(value)
     else:
-        res = history.IntFrontendOp(0)
-        res.setint(value)
-        return res
+        return resoperation.InputArgInt(value)
 
 @specialize.arg(0)
 def equal_whatever(TYPE, x, y):
@@ -137,7 +130,6 @@ JC_TRACING         = 0x01
 JC_DONT_TRACE_HERE = 0x02
 JC_TEMPORARY       = 0x04
 JC_TRACING_OCCURRED= 0x08
-JC_FORCE_FINISH    = 0x10
 
 class BaseJitCell(object):
     """Subclasses of BaseJitCell are used in tandem with the single
@@ -150,7 +142,7 @@ class BaseJitCell(object):
     app-level Python code.
 
     We create subclasses of BaseJitCell --one per jitdriver-- so that
-    they can store greenkeys of different types.
+    they can store greenkeys of different types.  
 
     Note that we don't create a JitCell the first time we see a given
     greenkey position in the interpreter.  At first, we only hash the
@@ -184,10 +176,6 @@ class BaseJitCell(object):
         this particular function.  (We only set this flag when aborting
         due to a trace too long, so we use the same flag as a hint to
         also mean "please trace from here as soon as possible".)
-
-        JC_FORCE_FINISH: when from a cell with that flag set, if the trace
-        becomes too long, "segment" it, ie finish it with a guard_always_fails.
-        this prevents re-tracing and failing this again and again.
     """
     flags = 0     # JC_xxx flags
     wref_procedure_token = None
@@ -224,10 +212,6 @@ class BaseJitCell(object):
             # we no longer have one, then remove me.  this prevents this
             # JitCell from being immortal.
             return self.has_seen_a_procedure_token()     # i.e. dead weakref
-        if self.flags & JC_FORCE_FINISH:
-            # don't remove, we need to remember that we should really finish a
-            # trace for this
-            return False
         return True   # Other JitCells can be removed.
 
 # ____________________________________________________________
@@ -265,10 +249,6 @@ class WarmEnterState(object):
         self.increment_trace_eagerness = self._compute_threshold(value)
 
     def set_param_trace_limit(self, value):
-        if value < 0:
-            raise ValueError
-        if value > self.warmrunnerdesc.metainterp_sd.opencoder_model.MAX_TRACE_LIMIT:
-            raise ValueError
         self.trace_limit = value
 
     def set_param_decay(self, decay):
@@ -306,9 +286,6 @@ class WarmEnterState(object):
             if self.warmrunnerdesc.memory_manager:
                 self.warmrunnerdesc.memory_manager.retrace_limit = value
 
-    def set_param_pureop_historylength(self, value):
-        self.pureop_historylength = value
-
     def set_param_max_retrace_guards(self, value):
         if self.warmrunnerdesc:
             if self.warmrunnerdesc.memory_manager:
@@ -324,14 +301,23 @@ class WarmEnterState(object):
             if self.warmrunnerdesc.memory_manager:
                 self.warmrunnerdesc.memory_manager.max_unroll_recursion = value
 
-    def set_param_vec(self, ivalue):
-        self.vec = bool(ivalue)
+    def set_param_vec(self, value):
+        self.vec = bool(value)
 
-    def set_param_vec_all(self, ivalue):
-        self.vec_all = bool(ivalue)
+    def set_param_vec_all(self, value):
+        self.vec_all = bool(value)
 
-    def set_param_vec_cost(self, ivalue):
-        self.vec_cost = ivalue
+    def set_param_vec_cost(self, value):
+        self.vec_cost = bool(value)
+
+    def set_param_vec_length(self, value):
+        self.vec_length = int(value)
+
+    def set_param_vec_ratio(self, value):
+        self.vec_ratio = value / 10.0
+
+    def set_param_vec_guard_ratio(self, value):
+        self.vec_guard_ratio = value / 10.0
 
     def disable_noninlinable_function(self, greenkey):
         cell = self.JitCell.ensure_jit_cell_at_key(greenkey)
@@ -356,9 +342,8 @@ class WarmEnterState(object):
 
     def make_entry_point(self):
         "NOT_RPYTHON"
-        from rpython.jit.metainterp import compile
-        if hasattr(self, 'entry_point_fns'):
-            return self.entry_point_fns
+        if hasattr(self, 'maybe_compile_and_run'):
+            return self.maybe_compile_and_run
 
         warmrunnerdesc = self.warmrunnerdesc
         metainterp_sd = warmrunnerdesc.metainterp_sd
@@ -371,8 +356,6 @@ class WarmEnterState(object):
         confirm_enter_jit = self.confirm_enter_jit
         range_red_args = unrolling_iterable(
             range(num_green_args, num_green_args + jitdriver_sd.num_red_args))
-        name_red_args = unrolling_iterable(
-            [(i, 'arg%d' % i) for i in range(jitdriver_sd.num_red_args)])
         # get a new specialized copy of the method
         ARGS = []
         for kind in jitdriver_sd.red_args_types:
@@ -387,7 +370,6 @@ class WarmEnterState(object):
         func_execute_token = self.cpu.make_execute_token(*ARGS)
         cpu = self.cpu
         jitcounter = self.warmrunnerdesc.jitcounter
-        result_type = jitdriver_sd.result_type
 
         def execute_assembler(loop_token, *args):
             # Call the backend to run the 'looptoken' with the given
@@ -399,7 +381,7 @@ class WarmEnterState(object):
             if vinfo is not None:
                 virtualizable = args[index_of_virtualizable]
                 vinfo.clear_vable_token(virtualizable)
-
+            
             deadframe = func_execute_token(loop_token, *args)
             #
             # Record in the memmgr that we just ran this loop,
@@ -408,40 +390,21 @@ class WarmEnterState(object):
             #
             # Handle the failure
             fail_descr = cpu.get_latest_descr(deadframe)
-            # First, a fast path to avoid raising and immediately catching
-            # a DoneWithThisFrame exception
-            if result_type == history.VOID:
-                if isinstance(fail_descr, compile.DoneWithThisFrameDescrVoid):
-                    return None
-            if result_type == history.INT:
-                if isinstance(fail_descr, compile.DoneWithThisFrameDescrInt):
-                    return fail_descr.get_result(cpu, deadframe)
-            if result_type == history.REF:
-                if isinstance(fail_descr, compile.DoneWithThisFrameDescrRef):
-                    return fail_descr.get_result(cpu, deadframe)
-            if result_type == history.FLOAT:
-                if isinstance(fail_descr, compile.DoneWithThisFrameDescrFloat):
-                    return fail_descr.get_result(cpu, deadframe)
-            #
-            # General case
             fail_descr.handle_fail(deadframe, metainterp_sd, jitdriver_sd)
+            #
             assert 0, "should have raised"
 
         def bound_reached(hash, cell, *args):
-            from rpython.jit.metainterp.pyjitpl import MetaInterp
             if not confirm_enter_jit(*args):
                 return
             jitcounter.decay_all_counters()
-            if rstack.stack_almost_full():
-                return
+            # start tracing
+            from rpython.jit.metainterp.pyjitpl import MetaInterp
+            metainterp = MetaInterp(metainterp_sd, jitdriver_sd)
             greenargs = args[:num_green_args]
             if cell is None:
                 cell = JitCell(*greenargs)
                 jitcounter.install_new_cell(hash, cell)
-            # start tracing
-            metainterp = MetaInterp(
-                metainterp_sd, jitdriver_sd,
-                force_finish_trace=bool(cell.flags & JC_FORCE_FINISH))
             cell.flags |= JC_TRACING | JC_TRACING_OCCURRED
             try:
                 metainterp.compile_and_run_once(jitdriver_sd, *args)
@@ -450,11 +413,8 @@ class WarmEnterState(object):
 
         def maybe_compile_and_run(increment_threshold, *args):
             """Entry point to the JIT.  Called at the point with the
-            can_enter_jit() hint, and at the start of a function
-            with a different threshold.
+            can_enter_jit() hint.
             """
-            if increment_threshold == 0:
-                return # jit is off
             # Look for the cell corresponding to the current greenargs.
             # Search for the JitCell that is of the correct subclass of
             # BaseJitCell, and that stores a key that compares equal.
@@ -473,6 +433,14 @@ class WarmEnterState(object):
                     bound_reached(hash, None, *args)
                 return
 
+            # Workaround for issue #2200, maybe temporary.  This is not
+            # a proper fix, but only a hack that should work well enough
+            # for PyPy's main jitdriver...  See test_issue2200_recursion
+            from rpython.jit.metainterp.blackhole import workaround2200
+            if workaround2200.active:
+                workaround2200.active = False
+                return
+
             # Here, we have found 'cell'.
             #
             if cell.flags & (JC_TRACING | JC_TEMPORARY):
@@ -510,27 +478,14 @@ class WarmEnterState(object):
             execute_args = ()
             for i in range_red_args:
                 execute_args += (unspecialize_value(args[i]), )
-            # run it, but from outside in ll_portal_runner, not from here
-            # (this avoids RPython-level recursion with no corresponding
-            # app-level recursion, as shown by issues 2200 and 2335)
-            raise EnterJitAssembler(procedure_token, *execute_args)
-
-        class EnterJitAssembler(jitexc.JitException):
-            def __init__(self, procedure_token, *args):
-                self.procedure_token = procedure_token
-                for i, argname in name_red_args:
-                    setattr(self, argname, args[i])
-            def execute(self):
-                args = ()
-                for i, argname in name_red_args:
-                    args += (getattr(self, argname), )
-                return execute_assembler(self.procedure_token, *args)
+            # run it!  this executes until interrupted by an exception
+            execute_assembler(procedure_token, *execute_args)
+            assert 0, "should not reach this point"
 
         maybe_compile_and_run._dont_inline_ = True
+        self.maybe_compile_and_run = maybe_compile_and_run
         self.execute_assembler = execute_assembler
-        self.entry_point_fns = (maybe_compile_and_run,
-                                EnterJitAssembler)
-        return self.entry_point_fns
+        return maybe_compile_and_run
 
     # ----------
 
@@ -649,11 +604,6 @@ class WarmEnterState(object):
             def dont_trace_here(*greenargs):
                 cell = JitCell._ensure_jit_cell_at_key(*greenargs)
                 cell.flags |= JC_DONT_TRACE_HERE
-
-            @staticmethod
-            def mark_as_being_traced(*greenargs):
-                cell = JitCell._ensure_jit_cell_at_key(*greenargs)
-                cell.flags |= JC_TRACING
         #
         self.JitCell = JitCell
         return JitCell
@@ -691,18 +641,6 @@ class WarmEnterState(object):
             cell.flags |= JC_DONT_TRACE_HERE
         self.dont_trace_here = dont_trace_here
 
-        def mark_as_being_traced(greenkey):
-            cell = JitCell.ensure_jit_cell_at_key(greenkey)
-            cell.flags |= JC_TRACING
-        self.mark_as_being_traced = mark_as_being_traced
-
-        def mark_force_finish_tracing(greenkey):
-            """ mark greenkey as "please definitely finish a trace for it the
-            next time" """
-            cell = JitCell.ensure_jit_cell_at_key(greenkey)
-            cell.flags |= JC_FORCE_FINISH
-        self.mark_force_finish_tracing = mark_force_finish_tracing
-
         if jd._should_unroll_one_iteration_ptr is None:
             def should_unroll_one_iteration(greenkey):
                 return False
@@ -734,35 +672,8 @@ class WarmEnterState(object):
             drivername = jitdriver.name
         else:
             drivername = '<unknown jitdriver>'
-        # get_location returns
-        get_location_ptr = getattr(self.jitdriver_sd, '_get_location_ptr', None)
-        if get_location_ptr is not None:
-            types = self.jitdriver_sd._get_loc_types
-            unwrap_greenkey = self.make_unwrap_greenkey()
-            unrolled_types = unrolling_iterable(enumerate(types))
-            def get_location(greenkey):
-                greenargs = unwrap_greenkey(greenkey)
-                fn = support.maybe_on_top_of_llinterp(rtyper, get_location_ptr)
-                value_tuple = fn(*greenargs)
-                values = []
-                for i, (sem_type,gen_type) in unrolled_types:
-                    if gen_type == "s":
-                        value = getattr(value_tuple, 'item' + str(i))
-                        values.append(jl.wrap(sem_type,gen_type,hlstr(value)))
-                    elif gen_type == "i":
-                        value = getattr(value_tuple, 'item' + str(i))
-                        values.append(jl.wrap(sem_type,gen_type,intmask(value)))
-                    else:
-                        raise NotImplementedError
-                return values
-            self.get_location_types = list(types)
-            self.get_location = get_location
-        else:
-            self.get_location_types = None
-            self.get_location = None
-        #
-        printable_loc_ptr = self.jitdriver_sd._get_printable_location_ptr
-        if printable_loc_ptr is None:
+        get_location_ptr = self.jitdriver_sd._get_printable_location_ptr
+        if get_location_ptr is None:
             missing = '(%s: no get_printable_location)' % drivername
             def get_location_str(greenkey):
                 return missing
@@ -778,7 +689,7 @@ class WarmEnterState(object):
                 if not have_debug_prints_for("jit-"):
                     return missing
                 greenargs = unwrap_greenkey(greenkey)
-                fn = support.maybe_on_top_of_llinterp(rtyper, printable_loc_ptr)
+                fn = support.maybe_on_top_of_llinterp(rtyper, get_location_ptr)
                 llres = fn(*greenargs)
                 if not we_are_translated() and isinstance(llres, str):
                     return llres
