@@ -7,7 +7,6 @@ from rpython.rlib.debug import debug_print, debug_start, debug_stop
 from rpython.rlib.rstring import assert_str0
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.rtyper.lltypesystem.lloperation import llop
-from rpython.translator.tool.cbuild import ExternalCompilationInfo
 
 # ____________________________________________________________
 # Reading env vars.  Supports returning ints, uints or floats,
@@ -138,8 +137,6 @@ def get_L2cache_linux2():
         return get_L2cache_linux2_cpuinfo()
     if arch in ('alpha', 'ppc'):
         return get_L2cache_linux2_cpuinfo(label='L2 cache')
-    #if arch == 's390x':    untested
-    #    return get_L2cache_linux2_cpuinfo_s390x()
     if arch == 'ia64':
         return get_L2cache_linux2_ia64()
     if arch in ('parisc', 'parisc64'):
@@ -211,57 +208,6 @@ def get_L2cache_linux2_cpuinfo(filename="/proc/cpuinfo", label='cache size'):
             "Warning: cannot find your CPU L2 cache size in /proc/cpuinfo")
         return -1
 
-def get_L2cache_linux2_cpuinfo_s390x(filename="/proc/cpuinfo", label='cache2'):
-    debug_start("gc-hardware")
-    L2cache = sys.maxint
-    try:
-        fd = os.open(filename, os.O_RDONLY, 0644)
-        try:
-            data = []
-            while True:
-                buf = os.read(fd, 4096)
-                if not buf:
-                    break
-                data.append(buf)
-        finally:
-            os.close(fd)
-    except OSError:
-        pass
-    else:
-        data = ''.join(data)
-        linepos = 0
-        while True:
-            start = _findend(data, '\n' + label, linepos)
-            if start < 0:
-                break    # done
-            start = _findend(data, 'size=', start)
-            if start < 0:
-                break
-            end = _findend(data, ' ', start) - 1
-            if end < 0:
-                break
-            linepos = end
-            size = data[start:end]
-            last_char = len(size)-1
-            assert 0 <= last_char < len(size)
-            if size[last_char] not in ('K', 'k'):    # assume kilobytes for now
-                continue
-            number = int(size[:last_char])* 1024
-            # for now we look for the smallest of the L2 caches of the CPUs
-            if number < L2cache:
-                L2cache = number
-
-    debug_print("L2cache =", L2cache)
-    debug_stop("gc-hardware")
-
-    if L2cache < sys.maxint:
-        return L2cache
-    else:
-        # Print a top-level warning even in non-debug builds
-        llop.debug_print(lltype.Void,
-            "Warning: cannot find your CPU L2 cache size in /proc/cpuinfo")
-        return -1
-
 def get_L2cache_linux2_sparc():
     debug_start("gc-hardware")
     cpu = 0
@@ -376,13 +322,11 @@ def _skipspace(data, pos):
 
 # ---------- Darwin ----------
 
-sysctlbyname_eci = ExternalCompilationInfo(includes=["sys/sysctl.h"])
 sysctlbyname = rffi.llexternal('sysctlbyname',
                                [rffi.CCHARP, rffi.VOIDP, rffi.SIZE_TP,
                                 rffi.VOIDP, rffi.SIZE_T],
                                rffi.INT,
-                               sandboxsafe=True,
-                               compilation_info=sysctlbyname_eci)
+                               sandboxsafe=True)
 
 def get_darwin_sysctl_signed(sysctl_name):
     rval_p = lltype.malloc(rffi.LONGLONGP.TO, 1, flavor='raw')
