@@ -208,7 +208,7 @@ class GenerationGC(SemiSpaceGC):
             # due to inlining).
             maxlength_for_minimal_nursery = (self.min_nursery_size // 4 //
                                              raw_malloc_usage(itemsize))
-
+            
             # The actual maximum length for our nursery depends on how
             # many times our nursery is bigger than the minimal size.
             # The computation is done in this roundabout way so that
@@ -329,7 +329,7 @@ class GenerationGC(SemiSpaceGC):
             self.header(obj).tid |= GCFLAG_NO_HEAP_PTRS
             # ^^^ the flag we just added will be removed immediately if
             # the object still contains pointers to younger objects
-            self.trace(obj, self.make_callback('_trace_external_obj'), self, obj)
+            self.trace(obj, self._trace_external_obj, obj)
         stack.delete()
 
     def _trace_external_obj(self, pointer, obj):
@@ -355,7 +355,6 @@ class GenerationGC(SemiSpaceGC):
             scan = beginning = self.free
             self.collect_oldrefs_to_nursery()
             self.collect_roots_in_nursery()
-            self.collect_young_objects_with_finalizers()
             scan = self.scan_objects_just_copied_out_of_nursery(scan)
             # at this point, all static and old objects have got their
             # GCFLAG_NO_YOUNG_PTRS set again by trace_and_drag_out_of_nursery
@@ -423,19 +422,6 @@ class GenerationGC(SemiSpaceGC):
         if self.is_in_nursery(obj):
             root.address[0] = self.copy(obj)
 
-    def collect_young_objects_with_finalizers(self):
-        # XXX always walk the whole 'objects_with_finalizers' list here
-        new = self.AddressDeque()
-        while self.objects_with_finalizers.non_empty():
-            obj = self.objects_with_finalizers.popleft()
-            fq_nr = self.objects_with_finalizers.popleft()
-            if self.is_in_nursery(obj):
-                obj = self.copy(obj)
-            new.append(obj)
-            new.append(fq_nr)
-        self.objects_with_finalizers.delete()
-        self.objects_with_finalizers = new
-
     def scan_objects_just_copied_out_of_nursery(self, scan):
         while scan < self.free:
             curr = scan + self.size_gc_header()
@@ -447,7 +433,7 @@ class GenerationGC(SemiSpaceGC):
         """obj must not be in the nursery.  This copies all the
         young objects it references out of the nursery.
         """
-        self.trace(obj, self.make_callback('_trace_drag_out'), self, None)
+        self.trace(obj, self._trace_drag_out, None)
 
     def _trace_drag_out(self, pointer, ignored):
         if self.is_in_nursery(pointer.address[0]):
@@ -543,9 +529,6 @@ class GenerationGC(SemiSpaceGC):
                 self.last_generation_root_objects.append(dest_addr)
         return True
 
-    def writebarrier_before_move(self, array_addr):
-        pass      # nothing to do
-
     def is_last_generation(self, obj):
         # overridden by HybridGC
         return (self.header(obj).tid & GCFLAG_EXTERNAL) != 0
@@ -591,14 +574,14 @@ class GenerationGC(SemiSpaceGC):
         if tid & GCFLAG_NO_YOUNG_PTRS:
             ll_assert(not self.is_in_nursery(obj),
                       "nursery object with GCFLAG_NO_YOUNG_PTRS")
-            self.trace(obj, self.make_callback('_debug_no_nursery_pointer'), self, None)
+            self.trace(obj, self._debug_no_nursery_pointer, None)
         elif not self.is_in_nursery(obj):
             ll_assert(self._d_oopty.contains(obj),
                       "missing from old_objects_pointing_to_young")
         if tid & GCFLAG_NO_HEAP_PTRS:
             ll_assert(self.is_last_generation(obj),
                       "GCFLAG_NO_HEAP_PTRS on non-3rd-generation object")
-            self.trace(obj, self.make_callback('_debug_no_gen1or2_pointer'), self, None)
+            self.trace(obj, self._debug_no_gen1or2_pointer, None)
         elif self.is_last_generation(obj):
             ll_assert(self._d_lgro.contains(obj),
                       "missing from last_generation_root_objects")
