@@ -46,16 +46,15 @@ def stacklet_destructor(stacklet):
 
 SIZEADDR = llmemory.sizeof(llmemory.Address)
 
-def customtrace(gc, obj, callback, arg1, arg2):
-    from rpython.memory.gctransform.shadowstack import walk_stack_root
-
+def customtrace(gc, obj, callback, arg):
     stacklet = llmemory.cast_adr_to_ptr(obj, STACKLET_PTR)
     sscopy = stacklet.s_sscopy
     if sscopy:
         length_bytes = sscopy.signed[0]
-        walk_stack_root(gc._trace_callback, callback, arg1, arg2,
-                        sscopy + SIZEADDR, sscopy + SIZEADDR + length_bytes,
-                        is_minor=False)
+        while length_bytes > 0:
+            addr = sscopy + length_bytes
+            gc._trace_callback(callback, arg, addr)
+            length_bytes -= SIZEADDR
 lambda_customtrace = lambda: customtrace
 
 def sscopy_detach_shadow_stack():
@@ -77,7 +76,6 @@ def sscopy_attach_shadow_stack(sscopy):
     llmemory.raw_memcopy(sscopy + SIZEADDR, base, length_bytes)
     llop.gc_adr_of_root_stack_top(llmemory.Address).address[0] = (
         base + length_bytes)
-    llop.gc_modified_shadowstack(lltype.Void)
     llmemory.raw_free(sscopy)
 
 def alloc_stacklet():
