@@ -1,11 +1,9 @@
 import sys
 import time
-from collections import Counter
 
-from rpython.rlib.objectmodel import enforceargs
 from rpython.rtyper.extregistry import ExtRegistryEntry
-from rpython.rlib.objectmodel import we_are_translated, always_inline
-from rpython.rlib.rarithmetic import is_valid_int, r_longlong
+from rpython.rlib.objectmodel import we_are_translated
+from rpython.rlib.rarithmetic import is_valid_int
 from rpython.rtyper.extfunc import register_external
 from rpython.rtyper.lltypesystem import lltype
 from rpython.rtyper.lltypesystem import rffi
@@ -13,8 +11,7 @@ from rpython.translator.tool.cbuild import ExternalCompilationInfo
 
 # Expose these here (public interface)
 from rpython.rtyper.debug import (
-    ll_assert, FatalError, fatalerror, fatalerror_notb, debug_print_traceback,
-    ll_assert_not_none)
+    ll_assert, FatalError, fatalerror, fatalerror_notb)
 
 
 class DebugLog(list):
@@ -39,23 +36,6 @@ class DebugLog(list):
         assert False, ("nesting error: no start corresponding to stop %r" %
                        (category,))
 
-    def reset(self):
-        # only for tests: empty the log
-        self[:] = []
-
-    def summary(self, flatten=False):
-        res = Counter()
-        def visit(lst):
-            for section, sublist in lst:
-                if section == 'debug_print':
-                    continue
-                res[section] += 1
-                if flatten:
-                    visit(sublist)
-        #
-        visit(self)
-        return res
-
     def __repr__(self):
         import pprint
         return pprint.pformat(list(self))
@@ -93,68 +73,34 @@ else:
     _start_colors_2 = ""
     _stop_colors = ""
 
-@always_inline
-@enforceargs(str, bool)
-def debug_start(category, timestamp=False):
-    """
-    Start a PYPYLOG section.
-
-    By default, the return value is undefined.  If timestamp is True, always
-    return the current timestamp, even if PYPYLOG is not set.
-    """
-    return _debug_start(category, timestamp)
-
-@always_inline
-@enforceargs(str, bool)
-def debug_stop(category, timestamp=False):
-    """
-    Stop a PYPYLOG section. See debug_start for docs about timestamp
-    """
-    return _debug_stop(category, timestamp)
-
-
-def _debug_start(category, timestamp):
+def debug_start(category):
     c = int(time.clock() * 100)
     print >> sys.stderr, '%s[%x] {%s%s' % (_start_colors_1, c,
                                            category, _stop_colors)
     if _log is not None:
         _log.debug_start(category)
 
-    if timestamp:
-        return r_longlong(c)
-    return r_longlong(-42) # random undefined value
-
-def _debug_stop(category, timestamp):
+def debug_stop(category):
     c = int(time.clock() * 100)
     print >> sys.stderr, '%s[%x] %s}%s' % (_start_colors_2, c,
                                            category, _stop_colors)
     if _log is not None:
         _log.debug_stop(category)
 
-    if timestamp:
-        return r_longlong(c)
-    return r_longlong(-42) # random undefined value
-
 class Entry(ExtRegistryEntry):
-    _about_ = _debug_start, _debug_stop
+    _about_ = debug_start, debug_stop
 
-    def compute_result_annotation(self, s_category, s_timestamp):
-        from rpython.rlib.rtimer import s_TIMESTAMP
-        return s_TIMESTAMP
+    def compute_result_annotation(self, s_category):
+        return None
 
     def specialize_call(self, hop):
         from rpython.rtyper.lltypesystem.rstr import string_repr
-        from rpython.rlib.rtimer import TIMESTAMP_type
         fn = self.instance
-        _, r_timestamp = hop.args_r
-        vlist = hop.inputargs(string_repr, r_timestamp)
+        vlist = hop.inputargs(string_repr)
         hop.exception_cannot_occur()
         t = hop.rtyper.annotator.translator
         if t.config.translation.log:
-            opname = fn.__name__[1:] # remove the '_'
-            return hop.genop(opname, vlist, resulttype=TIMESTAMP_type)
-        else:
-            return hop.inputconst(TIMESTAMP_type, r_longlong(0))
+            hop.genop(fn.__name__, vlist)
 
 
 def have_debug_prints():
@@ -212,9 +158,7 @@ class Entry(ExtRegistryEntry):
 
 
 def debug_flush():
-    """ Flushes the debug file.
-
-    With the reverse-debugger, it also closes the output log.
+    """ Flushes the debug file
     """
     pass
 
@@ -309,22 +253,6 @@ class Entry(ExtRegistryEntry):
         hop.exception_cannot_occur()
         return hop.inputarg(hop.args_r[0], arg=0)
 
-def check_not_access_directly(arg):
-    """ check that arg does not have the access_directly=True hint set """
-    return arg
-
-class Entry(ExtRegistryEntry):
-    _about_ = check_not_access_directly
-
-    def compute_result_annotation(self, s_arg):
-        assert not s_arg.flags.get('access_directly', False)
-        return s_arg
-
-    def specialize_call(self, hop):
-        hop.exception_cannot_occur()
-        return hop.inputarg(hop.args_r[0], arg=0)
-
-
 def make_sure_not_resized(arg):
     """ Function checking whether annotation of SomeList is never resized,
     useful for debugging. Does nothing when run directly
@@ -359,9 +287,6 @@ class Entry(ExtRegistryEntry):
 def mark_dict_non_null(d):
     """ Mark dictionary as having non-null keys and values. A warning would
     be emitted (not an error!) in case annotation disagrees.
-
-    This doesn't work for r_dicts. For them, pass
-    r_dict(..., force_non_null=True) to the constructor.
     """
     assert isinstance(d, dict)
     return d
@@ -390,21 +315,14 @@ class UnexpectedRUInt(Exception):
 class ExpectedRegularInt(Exception):
     pass
 
-class NegativeArgumentNotAllowed(Exception):
-    pass
-
 def check_nonneg(x):
     """Give a translation-time error if 'x' is not known to be non-negative.
     To help debugging, this also gives a translation-time error if 'x' is
     actually typed as an r_uint (in which case the call to check_nonneg()
     is a bit strange and probably unexpected).
     """
-    try:
-        assert type(x)(-1) < 0     # otherwise, 'x' is a r_uint or similar
-    except NegativeArgumentNotAllowed:
-        pass
-    else:
-        assert x >= 0
+    assert type(x)(-1) < 0     # otherwise, 'x' is a r_uint or similar
+    assert x >= 0
     return x
 
 class Entry(ExtRegistryEntry):
@@ -522,91 +440,13 @@ static void pypy__allow_attach(void) {
             except OSError as e:
                 os.write(2, "Could not start GDB: %s" % (
                     os.strerror(e.errno)))
-                os._exit(1)
+                raise SystemExit
         else:
             time.sleep(1)  # give the GDB time to attach
 
 else:
-    def make_vs_attach_eci():
-        # The COM interface to the Debugger has to be compiled as a .cpp file by
-        # Visual C. So we generate the source and then add a commandline switch
-        # to treat this source file as C++
-        import os
-        eci = ExternalCompilationInfo(post_include_bits=["""
-#ifdef __cplusplus
-extern "C" {
-#endif
-RPY_EXPORTED void AttachToVS();
-#ifdef __cplusplus
-}
-#endif
-                                      """],
-                                      separate_module_sources=["""
-#import "libid:80cc9f66-e7d8-4ddd-85b6-d9e6cd0e93e2" version("8.0") lcid("0") raw_interfaces_only named_guids
-extern "C" RPY_EXPORTED void AttachToVS() {
-    CoInitialize(0);
-    HRESULT hr;
-    CLSID Clsid;
-
-    CLSIDFromProgID(L"VisualStudio.DTE", &Clsid);
-    IUnknown *Unknown;
-    if (FAILED(GetActiveObject(Clsid, 0, &Unknown))) {
-        puts("Could not attach to Visual Studio (is it not running?");
-        return;
-    }
-
-    EnvDTE::_DTE *Interface;
-    hr = Unknown->QueryInterface(&Interface);
-    if (FAILED(GetActiveObject(Clsid, 0, &Unknown))) {
-        puts("Could not open COM interface to Visual Studio (no permissions?)");
-        return;
-    }
-
-    EnvDTE::Debugger *Debugger;
-    puts("Waiting for Visual Studio Debugger to become idle");
-    while (FAILED(Interface->get_Debugger(&Debugger)));
-
-    EnvDTE::Processes *Processes;
-    while (FAILED(Debugger->get_LocalProcesses(&Processes)));
-
-    long Count = 0;
-    if (FAILED(Processes->get_Count(&Count))) {
-        puts("Cannot query Process count");
-    }
-
-    for (int i = 0; i <= Count; i++) {
-        EnvDTE::Process *Process;
-        if (FAILED(Processes->Item(variant_t(i), &Process))) {
-            continue;
-        }
-
-        long ProcessID;
-        while (FAILED(Process->get_ProcessID(&ProcessID)));
-
-        if (ProcessID == GetProcessId(GetCurrentProcess())) {
-            printf("Found process ID %d\\n", ProcessID);
-            Process->Attach();
-            Debugger->Break(false);
-            CoUninitialize();
-            return;
-        }
-    }
-}
-                                      """]
-        )
-        eci = eci.convert_sources_to_files()
-        d = eci._copy_attributes()
-        cfile = d['separate_module_files'][0]
-        cppfile = cfile.replace(".c", "_vsdebug.cpp")
-        os.rename(cfile, cppfile)
-        d['separate_module_files'] = [cppfile]
-        return ExternalCompilationInfo(**d)
-
-    ll_attach = rffi.llexternal("AttachToVS", [], lltype.Void,
-                                compilation_info=make_vs_attach_eci())
     def impl_attach_gdb():
-        #ll_attach()
-        print "AttachToVS is disabled at the moment (compilation failure)"
+        print "Don't know how to attach GDB on Windows"
 
 register_external(attach_gdb, [], result=None,
                   export_name="impl_attach_gdb", llimpl=impl_attach_gdb)
