@@ -11,11 +11,9 @@ r_uint   an unsigned integer which has no overflow
 intmask  mask a possibly long value when running on CPython
          back to a signed int value
 ovfcheck check on CPython whether the result of a signed
-         integer operation did overflow (add, sub, mul)
+         integer operation did overflow
 ovfcheck_float_to_int
          convert to an integer or raise OverflowError
-ovfcheck_float_to_longlong
-         convert to a longlong or raise OverflowError
 r_longlong
          like r_int but double word size
 r_ulonglong
@@ -25,10 +23,6 @@ widen(x)
          lltype.Unsigned, widen it to lltype.Signed.
          Useful because the translator doesn't support
          arithmetic on the smaller types.
-ovfcheck_int32_add/sub/mul(x, y)
-         perform an add/sub/mul between two regular integers,
-         but assumes that they fit inside signed 32-bit ints
-         and raises OverflowError if the result no longer does
 
 These are meant to be erased by translation, r_uint
 in the process should mark unsigned values, ovfcheck should
@@ -36,12 +30,11 @@ mark where overflow checking is required.
 
 
 """
-import sys, struct, math
+import sys, struct
 from rpython.rtyper import extregistry
 from rpython.rlib import objectmodel
 from rpython.flowspace.model import Constant, const
 from rpython.flowspace.specialcase import register_flow_sc
-from rpython.rlib.objectmodel import specialize, not_rpython
 
 """
 Long-term target:
@@ -64,23 +57,23 @@ all typecodes.
 def _get_bitsize(typecode):
     return len(struct.pack(typecode, 1)) * 8
 
-long_typecode = 'l'
+_long_typecode = 'l'
 if _get_bitsize('P') > _get_bitsize('l'):
-    long_typecode = 'P'
+    _long_typecode = 'P'
 
 def _get_long_bit():
     # whatever size a long has, make it big enough for a pointer.
-    return _get_bitsize(long_typecode)
+    return _get_bitsize(_long_typecode)
 
 # exported for now for testing array values.
 # might go into its own module.
 def get_long_pattern(x):
     """get the bit pattern for a long, adjusted to pointer size"""
-    return struct.pack(long_typecode, x)
+    return struct.pack(_long_typecode, x)
 
 # used in tests for ctypes and for genc and friends
 # to handle the win64 special case:
-is_emulated_long = long_typecode != 'l'
+is_emulated_long = _long_typecode != 'l'
 
 LONG_BIT = _get_long_bit()
 LONG_MASK = (2**LONG_BIT)-1
@@ -109,8 +102,10 @@ to use long everywhere.
 # XXX returning int(n) should not be necessary and should be simply n.
 # XXX TODO: replace all int(n) by long(n) and fix everything that breaks.
 # XXX       Then relax it and replace int(n) by n.
-@not_rpython
 def intmask(n):
+    """
+    NOT_RPYTHON
+    """
     if isinstance(n, objectmodel.Symbolic):
         return n        # assume Symbolics don't overflow
     assert not isinstance(n, float)
@@ -122,8 +117,10 @@ def intmask(n):
         n -= 2*LONG_TEST
     return int(n)
 
-@not_rpython
 def longlongmask(n):
+    """
+    NOT_RPYTHON
+    """
     assert isinstance(n, (int, long))
     n = long(n)
     n &= LONGLONG_MASK
@@ -136,15 +133,14 @@ def longlonglongmask(n):
     # We deal directly with overflow there anyway.
     return r_longlonglong(n)
 
-@specialize.argtype(0)
 def widen(n):
     from rpython.rtyper.lltypesystem import lltype
     if _should_widen_type(lltype.typeOf(n)):
         return intmask(n)
     else:
         return n
+widen._annspecialcase_ = 'specialize:argtype(0)'
 
-@specialize.memo()
 def _should_widen_type(tp):
     from rpython.rtyper.lltypesystem import lltype, rffi
     if tp is lltype.Bool:
@@ -155,24 +151,24 @@ def _should_widen_type(tp):
     assert issubclass(r_class, base_int)
     return r_class.BITS < LONG_BIT or (
         r_class.BITS == LONG_BIT and r_class.SIGNED)
+_should_widen_type._annspecialcase_ = 'specialize:memo'
 
 # the replacement for sys.maxint
 maxint = int(LONG_TEST - 1)
 # for now, it should be equal to sys.maxint on all supported platforms
 assert maxint == sys.maxint
 
-@specialize.argtype(0)
 def is_valid_int(r):
     if objectmodel.we_are_translated():
         return isinstance(r, int)
     return isinstance(r, (base_int, int, long, bool)) and (
         -maxint - 1 <= r <= maxint)
+is_valid_int._annspecialcase_ = 'specialize:argtype(0)'
 
-@not_rpython
 def ovfcheck(r):
+    "NOT_RPYTHON"
     # to be used as ovfcheck(x <op> y)
     # raise OverflowError if the operation did overflow
-    # Nowadays, only supports '+', '-' or '*' as the operation.
     assert not isinstance(r, r_uint), "unexpected ovf check on unsigned"
     assert not isinstance(r, r_longlong), "ovfcheck not supported on r_longlong"
     assert not isinstance(r, r_ulonglong), "ovfcheck not supported on r_ulonglong"
@@ -186,27 +182,25 @@ def ovfcheck(r):
 # int(float(i)) != i  because of rounding issues.
 # These are the minimum and maximum float value that can
 # successfully be casted to an int.
-
-# The following values are not quite +/-sys.maxint.
-# Note the "<= x <" here, as opposed to "< x <" above.
-# This is justified by test_typed in translator/c/test.
-def ovfcheck_float_to_longlong(x):
-    if math.isnan(x):
-        raise OverflowError
-    if -9223372036854776832.0 <= x < 9223372036854775296.0:
-        return r_longlong(x)
-    raise OverflowError
-
 if sys.maxint == 2147483647:
     def ovfcheck_float_to_int(x):
-        if math.isnan(x):
+        from rpython.rlib.rfloat import isnan
+        if isnan(x):
             raise OverflowError
         if -2147483649.0 < x < 2147483648.0:
             return int(x)
         raise OverflowError
 else:
+    # The following values are not quite +/-sys.maxint.
+    # Note the "<= x <" here, as opposed to "< x <" above.
+    # This is justified by test_typed in translator/c/test.
     def ovfcheck_float_to_int(x):
-        return int(ovfcheck_float_to_longlong(x))
+        from rpython.rlib.rfloat import isnan
+        if isnan(x):
+            raise OverflowError
+        if -9223372036854776832.0 <= x < 9223372036854775296.0:
+            return int(x)
+        raise OverflowError
 
 def compute_restype(self_type, other_type):
     if self_type is other_type:
@@ -219,18 +213,16 @@ def compute_restype(self_type, other_type):
         return self_type
     if self_type in (bool, int, long):
         return other_type
-    if self_type is float or other_type is float:
-        return float
     if self_type.SIGNED == other_type.SIGNED:
         return build_int(None, self_type.SIGNED, max(self_type.BITS, other_type.BITS))
     raise AssertionError("Merging these types (%s, %s) is not supported" % (self_type, other_type))
 
-@specialize.memo()
 def signedtype(t):
     if t in (bool, int, long):
         return True
     else:
         return t.SIGNED
+signedtype._annspecialcase_ = 'specialize:memo'
 
 def normalizedinttype(t):
     if t is int:
@@ -241,12 +233,11 @@ def normalizedinttype(t):
         assert t.BITS <= r_longlong.BITS
         return build_int(None, t.SIGNED, r_longlong.BITS)
 
-@specialize.argtype(0)
 def most_neg_value_of_same_type(x):
     from rpython.rtyper.lltypesystem import lltype
     return most_neg_value_of(lltype.typeOf(x))
+most_neg_value_of_same_type._annspecialcase_ = 'specialize:argtype(0)'
 
-@specialize.memo()
 def most_neg_value_of(tp):
     from rpython.rtyper.lltypesystem import lltype, rffi
     if tp is lltype.Signed:
@@ -257,13 +248,13 @@ def most_neg_value_of(tp):
         return r_class(-(r_class.MASK >> 1) - 1)
     else:
         return r_class(0)
+most_neg_value_of._annspecialcase_ = 'specialize:memo'
 
-@specialize.argtype(0)
 def most_pos_value_of_same_type(x):
     from rpython.rtyper.lltypesystem import lltype
     return most_pos_value_of(lltype.typeOf(x))
+most_pos_value_of_same_type._annspecialcase_ = 'specialize:argtype(0)'
 
-@specialize.memo()
 def most_pos_value_of(tp):
     from rpython.rtyper.lltypesystem import lltype, rffi
     if tp is lltype.Signed:
@@ -274,8 +265,8 @@ def most_pos_value_of(tp):
         return r_class(r_class.MASK >> 1)
     else:
         return r_class(r_class.MASK)
+most_pos_value_of._annspecialcase_ = 'specialize:memo'
 
-@specialize.memo()
 def is_signed_integer_type(tp):
     from rpython.rtyper.lltypesystem import lltype, rffi
     if tp is lltype.Signed:
@@ -285,6 +276,7 @@ def is_signed_integer_type(tp):
         return r_class.SIGNED
     except KeyError:
         return False   # not an integer type
+is_signed_integer_type._annspecialcase_ = 'specialize:memo'
 
 def highest_bit(n):
     """
@@ -305,7 +297,6 @@ class base_int(long):
     def _widen(self, other, value):
         """
         if one argument is int or long, the other type wins.
-        if one argument is float, the result is float.
         otherwise, produce the largest class to hold the result.
         """
         self_type = type(self)
@@ -326,29 +317,17 @@ class base_int(long):
 
     def __add__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return x + other
-        y = long(other)
-        return self._widen(other, x + y)
-
-    def __radd__(self, other):
-        x = long(self)
-        if not isinstance(other, (int, long)):
-            return other + x
         y = long(other)
         return self._widen(other, x + y)
+    __radd__ = __add__
 
     def __sub__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return x - other
         y = long(other)
         return self._widen(other, x - y)
 
     def __rsub__(self, other):
         y = long(self)
-        if not isinstance(other, (int, long)):
-            return other - y
         x = long(other)
         return self._widen(other, x - y)
 
@@ -358,133 +337,75 @@ class base_int(long):
             return x * other
         y = long(other)
         return self._widen(other, x * y)
-
-    def __rmul__(self, other):
-        x = long(self)
-        if not isinstance(other, (int, long)):
-            return other * x
-        y = long(other)
-        return self._widen(other, x * y)
+    __rmul__ = __mul__
 
     def __div__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return x / other
         y = long(other)
         return self._widen(other, x // y)
 
+    __floordiv__ = __div__
+
     def __rdiv__(self, other):
         y = long(self)
-        if not isinstance(other, (int, long)):
-            return other / y
         x = long(other)
         return self._widen(other, x // y)
 
-    def __floordiv__(self, other):
-        x = long(self)
-        if not isinstance(other, (int, long)):
-            return x // other
-        y = long(other)
-        return self._widen(other, x // y)
-
-    def __rfloordiv__(self, other):
-        y = long(self)
-        if not isinstance(other, (int, long)):
-            return other // y
-        x = long(other)
-        return self._widen(other, x // y)
+    __rfloordiv__ = __rdiv__
 
     def __mod__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return x % other
         y = long(other)
         return self._widen(other, x % y)
 
     def __rmod__(self, other):
         y = long(self)
-        if not isinstance(other, (int, long)):
-            return other % y
         x = long(other)
         return self._widen(other, x % y)
 
     def __divmod__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return divmod(x, other)
         y = long(other)
         res = divmod(x, y)
-        return (self._widen(other, res[0]), self._widen(other, res[1]))
+        return (self.__class__(res[0]), self.__class__(res[1]))
 
     def __lshift__(self, n):
         x = long(self)
-        if not isinstance(n, (int, long)):
-            raise TypeError
         y = long(n)
         return self.__class__(x << y)
 
     def __rlshift__(self, n):
         y = long(self)
-        if not isinstance(n, (int, long)):
-            raise TypeError
         x = long(n)
-        return n.__class__(x << y)
+        return self._widen(n, x << y)
 
     def __rshift__(self, n):
         x = long(self)
-        if not isinstance(n, (int, long)):
-            raise TypeError
         y = long(n)
-        return self.__class__(x >> y)
+        return self._widen(n, x >> y)
 
     def __rrshift__(self, n):
         y = long(self)
-        if not isinstance(n, (int, long)):
-            raise TypeError
         x = long(n)
-        return n.__class__(x >> y)
+        return self._widen(n, x >> y)
 
     def __or__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return x | other
-        y = long(other)
-        return self._widen(other, x | y)
-
-    def __ror__(self, other):
-        x = long(self)
-        if not isinstance(other, (int, long)):
-            return other | x
         y = long(other)
         return self._widen(other, x | y)
+    __ror__ = __or__
 
     def __and__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return x & other
-        y = long(other)
-        return self._widen(other, x & y)
-
-    def __rand__(self, other):
-        x = long(self)
-        if not isinstance(other, (int, long)):
-            return other & x
         y = long(other)
         return self._widen(other, x & y)
+    __rand__ = __and__
 
     def __xor__(self, other):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return x ^ other
-        y = long(other)
-        return self._widen(other, x ^ y)
-
-    def __rxor__(self, other):
-        x = long(self)
-        if not isinstance(other, (int, long)):
-            return other ^ x
         y = long(other)
         return self._widen(other, x ^ y)
+    __rxor__ = __xor__
 
     def __neg__(self):
         x = long(self)
@@ -503,16 +424,12 @@ class base_int(long):
 
     def __pow__(self, other, m=None):
         x = long(self)
-        if not isinstance(other, (int, long)):
-            return pow(x, other, m)
         y = long(other)
         res = pow(x, y, m)
         return self._widen(other, res)
 
     def __rpow__(self, other, m=None):
         y = long(self)
-        if not isinstance(other, (int, long)):
-            return pow(other, y, m)
         x = long(other)
         res = pow(x, y, m)
         return self._widen(other, res)
@@ -612,20 +529,15 @@ r_longlong = build_int('r_longlong', True, 64)
 r_ulonglong = build_int('r_ulonglong', False, 64)
 
 r_longlonglong = build_int('r_longlonglong', True, 128)
-r_ulonglonglong = build_int('r_ulonglonglong', False, 128)
 longlongmax = r_longlong(LONGLONG_TEST - 1)
 
 if r_longlong is not r_int:
     r_int64 = r_longlong
-    r_uint64 = r_ulonglong
-    r_int32 = int # XXX: what about r_int
-    r_uint32 = r_uint
 else:
-    r_int64 = int # XXX: what about r_int
-    r_uint64 = r_uint # is r_ulonglong
-    r_int32 = build_int('r_int32', True, 32)     # also needed for rposix_stat.time_t_to_FILE_TIME in the 64 bit case
-    r_uint32 = build_int('r_uint32', False, 32)
+    r_int64 = int
 
+# needed for rposix_stat.time_t_to_FILE_TIME in the 64 bit case
+r_uint32 = build_int('r_uint32', False, 32)
 
 SHRT_MIN = -2**(_get_bitsize('h') - 1)
 SHRT_MAX = 2**(_get_bitsize('h') - 1) - 1
@@ -665,9 +577,6 @@ class r_singlefloat(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def __hash__(self):
-        return hash(self._bytes)
-
     def __repr__(self):
         return 'r_singlefloat(%s)' % (float(self),)
 
@@ -695,9 +604,6 @@ class r_longfloat(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def __hash__(self):
-        return hash(self.value)
-
 
 class For_r_singlefloat_values_Entry(extregistry.ExtRegistryEntry):
     _type_ = r_singlefloat
@@ -721,13 +627,6 @@ class For_r_singlefloat_type_Entry(extregistry.ExtRegistryEntry):
         return hop.genop('cast_primitive', [v],
                          resulttype = lltype.SingleFloat)
 
-class For_r_longfloat_values_Entry(extregistry.ExtRegistryEntry):
-    _type_ = r_longfloat
-
-    def compute_annotation(self):
-        from rpython.annotator import model as annmodel
-        return annmodel.SomeLongFloat()
-
 
 def int_between(n, m, p):
     """ check that n <= m < p. This assumes that n <= p. This is useful because
@@ -742,31 +641,9 @@ def int_force_ge_zero(n):
     """ The JIT special-cases this too. """
     from rpython.rtyper.lltypesystem import lltype
     from rpython.rtyper.lltypesystem.lloperation import llop
-    n = llop.int_force_ge_zero(lltype.Signed, n)
-    assert n >= 0
-    return n
+    return llop.int_force_ge_zero(lltype.Signed, n)
 
-def int_c_div(x, y):
-    """Return the result of the C-style 'x / y'.  This differs from the
-    Python-style division if (x < 0  xor y < 0).  The JIT implements it
-    with a Python-style division followed by correction code.  This
-    is not that bad, because the JIT removes the correction code if
-    x and y are both nonnegative, and if y is any nonnegative constant
-    then the division turns into a rshift or a mul.
-    """
-    from rpython.rtyper.lltypesystem import lltype
-    from rpython.rtyper.lltypesystem.lloperation import llop
-    return llop.int_floordiv(lltype.Signed, x, y)
-
-def int_c_mod(x, y):
-    """Return the result of the C-style 'x % y'.  This differs from the
-    Python-style division if (x < 0  xor y < 0).
-    """
-    from rpython.rtyper.lltypesystem import lltype
-    from rpython.rtyper.lltypesystem.lloperation import llop
-    return llop.int_mod(lltype.Signed, x, y)
-
-@specialize.ll()
+@objectmodel.specialize.ll()
 def byteswap(arg):
     """ Convert little->big endian and the opposite
     """
@@ -814,113 +691,20 @@ def byteswap(arg):
         return longlong2float(rffi.cast(rffi.LONGLONG, res))
     return rffi.cast(T, res)
 
-if sys.maxint == 2147483647:
-    def ovfcheck_int32_add(x, y):
-        return ovfcheck(x + y)
-    def ovfcheck_int32_sub(x, y):
-        return ovfcheck(x - y)
-    def ovfcheck_int32_mul(x, y):
-        return ovfcheck(x * y)
-else:
-    def ovfcheck_int32_add(x, y):
-        """x and y are assumed to fit inside the 32-bit rffi.INT;
-        raises OverflowError if the result doesn't fit rffi.INT"""
-        from rpython.rtyper.lltypesystem import lltype, rffi
-        x = rffi.cast(lltype.Signed, x)
-        y = rffi.cast(lltype.Signed, y)
-        z = x + y
-        if z != rffi.cast(lltype.Signed, rffi.cast(rffi.INT, z)):
-            raise OverflowError
-        return z
-
-    def ovfcheck_int32_sub(x, y):
-        """x and y are assumed to fit inside the 32-bit rffi.INT;
-        raises OverflowError if the result doesn't fit rffi.INT"""
-        from rpython.rtyper.lltypesystem import lltype, rffi
-        x = rffi.cast(lltype.Signed, x)
-        y = rffi.cast(lltype.Signed, y)
-        z = x - y
-        if z != rffi.cast(lltype.Signed, rffi.cast(rffi.INT, z)):
-            raise OverflowError
-        return z
-
-    def ovfcheck_int32_mul(x, y):
-        """x and y are assumed to fit inside the 32-bit rffi.INT;
-        raises OverflowError if the result doesn't fit rffi.INT"""
-        from rpython.rtyper.lltypesystem import lltype, rffi
-        x = rffi.cast(lltype.Signed, x)
-        y = rffi.cast(lltype.Signed, y)
-        z = x * y
-        if z != rffi.cast(lltype.Signed, rffi.cast(rffi.INT, z)):
-            raise OverflowError
-        return z
-
-
-@specialize.memo()
-def check_support_int128():
-    from rpython.rtyper.lltypesystem import rffi
-    return hasattr(rffi, '__INT128_T')
-
-def mulmod(a, b, c):
-    """Computes (a * b) % c.
-    Assumes c > 0, and returns a nonnegative result.
-    """
-    assert c > 0
-    if LONG_BIT < LONGLONG_BIT:
-        a = r_longlong(a)
-        b = r_longlong(b)
-        return intmask((a * b) % c)
-    elif check_support_int128():
-        a = r_longlonglong(a)
-        b = r_longlonglong(b)
-        return intmask((a * b) % c)
-    else:
-        from rpython.rlib.rbigint import rbigint
-        a = rbigint.fromint(a)
-        b = rbigint.fromint(b)
-        return a.mul(b).int_mod(c).toint()
-
 
 # String parsing support
 # ---------------------------
 
-OVF_DIGITS = len(str(sys.maxint))
-
-def string_to_int(s, base=10, allow_underscores=False, no_implicit_octal=False):
+def string_to_int(s, base=10):
     """Utility to converts a string to an integer.
     If base is 0, the proper base is guessed based on the leading
     characters of 's'.  Raises ParseStringError in case of error.
     Raises ParseStringOverflowError in case the result does not fit.
     """
     from rpython.rlib.rstring import (
-        NumberStringParser, ParseStringOverflowError)
-
-    if base == 10 and 0 < len(s) < OVF_DIGITS:
-        # fast path for simple cases, just supporting (+/-)[0-9]* with not too
-        # many digits
-        start = 0
-        sign = 1
-        if s[0] == "-":
-            start = 1
-            sign = -1
-        elif s[0] == "+":
-            start = 1
-        if start != len(s):
-            result = 0
-            for i in range(start, len(s)):
-                char = s[i]
-                value = ord(char) - ord('0')
-                if 0 <= value <= 9:
-                    result = result * 10 + value
-                else:
-                    # non digit char, let the NumberStringParser do the work
-                    break
-            else:
-                return result * sign
-
-    p = NumberStringParser(s, s, base, 'int',
-                           allow_underscores=allow_underscores,
-                           no_implicit_octal=no_implicit_octal)
+        NumberStringParser, ParseStringOverflowError, strip_spaces)
+    s = literal = strip_spaces(s)
+    p = NumberStringParser(s, literal, base, 'int')
     base = p.base
     result = 0
     while True:
@@ -936,4 +720,4 @@ def string_to_int(s, base=10, allow_underscores=False, no_implicit_octal=False):
             result = ovfcheck(result + digit)
         except OverflowError:
             raise ParseStringOverflowError(p)
-string_to_int._elidable_function_ = True # can't use decorator due to circular imports
+string_to_int._elidable_function_ = True
