@@ -3,7 +3,7 @@
 
 from rpython.rtyper.tool import rffi_platform
 from rpython.rtyper.lltypesystem import rffi, lltype
-from rpython.rlib.objectmodel import we_are_translated, not_rpython
+from rpython.rlib.objectmodel import we_are_translated
 from rpython.rlib.rarithmetic import r_uint
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
 from rpython.translator.platform import platform
@@ -26,7 +26,7 @@ else:
 
 if _MAC_OS:
     pre_include_bits = ['#define MACOSX']
-else:
+else: 
     pre_include_bits = []
 
 if _FREEBSD or _NETBSD or _WIN32:
@@ -34,7 +34,6 @@ if _FREEBSD or _NETBSD or _WIN32:
 else:
     libraries = ['dl']
 
-# this 'eci' is also used in pypy/module/sys/initpath.py
 eci = ExternalCompilationInfo(
     pre_include_bits = pre_include_bits,
     includes = includes,
@@ -92,23 +91,15 @@ if not _WIN32:
             return ""
         return rffi.charp2str(res)
 
-    @not_rpython
     def _dlerror_on_dlopen_untranslated(name):
-        # aaargh
+        "NOT_RPYTHON: aaargh"
         import ctypes
         name = rffi.charp2str(name)
         try:
             ctypes.CDLL(name)
-        except OSError as e:
-            # common case: ctypes fails too, with the real dlerror()
-            # message in str(e).  Return that error message.
+        except OSError, e:
             return str(e)
         else:
-            # uncommon case: may happen if 'name' is a linker script
-            # (which the C-level dlopen() can't handle) and we are
-            # directly running on pypy (whose implementation of ctypes
-            # or cffi will resolve linker scripts).  In that case, 
-            # unsure what we can do.
             return ("opening %r with ctypes.CDLL() works, "
                     "but not with c_dlopen()??" % (name,))
 
@@ -154,33 +145,17 @@ if not _WIN32:
         else:
             return lltype.nullptr(rffi.VOIDP.TO)
 
-    def _dlopen_default_mode():
-        """ The default dlopen mode if it hasn't been changed by the user.
-        """
-        mode = RTLD_NOW
-        if RTLD_LOCAL is not None:
-            mode |= RTLD_LOCAL
-        return mode
-
     def dlopen(name, mode=-1):
         """ Wrapper around C-level dlopen
         """
         if mode == -1:
-            mode = _dlopen_default_mode()
-        elif (mode & (RTLD_LAZY | RTLD_NOW)) == 0:
+            if RTLD_LOCAL is not None:
+                mode = RTLD_LOCAL
+            else:
+                mode = 0
+        if (mode & (RTLD_LAZY | RTLD_NOW)) == 0:
             mode |= RTLD_NOW
-        #
-        # haaaack for 'pypy py.test -A' if libm.so is a linker script
-        # (see reason in _dlerror_on_dlopen_untranslated())
-        must_free = False
-        if not we_are_translated() and platform.name == "linux":
-            if name and rffi.charp2str(name) == 'libm.so':
-                name = rffi.str2charp('libm.so.6')
-                must_free = True
-        #
         res = c_dlopen(name, rffi.cast(rffi.INT, mode))
-        if must_free:
-            rffi.free_charp(name)
         if not res:
             if not we_are_translated():
                 err = _dlerror_on_dlopen_untranslated(name)
@@ -218,38 +193,12 @@ else:  # _WIN32
     DLLHANDLE = rwin32.HMODULE
     RTLD_GLOBAL = None
 
-    def _dlopen_default_mode():
-        """ The default dlopen mode if it hasn't been changed by the user.
-        """
-        return 0
-
     def dlopen(name, mode=-1):
         # mode is unused on windows, but a consistant signature
         res = rwin32.LoadLibrary(name)
         if not res:
             err = rwin32.GetLastError_saved()
-            ustr, lgt = rwin32.FormatErrorW(err)
-            raise DLOpenError(ustr)
-        return res
-
-    def dlopenex(name, flags=rwin32.LOAD_WITH_ALTERED_SEARCH_PATH):
-        # Don't display a message box when Python can't load a DLL */
-        old_mode = rwin32.SetErrorMode(rwin32.SEM_FAILCRITICALERRORS)
-        res = rwin32.LoadLibraryExA(name, flags)
-        rwin32.SetErrorMode(old_mode)
-        if not res:
-            err = rwin32.GetLastError_saved()
-            ustr, lgt = rwin32.FormatErrorW(err)
-            raise DLOpenError(ustr)
-        return res
-
-    def dlopenU(name, mode=-1):
-        # mode is unused on windows, but a consistant signature
-        res = rwin32.LoadLibraryW(name)
-        if not res:
-            err = rwin32.GetLastError_saved()
-            ustr, lgt = rwin32.FormatErrorW(err)
-            raise DLOpenError(ustr)
+            raise DLOpenError(rwin32.FormatError(err))
         return res
 
     def dlclose(handle):
