@@ -4,7 +4,7 @@ python builtin open()
 """
 
 import os, stat, errno, sys
-from rpython.rlib import rposix, rgc
+from rpython.rlib import rposix
 from rpython.rlib.objectmodel import enforceargs
 from rpython.rlib.rarithmetic import intmask
 from rpython.rlib.rstring import StringBuilder
@@ -75,30 +75,13 @@ c_fclose = llexternal('fclose', [FILEP], rffi.INT,
 c_pclose = llexternal('pclose', [FILEP], rffi.INT,
                       save_err=rffi.RFFI_SAVE_ERRNO)
 
-def wrap_fclose(filep):
-    with rposix.SuppressIPH():
-        return c_fclose(filep)
-
-def wrap_pclose(filep):
-    with rposix.SuppressIPH():
-        return c_fclose(filep)
-
 # Note: the following two functions are called from __del__ methods,
 # so must be 'releasegil=False'.  Otherwise, a program using both
 # threads and the RFile class cannot translate.  See c684bf704d1f
 c_fclose_in_del = llexternal('fclose', [FILEP], rffi.INT, releasegil=False)
 c_pclose_in_del = llexternal('pclose', [FILEP], rffi.INT, releasegil=False)
-
-def wrap_fclose_in_del(filep):
-    with rposix.SuppressIPH_del():
-        return c_fclose_in_del(filep)
-
-def wrap_pclose_in_del(filep):
-    with rposix.SuppressIPH_del():
-        return c_pclose_in_del(filep)
-
-_fclose2 = (wrap_fclose, wrap_fclose_in_del)
-_pclose2 = (wrap_pclose, wrap_pclose_in_del)
+_fclose2 = (c_fclose, c_fclose_in_del)
+_pclose2 = (c_pclose, c_pclose_in_del)
 
 c_getc = llexternal('getc', [FILEP], rffi.INT, macro=True)
 c_ungetc = llexternal('ungetc', [rffi.INT, FILEP], rffi.INT)
@@ -124,11 +107,11 @@ c_ferror = llexternal('ferror', [FILEP], rffi.INT)
 c_clearerr = llexternal('clearerr', [FILEP], lltype.Void)
 
 c_stdin = rffi.CExternVariable(FILEP, 'stdin', eci, c_type='FILE*',
-                               getter_only=True, declare_as_extern=False)
+                               getter_only=True)
 c_stdout = rffi.CExternVariable(FILEP, 'stdout', eci, c_type='FILE*',
-                                getter_only=True, declare_as_extern=False)
+                                getter_only=True)
 c_stderr = rffi.CExternVariable(FILEP, 'stderr', eci, c_type='FILE*',
-                                getter_only=True, declare_as_extern=False)
+                                getter_only=True)
 
 
 def _error(ll_file):
@@ -190,10 +173,10 @@ def create_file(filename, mode="r", buffering=-1):
 
 def create_fdopen_rfile(fd, mode="r", buffering=-1):
     newmode = _sanitize_mode(mode)
+    rposix.validate_fd(fd)
     ll_mode = rffi.str2charp(newmode)
     try:
-        with rposix.SuppressIPH():
-            ll_file = c_fdopen(fd, ll_mode)
+        ll_file = c_fdopen(fd, ll_mode)
         if not ll_file:
             errno = rposix.get_saved_errno()
             raise OSError(errno, os.strerror(errno))
@@ -311,7 +294,6 @@ class RFile(object):
         if ll_file:
             # double close is allowed
             self._ll_file = lltype.nullptr(FILEP.TO)
-            rgc.may_ignore_finalizer(self)
             do_close = self._close2[0]
             try:
                 if do_close:
@@ -329,10 +311,6 @@ class RFile(object):
         if not self._ll_file:
             raise ValueError("I/O operation on closed file")
 
-    @property
-    def closed(self):
-        return not self._ll_file
-
     def _fread(self, buf, n, stream):
         if not self._univ_newline:
             return c_fread(buf, 1, n, stream)
