@@ -17,13 +17,12 @@ from rpython.rtyper.rint import IntegerRepr
 from rpython.rtyper.error import TyperError
 
 from rpython.rlib._os_support import _preferred_traits, string_traits
-from rpython.rlib.objectmodel import specialize, we_are_translated, not_rpython
+from rpython.rlib.objectmodel import specialize
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
 from rpython.rlib.rarithmetic import intmask
 from rpython.rlib.rposix import (
     replace_os_function, handle_posix_error, _as_bytes0)
-from rpython.rlib import rposix
 
 _WIN32 = sys.platform.startswith('win')
 _LINUX = sys.platform.startswith('linux')
@@ -36,12 +35,14 @@ if _WIN32:
 # - ALL_STAT_FIELDS contains Float fields if the system can retrieve
 #   sub-second timestamps.
 # - TIMESPEC is defined when the "struct stat" contains st_atim field.
+
 if sys.platform.startswith('linux') or sys.platform.startswith('openbsd'):
-    from rpython.rlib.rposix import TIMESPEC
+    TIMESPEC = platform.Struct('struct timespec',
+                               [('tv_sec', rffi.TIME_T),
+                                ('tv_nsec', rffi.LONG)])
 else:
     TIMESPEC = None
 
-
 # all possible fields - some of them are not available on all platforms
 ALL_STAT_FIELDS = [
     ("st_mode",      lltype.Signed),
@@ -51,18 +52,15 @@ ALL_STAT_FIELDS = [
     ("st_uid",       lltype.Signed),
     ("st_gid",       lltype.Signed),
     ("st_size",      lltype.SignedLongLong),
-    ("st_atime",     lltype.SignedLongLong),   # integral number of seconds
-    ("st_mtime",     lltype.SignedLongLong),   #
-    ("st_ctime",     lltype.SignedLongLong),   #
+    ("st_atime",     lltype.Float),
+    ("st_mtime",     lltype.Float),
+    ("st_ctime",     lltype.Float),
     ("st_blksize",   lltype.Signed),
     ("st_blocks",    lltype.Signed),
     ("st_rdev",      lltype.Signed),
     ("st_flags",     lltype.Signed),
     #("st_gen",       lltype.Signed),     -- new in CPy 2.5, not implemented
     #("st_birthtime", lltype.Float),      -- new in CPy 2.5, not implemented
-    ("nsec_atime",   lltype.Signed),   # number of nanoseconds
-    ("nsec_mtime",   lltype.Signed),   #
-    ("nsec_ctime",   lltype.Signed),   #
 ]
 N_INDEXABLE_FIELDS = 10
 
@@ -82,37 +80,6 @@ STATVFS_FIELDS = [
     ("f_namemax", lltype.Signed),
 ]
 
-@specialize.arg(1)
-def get_stat_ns_as_bigint(st, name):
-    """'name' is one of the strings "atime", "mtime" or "ctime".
-    Returns a bigint that represents the number of nanoseconds
-    stored inside the RPython-level os.stat_result 'st'.
-
-    Note that when running untranslated, the os.stat_result type
-    is from Python 2.7, which doesn't store more precision than
-    a float anyway.  You will only get more after translation.
-    """
-    from rpython.rlib.rbigint import rbigint
-
-    if not we_are_translated():
-        as_float = getattr(st, "st_" + name)
-        return rbigint.fromfloat(as_float * 1e9)
-
-    if name == "atime":
-        i, j = 7, -3
-    elif name == "mtime":
-        i, j = 8, -2
-    elif name == "ctime":
-        i, j = 9, -1
-    else:
-        raise AssertionError(name)
-
-    sec = st[i]
-    nsec = st[j]
-    result = rbigint.fromrarith_int(sec).int_mul(1000000000)
-    result = result.int_add(nsec)
-    return result
-
 
 # ____________________________________________________________
 #
@@ -128,18 +95,9 @@ class SomeStatResult(annmodel.SomeObject):
         return self.__class__,
 
     def getattr(self, s_attr):
-        if not s_attr.is_constant():
-            raise annmodel.AnnotatorError("non-constant attr name in getattr()")
+        assert s_attr.is_constant(), "non-constant attr name in getattr()"
         attrname = s_attr.const
-        if attrname in ('st_atime', 'st_mtime', 'st_ctime'):
-            # like CPython, in RPython we can read the st_Xtime
-            # attribute and get a floating-point result.  We can also
-            # get a full-precision bigint with get_stat_ns_as_bigint().
-            # The floating-point result is computed like a property
-            # by _ll_get_st_Xtime().
-            TYPE = lltype.Float
-        else:
-            TYPE = STAT_FIELD_TYPES[attrname]
+        TYPE = STAT_FIELD_TYPES[attrname]
         return lltype_to_annotation(TYPE)
 
     def _get_rmarshall_support_(self):     # for rlib.rmarshal
@@ -147,21 +105,13 @@ class SomeStatResult(annmodel.SomeObject):
         # (we ignore the extra values here for simplicity and portability)
         def stat_result_reduce(st):
             return (st[0], st[1], st[2], st[3], st[4],
-                    st[5], st[6], st.st_atime, st.st_mtime, st.st_ctime)
+                    st[5], st[6], st[7], st[8], st[9])
 
         def stat_result_recreate(tup):
-            atime, mtime, ctime = tup[7:]
-            result = tup[:7]
-            result += (int(atime), int(mtime), int(ctime))
-            result += extra_zeroes
-            result += (int((atime - result[7]) * 1e9),
-                       int((mtime - result[8]) * 1e9),
-                       int((ctime - result[9]) * 1e9))
-            return make_stat_result(result)
+            return make_stat_result(tup + extra_zeroes)
         s_reduced = annmodel.SomeTuple([lltype_to_annotation(TYPE)
-                                    for name, TYPE in PORTABLE_STAT_FIELDS[:7]]
-                                 + 3 * [lltype_to_annotation(lltype.Float)])
-        extra_zeroes = (0,) * (len(STAT_FIELDS) - len(PORTABLE_STAT_FIELDS) - 3)
+                                       for name, TYPE in PORTABLE_STAT_FIELDS])
+        extra_zeroes = (0,) * (len(STAT_FIELDS) - len(PORTABLE_STAT_FIELDS))
         return s_reduced, stat_result_reduce, stat_result_recreate
 
 
@@ -169,7 +119,7 @@ class __extend__(pairtype(SomeStatResult, annmodel.SomeInteger)):
     def getitem((s_sta, s_int)):
         assert s_int.is_constant(), "os.stat()[index]: index must be constant"
         index = s_int.const
-        assert -3 <= index < N_INDEXABLE_FIELDS, "os.stat()[index] out of range"
+        assert 0 <= index < N_INDEXABLE_FIELDS, "os.stat()[index] out of range"
         name, TYPE = STAT_FIELDS[index]
         return lltype_to_annotation(TYPE)
 
@@ -202,61 +152,28 @@ class StatResultRepr(Repr):
     def rtype_getattr(self, hop):
         s_attr = hop.args_s[1]
         attr = s_attr.const
-        if attr in ('st_atime', 'st_mtime', 'st_ctime'):
-            ll_func = globals()['_ll_get_' + attr]
-            v_tuple = hop.inputarg(self, arg=0)
-            return hop.gendirectcall(ll_func, v_tuple)
         try:
             index = self.stat_field_indexes[attr]
         except KeyError:
             raise TyperError("os.stat().%s: field not available" % (attr,))
         return self.redispatch_getfield(hop, index)
 
-@specialize.memo()
-def _stfld(name):
-    index = STAT_FIELD_NAMES.index(name)
-    return 'item%d' % index
-
-def _ll_get_st_atime(tup):
-    return (float(getattr(tup, _stfld("st_atime"))) +
-            1E-9 * getattr(tup, _stfld("nsec_atime")))
-
-def _ll_get_st_mtime(tup):
-    return (float(getattr(tup, _stfld("st_mtime"))) +
-            1E-9 * getattr(tup, _stfld("nsec_mtime")))
-
-def _ll_get_st_ctime(tup):
-    return (float(getattr(tup, _stfld("st_ctime"))) +
-            1E-9 * getattr(tup, _stfld("nsec_ctime")))
-
 
 class __extend__(pairtype(StatResultRepr, IntegerRepr)):
     def rtype_getitem((r_sta, r_int), hop):
         s_int = hop.args_s[1]
         index = s_int.const
-        if index < 0:
-            index += len(STAT_FIELDS)
         return r_sta.redispatch_getfield(hop, index)
 
 s_StatResult = SomeStatResult()
 
-@not_rpython
 def make_stat_result(tup):
     """Turn a tuple into an os.stat_result object."""
-    assert len(tup) == len(STAT_FIELDS)
-    assert float not in [type(x) for x in tup]
-    positional = []
-    for i in range(N_INDEXABLE_FIELDS):
-        name, TYPE = STAT_FIELDS[i]
-        value = lltype.cast_primitive(TYPE, tup[i])
-        positional.append(value)
+    positional = tuple(
+        lltype.cast_primitive(TYPE, value) for value, (name, TYPE) in
+            zip(tup, STAT_FIELDS)[:N_INDEXABLE_FIELDS])
     kwds = {}
-    kwds['st_atime'] = tup[7] + 1e-9 * tup[-3]
-    kwds['st_mtime'] = tup[8] + 1e-9 * tup[-2]
-    kwds['st_ctime'] = tup[9] + 1e-9 * tup[-1]
     for value, (name, TYPE) in zip(tup, STAT_FIELDS)[N_INDEXABLE_FIELDS:]:
-        if name.startswith('nsec_'):
-            continue   # ignore the nsec_Xtime here
         kwds[name] = lltype.cast_primitive(TYPE, value)
     return os.stat_result(positional, kwds)
 
@@ -383,6 +300,13 @@ compilation_info = ExternalCompilationInfo(
     includes=INCLUDES
 )
 
+if TIMESPEC is not None:
+    class CConfig_for_timespec:
+        _compilation_info_ = compilation_info
+        TIMESPEC = TIMESPEC
+    TIMESPEC = lltype.Ptr(
+        platform.configure(CConfig_for_timespec)['TIMESPEC'])
+
 
 def posix_declaration(try_to_add=None):
     global STAT_STRUCT, STATVFS_STRUCT
@@ -398,7 +322,7 @@ def posix_declaration(try_to_add=None):
                 if _name == originalname:
                     # replace the 'st_atime' field of type rffi.DOUBLE
                     # with a field 'st_atim' of type 'struct timespec'
-                    lst[i] = (timespecname, TIMESPEC)
+                    lst[i] = (timespecname, TIMESPEC.TO)
                     break
 
         _expand(LL_STAT_FIELDS, 'st_atime', 'st_atim')
@@ -443,8 +367,6 @@ if sys.platform != 'win32':
         posix_declaration(ALL_STAT_FIELDS[_i])
     del _i
 
-STAT_FIELDS += ALL_STAT_FIELDS[-3:]   # nsec_Xtime
-
 # these two global vars only list the fields defined in the underlying platform
 STAT_FIELD_TYPES = dict(STAT_FIELDS)      # {'st_xxx': TYPE}
 STAT_FIELD_NAMES = [_name for (_name, _TYPE) in STAT_FIELDS]
@@ -453,20 +375,16 @@ del _name, _TYPE
 STATVFS_FIELD_TYPES = dict(STATVFS_FIELDS)
 STATVFS_FIELD_NAMES = [name for name, tp in STATVFS_FIELDS]
 
-
 def build_stat_result(st):
     # only for LL backends
     if TIMESPEC is not None:
-        atim = st.c_st_atim
-        mtim = st.c_st_mtim
-        ctim = st.c_st_ctim
-        atime, extra_atime = atim.c_tv_sec, int(atim.c_tv_nsec)
-        mtime, extra_mtime = mtim.c_tv_sec, int(mtim.c_tv_nsec)
-        ctime, extra_ctime = ctim.c_tv_sec, int(ctim.c_tv_nsec)
+        atim = st.c_st_atim; atime = int(atim.c_tv_sec) + 1E-9 * int(atim.c_tv_nsec)
+        mtim = st.c_st_mtim; mtime = int(mtim.c_tv_sec) + 1E-9 * int(mtim.c_tv_nsec)
+        ctim = st.c_st_ctim; ctime = int(ctim.c_tv_sec) + 1E-9 * int(ctim.c_tv_nsec)
     else:
-        atime, extra_atime = st.c_st_atime, 0
-        mtime, extra_mtime = st.c_st_mtime, 0
-        ctime, extra_ctime = st.c_st_ctime, 0
+        atime = st.c_st_atime
+        mtime = st.c_st_mtime
+        ctime = st.c_st_ctime
 
     result = (st.c_st_mode,
               st.c_st_ino,
@@ -484,10 +402,6 @@ def build_stat_result(st):
     if "st_rdev"    in STAT_FIELD_TYPES: result += (st.c_st_rdev,)
     if "st_flags"   in STAT_FIELD_TYPES: result += (st.c_st_flags,)
 
-    result += (extra_atime,
-               extra_mtime,
-               extra_ctime)
-
     return make_stat_result(result)
 
 
@@ -548,14 +462,12 @@ def fstat(fd):
             # console or LPT device
             return make_stat_result((win32traits._S_IFCHR,
                                      0, 0, 0, 0, 0,
-                                     0, 0, 0, 0,
-                                     0, 0, 0))
+                                     0, 0, 0, 0))
         elif filetype == win32traits.FILE_TYPE_PIPE:
             # socket or named pipe
             return make_stat_result((win32traits._S_IFIFO,
                                      0, 0, 0, 0, 0,
-                                     0, 0, 0, 0,
-                                     0, 0, 0))
+                                     0, 0, 0, 0))
         elif filetype == win32traits.FILE_TYPE_UNKNOWN:
             error = rwin32.GetLastError_saved()
             if error != 0:
@@ -600,45 +512,6 @@ def lstat(path):
         path = traits.as_str0(path)
         return win32_xstat(traits, path, traverse=False)
 
-@specialize.argtype(0)
-def stat3(path):
-    # On Windows, the algorithm behind os.stat() changed a lot between
-    # Python 2 and Python 3.  This is the Python 3 version.
-    if not _WIN32:
-        return stat(path)
-    else:
-        traits = _preferred_traits(path)
-        path = traits.as_str0(path)
-        return win32_xstat3(traits, path, traverse=True)
-
-@specialize.argtype(0)
-def lstat3(path):
-    # On Windows, the algorithm behind os.lstat() changed a lot between
-    # Python 2 and Python 3.  This is the Python 3 version.
-    if not _WIN32:
-        return lstat(path)
-    else:
-        traits = _preferred_traits(path)
-        path = traits.as_str0(path)
-        return win32_xstat3(traits, path, traverse=False)
-
-if rposix.HAVE_FSTATAT:
-    from rpython.rlib.rposix import AT_FDCWD, AT_SYMLINK_NOFOLLOW
-    c_fstatat = rffi.llexternal('fstatat64' if _LINUX else 'fstatat',
-        [rffi.INT, rffi.CCHARP, STAT_STRUCT, rffi.INT], rffi.INT,
-        compilation_info=compilation_info,
-        save_err=rffi.RFFI_SAVE_ERRNO, macro=True)
-
-    def fstatat(pathname, dir_fd=AT_FDCWD, follow_symlinks=True):
-        if follow_symlinks:
-            flags = 0
-        else:
-            flags = AT_SYMLINK_NOFOLLOW
-        with lltype.scoped_alloc(STAT_STRUCT.TO) as stresult:
-            error = c_fstatat(dir_fd, pathname, stresult, flags)
-            handle_posix_error('fstatat', error)
-            return build_stat_result(stresult)
-
 @replace_os_function('fstatvfs')
 def fstatvfs(fd):
     with lltype.scoped_alloc(STATVFS_STRUCT.TO) as stresult:
@@ -656,14 +529,16 @@ def statvfs(path):
 #__________________________________________________
 # Helper functions for win32
 if _WIN32:
-    from rpython.rlib.rwin32file import FILE_TIME_to_time_t_nsec
+    from rpython.rlib.rwin32file import FILE_TIME_to_time_t_float
 
     def make_longlong(high, low):
         return (rffi.r_longlong(high) << 32) + rffi.r_longlong(low)
 
+    # Seconds between 1.1.1601 and 1.1.1970
+    secs_between_epochs = rffi.r_longlong(11644473600)
+
     @specialize.arg(0)
     def win32_xstat(traits, path, traverse=False):
-        # XXX 'traverse' is ignored
         win32traits = make_win32_traits(traits)
         with lltype.scoped_alloc(
                 win32traits.WIN32_FILE_ATTRIBUTE_DATA) as data:
@@ -679,45 +554,6 @@ if _WIN32:
                 raise WindowsError(errcode, "os_stat failed")
             return win32_attribute_data_to_stat(win32traits, data)
 
-    @specialize.arg(0)
-    def win32_xstat3(traits, path, traverse=False):
-        # This is the Python3 version of os.stat() or lstat().
-        # XXX 'traverse' is ignored, and everything related to
-        # the "reparse points" is missing
-        win32traits = make_win32_traits(traits)
-
-        hFile = win32traits.CreateFile(path,
-            win32traits.FILE_READ_ATTRIBUTES,
-            0,
-            lltype.nullptr(rwin32.LPSECURITY_ATTRIBUTES.TO),
-            win32traits.OPEN_EXISTING,
-            win32traits.FILE_ATTRIBUTE_NORMAL |
-                win32traits.FILE_FLAG_BACKUP_SEMANTICS |
-                0,  # win32traits.FILE_FLAG_OPEN_REPARSE_POINT,
-            rwin32.NULL_HANDLE)
-
-        if hFile == rwin32.INVALID_HANDLE_VALUE:
-            errcode = rwin32.GetLastError_saved()
-            if (errcode != win32traits.ERROR_ACCESS_DENIED and
-                errcode != win32traits.ERROR_SHARING_VIOLATION):
-                raise WindowsError(errcode, "os_stat failed")
-
-            with lltype.scoped_alloc(
-                    win32traits.WIN32_FILE_ATTRIBUTE_DATA) as data:
-                if win32_attributes_from_dir(win32traits, path, data) == 0:
-                    raise WindowsError(rwin32.GetLastError_saved(),
-                                       "win32_attributes_from_dir failed")
-                return win32_attribute_data_to_stat(win32traits, data)
-
-        with lltype.scoped_alloc(
-                win32traits.BY_HANDLE_FILE_INFORMATION, zero=True) as data:
-            res = win32traits.GetFileInformationByHandle(hFile, data)
-            errcode = rwin32.GetLastError_saved()
-            rwin32.CloseHandle(hFile)
-            if res == 0:
-                raise WindowsError(errcode, "GetFileInformationByHandle failed")
-            return win32_by_handle_info_to_stat(win32traits, data)
-
     @specialize.arg(0)
     def win32_attributes_to_mode(win32traits, attributes):
         m = 0
@@ -736,37 +572,33 @@ if _WIN32:
     def win32_attribute_data_to_stat(win32traits, info):
         st_mode = win32_attributes_to_mode(win32traits, info.c_dwFileAttributes)
         st_size = make_longlong(info.c_nFileSizeHigh, info.c_nFileSizeLow)
-        ctime, extra_ctime = FILE_TIME_to_time_t_nsec(info.c_ftCreationTime)
-        mtime, extra_mtime = FILE_TIME_to_time_t_nsec(info.c_ftLastWriteTime)
-        atime, extra_atime = FILE_TIME_to_time_t_nsec(info.c_ftLastAccessTime)
+        ctime = FILE_TIME_to_time_t_float(info.c_ftCreationTime)
+        mtime = FILE_TIME_to_time_t_float(info.c_ftLastWriteTime)
+        atime = FILE_TIME_to_time_t_float(info.c_ftLastAccessTime)
 
         result = (st_mode,
                   0, 0, 0, 0, 0,
                   st_size,
-                  atime, mtime, ctime,
-                  extra_atime, extra_mtime, extra_ctime)
+                  atime, mtime, ctime)
 
         return make_stat_result(result)
 
-    @specialize.arg(0)
     def win32_by_handle_info_to_stat(win32traits, info):
         # similar to the one above
         st_mode = win32_attributes_to_mode(win32traits, info.c_dwFileAttributes)
         st_size = make_longlong(info.c_nFileSizeHigh, info.c_nFileSizeLow)
-        ctime, extra_ctime = FILE_TIME_to_time_t_nsec(info.c_ftCreationTime)
-        mtime, extra_mtime = FILE_TIME_to_time_t_nsec(info.c_ftLastWriteTime)
-        atime, extra_atime = FILE_TIME_to_time_t_nsec(info.c_ftLastAccessTime)
+        ctime = FILE_TIME_to_time_t_float(info.c_ftCreationTime)
+        mtime = FILE_TIME_to_time_t_float(info.c_ftLastWriteTime)
+        atime = FILE_TIME_to_time_t_float(info.c_ftLastAccessTime)
 
         # specific to fstat()
         st_ino = make_longlong(info.c_nFileIndexHigh, info.c_nFileIndexLow)
-        st_dev = info.c_dwVolumeSerialNumber
         st_nlink = info.c_nNumberOfLinks
 
         result = (st_mode,
-                  st_ino, st_dev, st_nlink, 0, 0,
+                  st_ino, 0, st_nlink, 0, 0,
                   st_size,
-                  atime, mtime, ctime,
-                  extra_atime, extra_mtime, extra_ctime)
+                  atime, mtime, ctime)
 
         return make_stat_result(result)
 
@@ -787,3 +619,4 @@ if _WIN32:
             return 1
         finally:
             lltype.free(filedata, flavor='raw')
+
