@@ -1,9 +1,9 @@
-
 from rpython.rlib.rsre import rsre_char
 from rpython.rlib.rarithmetic import intmask
 
-
+VERSION = "2.7.6"
 MAGIC = 20031017
+MAXREPEAT = rsre_char.MAXREPEAT
 CODESIZE = rsre_char.CODESIZE
 getlower = rsre_char.getlower
 
