@@ -1,5 +1,5 @@
 from collections import OrderedDict
-import pytest
+import py
 from rpython.rlib.objectmodel import (
     r_dict, UnboxedValue, Symbolic, compute_hash, compute_identity_hash,
     compute_unique_id, current_object_addr_as_int, we_are_translated,
@@ -7,12 +7,10 @@ from rpython.rlib.objectmodel import (
     resizelist_hint, is_annotation_constant, always_inline, NOT_CONSTANT,
     iterkeys_with_hash, iteritems_with_hash, contains_with_hash,
     setitem_with_hash, getitem_with_hash, delitem_with_hash, import_from_mixin,
-    fetch_translated_config, try_inline, delitem_if_value_is, move_to_end,
-    never_allocate, dont_inline, list_get_physical_size,
-    dict_to_switch)
+    fetch_translated_config)
 from rpython.translator.translator import TranslationContext, graphof
 from rpython.rtyper.test.tool import BaseRtypingTest
-from rpython.rtyper.test.test_llinterp import interpret, LLException, gengraph
+from rpython.rtyper.test.test_llinterp import interpret
 from rpython.conftest import option
 
 def strange_key_eq(key1, key2):
@@ -146,13 +144,13 @@ def test_unboxed_value():
     assert A(12098).get_untagged_value() == 12098
 
 def test_symbolic():
-    pytest.skip("xxx no test here")
+    py.test.skip("xxx no test here")
 
 def test_symbolic_raises():
     s1 = Symbolic()
     s2 = Symbolic()
-    pytest.raises(TypeError, "s1 < s2")
-    pytest.raises(TypeError, "hash(s1)")
+    py.test.raises(TypeError, "s1 < s2")
+    py.test.raises(TypeError, "hash(s1)")
 
 def test_compute_hash():
     from rpython.rlib.objectmodel import _hash_string, _hash_float, _hash_tuple
@@ -168,6 +166,7 @@ def test_compute_hash():
     foo = Foo()
     h = compute_hash(foo)
     assert h == object.__hash__(foo)
+    assert h == getattr(foo, '__precomputed_identity_hash')
     assert compute_hash(None) == 0
 
 def test_compute_hash_float():
@@ -183,6 +182,7 @@ def test_compute_identity_hash():
     foo = Foo()
     h = compute_identity_hash(foo)
     assert h == object.__hash__(foo)
+    assert h == getattr(foo, '__precomputed_identity_hash')
 
 def test_compute_unique_id():
     from rpython.rlib.rarithmetic import intmask
@@ -332,13 +332,6 @@ class TestObjectModel(BaseRtypingTest):
         res = self.interpret(g, [3])
         assert res == 77
 
-    def test_r_dict_fast_functions(self):
-        def fn():
-            d1 = r_dict(strange_key_eq, strange_key_hash, simple_hash_eq=True)
-            return play_with_r_dict(d1)
-        res = self.interpret(fn, [])
-        assert res
-
     def test_prepare_dict_update(self):
         def g(n):
             d = {}
@@ -417,6 +410,36 @@ class TestObjectModel(BaseRtypingTest):
         res = self.interpret(f, [])
         assert res == 1
 
+    def test_compute_hash_across_translation(self):
+        class Foo(object):
+            pass
+        q = Foo()
+
+        def f(i):
+            assert compute_hash(None) == 0
+            assert compute_hash(i) == h_42
+            assert compute_hash(i + 1.0) == h_43_dot_0
+            assert compute_hash((i + 3) / 6.0) == h_7_dot_5
+            assert compute_hash("Hello" + str(i)) == h_Hello42
+            if i == 42:
+                p = None
+            else:
+                p = Foo()
+            assert compute_hash(p) == h_None
+            assert compute_hash(("world", None, i, 7.5)) == h_tuple
+            assert compute_hash(q) == h_q
+            return i * 2
+        h_42 = compute_hash(42)
+        h_43_dot_0 = compute_hash(43.0)
+        h_7_dot_5 = compute_hash(7.5)
+        h_Hello42 = compute_hash("Hello42")
+        h_None = compute_hash(None)
+        h_tuple = compute_hash(("world", None, 42, 7.5))
+        h_q = compute_hash(q)
+
+        res = self.interpret(f, [42])
+        assert res == 84
+
     def test_fetch_translated_config(self):
         assert fetch_translated_config() is None
         def f():
@@ -447,12 +470,12 @@ def test_enforceargs_decorator():
         return a, b, c
     f.foo = 'foo'
     assert f._annenforceargs_ == (int, str, None)
-    assert f.__name__ == 'f'
+    assert f.func_name == 'f'
     assert f.foo == 'foo'
     assert f(1, 'hello', 42) == (1, 'hello', 42)
-    exc = pytest.raises(TypeError, "f(1, 2, 3)")
+    exc = py.test.raises(TypeError, "f(1, 2, 3)")
     assert exc.value.message == "f argument 'b' must be of type <type 'str'>"
-    pytest.raises(TypeError, "f('hello', 'world', 3)")
+    py.test.raises(TypeError, "f('hello', 'world', 3)")
 
 def test_always_inline():
     @always_inline
@@ -460,13 +483,6 @@ def test_always_inline():
         return a, b, c
     assert f._always_inline_ is True
 
-def test_try_inline():
-    @try_inline
-    def f(a, b, c):
-        return a, b, c
-    assert f._always_inline_ == "try"
-
-
 def test_enforceargs_defaults():
     @enforceargs(int, int)
     def f(a, b=40):
@@ -486,6 +502,12 @@ def test_enforceargs_int_float_promotion():
     # in RPython there is an implicit int->float promotion
     assert f(42) == 42
 
+def test_enforceargs_None_string():
+    @enforceargs(str, unicode)
+    def f(a, b):
+        return a, b
+    assert f(None, None) == (None, None)
+
 def test_enforceargs_complex_types():
     @enforceargs([int], {str: int})
     def f(a, b):
@@ -495,9 +517,9 @@ def test_enforceargs_complex_types():
     assert f(x, y) == (x, y)
     assert f([], {}) == ([], {})
     assert f(None, None) == (None, None)
-    pytest.raises(TypeError, "f(['hello'], y)")
-    pytest.raises(TypeError, "f(x, {'a': 'hello'})")
-    pytest.raises(TypeError, "f(x, {0: 42})")
+    py.test.raises(TypeError, "f(['hello'], y)")
+    py.test.raises(TypeError, "f(x, {'a': 'hello'})")
+    py.test.raises(TypeError, "f(x, {0: 42})")
 
 def test_enforceargs_no_typecheck():
     @enforceargs(int, str, None, typecheck=False)
@@ -591,24 +613,6 @@ def test_resizelist_hint_len():
     r = interpret(f, [29])
     assert r == 1
 
-def test_list_get_physical_size():
-    def f(z):
-        l = [z, z + 1]
-        if z:
-            l.append(z)
-        return list_get_physical_size(l)
-
-    r = interpret(f, [29])
-    assert r == 6
-
-    # now with fixed-sized list
-    def f(z):
-        l = [z, z + 1]
-        return list_get_physical_size(l)
-
-    r = interpret(f, [29])
-    assert r == 2
-
 def test_iterkeys_with_hash():
     def f(i):
         d = {i + .0: 5, i + .5: 6}
@@ -682,24 +686,6 @@ def test_delitem_with_hash():
     f(29)
     interpret(f, [27])
 
-def test_delitem_if_value_is():
-    class X:
-        pass
-    def f(i):
-        x42 = X()
-        x612 = X()
-        d = {i + .5: x42, i + .6: x612}
-        delitem_if_value_is(d, i + .5, x612)
-        assert (i + .5) in d
-        delitem_if_value_is(d, i + .5, x42)
-        assert (i + .5) not in d
-        delitem_if_value_is(d, i + .5, x612)
-        assert (i + .5) not in d
-        return 0
-
-    f(29)
-    interpret(f, [27])
-
 def test_rdict_with_hash():
     def f(i):
         d = r_dict(strange_key_eq, strange_key_hash)
@@ -718,25 +704,6 @@ def test_rdict_with_hash():
     assert f(29) == 0
     interpret(f, [27])
 
-def test_rordereddict_move_to_end():
-    d = OrderedDict()
-    d['key1'] = 'val1'
-    d['key2'] = 'val2'
-    d['key3'] = 'val3'
-    move_to_end(d, 'key1')
-    assert d.items() == [('key2', 'val2'), ('key3', 'val3'), ('key1', 'val1')]
-    move_to_end(d, 'key1', last=False)
-    assert d.items() == [('key1', 'val1'), ('key2', 'val2'), ('key3', 'val3')]
-
-def test_r_dict_move_to_end():
-    d = r_dict(strange_key_eq, strange_key_hash)
-    d['1key'] = 'val1'
-    d['2key'] = 'val2'
-    d['3key'] = 'val3'
-    # does not crash, we can't check that it actually moves to end on CPython
-    move_to_end(d, '1key')
-    move_to_end(d, '1key', last=False)
-
 def test_import_from_mixin():
     class M:    # old-style
         def f(self):
@@ -797,7 +764,7 @@ def test_import_from_mixin():
         class B(object):
             a = 63
             import_from_mixin(M)
-    except Exception as e:
+    except Exception, e:
         assert ("would overwrite the value already defined locally for 'a'"
                 in str(e))
     else:
@@ -823,10 +790,10 @@ def test_import_from_mixin():
     assert B().foo == 42
 
     d = dict(__name__='foo')
-    exec("""class M(object):
+    exec """class M(object):
                 @staticmethod
                 def f(): pass
-    """, d)
+    """ in d
     M = d['M']
     class A(object):
         import_from_mixin(M)
@@ -871,91 +838,3 @@ def test_import_from_mixin_immutable_fields():
         import_from_mixin(C)
 
     assert BA._immutable_fields_ == ['c', 'a']
-
-
-def test_never_allocate():
-    from rpython.translator.c.test.test_genc import compile as c_compile
-    from rpython.memory.gctransform.transform import GCTransformError
-
-    @never_allocate
-    class MyClass(object):
-        def __init__(self, x):
-            self.x = x + 1
-
-    @dont_inline
-    def allocate_MyClass(x):
-        return MyClass(x)
-
-    def f(x):
-        # this fails because the allocation of MyClass can't be
-        # constant-folded (because it's inside a @dont_inline function)
-        return allocate_MyClass(x).x
-
-    def g(x):
-        # this works because MyClass is constant folded, so the GC transformer
-        # never sees a malloc(MyClass)
-        return MyClass(x).x
-
-    # test what happens if MyClass escapes
-    with pytest.raises(GCTransformError) as exc:
-        c_compile(f, [int])
-    assert '[function allocate_MyClass]' in str(exc)
-    assert 'was marked as @never_allocate' in str(exc)
-
-    # test that it works in the "normal" case
-    compiled_g = c_compile(g, [int])
-    assert compiled_g(41) == 42
-
-def test_dict_to_switch():
-    d = dict_to_switch({1: "one", 2: "two"})
-    assert d(1) == "one"
-    assert d(2) == "two"
-    with pytest.raises(KeyError):
-        d(3)
-
-    d = dict_to_switch({"1": "one", "2": "two"})
-    assert d('1') == "one"
-    assert d('2') == "two"
-    with pytest.raises(KeyError):
-        d('3')
-    with pytest.raises(KeyError):
-        d('33454')
-
-    # various errors
-    with pytest.raises(AssertionError):
-        dict_to_switch({"abc": 1})
-    with pytest.raises(AssertionError):
-        dict_to_switch({(1, 2, 3): 1})
-
-def test_dict_to_switch_translate():
-    from rpython.flowspace.model import summary
-    d = dict_to_switch({1: 17, 2: 19})
-    r = interpret(d, [1])
-    assert r == 17
-    r = interpret(d, [2])
-    assert r == 19
-    with pytest.raises(LLException):
-        interpret(d, [10])
-
-    d = dict_to_switch({'1': 17, '2': 19})
-    r = interpret(d, ['1'])
-    assert r == 17
-    r = interpret(d, ['2'])
-    assert r == 19
-    with pytest.raises(LLException):
-        interpret(d, ['a'])
-
-    d = dict_to_switch({'1': 17, '2': 19, '3': 1121, 'a': -12})
-    def f(i):
-        c = chr(i)
-        if not i:
-            c = 'abc'
-        try:
-            return d(c)
-        except KeyError:
-            return -1
-
-
-    t, typer, g = gengraph(f, [int])
-    lookupg = t.graphs[1]
-    assert summary(lookupg)['direct_call'] == 1 # should be char based
