@@ -1,9 +1,7 @@
 from rpython.rtyper.test.tool import BaseRtypingTest
 from rpython.rtyper.test.test_llinterp import interpret
-from rpython.rlib import rarithmetic
 from rpython.rlib.rarithmetic import *
 from rpython.rlib.rstring import ParseStringError, ParseStringOverflowError
-from hypothesis import given, strategies, assume
 import sys
 import py
 
@@ -20,11 +18,11 @@ while i == l and type(i) is int:
 
 class Test_r_int:
     def test__add__(self):
-        self.binary_test(lambda x, y: x + y, includes_floats=True)
+        self.binary_test(lambda x, y: x + y)
     def test__sub__(self):
-        self.binary_test(lambda x, y: x - y, includes_floats=True)
+        self.binary_test(lambda x, y: x - y)
     def test__mul__(self):
-        self.binary_test(lambda x, y: x * y, includes_floats=True)
+        self.binary_test(lambda x, y: x * y)
         x = 3; y = [2]
         assert x*y == r_int(x)*y
         assert y*x == y*r_int(x)
@@ -60,15 +58,12 @@ class Test_r_int:
             cmp = f(r_int(arg))
             assert res == cmp
 
-    def binary_test(self, f, rargs=None, includes_floats=False):
+    def binary_test(self, f, rargs = None):
         if not rargs:
             rargs = (-10, -1, 3, 55)
-        types_list = [(int, r_int), (r_int, int), (r_int, r_int)]
-        if includes_floats:
-            types_list += [(float, r_int), (r_int, float)]
         for larg in (-10, -1, 0, 3, 1234):
             for rarg in rargs:
-                for types in types_list:
+                for types in ((int, r_int), (r_int, int), (r_int, r_int)):
                     res = f(larg, rarg)
                     left, right = types
                     cmp = f(left(larg), right(rarg))
@@ -395,26 +390,6 @@ def test_int_between():
     assert not int_between(1, 2, 2)
     assert not int_between(1, 1, 1)
 
-def test_int_force_ge_zero():
-    assert int_force_ge_zero(42) == 42
-    assert int_force_ge_zero(0) == 0
-    assert int_force_ge_zero(-42) == 0
-
-@given(strategies.integers(min_value=0, max_value=sys.maxint),
-       strategies.integers(min_value=1, max_value=sys.maxint))
-def test_int_c_div_mod(x, y):
-    assert int_c_div(~x, y) == -(abs(~x) // y)
-    assert int_c_div( x,-y) == -(x // y)
-
-@given(strategies.integers(min_value=0, max_value=sys.maxint),
-       strategies.integers(min_value=1, max_value=sys.maxint))
-def test_int_c_div_mod_2(x, y):
-    assume((x, y) != (sys.maxint, 1))  # This case would overflow
-    assert int_c_div(~x,-y) == +(abs(~x) // y)
-    for x1 in [x, ~x]:
-        for y1 in [y, -y]:
-            assert int_c_div(x1, y1) * y1 + int_c_mod(x1, y1) == x1
-
 # these can't be prebuilt on 32bit
 U1 = r_ulonglong(0x0102030405060708L)
 U2 = r_ulonglong(0x0807060504030201L)
@@ -553,221 +528,3 @@ class TestStringToInt:
             py.test.raises(ParseStringError, string_to_int, s+'  ', base)
             py.test.raises(ParseStringError, string_to_int, '+'+s, base)
             py.test.raises(ParseStringError, string_to_int, '-'+s, base)
-
-    @py.test.mark.parametrize('s', [
-        '0_0_0',
-        '4_2',
-        '1_0000_0000',
-        '0b1001_0100',
-        '0xfff_ffff',
-        '0o5_7_7',
-        '0b_0',
-        '0x_f',
-        '0o_5',
-    ])
-    def test_valid_underscores(self, s):
-        result = string_to_int(
-            s, base=0, allow_underscores=True, no_implicit_octal=True)
-        assert result == int(s.replace('_', ''), base=0)
-
-    @py.test.mark.parametrize('s', [
-        # Leading underscores
-        '_100',
-        '_',
-        '_0b1001_0100',
-        # Trailing underscores:
-        '0_',
-        '42_',
-        '1.4j_',
-        '0x_',
-        '0b1_',
-        '0xf_',
-        '0o5_',
-        # Underscores in the base selector:
-        '0_b0',
-        '0_xf',
-        '0_o5',
-        # Old-style octal, still disallowed:
-        '09_99',
-        # Multiple consecutive underscores:
-        '4_______2',
-        '0b1001__0100',
-        '0xfff__ffff',
-        '0x___',
-        '0o5__77',
-        '1e1__0',
-    ])
-    def test_invalid_underscores(self, s):
-        with py.test.raises(ParseStringError):
-            string_to_int(s, base=0, allow_underscores=True)
-
-    def test_no_implicit_octal(self):
-        TESTS = ['00', '000', '00_00', '02', '0377', '02_34']
-        for x in TESTS:
-            for valid_underscore in [False, True]:
-                for no_implicit_octal in [False, True]:
-                    print x, valid_underscore, no_implicit_octal
-                    expected_ok = True
-                    if no_implicit_octal and any('1' <= c <= '7' for c in x):
-                        expected_ok = False
-                    if not valid_underscore and '_' in x:
-                        expected_ok = False
-                    if expected_ok:
-                        y = string_to_int(x, base=0,
-                                          allow_underscores=valid_underscore,
-                                          no_implicit_octal=no_implicit_octal)
-                        assert y == int(x.replace('_', ''), base=8)
-                    else:
-                        py.test.raises(ParseStringError, string_to_int, x,
-                                       base=0,
-                                       allow_underscores=valid_underscore,
-                                       no_implicit_octal=no_implicit_octal)
-
-
-class TestExplicitIntsizes:
-
-    _32_max =            2147483647
-    _32_min =           -2147483648
-    _32_umax =           4294967295
-    _64_max =   9223372036854775807
-    _64_min =  -9223372036854775808
-    _64_umax = 18446744073709551615
-
-    def test_explicit_32(self):
-
-        assert type(r_int32(0)) == r_int32
-        assert type(r_int32(self._32_max)) == r_int32
-        assert type(r_int32(self._32_min)) == r_int32
-
-        assert type(r_uint32(0)) == r_uint32
-        assert type(r_uint32(self._32_umax)) == r_uint32
-
-        with py.test.raises(OverflowError):
-            ovfcheck(r_int32(self._32_max) + r_int32(1))
-            ovfcheck(r_int32(self._32_min) - r_int32(1))
-
-        assert most_pos_value_of_same_type(r_int32(1)) == self._32_max
-        assert most_neg_value_of_same_type(r_int32(1)) == self._32_min
-
-        assert most_pos_value_of_same_type(r_uint32(1)) == self._32_umax
-        assert most_neg_value_of_same_type(r_uint32(1)) == 0
-
-        assert r_uint32(self._32_umax) + r_uint32(1) == r_uint32(0)
-        assert r_uint32(0) - r_uint32(1) == r_uint32(self._32_umax)
-
-    def test_explicit_64(self):
-
-        assert type(r_int64(0)) == r_int64
-        assert type(r_int64(self._64_max)) == r_int64
-        assert type(r_int64(self._64_min)) == r_int64
-
-        assert type(r_uint64(0)) == r_uint64
-        assert type(r_uint64(self._64_umax)) == r_uint64
-
-        with py.test.raises(OverflowError):
-            ovfcheck(r_int64(self._64_max) + r_int64(1))
-            ovfcheck(r_int64(self._64_min) - r_int64(1))
-
-        assert most_pos_value_of_same_type(r_int64(1)) == self._64_max
-        assert most_neg_value_of_same_type(r_int64(1)) == self._64_min
-
-        assert most_pos_value_of_same_type(r_uint64(1)) == self._64_umax
-        assert most_neg_value_of_same_type(r_uint64(1)) == 0
-
-        assert r_uint64(self._64_umax) + r_uint64(1) == r_uint64(0)
-        assert r_uint64(0) - r_uint64(1) == r_uint64(self._64_umax)
-
-
-def test_operation_with_float():
-    def f(x):
-        assert r_longlong(x) + 0.5 == 43.5
-        assert r_longlong(x) - 0.5 == 42.5
-        assert r_longlong(x) * 0.5 == 21.5
-        assert r_longlong(x) / 0.8 == 53.75
-    f(43)
-    interpret(f, [43])
-
-def test_int64_plus_int32():
-    assert r_uint64(1234567891234) + r_uint32(1) == r_uint64(1234567891235)
-
-def test_fallback_paths():
-
-    def make(pattern):
-        def method(self, other, *extra):
-            if extra:
-                assert extra == (None,)   # for 'pow'
-            if type(other) is long:
-                return pattern % other
-            else:
-                return NotImplemented
-        return method
-
-    class A(object):
-        __add__  = make("a+%d")
-        __radd__ = make("%d+a")
-        __sub__  = make("a-%d")
-        __rsub__ = make("%d-a")
-        __mul__  = make("a*%d")
-        __rmul__ = make("%d*a")
-        __div__  = make("a/%d")
-        __rdiv__ = make("%d/a")
-        __floordiv__  = make("a//%d")
-        __rfloordiv__ = make("%d//a")
-        __mod__  = make("a%%%d")
-        __rmod__ = make("%d%%a")
-        __and__  = make("a&%d")
-        __rand__ = make("%d&a")
-        __or__   = make("a|%d")
-        __ror__  = make("%d|a")
-        __xor__  = make("a^%d")
-        __rxor__ = make("%d^a")
-        __pow__  = make("a**%d")
-        __rpow__ = make("%d**a")
-
-    a = A()
-    assert r_uint32(42) + a == "42+a"
-    assert a + r_uint32(42) == "a+42"
-    assert r_uint32(42) - a == "42-a"
-    assert a - r_uint32(42) == "a-42"
-    assert r_uint32(42) * a == "42*a"
-    assert a * r_uint32(42) == "a*42"
-    assert r_uint32(42) / a == "42/a"
-    assert a / r_uint32(42) == "a/42"
-    assert r_uint32(42) // a == "42//a"
-    assert a // r_uint32(42) == "a//42"
-    assert r_uint32(42) % a == "42%a"
-    assert a % r_uint32(42) == "a%42"
-    py.test.raises(TypeError, "a << r_uint32(42)")
-    py.test.raises(TypeError, "r_uint32(42) << a")
-    py.test.raises(TypeError, "a >> r_uint32(42)")
-    py.test.raises(TypeError, "r_uint32(42) >> a")
-    assert r_uint32(42) & a == "42&a"
-    assert a & r_uint32(42) == "a&42"
-    assert r_uint32(42) | a == "42|a"
-    assert a | r_uint32(42) == "a|42"
-    assert r_uint32(42) ^ a == "42^a"
-    assert a ^ r_uint32(42) == "a^42"
-    assert r_uint32(42) ** a == "42**a"
-    assert a ** r_uint32(42) == "a**42"
-
-def test_ovfcheck_int32():
-    assert ovfcheck_int32_add(-2**30, -2**30) == -2**31
-    py.test.raises(OverflowError, ovfcheck_int32_add, 2**30, 2**30)
-    assert ovfcheck_int32_sub(-2**30, 2**30) == -2**31
-    py.test.raises(OverflowError, ovfcheck_int32_sub, 2**30, -2**30)
-    assert ovfcheck_int32_mul(-2**16, 2**15) == -2**31
-    py.test.raises(OverflowError, ovfcheck_int32_mul, -2**16, -2**15)
-
-@given(strategies.integers(min_value=-sys.maxint-1, max_value=sys.maxint),
-       strategies.integers(min_value=-sys.maxint-1, max_value=sys.maxint),
-       strategies.integers(min_value=1, max_value=sys.maxint))
-def test_mulmod(a, b, c):
-    assert mulmod(a, b, c) == (a * b) % c
-    #
-    import rpython.rlib.rbigint  # import before patching check_support_int128
-    prev = rarithmetic.check_support_int128
-    try:
-        rarithmetic.check_support_int128 = lambda: False
-        assert mulmod(a, b, c) == (a * b) % c
-    finally:
-        rarithmetic.check_support_int128 = prev
