@@ -657,23 +657,6 @@ class BaseTestTextCRLFFilter(BaseRtypingTest):
             assert line == ''
         self.interpret(f, [])
 
-    def test_read1(self):
-        s_input = "abc\r\nabc\nd\r\nef\r\ngha\rbc\rdef\n\r\n\r"
-        s_output = "abc\nabc\nd\nef\ngha\rbc\rdef\n\n\r"
-        assert s_output == s_input.replace('\r\n', '\n')
-        packets = list(s_input)
-        expected = list(s_output)
-        crlf = streamio.TextCRLFFilter(TSource(packets))
-        def f():
-            blocks = []
-            while True:
-                block = crlf.read(1)
-                if not block:
-                    break
-                blocks.append(block)
-            assert blocks == expected
-        self.interpret(f, [])
-
 class TestTextCRLFFilterLLInterp(BaseTestTextCRLFFilter):
     pass
 
@@ -692,7 +675,7 @@ class TestMMapFile(BaseTestBufferingInputStreamTests):
             self.tfn = None
             try:
                 os.remove(tfn)
-            except os.error as msg:
+            except os.error, msg:
                 print "can't remove %s: %s" % (tfn, msg)
 
     def makeStream(self, tell=None, seek=None, bufsize=-1, mode="r"):
@@ -1123,7 +1106,6 @@ class TestDiskFile:
             alarm(0)
             signal(SIGALRM, SIG_DFL)
 
-    @pytest.mark.skipif("sys.platform == 'darwin'")
     def test_append_mode(self):
         tfn = str(udir.join('streamio-append-mode'))
         fo = streamio.open_file_as_stream # shorthand
@@ -1139,7 +1121,6 @@ class TestDiskFile:
         assert x.read() == 'abc123456'
         x.close()
 
-    @pytest.mark.skipif("sys.platform == 'darwin'")
     def test_seek_changed_underlying_position(self):
         tfn = str(udir.join('seek_changed_underlying_position'))
         fo = streamio.open_file_as_stream # shorthand
@@ -1157,7 +1138,6 @@ class TestDiskFile:
         assert x.tell() == 0    # detected in this case.  not always.
         # the point of the test is that we don't crash in an assert.
 
-    @pytest.mark.skipif("sys.platform == 'darwin'")
     def test_ignore_ioerror_in_readall_if_nonempty_result(self):
         # this is the behavior of regular files in CPython 2.7, as
         # well as of _io.FileIO at least in CPython 3.3.  This is
