@@ -8,7 +8,7 @@ from rpython.rlib.objectmodel import Symbolic
 from rpython.rlib.rarithmetic import (
     base_int, intmask, is_emulated_long, is_valid_int, longlonglongmask,
     longlongmask, maxint, normalizedinttype, r_int, r_longfloat, r_longlong,
-    r_longlonglong, r_singlefloat, r_uint, r_ulonglong, r_ulonglonglong)
+    r_longlonglong, r_singlefloat, r_uint, r_ulonglong)
 from rpython.rtyper.extregistry import ExtRegistryEntry
 from rpython.tool import leakfinder
 from rpython.tool.identity_dict import identity_dict
@@ -467,16 +467,12 @@ class Array(ContainerType):
     _str_fields = saferecursive(_str_fields, '...')
 
     def __str__(self):
-        hints = (' ' + str(self._hints)) if self._hints else ''
-        return "%s of %s%s " % (self.__class__.__name__,
-                                self._str_fields(),
-                                hints)
+        return "%s of %s " % (self.__class__.__name__,
+                               self._str_fields(),)
 
     def _short_name(self):
-        hints = (' ' + str(self._hints)) if self._hints else ''
-        return "%s %s%s" % (self.__class__.__name__,
-                            self.OF._short_name(),
-                            hints)
+        return "%s %s" % (self.__class__.__name__,
+                          self.OF._short_name(),)
     _short_name = saferecursive(_short_name, '...')
 
     def _container_example(self):
@@ -584,7 +580,7 @@ class OpaqueType(ContainerType):
         """
         self.tag = tag
         self.__name__ = tag
-        self._hints = frozendict(hints)
+        self.hints = frozendict(hints)
 
     def __str__(self):
         return "%s (opaque)" % self.tag
@@ -680,7 +676,6 @@ _numbertypes = {int: Number("Signed", int, intmask)}
 _numbertypes[r_int] = _numbertypes[int]
 _numbertypes[r_longlonglong] = Number("SignedLongLongLong", r_longlonglong,
                                       longlonglongmask)
-
 if r_longlong is not r_int:
     _numbertypes[r_longlong] = Number("SignedLongLong", r_longlong,
                                       longlongmask)
@@ -705,7 +700,6 @@ Unsigned = build_number("Unsigned", r_uint)
 SignedLongLong = build_number("SignedLongLong", r_longlong)
 SignedLongLongLong = build_number("SignedLongLongLong", r_longlonglong)
 UnsignedLongLong = build_number("UnsignedLongLong", r_ulonglong)
-UnsignedLongLongLong = build_number("UnsignedLongLongLong", r_ulonglonglong)
 
 Float       = Primitive("Float",       0.0)                  # C type 'double'
 SingleFloat = Primitive("SingleFloat", r_singlefloat(0.0))   # 'float'
@@ -818,14 +812,8 @@ def typeOf(val):
         if tp is long:
             if -maxint-1 <= val <= maxint:
                 return Signed
-            elif longlongmask(val) == val:
-                raise OverflowError("integer %r is out of bounds for Signed "
-                                    "(it would fit SignedLongLong, but we "
-                                    "won't implicitly return SignedLongLong "
-                                    "for typeOf(%r) where type(%r) is long)"
-                                    % (val, val, val))
             else:
-                raise OverflowError("integer %r is out of bounds" % (val,))
+                return SignedLongLong
         if tp is bool:
             return Bool
         if issubclass(tp, base_int):
@@ -1390,11 +1378,20 @@ class _abstract_ptr(object):
             return callb(*args)
         raise TypeError("%r instance is not a function" % (self._T,))
 
-    def _identityhash(self):
+    def _identityhash(self, cache=True):
         p = normalizeptr(self)
-        assert self._T._gckind == 'gc'
-        assert self      # not for NULL
-        return hash(p._obj)
+        try:
+            return p._obj._hash_cache_
+        except AttributeError:
+            assert self._T._gckind == 'gc'
+            assert self      # not for NULL
+            result = hash(p._obj)
+            if cache:
+                try:
+                    p._obj._hash_cache_ = result
+                except AttributeError:
+                    pass
+            return result
 
 class _ptr(_abstract_ptr):
     __slots__ = ('_TYPE',
@@ -1760,7 +1757,7 @@ def _get_empty_instance_of_struct_variety(flds):
 class _struct(_parentable):
     _kind = "structure"
 
-    __slots__ = ('_compilation_info',)
+    __slots__ = ('_hash_cache_', '_compilation_info')
 
     def __new__(self, TYPE, n=None, initialization=None, parent=None,
                 parentindex=None):
@@ -1927,29 +1924,14 @@ class _array(_parentable):
         return 0, stop
 
     def getitem(self, index, uninitialized_ok=False):
-        try:
-            v = self.items[index]
-        except IndexError:
-            if (index == len(self.items) and uninitialized_ok == 2 and
-                self._TYPE._hints.get('extra_item_after_alloc')):
-                # special case: reading the extra final char returns
-                # an uninitialized, if 'uninitialized_ok==2'
-                return _uninitialized(self._TYPE.OF)
-            raise
+        v = self.items[index]
         if isinstance(v, _uninitialized) and not uninitialized_ok:
             raise UninitializedMemoryAccess("%r[%s]"%(self, index))
         return v
 
     def setitem(self, index, value):
         assert typeOf(value) == self._TYPE.OF
-        try:
-            self.items[index] = value
-        except IndexError:
-            if (index == len(self.items) and value == '\x00' and
-                self._TYPE._hints.get('extra_item_after_alloc')):
-                # special case: writing NULL to the extra final char
-                return
-            raise
+        self.items[index] = value
 
 assert not '__dict__' in dir(_array)
 assert not '__dict__' in dir(_struct)
@@ -2192,8 +2174,7 @@ class _opaque(_parentable):
 
 
 def malloc(T, n=None, flavor='gc', immortal=False, zero=False,
-           track_allocation=True, add_memory_pressure=False,
-           nonmovable=False):
+           track_allocation=True, add_memory_pressure=False):
     assert flavor in ('gc', 'raw')
     if zero or immortal:
         initialization = 'example'
@@ -2218,9 +2199,8 @@ def malloc(T, n=None, flavor='gc', immortal=False, zero=False,
     return _ptr(Ptr(T), o, solid)
 
 @analyzer_for(malloc)
-def ann_malloc(s_T, s_n=None, s_flavor=None, s_immortal=None, s_zero=None,
-               s_track_allocation=None, s_add_memory_pressure=None,
-               s_nonmovable=None):
+def ann_malloc(s_T, s_n=None, s_flavor=None, s_zero=None,
+               s_track_allocation=None, s_add_memory_pressure=None):
     assert (s_n is None or s_n.knowntype == int
             or issubclass(s_n.knowntype, base_int))
     assert s_T.is_constant()
@@ -2238,7 +2218,6 @@ def ann_malloc(s_T, s_n=None, s_flavor=None, s_immortal=None, s_zero=None,
         assert s_track_allocation is None or s_track_allocation.is_constant()
         assert (s_add_memory_pressure is None or
                 s_add_memory_pressure.is_constant())
-        assert s_nonmovable is None or s_nonmovable.is_constant()
         # not sure how to call malloc() for the example 'p' in the
         # presence of s_extraargs
         r = SomePtr(Ptr(s_T.const))
@@ -2443,6 +2422,24 @@ def ann_identityhash(s_obj):
     return SomeInteger()
 
 
+def identityhash_nocache(p):
+    """Version of identityhash() to use from backends that don't care about
+    caching."""
+    assert p
+    return p._identityhash(cache=False)
+
+def init_identity_hash(p, value):
+    """For a prebuilt object p, initialize its hash value to 'value'."""
+    assert isinstance(typeOf(p), Ptr)
+    p = normalizeptr(p)
+    if not p:
+        raise ValueError("cannot change hash(NULL)!")
+    if hasattr(p._obj, '_hash_cache_'):
+        raise ValueError("the hash of %r was already computed" % (p,))
+    if typeOf(p).TO._is_varsize():
+        raise ValueError("init_identity_hash(): not for varsized types")
+    p._obj._hash_cache_ = intmask(value)
+
 def isCompatibleType(TYPE1, TYPE2):
     return TYPE1._is_compatible(TYPE2)
 
