@@ -9,11 +9,11 @@ from rpython.rtyper.exceptiondata import UnknownException
 from rpython.translator.translator import TranslationContext, graphof
 from rpython.rtyper.lltypesystem import lltype
 from rpython.annotator import model as annmodel
-from rpython.annotator.model import lltype_to_annotation
+from rpython.rtyper.llannotation import lltype_to_annotation
 from rpython.rlib.rarithmetic import r_uint, ovfcheck
 from rpython.tool import leakfinder
 from rpython.conftest import option
-
+from rpython.rtyper.rtyper import llinterp_backend
 
 # switch on logging of interp to show more info on failing tests
 
@@ -25,22 +25,13 @@ def teardown_module(mod):
     py.log._setstate(mod.logstate)
 
 
-
-def timelog(prefix, call, *args, **kwds):
-    #import time
-    #print prefix, "...",
-    #start = time.time()
-    res = call(*args, **kwds)
-    #elapsed = time.time() - start
-    #print "%.2f secs" % (elapsed,)
-    return res
-
 def gengraph(func, argtypes=[], viewbefore='auto', policy=None,
              backendopt=False, config=None, **extraconfigopts):
     t = TranslationContext(config=config)
     t.config.set(**extraconfigopts)
     a = t.buildannotator(policy=policy)
-    timelog("annotating", a.build_types, func, argtypes, main_entry_point=True)
+    a.build_types(func, argtypes, main_entry_point=True)
+    a.validate()
     if viewbefore == 'auto':
         viewbefore = getattr(option, 'view', False)
     if viewbefore:
@@ -48,13 +39,14 @@ def gengraph(func, argtypes=[], viewbefore='auto', policy=None,
         t.view()
     global typer # we need it for find_exception
     typer = t.buildrtyper()
-    timelog("rtyper-specializing", typer.specialize)
+    typer.backend = llinterp_backend
+    typer.specialize()
     #t.view()
-    timelog("checking graphs", t.checkgraphs)
+    t.checkgraphs()
     if backendopt:
         from rpython.translator.backendopt.all import backend_optimizations
         backend_optimizations(t)
-        timelog("checking graphs", t.checkgraphs)
+        t.checkgraphs()
         if viewbefore:
             t.view()
     desc = t.annotator.bookkeeper.getdesc(func)
@@ -68,8 +60,7 @@ def clear_tcache():
     _tcache.clear()
 
 def get_interpreter(func, values, view='auto', viewbefore='auto', policy=None,
-                    type_system="lltype", backendopt=False, config=None,
-                    **extraconfigopts):
+                    backendopt=False, config=None, **extraconfigopts):
     extra_key = [(key, value) for key, value in extraconfigopts.iteritems()]
     extra_key.sort()
     extra_key = tuple(extra_key)
@@ -102,10 +93,8 @@ def get_interpreter(func, values, view='auto', viewbefore='auto', policy=None,
     return interp, graph
 
 def interpret(func, values, view='auto', viewbefore='auto', policy=None,
-              type_system="lltype", backendopt=False, config=None,
-              malloc_check=True, **kwargs):
+              backendopt=False, config=None, malloc_check=True, **kwargs):
     interp, graph = get_interpreter(func, values, view, viewbefore, policy,
-                                    type_system=type_system,
                                     backendopt=backendopt, config=config,
                                     **kwargs)
     if not malloc_check:
@@ -121,15 +110,14 @@ def interpret(func, values, view='auto', viewbefore='auto', policy=None,
     return result
 
 def interpret_raises(exc, func, values, view='auto', viewbefore='auto',
-                     policy=None, type_system="lltype",
+                     policy=None,
                      backendopt=False):
     interp, graph  = get_interpreter(func, values, view, viewbefore, policy,
-                                     type_system=type_system,
                                      backendopt=backendopt)
     info = py.test.raises(LLException, "interp.eval_graph(graph, values)")
     try:
         got = interp.find_exception(info.value)
-    except ValueError, message:
+    except ValueError as message:
         got = 'None %r' % message
     assert got is exc, "wrong exception type, expected %r got %r" % (exc, got)
 
@@ -336,14 +324,15 @@ def test_mod_ovf_zer():
 
 def test_funny_links():
     from rpython.flowspace.model import Block, FunctionGraph, \
-         SpaceOperation, Variable, Constant, Link
+         Variable, Constant, Link
+    from rpython.flowspace.operation import op
     for i in range(2):
         v_i = Variable("i")
-        v_case = Variable("case")
         block = Block([v_i])
         g = FunctionGraph("is_one", block)
-        block.operations.append(SpaceOperation("eq", [v_i, Constant(1)], v_case))
-        block.exitswitch = v_case
+        op1 = op.eq(v_i, Constant(1))
+        block.operations.append(op1)
+        block.exitswitch = op1.result
         tlink = Link([Constant(1)], g.returnblock, True)
         flink = Link([Constant(0)], g.returnblock, False)
         links = [tlink, flink]
