@@ -1,8 +1,10 @@
 import py
 
-from rpython.annotator import policy, specialize
+from rpython.annotator import model as annmodel
+from rpython.annotator import specialize
 from rpython.rtyper.lltypesystem.lltype import typeOf
 from rpython.rtyper.test.tool import BaseRtypingTest
+from rpython.rtyper.llannotation import SomePtr, lltype_to_annotation
 
 
 class MyBase:
@@ -1640,68 +1642,60 @@ class TestRPBC(BaseRtypingTest):
         res = self.interpret(g, [])
         assert res == False
 
+    def test_class___name__(self):
+        class Base(object): pass
+        class ASub(Base): pass
+        def g(n):
+            if n == 1:
+                x = Base()
+            else:
+                x = ASub()
+            return x.__class__.__name__
+        res = self.interpret(g, [1])
+        assert self.ll_to_string(res) == "Base"
+        res = self.interpret(g, [2])
+        assert self.ll_to_string(res) == "ASub"
+
+    def test_str_class(self):
+        class Base(object): pass
+        class ASub(Base): pass
+        def g(n):
+            if n == 1:
+                x = Base()
+            else:
+                x = ASub()
+            return str(x.__class__)
+        res = self.interpret(g, [1])
+        assert self.ll_to_string(res) == "Base"
+        res = self.interpret(g, [2])
+        assert self.ll_to_string(res) == "ASub"
+
+    def test_bug_callfamily(self):
+        def cb1():
+            xxx    # never actually called
+        def cb2():
+            pass
+        def g(cb, result):
+            assert (cb is None) == (result == 0)
+        def h(cb):
+            cb()
+        def f():
+            g(None, 0)
+            g(cb1, 1)
+            g(cb2, 2)
+            h(cb2)
+            return 42
+        res = self.interpret(f, [])
+        assert res == 42
+
 # ____________________________________________________________
 
-class TestRPBCExtra(BaseRtypingTest):
-
-    def test_folding_specialize_support(self):
-
-        class S(object):
-
-            def w(s, x):
-                if isinstance(x, int):
-                    return x
-                if isinstance(x, str):
-                    return len(x)
-                return -1
-            w._annspecialcase_ = "specialize:w"
-
-            def _freeze_(self):
-                return True
-
-        s = S()
-
-        def f(i, n):
-            w = s.w
-            if i == 0:
-                return w(0)
-            elif i == 1:
-                return w("abc")
-            elif i == 2:
-                return w(3*n)
-            elif i == 3:
-                return w(str(n))
-            return -1
-
-        class P(policy.AnnotatorPolicy):
-            def specialize__w(pol, funcdesc, args_s):
-                typ = args_s[1].knowntype
-                if args_s[0].is_constant() and args_s[1].is_constant():
-                    x = args_s[1].const
-                    v = s.w(x)
-                    builder = specialize.make_constgraphbuilder(2, v)
-                    return funcdesc.cachedgraph(x, builder=builder)
-                return funcdesc.cachedgraph(typ)
-
-        p = P()
-
-        res = self.interpret(f, [0, 66], policy=p)
-        assert res == 0
-        res = self.interpret(f, [1, 66], policy=p)
-        assert res == 3
-        res = self.interpret(f, [2, 4], policy=p)
-        assert res == 12
-        res = self.interpret(f, [3, 5555], policy=p)
-        assert res == 4
-
-
 def test_hlinvoke_simple():
     def f(a,b):
         return a + b
     from rpython.translator import translator
     from rpython.annotator import annrpython
     a = annrpython.RPythonAnnotator()
-    from rpython.annotator import model as annmodel
 
     s_f = a.bookkeeper.immutablevalue(f)
     a.bookkeeper.emulate_pbc_call('f', s_f, [annmodel.SomeInteger(), annmodel.SomeInteger()])
@@ -1720,7 +1714,7 @@ def test_hlinvoke_simple():
     r_f = rt.getrepr(s_f)
 
     s_R = a.bookkeeper.immutablevalue(r_f)
-    s_ll_f = annmodel.lltype_to_annotation(r_f.lowleveltype)
+    s_ll_f = lltype_to_annotation(r_f.lowleveltype)
     ll_h_graph = annlowlevel.annotate_lowlevel_helper(a, ll_h, [s_R, s_ll_f, annmodel.SomeInteger()])
     assert a.binding(ll_h_graph.getreturnvar()).knowntype == int
     rt.specialize_more_blocks()
@@ -1739,7 +1733,6 @@ def test_hlinvoke_simple2():
         return a - b
     from rpython.annotator import annrpython
     a = annrpython.RPythonAnnotator()
-    from rpython.annotator import model as annmodel
 
     def g(i):
         if i:
@@ -1768,7 +1761,7 @@ def test_hlinvoke_simple2():
     r_f = rt.getrepr(s_f)
 
     s_R = a.bookkeeper.immutablevalue(r_f)
-    s_ll_f = annmodel.lltype_to_annotation(r_f.lowleveltype)
+    s_ll_f = lltype_to_annotation(r_f.lowleveltype)
     ll_h_graph= annlowlevel.annotate_lowlevel_helper(a, ll_h, [s_R, s_ll_f, annmodel.SomeInteger()])
     assert a.binding(ll_h_graph.getreturnvar()).knowntype == int
     rt.specialize_more_blocks()
@@ -1791,7 +1784,6 @@ def test_hlinvoke_hltype():
 
     from rpython.annotator import annrpython
     a = annrpython.RPythonAnnotator()
-    from rpython.annotator import model as annmodel
 
     def g():
         a = A(None)
@@ -1814,10 +1806,11 @@ def test_hlinvoke_hltype():
     r_f = rt.getrepr(s_f)
 
     s_R = a.bookkeeper.immutablevalue(r_f)
-    s_ll_f = annmodel.lltype_to_annotation(r_f.lowleveltype)
+    s_ll_f = lltype_to_annotation(r_f.lowleveltype)
     A_repr = rclass.getinstancerepr(rt, a.bookkeeper.getdesc(A).
                                     getuniqueclassdef())
-    ll_h_graph = annlowlevel.annotate_lowlevel_helper(a, ll_h, [s_R, s_ll_f, annmodel.SomePtr(A_repr.lowleveltype)])
+    ll_h_graph = annlowlevel.annotate_lowlevel_helper(
+        a, ll_h, [s_R, s_ll_f, SomePtr(A_repr.lowleveltype)])
     s = a.binding(ll_h_graph.getreturnvar())
     assert s.ll_ptrtype == A_repr.lowleveltype
     rt.specialize_more_blocks()
@@ -1840,7 +1833,6 @@ def test_hlinvoke_method_hltype():
 
     from rpython.annotator import annrpython
     a = annrpython.RPythonAnnotator()
-    from rpython.annotator import model as annmodel
 
     def g():
         a = A(None)
@@ -1870,10 +1862,11 @@ def test_hlinvoke_method_hltype():
     r_f = rt.getrepr(s_f)
 
     s_R = a.bookkeeper.immutablevalue(r_f)
-    s_ll_f = annmodel.lltype_to_annotation(r_f.lowleveltype)
+    s_ll_f = lltype_to_annotation(r_f.lowleveltype)
     A_repr = rclass.getinstancerepr(rt, a.bookkeeper.getdesc(A).
                                     getuniqueclassdef())
-    ll_h_graph = annlowlevel.annotate_lowlevel_helper(a, ll_h, [s_R, s_ll_f, annmodel.SomePtr(A_repr.lowleveltype)])
+    ll_h_graph = annlowlevel.annotate_lowlevel_helper(
+        a, ll_h, [s_R, s_ll_f, SomePtr(A_repr.lowleveltype)])
     s = a.binding(ll_h_graph.getreturnvar())
     assert s.ll_ptrtype == A_repr.lowleveltype
     rt.specialize_more_blocks()
@@ -1900,7 +1893,6 @@ def test_hlinvoke_pbc_method_hltype():
 
     from rpython.annotator import annrpython
     a = annrpython.RPythonAnnotator()
-    from rpython.annotator import model as annmodel
 
     i = Impl()
 
@@ -1925,11 +1917,12 @@ def test_hlinvoke_pbc_method_hltype():
     r_f = rt.getrepr(s_f)
 
     s_R = a.bookkeeper.immutablevalue(r_f)
-    s_ll_f = annmodel.lltype_to_annotation(r_f.lowleveltype)
+    s_ll_f = lltype_to_annotation(r_f.lowleveltype)
 
     A_repr = rclass.getinstancerepr(rt, a.bookkeeper.getdesc(A).
                                     getuniqueclassdef())
-    ll_h_graph = annlowlevel.annotate_lowlevel_helper(a, ll_h, [s_R, s_ll_f, annmodel.SomePtr(A_repr.lowleveltype)])
+    ll_h_graph = annlowlevel.annotate_lowlevel_helper(
+        a, ll_h, [s_R, s_ll_f, SomePtr(A_repr.lowleveltype)])
     s = a.binding(ll_h_graph.getreturnvar())
     assert s.ll_ptrtype == A_repr.lowleveltype
     rt.specialize_more_blocks()
@@ -1952,7 +1945,7 @@ class TestSmallFuncSets(TestRPBC):
 
     def interpret(self, fn, args, **kwds):
         kwds['config'] = self.config
-        return TestRPBC.interpret(self, fn, args, **kwds)
+        return TestRPBC.interpret(fn, args, **kwds)
 
 def test_smallfuncsets_basic():
     from rpython.translator.translator import TranslationContext, graphof
