@@ -1,5 +1,3 @@
-from __future__ import print_function
-import time
 import py
 import os
 from subprocess import Popen, PIPE
@@ -12,25 +10,14 @@ def maywarn(err, repo_type='Mercurial'):
     if not err:
         return
 
-    from rpython.tool.ansi_print import AnsiLogger
-    log = AnsiLogger("version")
+    from rpython.tool.ansi_print import ansi_log
+    log = py.log.Producer("version")
+    py.log.setconsumer("version", ansi_log)
     log.WARNING('Errors getting %s information: %s' % (repo_type, err))
 
-CACHED_RESULT = None
-CACHED_ARGS = None
-
 def get_repo_version_info(hgexe=None, root=rpythonroot):
     '''Obtain version information by invoking the 'hg' or 'git' commands.'''
-    global CACHED_RESULT, CACHED_ARGS
-    key = (hgexe, root)
-    if CACHED_RESULT is not None and CACHED_ARGS == key:
-        return CACHED_RESULT
-    res = _get_repo_version_info(hgexe, root)
-    CACHED_RESULT = res
-    CACHED_ARGS = key
-    return res
 
-def _get_repo_version_info(hgexe, root):
     # Try to see if we can get info from Git if hgexe is not specified.
     if not hgexe:
         if os.path.isdir(os.path.join(root, '.git')):
@@ -61,7 +48,7 @@ def _get_hg_version(hgexe, root):
 
     try:
         p = Popen([str(hgexe), 'version', '-q'],
-                  stdout=PIPE, stderr=PIPE, env=env, universal_newlines=True)
+                  stdout=PIPE, stderr=PIPE, env=env)
     except OSError as e:
         maywarn(e)
         return default_retval
@@ -70,27 +57,38 @@ def _get_hg_version(hgexe, root):
         maywarn('command does not identify itself as Mercurial')
         return default_retval
 
-    p = Popen([str(hgexe), 'id', '--template', r"{id}\n{tags}\n{branch}\n", root],
-              stdout=PIPE, stderr=PIPE, env=env,
-              universal_newlines=True)
-    hgout = p.stdout.read().strip()
+    p = Popen([str(hgexe), 'id', '-i', root],
+              stdout=PIPE, stderr=PIPE, env=env)
+    hgid = p.stdout.read().strip()
+    maywarn(p.stderr.read())
     if p.wait() != 0:
-        maywarn(p.stderr.read())
-        hgout = '?\n?\n?'
-    hgid, hgtags, hgbranch = hgout.strip().split("\n", 3)
-    hgtags = [t for t in hgtags.strip().split() if t != 'tip']
+        hgid = '?'
+
+    p = Popen([str(hgexe), 'id', '-t', root],
+              stdout=PIPE, stderr=PIPE, env=env)
+    hgtags = [t for t in p.stdout.read().strip().split() if t != 'tip']
+    maywarn(p.stderr.read())
+    if p.wait() != 0:
+        hgtags = ['?']
 
     if hgtags:
         return hgtags[0], hgid
     else:
+        # use the branch instead
+        p = Popen([str(hgexe), 'id', '-b', root],
+                  stdout=PIPE, stderr=PIPE, env=env)
+        hgbranch = p.stdout.read().strip()
+        maywarn(p.stderr.read())
+
         return hgbranch, hgid
 
 
 def _get_hg_archive_version(path):
-    with open(path) as fp:
-        # reverse the order since there may be more than one tag
-        # and the latest tag will be first, so make it last instead
-        data = dict((x.split(': ', 1) for x in fp.read().splitlines()[::-1]))
+    fp = open(path)
+    try:
+        data = dict(x.split(': ', 1) for x in fp.read().splitlines())
+    finally:
+        fp.close()
     if 'tag' in data:
         return data['tag'], data['node']
     else:
@@ -107,8 +105,7 @@ def _get_git_version(root):
     try:
         p = Popen(
             [str(gitexe), 'rev-parse', 'HEAD'],
-            stdout=PIPE, stderr=PIPE, cwd=root,
-            universal_newlines=True,
+            stdout=PIPE, stderr=PIPE, cwd=root
             )
     except OSError as e:
         maywarn(e, 'Git')
@@ -119,13 +116,12 @@ def _get_git_version(root):
     revision_id = p.stdout.read().strip()[:12]
     p = Popen(
         [str(gitexe), 'describe', '--tags', '--exact-match'],
-        stdout=PIPE, stderr=PIPE, cwd=root,
-        universal_newlines=True,
+        stdout=PIPE, stderr=PIPE, cwd=root
         )
     if p.wait() != 0:
         p = Popen(
             [str(gitexe), 'branch'], stdout=PIPE, stderr=PIPE,
-            cwd=root, universal_newlines=True,
+            cwd=root
             )
         if p.wait() != 0:
             maywarn(p.stderr.read(), 'Git')
@@ -142,4 +138,4 @@ def _get_git_version(root):
 
 
 if __name__ == '__main__':
-    print(get_repo_version_info())
+    print get_repo_version_info()
