@@ -1,8 +1,8 @@
+from rpython.translator.backendopt.raisingop2direct_call import raisingop2direct_call
 from rpython.translator.backendopt import removenoops
 from rpython.translator.backendopt import inline
 from rpython.translator.backendopt.malloc import remove_mallocs
 from rpython.translator.backendopt.constfold import constant_fold_graph
-from rpython.translator.backendopt.constfold import replace_we_are_jitted
 from rpython.translator.backendopt.stat import print_statistics
 from rpython.translator.backendopt.merge_if_blocks import merge_if_blocks
 from rpython.translator import simplify
@@ -21,7 +21,7 @@ def get_function(dottedname):
     name = parts[-1]
     try:
         mod = __import__(module, {}, {}, ['__doc__'])
-    except ImportError as e:
+    except ImportError, e:
         raise Exception("Import error loading %s: %s" % (dottedname, e))
 
     try:
@@ -34,10 +34,9 @@ def get_function(dottedname):
 def backend_optimizations(translator, graphs=None, secondary=False,
                           inline_graph_from_anywhere=False, **kwds):
     # sensible keywords are
-    # inline_threshold, mallocs
+    # raisingop2direct_call, inline_threshold, mallocs
     # merge_if_blocks, constfold, heap2stack
     # clever_malloc_removal, remove_asserts
-    # replace_we_are_jitted
 
     config = translator.config.translation.backendopt.copy(as_default=True)
     config.set(**kwds)
@@ -51,9 +50,8 @@ def backend_optimizations(translator, graphs=None, secondary=False,
         print "before optimizations:"
         print_statistics(translator.graphs[0], translator, "per-graph.txt")
 
-    if config.replace_we_are_jitted:
-        for graph in graphs:
-            replace_we_are_jitted(graph)
+    if config.raisingop2direct_call:
+        raisingop2direct_call(translator, graphs)
 
     if config.remove_asserts:
         constfold(config, graphs)
