@@ -1,4 +1,3 @@
-import math
 from rpython.rtyper.lltypesystem.lltype import (Struct, Array, FixedSizeArray,
     FuncType, typeOf, GcStruct, GcArray, RttiStruct, ContainerType, parentlink,
     Void, OpaqueType, Float, RuntimeTypeInfo, getRuntimeTypeInfo, Char,
@@ -9,13 +8,11 @@ from rpython.translator.c.support import USESLOTS # set to False if necessary wh
 from rpython.translator.c.support import cdecl, forward_cdecl, somelettersfrom
 from rpython.translator.c.support import c_char_array_constant, barebonearray
 from rpython.translator.c.primitive import PrimitiveType, name_signed
-from rpython.rlib import exports, objectmodel
-from rpython.rlib.rfloat import isfinite
+from rpython.rlib import exports
+from rpython.rlib.rfloat import isfinite, isinf
 
 
-def needs_gcheader(gctransformer, T):
-    if getattr(gctransformer, 'NO_HEADER', False):   # for boehm
-        return False
+def needs_gcheader(T):
     if not isinstance(T, ContainerType):
         return False
     if T._gckind != 'gc':
@@ -90,7 +87,7 @@ class StructDefNode(NodeWithDependencies):
         STRUCT = self.STRUCT
         if self.varlength is not None:
             self.normalizedtypename = db.gettype(STRUCT, who_asks=self)
-        if needs_gcheader(db.gctransformer, self.STRUCT):
+        if needs_gcheader(self.STRUCT):
             HDR = db.gcpolicy.struct_gcheader_definition(self)
             if HDR is not None:
                 gc_field = ("_gcheader", db.gettype(HDR, who_asks=self))
@@ -216,7 +213,7 @@ class ArrayDefNode(NodeWithDependencies):
         self.computegcinfo(db.gcpolicy)
         if self.varlength is not None:
             self.normalizedtypename = db.gettype(ARRAY, who_asks=self)
-        if needs_gcheader(db.gctransformer, ARRAY):
+        if needs_gcheader(ARRAY):
             HDR = db.gcpolicy.array_gcheader_definition(self)
             if HDR is not None:
                 gc_field = ("_gcheader", db.gettype(HDR, who_asks=self))
@@ -255,12 +252,9 @@ class ArrayDefNode(NodeWithDependencies):
         for fname, typename in self.gcfields:
             yield '\t' + cdecl(typename, fname) + ';'
         if not self.ARRAY._hints.get('nolength', False):
-            yield '\tSigned length;'
-        varlength = self.varlength
-        if varlength is not None:
-            varlength += self.ARRAY._hints.get('extra_item_after_alloc', 0)
+            yield '\tlong length;'
         line = '%s;' % cdecl(self.itemtypename,
-                             'items[%s]' % deflength(varlength))
+                             'items[%s]' % deflength(self.varlength))
         if self.ARRAY.OF is Void:    # strange
             line = '/* array of void */'
             if self.ARRAY._hints.get('nolength', False):
@@ -503,14 +497,14 @@ class ContainerNode(Node):
             assert self.implementationtypename.startswith('struct ')
             assert self.implementationtypename.endswith(' @')
             uniontypename = 'union %su @' % self.implementationtypename[7:-2]
-            return uniontypename, self.name[:-2], True
+            return uniontypename, self.name[:-2]
         else:
-            return self.implementationtypename, self.name, False
+            return self.implementationtypename, self.name
 
     def forward_declaration(self):
         if llgroup.member_of_group(self.obj):
             return
-        type, name, is_union = self.get_declaration()
+        type, name = self.get_declaration()
         yield '%s;' % (
             forward_cdecl(type, name, self.db.standalone,
                           is_thread_local=self.is_thread_local(),
@@ -520,12 +514,12 @@ class ContainerNode(Node):
         if llgroup.member_of_group(self.obj):
             return []
         lines = list(self.initializationexpr())
-        type, name, is_union = self.get_declaration()
-        if is_union and len(lines) < 2:
+        type, name = self.get_declaration()
+        if name != self.name and len(lines) < 2:
             # a union with length 0
             lines[0] = cdecl(type, name, self.is_thread_local())
         else:
-            if is_union:
+            if name != self.name:
                 lines[0] = '{ ' + lines[0]    # extra braces around the 'a' part
                 lines[-1] += ' }'             # of the union
             lines[0] = '%s = %s' % (
@@ -549,10 +543,11 @@ class StructNode(ContainerNode):
 
     def __init__(self, db, T, obj):
         ContainerNode.__init__(self, db, T, obj)
-        gct = self.db.gctransformer
-        if needs_gcheader(gct, T):
+        if needs_gcheader(T):
+            gct = self.db.gctransformer
             if gct is not None:
                 self.gc_init = gct.gcheader_initdata(self.obj)
+                db.getcontainernode(self.gc_init)
             else:
                 self.gc_init = None
 
@@ -580,28 +575,18 @@ class StructNode(ContainerNode):
 
         data = []
 
-        if needs_gcheader(self.db.gctransformer, T):
+        if needs_gcheader(T):
             data.append(('gcheader', self.gc_init))
 
         for name in defnode.fieldnames:
             data.append((name, getattr(self.obj, name)))
 
-        if T._hints.get('remove_hash'):
-            # hack for rstr.STR and UNICODE: remove their .hash value
-            # and write 0 in the C sources, if we're using a non-default
-            # hash function.
-            if hasattr(self.db.translator, 'll_hash_string'):
-                i = 0
-                while data[i][0] != 'hash':
-                    i += 1
-                data[i] = ('hash', 0)
-
         # Reasonably, you should only initialise one of the fields of a union
         # in C.  This is possible with the syntax '.fieldname value' or
         # '.fieldname = value'.  But here we don't know which of the
         # fields need initialization, so XXX we pick the first one
         # arbitrarily.
-        if T._hints.get('union'):
+        if hasattr(T, "_hints") and T._hints.get('union'):
             data = data[0:1]
 
         if 'get_padding_drop' in T._hints:
@@ -613,8 +598,8 @@ class StructNode(ContainerNode):
             padding_drop = T._hints['get_padding_drop'](d)
         else:
             padding_drop = []
-        type, name, is_union = self.get_declaration()
-        if is_union and self.getvarlength() < 1 and len(data) < 2:
+        type, name = self.get_declaration()
+        if name != self.name and self.getvarlength() < 1 and len(data) < 2:
             # an empty union
             yield ''
             return
@@ -636,6 +621,51 @@ class StructNode(ContainerNode):
 
 assert not USESLOTS or '__dict__' not in dir(StructNode)
 
+class GcStructNodeWithHash(StructNode):
+    # for the outermost level of nested structures, if it has a _hash_cache_.
+    nodekind = 'struct'
+    if USESLOTS:
+        __slots__ = ()
+
+    def get_hash_typename(self):
+        return 'struct _hashT_%s @' % self.name
+
+    def forward_declaration(self):
+        T = self.getTYPE()
+        assert self.typename == self.implementationtypename  # no array part
+        hash_typename = self.get_hash_typename()
+        hash_offset = self.db.gctransformer.get_hash_offset(T)
+        yield '%s {' % cdecl(hash_typename, '')
+        yield '\tunion {'
+        yield '\t\t%s;' % cdecl(self.implementationtypename, 'head')
+        yield '\t\tchar pad[%s];' % name_signed(hash_offset, self.db)
+        yield '\t} u;'
+        yield '\tlong hash;'
+        yield '};'
+        yield '%s;' % (
+            forward_cdecl(hash_typename, '_hash_' + self.name,
+                          self.db.standalone, self.is_thread_local()),)
+        yield '#define %s _hash_%s.u.head' % (self.name, self.name)
+
+    def implementation(self):
+        hash_typename = self.get_hash_typename()
+        hash = self.db.gctransformer.get_prebuilt_hash(self.obj)
+        assert hash is not None
+        lines = list(self.initializationexpr())
+        lines.insert(0, '%s = { {' % (
+            cdecl(hash_typename, '_hash_' + self.name,
+                  self.is_thread_local()),))
+        lines.append('}, %s /* hash */ };' % name_signed(hash, self.db))
+        return lines
+
+def gcstructnode_factory(db, T, obj):
+    if (db.gctransformer and
+            db.gctransformer.get_prebuilt_hash(obj) is not None):
+        cls = GcStructNodeWithHash
+    else:
+        cls = StructNode
+    return cls(db, T, obj)
+
 
 class ArrayNode(ContainerNode):
     nodekind = 'array'
@@ -644,10 +674,11 @@ class ArrayNode(ContainerNode):
 
     def __init__(self, db, T, obj):
         ContainerNode.__init__(self, db, T, obj)
-        gct = self.db.gctransformer
-        if needs_gcheader(gct, T):
+        if needs_gcheader(T):
+            gct = self.db.gctransformer
             if gct is not None:
                 self.gc_init = gct.gcheader_initdata(self.obj)
+                db.getcontainernode(self.gc_init)
             else:
                 self.gc_init = None
 
@@ -668,7 +699,7 @@ class ArrayNode(ContainerNode):
     def initializationexpr(self, decoration=''):
         T = self.getTYPE()
         yield '{'
-        if needs_gcheader(self.db.gctransformer, T):
+        if needs_gcheader(T):
             lines = generic_initializationexpr(self.db, self.gc_init, 'gcheader',
                                                '%sgcheader' % (decoration,))
             for line in lines:
@@ -757,7 +788,7 @@ def generic_initializationexpr(db, value, access_expr, decoration):
         comma = ','
         if typeOf(value) == Float and not isfinite(value):
             db.late_initializations.append(('%s' % access_expr, db.get(value)))
-            if math.isinf(value):
+            if isinf(value):
                 name = '-+'[value > 0] + 'inf'
             else:
                 name = 'NaN'
@@ -816,7 +847,8 @@ class FuncNode(FuncNodeBase):
                 self.name, self.db.standalone, is_exported=is_exported))
 
     def graphs_to_patch(self):
-        return self.funcgen.graphs_to_patch()
+        for i in self.funcgen.graphs_to_patch():
+            yield i
 
     def implementation(self):
         funcgen = self.funcgen
@@ -947,7 +979,7 @@ class ExtType_OpaqueNode(ContainerNode):
 def opaquenode_factory(db, T, obj):
     if T == RuntimeTypeInfo:
         return db.gcpolicy.rtti_node_factory()(db, T, obj)
-    if T._hints.get("render_structure", False):
+    if T.hints.get("render_structure", False):
         return ExtType_OpaqueNode(db, T, obj)
     raise Exception("don't know about %r" % (T,))
 
@@ -1024,7 +1056,7 @@ class GroupNode(ContainerNode):
 
 ContainerNodeFactory = {
     Struct:       StructNode,
-    GcStruct:     StructNode,
+    GcStruct:     gcstructnode_factory,
     Array:        ArrayNode,
     GcArray:      ArrayNode,
     FixedSizeArray: FixedSizeArrayNode,
