@@ -17,7 +17,6 @@ class TestTypedOptimizedTestCase(_TestTypedTestCase):
         assert fn(False) == 456
 
     def test__del__(self):
-        import gc
         class B(object):
             pass
         b = B()
@@ -35,7 +34,6 @@ class TestTypedOptimizedTestCase(_TestTypedTestCase):
             a = A()
             for i in range(x):
                 a = A()
-            gc.collect()
             return b.num_deleted
 
         fn = self.getcompiled(f, [int], gcpolicy='ref')
@@ -46,7 +44,6 @@ class TestTypedOptimizedTestCase(_TestTypedTestCase):
         assert res == 6
 
     def test_del_inheritance(self):
-        import gc
         class State:
             pass
         s = State()
@@ -67,7 +64,6 @@ class TestTypedOptimizedTestCase(_TestTypedTestCase):
             A()
             B()
             C()
-            gc.collect()
             if x:
                 return s.a_dels * 10 + s.b_dels
             else:
@@ -208,7 +204,7 @@ class TestTypedOptimizedSwitchTestCase:
 
 class TestTypedOptimizedRaisingOps:
     def getcompiled(self, func, argtypes):
-        return compile(func, argtypes)
+        return compile(func, argtypes, raisingop2direct_call=True)
 
     def test_int_floordiv_zer(self):
         def f(x):
