@@ -1,9 +1,10 @@
 import ctypes
+from collections import OrderedDict
 
 import py
 
 from rpython.rlib.rfloat import NAN, INFINITY
-from rpython.rlib.entrypoint import entrypoint
+from rpython.rlib.entrypoint import entrypoint_highlevel
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rlib.rarithmetic import r_longlong, r_ulonglong, r_uint, intmask
 from rpython.rlib.objectmodel import specialize
@@ -50,7 +51,7 @@ def parse_ulonglong(a):
                                            unsigned_ffffffff)
 
 def compile(fn, argtypes, view=False, gcpolicy="none", backendopt=True,
-            annotatorpolicy=None, thread=False):
+            annotatorpolicy=None, thread=False, **kwds):
     argtypes_unroll = unrolling_iterable(enumerate(argtypes))
 
     for argtype in argtypes:
@@ -98,7 +99,7 @@ def compile(fn, argtypes, view=False, gcpolicy="none", backendopt=True,
         return 0
 
     t = Translation(entry_point, None, gc=gcpolicy, backend="c",
-                    policy=annotatorpolicy, thread=thread)
+                    policy=annotatorpolicy, thread=thread, **kwds)
     if not backendopt:
         t.disable(["backendopt_lltype"])
     t.driver.config.translation.countmallocs = True
@@ -480,6 +481,7 @@ def test_name():
         return 3
 
     f.c_name = 'pypy_xyz_f'
+    f.exported_symbol = True
 
     t = Translation(f, [], backend="c")
     t.annotate()
@@ -493,7 +495,7 @@ def test_entrypoints():
         return 3
 
     key = "test_entrypoints42"
-    @entrypoint(key, [int], "foobar")
+    @entrypoint_highlevel(key, [int], "foobar")
     def g(x):
         return x + 42
 
@@ -512,9 +514,6 @@ def test_exportstruct():
     FOO = Struct("FOO", ("field1", Signed))
     foo = malloc(FOO, flavor="raw")
     foo.field1 = 43
-    # maybe export_struct should add the struct name to eci automatically?
-    # https://bugs.pypy.org/issue1361
-    foo._obj._compilation_info = ExternalCompilationInfo(export_symbols=['BarStruct'])
     export_struct("BarStruct", foo._obj)
     t = Translation(f, [], backend="c")
     t.annotate()
@@ -574,6 +573,17 @@ def test_recursive_llhelper():
     fn = compile(chooser, [bool])
     assert fn(True)
 
+def test_ordered_dict():
+    expected = [('ea', 1), ('bb', 2), ('c', 3), ('d', 4), ('e', 5),
+                ('ef', 6)]
+    d = OrderedDict(expected)
+
+    def f():
+        assert d.items() == expected
+
+    fn = compile(f, [])
+    fn()
+
 def test_inhibit_tail_call():
     def foobar_fn(n):
         return 42
@@ -586,7 +596,7 @@ def test_inhibit_tail_call():
     t.context._graphof(foobar_fn).inhibit_tail_call = True
     t.source_c()
     lines = t.driver.cbuilder.c_source_filename.join('..',
-                              'rpython_translator_c_test_test_genc.c').readlines()
+                              'rpython_translator_c_test.c').readlines()
     for i, line in enumerate(lines):
         if '= pypy_g_foobar_fn' in line:
             break
