@@ -5,7 +5,7 @@ import platform
 import sys
 from rpython.translator.platform.posix import BasePosix
 
-class Linux(BasePosix):
+class BaseLinux(BasePosix):
     name = "linux"
 
     link_flags = tuple(
@@ -14,17 +14,14 @@ class Linux(BasePosix):
     extra_libs = ('-lrt',)
     cflags = tuple(
              ['-O3', '-pthread', '-fomit-frame-pointer',
-              '-Wall', '-Wno-unused', '-Wno-address']
+              '-Wall', '-Wno-unused']
              + os.environ.get('CFLAGS', '').split())
     standalone_only = ()
     shared_only = ('-fPIC',)
     so_ext = 'so'
+    so_prefixes = ('lib', '')
 
-    if platform.machine() == 's390x':
-        from rpython.translator.platform.arch import s390x
-        cflags = s390x.update_cflags(cflags)
-
-    def _args_for_shared(self, args, **kwds):
+    def _args_for_shared(self, args):
         return ['-shared'] + args
 
     def _include_dirs_for_libffi(self):
@@ -36,3 +33,14 @@ class Linux(BasePosix):
         return self._pkg_config("libffi", "--libs-only-L",
                                 ['/usr/lib/libffi'],
                                 check_result_dir=True)
+
+
+class Linux(BaseLinux):
+    if platform.machine().startswith('arm'):
+        shared_only = ('-fPIC',) # ARM requires compiling with -fPIC
+    else:
+        shared_only = () # it seems that on 32-bit linux, compiling with -fPIC
+                         # gives assembler that asmgcc is not happy about.
+
+class LinuxPIC(BaseLinux):
+    pass
