@@ -10,11 +10,13 @@ import types
 from rpython.translator import simplify
 from rpython.flowspace.model import FunctionGraph, checkgraph, Block
 from rpython.flowspace.objspace import build_flow
-from rpython.tool.ansi_print import AnsiLogger
+from rpython.tool.ansi_print import ansi_log
 from rpython.tool.sourcetools import nice_repr_for_func
 from rpython.config.translationoption import get_platform
 
-log = AnsiLogger("flowgraph")
+import py
+log = py.log.Producer("flowgraph")
+py.log.setconsumer("flowgraph", ansi_log)
 
 class TranslationContext(object):
     FLOWING_FLAGS = {
@@ -38,7 +40,6 @@ class TranslationContext(object):
         self.graphs = []      # [graph]
         self.callgraph = {}   # {opaque_tag: (caller-graph, callee-graph)}
         self._prebuilt_graphs = {}   # only used by the pygame viewer
-        self._call_at_startup = []
 
     def buildflowgraph(self, func, mute_dot=False):
         """Get the flow graph for a function."""
@@ -49,12 +50,14 @@ class TranslationContext(object):
             graph = self._prebuilt_graphs.pop(func)
         else:
             if self.config.translation.verbose:
-                log(nice_repr_for_func(func))
+                log.start(nice_repr_for_func(func))
             graph = build_flow(func)
             simplify.simplify_graph(graph)
             if self.config.translation.list_comprehension_operations:
                 simplify.detect_list_comprehension(graph)
-            if not self.config.translation.verbose and not mute_dot:
+            if self.config.translation.verbose:
+                log.done(func.__name__)
+            elif not mute_dot:
                 log.dot()
             self.graphs.append(graph)   # store the graph in our list
         return graph
@@ -68,8 +71,7 @@ class TranslationContext(object):
         if self.annotator is not None:
             raise ValueError("we already have an annotator")
         from rpython.annotator.annrpython import RPythonAnnotator
-        self.annotator = RPythonAnnotator(
-            self, policy=policy, keepgoing=self.config.translation.keepgoing)
+        self.annotator = RPythonAnnotator(self, policy=policy)
         return self.annotator
 
     def buildrtyper(self):
@@ -141,9 +143,6 @@ def graphof(translator, func):
     if isinstance(func, FunctionGraph):
         return func
     result = []
-    if hasattr(func, 'im_func'):
-        # make it possible to translate bound methods
-        func = func.im_func
     for graph in translator.graphs:
         if getattr(graph, 'func', None) is func:
             result.append(graph)
